Type: Package
Package: mschart
Title: Chart Generation for 'Microsoft Word', 'Microsoft Excel' and
        'Microsoft PowerPoint' Documents
Version: 0.5.1
Authors@R: c(
    person("David", "Gohel", , "david.gohel@ardata.fr", role = c("aut", "cre")),
    person("ArData", role = "cph"),
    person("YouGov", role = "fnd"),
    person("Jan Marvin", "Garbuszus", role = "ctb",
           comment = "support for openxlsx2 and combo charts"),
    person("Stefan", "Moog", , "moogs@gmx.de", role = "ctb",
           comment = "support to set chart and plot area color and border, fix for line style none"),
    person("Eli", "Daniels", , "eli.daniels@ardata.fr", role = "ctb"),
    person("Marlon", "Molina", role = "ctb",
           comment = "added table feature"),
    person("Rokas", "Klydzia", role = "ctb",
           comment = "custom labels"),
    person("David", "Camposeco", , "david.camposeco.paulsen@gmail.com", role = "ctb",
           comment = "chart_data_smooth function"),
    person("Dan", "Joplin", role = "ctb",
           comment = "fix scatter plot data structure")
  )
Description: Create native charts for 'Microsoft PowerPoint', 'Microsoft
    Excel' and 'Microsoft Word' documents. The resulting charts can then
    be edited and annotated in the host application. It provides functions
    to create charts and to modify their content and formatting. The
    chart's underlying data is automatically saved within the 'Word',
    'Excel' or 'PowerPoint' file. It extends the 'officer' package, which
    does not provide native 'Microsoft' chart production.
License: MIT + file LICENSE
URL: https://ardata-fr.github.io/officeverse/,
        https://ardata-fr.github.io/mschart/
BugReports: https://github.com/ardata-fr/mschart/issues
Depends: R (>= 3.5)
Imports: cellranger, data.table, grDevices, htmltools, officer (>=
        0.7.5), scales, stats, utils, writexl, xml2 (>= 1.1.0)
Suggests: doconv (>= 0.4.0), testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-20 14:18:38 UTC; davidgohel
Author: David Gohel [aut, cre],
  ArData [cph],
  YouGov [fnd],
  Jan Marvin Garbuszus [ctb] (support for openxlsx2 and combo charts),
  Stefan Moog [ctb] (support to set chart and plot area color and border,
    fix for line style none),
  Eli Daniels [ctb],
  Marlon Molina [ctb] (added table feature),
  Rokas Klydzia [ctb] (custom labels),
  David Camposeco [ctb] (chart_data_smooth function),
  Dan Joplin [ctb] (fix scatter plot data structure)
Maintainer: David Gohel <david.gohel@ardata.fr>
Repository: CRAN
Date/Publication: 2026-07-20 14:50:02 UTC
Built: R 4.5.2; ; 2026-07-20 17:24:57 UTC; unix

Type: Package
Package: mschart
Title: Chart Generation for 'Microsoft Word', 'Microsoft Excel' and
        'Microsoft PowerPoint' Documents
Version: 0.5.0
Authors@R: c(
    person("David", "Gohel", , "david.gohel@ardata.fr", role = c("aut", "cre")),
    person("ArData", role = "cph"),
    person("YouGov", role = "fnd"),
    person("Jan Marvin", "Garbuszus", role = "ctb",
           comment = "support for openxlsx2 and combo charts"),
    person("Stefan", "Moog", , "moogs@gmx.de", role = "ctb",
           comment = "support to set chart and plot area color and border, fix for line style none"),
    person("Eli", "Daniels", , "eli.daniels@ardata.fr", role = "ctb"),
    person("Marlon", "Molina", role = "ctb",
           comment = "added table feature"),
    person("Rokas", "Klydzia", role = "ctb",
           comment = "custom labels"),
    person("David", "Camposeco", , "david.camposeco.paulsen@gmail.com", role = "ctb",
           comment = "chart_data_smooth function"),
    person("Dan", "Joplin", role = "ctb",
           comment = "fix scatter plot data structure")
  )
Description: Create native charts for 'Microsoft PowerPoint', 'Microsoft
    Excel' and 'Microsoft Word' documents. The resulting charts can then
    be edited and annotated in the host application. It provides functions
    to create charts and to modify their content and formatting. The
    chart's underlying data is automatically saved within the 'Word',
    'Excel' or 'PowerPoint' file. It extends the 'officer' package, which
    does not provide native 'Microsoft' chart production.
License: MIT + file LICENSE
URL: https://ardata-fr.github.io/officeverse/,
        https://ardata-fr.github.io/mschart/
BugReports: https://github.com/ardata-fr/mschart/issues
Depends: R (>= 3.5)
Imports: cellranger, data.table, grDevices, htmltools, officer (>=
        0.7.5), scales, stats, utils, writexl, xml2 (>= 1.1.0)
Suggests: doconv (>= 0.4.0), testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-16 15:18:39 UTC; davidgohel
Author: David Gohel [aut, cre],
  ArData [cph],
  YouGov [fnd],
  Jan Marvin Garbuszus [ctb] (support for openxlsx2 and combo charts),
  Stefan Moog [ctb] (support to set chart and plot area color and border,
    fix for line style none),
  Eli Daniels [ctb],
  Marlon Molina [ctb] (added table feature),
  Rokas Klydzia [ctb] (custom labels),
  David Camposeco [ctb] (chart_data_smooth function),
  Dan Joplin [ctb] (fix scatter plot data structure)
Maintainer: David Gohel <david.gohel@ardata.fr>
Repository: CRAN
Date/Publication: 2026-05-16 21:50:37 UTC
Built: R 4.5.2; ; 2026-05-17 02:03:08 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(chart_ax_x,default)
S3method(chart_ax_x,ms_chart_ex)
S3method(chart_ax_y,default)
S3method(chart_ax_y,ms_chart_ex)
S3method(chart_data_fill,default)
S3method(chart_data_fill,ms_chart_ex)
S3method(chart_data_labels,default)
S3method(chart_data_labels,ms_chart_ex)
S3method(chart_data_stroke,default)
S3method(chart_data_stroke,ms_chart_ex)
S3method(chart_extra_parts,ms_chart)
S3method(chart_extra_parts,ms_chart_ex)
S3method(chart_graphicdata_uri,ms_chart)
S3method(chart_graphicdata_uri,ms_chart_ex)
S3method(chart_labels_text,default)
S3method(chart_labels_text,ms_chart_ex)
S3method(chart_part_content_type,ms_chart)
S3method(chart_part_content_type,ms_chart_ex)
S3method(chart_part_rel_type,ms_chart)
S3method(chart_part_rel_type,ms_chart_ex)
S3method(chart_reference_xml,ms_chart)
S3method(chart_reference_xml,ms_chart_ex)
S3method(chart_settings,ms_areachart)
S3method(chart_settings,ms_barchart)
S3method(chart_settings,ms_boxplotchart)
S3method(chart_settings,ms_bubblechart)
S3method(chart_settings,ms_chart_ex)
S3method(chart_settings,ms_linechart)
S3method(chart_settings,ms_paretochart)
S3method(chart_settings,ms_piechart)
S3method(chart_settings,ms_radarchart)
S3method(chart_settings,ms_scatterchart)
S3method(chart_settings,ms_stockchart)
S3method(cx_fill_cat_column,default)
S3method(cx_fill_cat_column,ms_histogramchart)
S3method(cx_fill_cat_column,ms_paretochart)
S3method(cx_fill_cat_column,ms_sunburstchart)
S3method(cx_fill_cat_column,ms_treemapchart)
S3method(cx_uses_connectors,default)
S3method(cx_uses_connectors,ms_waterfallchart)
S3method(format,ms_boxplotchart)
S3method(format,ms_chart)
S3method(format,ms_funnelchart)
S3method(format,ms_histogramchart)
S3method(format,ms_paretochart)
S3method(format,ms_sunburstchart)
S3method(format,ms_treemapchart)
S3method(format,ms_waterfallchart)
S3method(ph_with,ms_chart)
S3method(print,ms_boxplotchart)
S3method(print,ms_chart)
S3method(print,ms_funnelchart)
S3method(print,ms_histogramchart)
S3method(print,ms_paretochart)
S3method(print,ms_sunburstchart)
S3method(print,ms_treemapchart)
S3method(print,ms_waterfallchart)
S3method(sheet_add_drawing,ms_chart)
S3method(to_pml,date_ref)
S3method(to_pml,label_ref)
S3method(to_pml,labels_options)
S3method(to_pml,ms_areachart)
S3method(to_pml,ms_barchart)
S3method(to_pml,ms_bubblechart)
S3method(to_pml,ms_linechart)
S3method(to_pml,ms_piechart)
S3method(to_pml,ms_radarchart)
S3method(to_pml,ms_scatterchart)
S3method(to_pml,ms_stockchart)
S3method(to_pml,num_ref)
S3method(to_pml,str_ref)
S3method(update,col_ref)
export(as_bar_stack)
export(body_add_chart)
export(chart_ax_x)
export(chart_ax_y)
export(chart_data_fill)
export(chart_data_labels)
export(chart_data_line_style)
export(chart_data_line_width)
export(chart_data_size)
export(chart_data_smooth)
export(chart_data_stroke)
export(chart_data_symbol)
export(chart_fill_ggplot2)
export(chart_labels)
export(chart_labels_text)
export(chart_settings)
export(chart_table)
export(chart_theme)
export(ms_areachart)
export(ms_barchart)
export(ms_boxplotchart)
export(ms_bubblechart)
export(ms_chart_combine)
export(ms_funnelchart)
export(ms_histogramchart)
export(ms_linechart)
export(ms_paretochart)
export(ms_piechart)
export(ms_radarchart)
export(ms_scatterchart)
export(ms_stockchart)
export(ms_sunburstchart)
export(ms_treemapchart)
export(ms_waterfallchart)
export(mschart_theme)
export(set_theme)
export(theme_ggplot2)
importFrom(cellranger,as.cell_limits)
importFrom(cellranger,as.range)
importFrom(cellranger,cell_limits)
importFrom(cellranger,ra_ref)
importFrom(cellranger,to_string)
importFrom(data.table,":=")
importFrom(data.table,.N)
importFrom(data.table,as.data.table)
importFrom(data.table,dcast.data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,setDF)
importFrom(data.table,setnames)
importFrom(data.table,setorderv)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colors)
importFrom(grDevices,rgb)
importFrom(htmltools,htmlEscape)
importFrom(officer,add_slide)
importFrom(officer,body_add_xml)
importFrom(officer,fp_border)
importFrom(officer,fp_text)
importFrom(officer,ph_location_fullsize)
importFrom(officer,ph_with)
importFrom(officer,read_pptx)
importFrom(officer,sheet_add_drawing)
importFrom(officer,sheet_write_data)
importFrom(officer,styles_info)
importFrom(officer,to_pml)
importFrom(officer,uuid_generate)
importFrom(officer,xlsx_drawing)
importFrom(scales,hue_pal)
importFrom(stats,as.formula)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(utils,browseURL)
importFrom(writexl,write_xlsx)
importFrom(xml2,"xml_attr<-")
importFrom(xml2,as_xml_document)
importFrom(xml2,read_xml)
importFrom(xml2,write_xml)
importFrom(xml2,xml_add_child)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_find_first)
importFrom(xml2,xml_remove)
importFrom(xml2,xml_replace)

mschart R package
================

<!-- README.md is generated from README.Rmd. Please edit that file -->

[![R build
status](https://github.com/ardata-fr/mschart/workflows/R-CMD-check/badge.svg)](https://github.com/ardata-fr/mschart/actions)
[![version](https://www.r-pkg.org/badges/version/mschart)](https://CRAN.R-project.org/package=mschart)
[![test
coverage](https://codecov.io/gh/ardata-fr/mschart/branch/master/graph/badge.svg)](https://app.codecov.io/gh/ardata-fr/mschart)
![Active](https://www.repostatus.org/badges/latest/active.svg)

The `mschart` package provides a framework for easily creating charts
for ‘Microsoft PowerPoint’ presentations, ‘Microsoft Word’ documents and
‘Microsoft Excel’ workbooks. It has to be used with package
[`officer`](https://davidgohel.github.io/officer/) that will produce the
charts in new or existing PowerPoint, Word or Excel files. With
‘Microsoft Charts’, the data is integrated into the document and linked
to the chart. The result can be edited, annotated and resized. If the
data is updated in the document, the chart is also updated.

## Chart types

Classical charts (Office 2007+): `ms_barchart()`, `ms_linechart()`,
`ms_areachart()`, `ms_scatterchart()`, `ms_piechart()`,
`ms_bubblechart()`, `ms_radarchart()`, `ms_stockchart()`.

chartEx charts (Office 2016+): `ms_boxplotchart()`, `ms_funnelchart()`,
`ms_histogramchart()`, `ms_paretochart()`, `ms_sunburstchart()`,
`ms_treemapchart()`, `ms_waterfallchart()`.

Classical charts can be combined on the same plot area with
`ms_chart_combine()`.

## Example

This is a basic example which shows you how to create a bar chart.

``` r
library(mschart)

sales <- data.frame(
  quarter = rep(c("Q1", "Q2", "Q3", "Q4"), each = 2),
  revenue = c(12, 9, 15, 11, 18, 14, 21, 17),
  region  = rep(c("EU", "US"), times = 4)
)

bars <- ms_barchart(
  data = sales, x = "quarter", y = "revenue", group = "region"
)
```

Then use package `officer` to send the object as a chart.

``` r
library(officer)
doc <- read_pptx()
doc <- add_slide(doc, layout = "Title and Content", master = "Office Theme")
doc <- ph_with(doc, value = bars, location = ph_location_fullsize())

print(doc, target = "example.pptx")
```

The same chart object can be dropped into an Excel sheet with
`officer::sheet_add_drawing()`. By default the chart’s underlying data
is written next to the chart automatically.

You can also write the data first with `officer::sheet_write_data()` and
ask the chart to reuse it by passing `write_data = FALSE`. This is
useful when the data should appear at a specific position, or when
several charts share the same dataset:

``` r
wb <- read_xlsx()
wb <- add_sheet(wb, label = "sales")

# 1. Write the data on the sheet at the desired position.
wb <- sheet_write_data(wb, sheet = "sales", value = bars$data_series,
                       start_col = 1, start_row = 1)

# 2. Add the chart and tell it to reference the data already in place.
wb <- sheet_add_drawing(wb, sheet = "sales", value = bars,
                        write_data = FALSE,
                        start_col = 1, start_row = 1,
                        left = 5, top = 0.5, width = 6, height = 4)

print(wb, target = "example.xlsx")
```

Use `print(your_chart, preview = TRUE)` to open the chart in PowerPoint
while iterating.

## Installation

You can get the development version from GitHub:

``` r
devtools::install_github("ardata-fr/mschart")
```

Or the latest version on CRAN:

``` r
install.packages("mschart")
```

## Contributing to the package

### Bug reports

When you file a [bug
report](https://github.com/ardata-fr/mschart/issues), please spend some
time making it easy for me to follow and reproduce. The more time you
spend on making the bug report coherent, the more time I can dedicate to
investigating the bug as opposed to the bug report.

#' @export
#' @title Add a chart to a Word document
#' @description Add a `ms_chart` to an rdocx object. The graphic will be
#' inserted in an empty paragraph.
#' @param x an rdocx object
#' @param chart an `ms_chart` object.
#' @param style paragraph style
#' @param pos where to add the new element relative to the cursor,
#' one of "after", "before", "on".
#' @param height,width height and width in inches.
#' @return An rdocx object.
#' @seealso [ph_with.ms_chart()]
#' @importFrom officer body_add_xml styles_info
#' @examples
#' library(officer)
#' my_barchart <- ms_barchart(data = browser_data,
#'   x = "browser", y = "value", group = "serie")
#' my_barchart <- chart_settings( my_barchart, grouping = "stacked",
#'   gap_width = 50, overlap = 100 )
#'
#' doc <- read_docx()
#' doc <- body_add_chart(doc, chart = my_barchart, style = "centered")
#' print(doc, target = tempfile(fileext = ".docx"))
body_add_chart <- function(
  x,
  chart,
  style = NULL,
  pos = "after",
  width = 5,
  height = 3
) {
  charts_dir <- file.path(x$package_dir, "word/charts")
  xlsx_dir <- file.path(x$package_dir, "word/embeddings")

  dir.create(charts_dir, recursive = TRUE, showWarnings = FALSE)
  dir.create(
    file.path(charts_dir, "_rels"),
    recursive = TRUE,
    showWarnings = FALSE
  )
  dir.create(xlsx_dir, recursive = TRUE, showWarnings = FALSE)

  chart_file <- tempfile(
    tmpdir = charts_dir,
    pattern = "chart",
    fileext = ".xml"
  )
  xlsx_file <- tempfile(tmpdir = xlsx_dir, pattern = "data", fileext = ".xlsx")
  rel_filename <- file.path(
    charts_dir,
    "_rels",
    paste0(basename(chart_file), ".rels")
  )

  rel_str <- paste0(
    "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>",
    "<Relationships  xmlns=\"http://schemas.openxmlformats.org/package/2006/relationships\"><Relationship Id=\"rId1\" Type=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships/package\" Target=\"../embeddings/%s\"/></Relationships>"
  )
  rel_str <- sprintf(rel_str, basename(xlsx_file))
  writeLines(rel_str, rel_filename, useBytes = TRUE)

  id_x = "64451712"
  id_y = "64453248"
  write_xlsx(x = list("sheet1" = chart$data_series), path = xlsx_file)
  xml_elt <- format(chart, id_x = id_x, id_y = id_y)
  writeLines(xml_elt, chart_file, useBytes = TRUE)

  next_id <- x$doc_obj$relationship()$get_next_id()
  x$doc_obj$relationship()$add(
    paste0("rId", next_id),
    type = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/chart",
    target = paste0("charts/", basename(chart_file))
  )
  reference_ <- "<c:chart xmlns:c=\"http://schemas.openxmlformats.org/drawingml/2006/chart\" xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\" r:id=\"rId%.0f\"/>"
  reference_ <- sprintf(reference_, next_id)

  drawing_str <- paste0(
    "<w:drawing><wp:inline distT=\"0\" distB=\"0\" distL=\"0\" distR=\"0\">",
    sprintf(
      "<wp:extent cx=\"%.0f\" cy=\"%.0f\"/>",
      width * 914400,
      height * 914400
    ),
    "<wp:effectExtent l=\"0\" t=\"0\" r=\"0\" b=\"0\"/>",
    "<wp:docPr id=\"\" name=\"\"/>",
    "<wp:cNvGraphicFramePr/>",
    "<a:graphic xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\">",
    "<a:graphicData uri=\"http://schemas.openxmlformats.org/drawingml/2006/chart\">",
    reference_,
    "</a:graphicData>",
    "</a:graphic>",
    "</wp:inline>",
    "</w:drawing>"
  )

  base_ns <- "xmlns:w=\"http://schemas.openxmlformats.org/wordprocessingml/2006/main\" xmlns:wp=\"http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing\" xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\" xmlns:w14=\"http://schemas.microsoft.com/office/word/2010/wordml\""
  if (is.null(style)) {
    style <- x$default_styles$paragraph
  }

  style_id <- get_style_id(
    data = styles_info(x),
    style = style,
    type = "paragraph"
  )
  par_elt <- paste0(
    sprintf("<%s %s>", "w:p", base_ns),
    "<w:pPr><w:pStyle w:val=\"",
    style_id,
    "\"/></w:pPr><w:r>",
    drawing_str,
    "</w:r></w:p>"
  )

  partname <- file.path("/word/charts", basename(chart_file))
  override <- setNames(
    "application/vnd.openxmlformats-officedocument.drawingml.chart+xml",
    partname
  )
  x$content_type$add_override(value = override)
  x$content_type$add_ext(
    extension = "xlsx",
    type = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
  )
  body_add_xml(x, str = par_elt, pos = pos)
}

get_style_id <- function(data, style, type) {
  ref <- data[data$style_type == type, ]

  if (!style %in% ref$style_name) {
    t_ <- shQuote(ref$style_name, type = "sh")
    t_ <- paste(t_, collapse = ", ")
    t_ <- paste0("c(", t_, ")")
    stop(
      "could not match any style named ",
      shQuote(style, type = "sh"),
      " in ",
      t_,
      call. = FALSE
    )
  }
  ref$style_id[ref$style_name == style]
}

#' @export
#' @title Modify axis and plot labels
#' @description Add labels to a chart, labels can be specified for
#' x axis, y axis and plot.
#' @param x an `ms_chart` object.
#' @param title title of the chart (displayed above the plot area). Use NULL to remove it.
#' @param xlab label for the x axis. Use NULL to remove it.
#' @param ylab label for the y axis. Use NULL to remove it.
#' @return An `ms_chart` object.
#' @seealso [chart_data_labels()], [chart_ax_x()], [chart_ax_y()]
#' @examples
#' mylc <- ms_linechart(
#'   data = browser_ts, x = "date", y = "freq",
#'   group = "browser"
#' )
#' mylc <- chart_labels(mylc,
#'   title = "my title", xlab = "my x label",
#'   ylab = "my y label"
#' )
chart_labels <- function(x, title = NULL, xlab = NULL, ylab = NULL) {
  validate_label(title, "title")
  validate_label(xlab, "xlab")
  validate_label(ylab, "ylab")

  if (!is.null(title)) {
    x$labels[["title"]] <- title
  } else {
    x$labels[["title"]] <- NULL
  }

  if (!is.null(xlab)) {
    x$labels[["x"]] <- xlab
  } else {
    x$labels[["x"]] <- NULL
  }

  if (!is.null(ylab)) {
    x$labels[["y"]] <- ylab
  } else {
    x$labels[["y"]] <- NULL
  }
  x
}

validate_label <- function(value, name) {
  if (is.null(value)) {
    return(invisible(NULL))
  }
  if (!is.character(value) || length(value) != 1 || is.na(value)) {
    stop(
      "`", name, "` must be NULL or a single non-NA character string.",
      call. = FALSE
    )
  }
  invisible(NULL)
}

test_that("UTF-8 characters in chart titles and data", {
  test <- browser_data
  test[test$browser == "Opera", 1] <- "\U0001f47d"

  chart_01 <- ms_barchart(
    data = test,
    x = "browser",
    y = "value",
    group = "serie"
  )
  chart_01 <- chart_labels(chart_01, title = "\U0001f47d", xlab = "\U0001f47d")

  xml <- format(
    chart_01,
    sheetname = "sheet1",
    id_x = "64451212",
    id_y = "64453248"
  )

  chart <- xml2::read_xml(xml)
  txt <- xml2::xml_find_all(chart, "//c:title/c:tx/c:rich/a:p/a:r/a:t")

  expect_equal(
    xml2::xml_text(txt),
    c("\U0001f47d", "\U0001f47d"),
    ignore_attr = TRUE
  )

  txt_data <- xml2::xml_find_all(
    chart,
    "//c:ser[1]/c:cat/c:strRef/c:strCache/c:pt"
  )
  expect_equal(
    xml2::xml_text(txt_data),
    c("Android", "Chrome", "Firefox", "IE", "Safari", "\U0001f47d"),
    ignore_attr = TRUE
  )
})

treemap_data <- function() {
  data.frame(
    region = c("EU", "EU", "EU", "AM", "AM"),
    country = c("FR", "FR", "DE", "US", "US"),
    city = c("Paris", "Lyon", "Berlin", "NYC", "LA"),
    value = c(10, 5, 12, 20, 8),
    stringsAsFactors = FALSE
  )
}

test_that("ms_treemapchart constructor validates inputs", {
  d <- treemap_data()

  expect_error(
    ms_treemapchart(d, path = c("region", "missing"), value = "value"),
    "not found in data"
  )
  expect_error(
    ms_treemapchart(d, path = "region", value = "missing"),
    "not found in data"
  )
  expect_error(
    ms_treemapchart(d, path = "region", value = "city"),
    "must be numeric"
  )
})

test_that("ms_treemapchart format produces well-formed cx XML", {
  tm <- ms_treemapchart(
    treemap_data(),
    path = c("region", "country", "city"),
    value = "value"
  )
  tm <- chart_labels(tm, title = "Sales by region")

  xml_str <- format(tm, sheetname = "sheet1")
  expect_true(nchar(xml_str) > 0)

  doc <- xml2::read_xml(xml_str)
  ns <- c(cx = "http://schemas.microsoft.com/office/drawing/2014/chartex")

  expect_equal(xml2::xml_name(xml2::xml_root(doc)), "chartSpace")

  # one <cx:data> with one strDim (cat) of 3 levels and one numDim (val)
  cat_lvls <- xml2::xml_find_all(doc, "//cx:strDim[@type='cat']/cx:lvl", ns)
  expect_length(cat_lvls, 3L)

  val_lvls <- xml2::xml_find_all(doc, "//cx:numDim[@type='size']/cx:lvl", ns)
  expect_length(val_lvls, 1L)

  # value level has 5 points
  pts <- xml2::xml_find_all(val_lvls[[1]], "./cx:pt", ns)
  expect_length(pts, 5L)

  # series with treemap layout
  series <- xml2::xml_find_all(doc, "//cx:series", ns)
  expect_length(series, 1L)
  expect_equal(xml2::xml_attr(series[[1]], "layoutId"), "treemap")

  # Office orders levels leaf-first: lvl[0] = city, lvl[2] = region
  leaf_pts <- xml2::xml_find_all(cat_lvls[[1]], "./cx:pt", ns)
  expect_equal(xml2::xml_text(leaf_pts[[1]]), "Paris")
  root_pts <- xml2::xml_find_all(cat_lvls[[3]], "./cx:pt", ns)
  expect_equal(xml2::xml_text(root_pts[[1]]), "EU")

  # title injected
  ttl <- xml2::xml_find_first(doc, "//cx:title", ns)
  expect_false(inherits(ttl, "xml_missing"))
})

test_that("ms_treemapchart can be inserted in a pptx end-to-end", {
  skip_if_not_installed("officer")
  skip_if_not_installed("writexl")

  tm <- ms_treemapchart(
    treemap_data(),
    path = c("region", "country", "city"),
    value = "value"
  )

  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(doc, tm, location = officer::ph_location_fullsize())

  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))

  # Inspect the produced package: chart part exists, content-type and
  # relationship use the chartEx URIs.
  unz_dir <- tempfile()
  dir.create(unz_dir)
  utils::unzip(out, exdir = unz_dir)

  chart_files <- list.files(
    file.path(unz_dir, "ppt", "charts"),
    pattern = "^chart.*\\.xml$",
    full.names = TRUE
  )
  expect_length(chart_files, 1L)
  chart_xml <- xml2::read_xml(chart_files[1])
  expect_equal(xml2::xml_name(xml2::xml_root(chart_xml)), "chartSpace")
  expect_match(
    xml2::xml_ns(chart_xml)[["cx"]],
    "office/drawing/2014/chartex",
    fixed = TRUE
  )

  ct_xml <- xml2::read_xml(file.path(unz_dir, "[Content_Types].xml"))
  ct_overrides <- xml2::xml_attr(
    xml2::xml_find_all(ct_xml, "//*[local-name()='Override']"),
    "ContentType"
  )
  expect_true(any(grepl("ms-office.chartex\\+xml", ct_overrides)))
})

#' @title Data table settings
#' @description Define visual settings for the data table displayed below the
#' chart. Requires `chart_settings(x, table = TRUE)` to be called first.
#' @param x an `ms_chart` object.
#' @param horizontal write horizontal lines in the table
#' @param vertical write vertical lines in the table
#' @param outline write an outline in the table
#' @param show_keys show keys in the table
#' @return An `ms_chart` object.
#' @seealso [chart_settings()]
#' @export
#' @examples
#' data <- data.frame(
#'   supp = factor(rep(c("OJ", "VC"), each = 3),
#'                 levels = c("OJ", "VC")),
#'   dose = factor(rep(c("low", "medium", "high"), 2),
#'                 levels = c("low", "medium", "high")),
#'   length = c(13.23, 22.7, 24.06, 7.98, 16.77, 26.14),
#'   label = LETTERS[1:6],
#'   stringsAsFactors = FALSE
#' )
#'
#' # example chart 03 -------
#' chart <- ms_linechart(
#'   data = data, x = "dose", y = "length",
#'   group = "supp", labels = "label"
#' )
#' chart <- chart_settings(
#'   x = chart, table = TRUE
#' )
#'
#' chart <- chart_table(chart,
#'   horizontal = TRUE, vertical = FALSE,
#'   outline = TRUE, show_keys = FALSE
#' )
chart_table <- function(x, horizontal, vertical, outline, show_keys) {
  stopifnot(inherits(x, "ms_chart"))

  options <- list(
    horizontal = ifelse(missing(horizontal), x$x_table$horizontal, horizontal),
    vertical = ifelse(missing(vertical), x$x_table$vertical, vertical),
    outline = ifelse(missing(outline), x$x_table$outline, outline),
    show_keys = ifelse(missing(show_keys), x$x_table$show_keys, show_keys)
  )

  x$x_table <- do.call(table_options, options)
  x
}


table_options <- function(
  horizontal = TRUE,
  vertical = TRUE,
  outline = TRUE,
  show_keys = TRUE
) {
  if (!is.logical(horizontal)) {
    stop("horizontal should be of type logical")
  }

  if (!is.logical(vertical)) {
    stop("vertical should be of type logical")
  }

  if (!is.logical(outline)) {
    stop("outline should be of type logical")
  }

  if (!is.logical(show_keys)) {
    stop("show_keys should be of type logical")
  }

  out <- list(
    horizontal = horizontal,
    vertical = vertical,
    outline = outline,
    show_keys = show_keys
  )

  class(out) <- "table_options"
  out
}

#' @title X axis settings
#' @description Define settings for an x axis.
#' @param x an `ms_chart` object.
#' @param orientation axis orientation, one of 'maxMin', 'minMax'.
#' @param crosses specifies how the axis crosses the perpendicular
#' axis, one of 'autoZero', 'max', 'min'.
#' @param cross_between specifies how the value axis crosses the
#' category axis between categories, one of 'between', 'midCat'.
#' @param major_tick_mark,minor_tick_mark tick marks position,
#' one of 'cross', 'in', 'none', 'out'.
#' @param tick_label_pos ticks labels position, one of 'high',
#' 'low', 'nextTo', 'none'.
#' @param display should the axis be displayed (a logical of
#' length 1).
#' @param num_fmt number formatting. See the num_fmt section for more details.
#' @param rotation rotation angle. Value should be between `-360`
#' and `360`.
#' @param limit_min minimum value on the axis. Date objects are also accepted
#' and will be converted automatically.
#' @param limit_max maximum value on the axis. Date objects are also accepted
#' and will be converted automatically.
#' @param position the value at which this axis crosses the perpendicular axis.
#' @param major_unit numeric, interval between major ticks and gridlines.
#' @param minor_unit numeric, interval between minor ticks and gridlines.
#' @param major_time_unit time unit for major ticks on date axes,
#' one of `"days"`, `"months"`, `"years"`.
#' @param minor_time_unit time unit for minor ticks on date axes,
#' one of `"days"`, `"months"`, `"years"`.
#' @section num_fmt:
#' All `%` need to be doubled, `0%%` means "a number
#' and percent symbol".
#'
#' To my current knowledge, depending on the chart type
#' and options, the following values are not systematically
#' used by office chart engine; i.e. when chart pre-compute
#' percentages, it seems using `0%%` will have no
#' effect.
#'
#' * `General`: default value
#' * `0`: display the number with no decimal
#' * `0.00`: display the number with two decimals
#' * `0%%`: display as percentages
#' * `0.00%%`: display as percentages with two decimal places
#' * `#,##0`
#' * `#,##0.00`
#' * `0.00E+00`
#' * `# ?/?`
#' * `# ??/??`
#' * `mm-dd-yy`
#' * `d-mmm-yy`
#' * `d-mmm`
#' * `mmm-yy`
#' * `h:mm AM/PM`
#' * `h:mm:ss AM/PM`
#' * `h:mm`
#' * `h:mm:ss`
#' * `m/d/yy h:mm`
#' * `#,##0 ;(#,##0)`
#' * `#,##0 ;[Red](#,##0)`
#' * `#,##0.00;(#,##0.00)`
#' * `#,##0.00;[Red](#,##0.00)`
#' * `mm:ss`
#' * `[h]:mm:ss`
#' * `mmss.0`
#' * `##0.0E+0`
#' * `@`
#'
#' @return An `ms_chart` object.
#' @export
#' @section Illustrations:
#'
#' \if{html}{\figure{fig_chart_ax_x_1.png}{options: width="500"}}
#' @examples
#' library(mschart)
#'
#' chart_01 <- ms_linechart(
#'   data = us_indus_prod,
#'   x = "date", y = "value",
#'   group = "type"
#' )
#'
#' chart_01 <- chart_ax_y(x = chart_01, limit_min = 20, limit_max = 120)
#' chart_01
#'
#' # control axis intervals
#' chart_01 <- chart_ax_x(chart_01,
#'   major_unit = 10, major_time_unit = "years"
#' )
#' chart_01 <- chart_ax_y(chart_01, major_unit = 20)
#' chart_01
#' @seealso [chart_ax_y()], [ms_areachart()], [ms_barchart()], [ms_scatterchart()],
#' [ms_linechart()]
chart_ax_x <- function(
  x,
  orientation,
  crosses,
  cross_between,
  major_tick_mark,
  minor_tick_mark,
  tick_label_pos,
  display,
  num_fmt,
  rotation,
  limit_min,
  limit_max,
  position,
  major_unit,
  minor_unit,
  major_time_unit,
  minor_time_unit
) {
  stopifnot(inherits(x, "ms_chart"))

  options <- list(
    orientation = ifelse(
      missing(orientation),
      x$x_axis$orientation,
      orientation
    ),
    axis_position = x$x_axis$axis_position,
    crosses = ifelse(missing(crosses), x$x_axis$crosses, crosses),
    cross_between = ifelse(
      missing(cross_between),
      x$x_axis$cross_between,
      cross_between
    ),
    major_tick_mark = ifelse(
      missing(major_tick_mark),
      x$x_axis$major_tick_mark,
      major_tick_mark
    ),
    minor_tick_mark = ifelse(
      missing(minor_tick_mark),
      x$x_axis$minor_tick_mark,
      minor_tick_mark
    ),
    tick_label_pos = ifelse(
      missing(tick_label_pos),
      x$x_axis$tick_label_pos,
      tick_label_pos
    ),
    delete = ifelse(missing(display), x$x_axis$delete, !display),
    rotation = ifelse(missing(rotation), x$x_axis$rotation, rotation)
  )
  if (missing(num_fmt)) {
    options$num_fmt <- x$x_axis$num_fmt
  } else {
    options$num_fmt <- num_fmt
  }

  if (missing(limit_min) && !is.null(x$x_axis$limit_min)) {
    options$limit_min <- x$x_axis$limit_min
  } else if (!missing(limit_min)) {
    if (inherits(limit_min, "Date")) {
      limit_min <- as.integer(limit_min - as.Date("1899-12-30"))
    }
    options$limit_min <- limit_min
  }
  if (missing(limit_max) && !is.null(x$x_axis$limit_max)) {
    options$limit_max <- x$x_axis$limit_max
  } else if (!missing(limit_max)) {
    if (inherits(limit_max, "Date")) {
      limit_max <- as.integer(limit_max - as.Date("1899-12-30"))
    }
    options$limit_max <- limit_max
  }
  if (missing(position) && !is.null(x$x_axis$position)) {
    options$position <- x$x_axis$position
  } else if (!missing(position)) {
    options$position <- position
  }
  if (missing(major_unit) && !is.null(x$x_axis$major_unit)) {
    options$major_unit <- x$x_axis$major_unit
  } else if (!missing(major_unit)) {
    options$major_unit <- major_unit
  }
  if (missing(minor_unit) && !is.null(x$x_axis$minor_unit)) {
    options$minor_unit <- x$x_axis$minor_unit
  } else if (!missing(minor_unit)) {
    options$minor_unit <- minor_unit
  }
  if (missing(major_time_unit) && !is.null(x$x_axis$major_time_unit)) {
    options$major_time_unit <- x$x_axis$major_time_unit
  } else if (!missing(major_time_unit)) {
    options$major_time_unit <- major_time_unit
  }
  if (missing(minor_time_unit) && !is.null(x$x_axis$minor_time_unit)) {
    options$minor_time_unit <- x$x_axis$minor_time_unit
  } else if (!missing(minor_time_unit)) {
    options$minor_time_unit <- minor_time_unit
  }

  x$x_axis <- do.call(axis_options, options)
  x
}


#' @title Y axis settings
#' @description Define settings for a y axis.
#' @inheritParams chart_ax_x
#' @inheritSection chart_ax_x num_fmt
#' @return An `ms_chart` object.
#' @export
#' @section Illustrations:
#'
#' \if{html}{\figure{fig_chart_ax_y_1.png}{options: width="500"}}
#' @examples
#' library(officer)
#' library(mschart)
#'
#' chart_01 <- ms_linechart(
#'   data = us_indus_prod,
#'   x = "date", y = "value",
#'   group = "type"
#' )
#' chart_01 <- chart_settings(chart_01, style = "marker")
#' chart_01 <- chart_ax_x(
#'   x = chart_01, num_fmt = "[$-fr-FR]mmm yyyy",
#'   limit_min = min(us_indus_prod$date),
#'   limit_max = as.Date("1992-01-01")
#' )
#' chart_01
#' @seealso [chart_ax_x()], [ms_areachart()], [ms_barchart()], [ms_scatterchart()],
#' [ms_linechart()]
chart_ax_y <- function(
  x,
  orientation,
  crosses,
  cross_between,
  major_tick_mark,
  minor_tick_mark,
  tick_label_pos,
  display,
  num_fmt,
  rotation,
  limit_min,
  limit_max,
  position,
  major_unit,
  minor_unit,
  major_time_unit,
  minor_time_unit
) {
  stopifnot(inherits(x, "ms_chart"))

  options <- list(
    orientation = ifelse(
      missing(orientation),
      x$y_axis$orientation,
      orientation
    ),
    axis_position = x$y_axis$axis_position,
    crosses = ifelse(missing(crosses), x$y_axis$crosses, crosses),
    cross_between = ifelse(
      missing(cross_between),
      x$y_axis$cross_between,
      cross_between
    ),
    major_tick_mark = ifelse(
      missing(major_tick_mark),
      x$y_axis$major_tick_mark,
      major_tick_mark
    ),
    minor_tick_mark = ifelse(
      missing(minor_tick_mark),
      x$y_axis$minor_tick_mark,
      minor_tick_mark
    ),
    tick_label_pos = ifelse(
      missing(tick_label_pos),
      x$y_axis$tick_label_pos,
      tick_label_pos
    ),
    delete = ifelse(missing(display), x$y_axis$delete, !display),
    rotation = ifelse(missing(rotation), x$y_axis$rotation, rotation)
  )
  if (missing(num_fmt)) {
    options$num_fmt <- x$y_axis$num_fmt
  } else {
    options$num_fmt <- num_fmt
  }
  if (missing(limit_min) && !is.null(x$y_axis$limit_min)) {
    options$limit_min <- x$y_axis$limit_min
  } else if (!missing(limit_min)) {
    if (inherits(limit_min, "Date")) {
      limit_min <- as.integer(limit_min - as.Date("1899-12-30"))
    }
    options$limit_min <- limit_min
  }
  if (missing(limit_max) && !is.null(x$y_axis$limit_max)) {
    options$limit_max <- x$y_axis$limit_max
  } else if (!missing(limit_max)) {
    if (inherits(limit_max, "Date")) {
      limit_max <- as.integer(limit_max - as.Date("1899-12-30"))
    }
    options$limit_max <- limit_max
  }
  if (missing(position) && !is.null(x$y_axis$position)) {
    options$position <- x$y_axis$position
  } else if (!missing(position)) {
    options$position <- position
  }
  if (missing(major_unit) && !is.null(x$y_axis$major_unit)) {
    options$major_unit <- x$y_axis$major_unit
  } else if (!missing(major_unit)) {
    options$major_unit <- major_unit
  }
  if (missing(minor_unit) && !is.null(x$y_axis$minor_unit)) {
    options$minor_unit <- x$y_axis$minor_unit
  } else if (!missing(minor_unit)) {
    options$minor_unit <- minor_unit
  }
  if (missing(major_time_unit) && !is.null(x$y_axis$major_time_unit)) {
    options$major_time_unit <- x$y_axis$major_time_unit
  } else if (!missing(major_time_unit)) {
    options$major_time_unit <- major_time_unit
  }
  if (missing(minor_time_unit) && !is.null(x$y_axis$minor_time_unit)) {
    options$minor_time_unit <- x$y_axis$minor_time_unit
  } else if (!missing(minor_time_unit)) {
    options$minor_time_unit <- minor_time_unit
  }

  x$y_axis <- do.call(axis_options, options)
  x
}


axis_options <- function(
  orientation = "minMax",
  axis_position = "b",
  crosses = "autoZero",
  cross_between = "between",
  major_tick_mark = "cross",
  minor_tick_mark = "none",
  tick_label_pos = "nextTo",
  delete = FALSE,
  num_fmt = NULL,
  rotation = 0,
  limit_min = NULL,
  limit_max = NULL,
  position = NULL,
  major_unit = NULL,
  minor_unit = NULL,
  major_time_unit = NULL,
  minor_time_unit = NULL
) {
  if (!orientation %in% st_orientation) {
    stop(
      "orientation should be one of ",
      paste0(shQuote(st_orientation), collapse = ", ")
    )
  }
  if (!axis_position %in% st_axpos) {
    stop(
      "axis_position should be one of ",
      paste0(shQuote(st_axpos), collapse = ", ")
    )
  }
  if (!crosses %in% st_crosses) {
    stop(
      "crosses should be one of ",
      paste0(shQuote(st_crosses), collapse = ", ")
    )
  }
  if (!cross_between %in% st_crossbetween) {
    stop(
      "cross_between should be one of ",
      paste0(shQuote(st_crossbetween), collapse = ", ")
    )
  }
  if (!major_tick_mark %in% st_tickmark) {
    stop(
      "major_tick_mark should be one of ",
      paste0(shQuote(st_tickmark), collapse = ", ")
    )
  }
  if (!minor_tick_mark %in% st_tickmark) {
    stop(
      "minor_tick_mark should be one of ",
      paste0(shQuote(st_tickmark), collapse = ", ")
    )
  }
  if (!tick_label_pos %in% st_ticklblpos) {
    stop(
      "tick_label_pos should be one of ",
      paste0(shQuote(st_ticklblpos), collapse = ", ")
    )
  }

  out <- list(
    orientation = orientation,
    axis_position = axis_position,
    crosses = crosses,
    cross_between = cross_between,
    delete = delete,
    num_fmt = num_fmt,
    major_tick_mark = major_tick_mark,
    minor_tick_mark = minor_tick_mark,
    tick_label_pos = tick_label_pos,
    rotation = rotation,
    limit_min = limit_min,
    limit_max = limit_max,
    position = position,
    major_unit = major_unit,
    minor_unit = minor_unit,
    major_time_unit = major_time_unit,
    minor_time_unit = minor_time_unit
  )
  class(out) <- "axis_options"
  out
}

test_that("areachart has no dLblPos in series", {
  chart_01 <- ms_areachart(
    data = browser_ts,
    x = "date",
    y = "freq",
    group = "browser"
  )
  chart_01 <- chart_ax_x(chart_01, num_fmt = "m/d/yy", rotation = -90)
  chart_01 <- chart_data_labels(chart_01, show_val = TRUE)
  chart_01 <- chart_labels_text(
    chart_01,
    values = fp_text(font.size = 7, color = "red")
  )

  xml <- format(
    chart_01,
    sheetname = "sheet1",
    id_x = "64451212",
    id_y = "64453248"
  )

  chart <- xml2::read_xml(xml)
  dLblPos <- xml2::xml_find_all(chart, "//c:ser/c:dLbls/c:dLblPos")

  expect_true(length(dLblPos) < 1)
})

cx_ns <- c(
  cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
  a = "http://schemas.openxmlformats.org/drawingml/2006/main"
)

test_that("chart_ax_y(num_fmt=) emits <cx:numFmt formatCode=...> on val axis", {
  d <- data.frame(step = c("A", "B"), amount = c(1.5, 2.5))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  wf <- chart_ax_y(wf, num_fmt = "0.00")
  doc <- xml2::read_xml(format(wf))
  nf <- xml2::xml_find_first(doc, "//cx:axis[@id='1']/cx:numFmt", cx_ns)
  expect_equal(xml2::xml_attr(nf, "formatCode"), "0.00")
  expect_equal(xml2::xml_attr(nf, "sourceLinked"), "0")
})

test_that("chart_ax_x(num_fmt=) emits <cx:numFmt> on cat axis", {
  d <- data.frame(step = c("A", "B"), amount = c(1, 2))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  wf <- chart_ax_x(wf, num_fmt = "@")
  doc <- xml2::read_xml(format(wf))
  nf <- xml2::xml_find_first(doc, "//cx:axis[@id='0']/cx:numFmt", cx_ns)
  expect_equal(xml2::xml_attr(nf, "formatCode"), "@")
})

test_that("chart_ax_y(major_grid=FALSE) hides val axis major gridlines", {
  d <- data.frame(step = c("A", "B"), amount = c(1, 2))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  wf <- chart_ax_y(wf, major_grid = FALSE)
  doc <- xml2::read_xml(format(wf))
  expect_length(
    xml2::xml_find_all(doc, "//cx:axis[@id='1']/cx:majorGridlines", cx_ns),
    0L
  )
})

test_that("default: val axis keeps major gridlines (back-compat)", {
  d <- data.frame(step = c("A", "B"), amount = c(1, 2))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  doc <- xml2::read_xml(format(wf))
  expect_length(
    xml2::xml_find_all(doc, "//cx:axis[@id='1']/cx:majorGridlines", cx_ns),
    1L
  )
})

test_that("chart_ax_y(minor_grid=TRUE) shows minor gridlines", {
  d <- data.frame(step = c("A", "B"), amount = c(1, 2))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  wf <- chart_ax_y(wf, minor_grid = TRUE)
  doc <- xml2::read_xml(format(wf))
  expect_length(
    xml2::xml_find_all(doc, "//cx:axis[@id='1']/cx:minorGridlines", cx_ns),
    1L
  )
})

test_that("chart_ax_y(limit_min=, limit_max=) embeds attributes on valScaling", {
  d <- data.frame(step = c("A", "B"), amount = c(1, 2))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  wf <- chart_ax_y(wf, limit_min = -10, limit_max = 200)
  doc <- xml2::read_xml(format(wf))
  vs <- xml2::xml_find_first(doc, "//cx:axis[@id='1']/cx:valScaling", cx_ns)
  expect_equal(xml2::xml_attr(vs, "min"), "-10")
  expect_equal(xml2::xml_attr(vs, "max"), "200")
})

test_that("chart_ax_y options work on histogram and pareto", {
  set.seed(1)
  hi <- ms_histogramchart(data.frame(x = rnorm(50)), value = "x")
  hi <- chart_ax_y(hi, num_fmt = "0", major_grid = FALSE)
  doc <- xml2::read_xml(format(hi))
  expect_equal(
    xml2::xml_attr(
      xml2::xml_find_first(doc, "//cx:axis[@id='1']/cx:numFmt", cx_ns),
      "formatCode"
    ),
    "0"
  )
  expect_length(
    xml2::xml_find_all(doc, "//cx:axis[@id='1']/cx:majorGridlines", cx_ns),
    0L
  )

  d <- data.frame(defect = c("A", "B"), n = c(10, 5))
  pa <- ms_paretochart(d, x = "defect", y = "n", aggregate = FALSE)
  pa <- chart_ax_y(pa, num_fmt = "#,##0")
  doc2 <- xml2::read_xml(format(pa))
  expect_equal(
    xml2::xml_attr(
      xml2::xml_find_first(doc2, "//cx:axis[@id='1']/cx:numFmt", cx_ns),
      "formatCode"
    ),
    "#,##0"
  )
  # axis 2 (percentage) is fixed and not affected
  expect_length(
    xml2::xml_find_all(doc2, "//cx:axis[@id='2']/cx:numFmt", cx_ns),
    0L
  )
})

test_that("chart_ax_x options work on funnel single axis", {
  d <- data.frame(stage = c("A", "B"), count = c(100, 50))
  fn <- ms_funnelchart(d, x = "stage", y = "count")
  fn <- chart_ax_x(fn, num_fmt = "@")
  doc <- xml2::read_xml(format(fn))
  expect_equal(
    xml2::xml_attr(
      xml2::xml_find_first(doc, "//cx:axis[@id='1']/cx:numFmt", cx_ns),
      "formatCode"
    ),
    "@"
  )
})

test_that("round-trip pptx still opens with axis options", {
  skip_if_not_installed("officer")
  d <- data.frame(step = c("A", "B", "C"), amount = c(10, -5, 8))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  wf <- chart_ax_y(
    wf,
    num_fmt = "0",
    limit_min = -10,
    limit_max = 50,
    major_grid = TRUE,
    minor_grid = TRUE
  )
  doc <- officer::read_pptx()
  doc <- officer::add_slide(doc, "Title and Content", "Office Theme")
  doc <- officer::ph_with(doc, wf, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

#' Index of US Industrial Production
#'
#' Index of US industrial production (1985 = 100).
#'
#' This is a transformation of the `USProdIndex` data
#' from package 'AER' into a simple data.frame.
#'
#' @name us_indus_prod
#' @format A data frame with 256 rows and 3 variables
NULL

#' Dummy dataset for barchart
#'
#' A dataset containing two categorical variables and one integer variable:
#'
#' * browser web browser
#' * serie id of series
#' * value integer values
#'
#' @name browser_data
#' @format A data frame with 18 rows and 3 variables
NULL

#' Dummy dataset for time series charts
#'
#' A dataset containing a date, a categorical, and an integer variable:
#'
#' * date date values
#' * browser web browser
#' * freq values in percent
#'
#' @name browser_ts
#' @format A data frame with 36 rows and 3 variables
NULL

#' @title xml code for chart and plot area properties
#' @param theme an object of class mschart_theme
#' @param what a character. One of `"plot"` or `"chart"`
#' @noRd
sppr_content_xml <- function(theme = NULL, what = "chart", ns = NULL) {
  if (!what %in% c("plot", "chart")) {
    stop("what sould be one of \"plot\" or \"chart\"")
  }

  fill <- theme[[paste0(what, "_background")]]
  border_properties <- theme[[paste0(what, "_border")]]

  if (!is.null(fill)) {
    fill_elts <- col2rgb(fill, alpha = TRUE)[, 1]
    fill_hex <- sprintf(
      "%02X%02X%02X",
      fill_elts[1],
      fill_elts[2],
      fill_elts[3]
    )
    fill_str <- sprintf(
      "<a:solidFill><a:srgbClr val=\"%s\"><a:alpha val=\"%.0f\"/></a:srgbClr></a:solidFill>",
      fill_hex,
      fill_elts[4] / 255.0 * 100000
    )
  } else {
    fill_str <- NULL
  }

  border_str <- ooxml_fp_border(border_properties)

  if (!is.null(ns)) {
    ns <- paste0(" ", ns)
  }

  sppr_str <- paste0(
    "<c:spPr",
    ns,
    ">",
    fill_str,
    border_str,
    "</c:spPr>"
  )

  sppr_str
}

#' @importFrom cellranger cell_limits as.cell_limits as.range ra_ref to_string
series_wb_name <- function(dataset, idx) {
  dims <- attr(dataset, "dims")
  sheetname <- attr(dataset, "sheet")

  dims <- dims[1, idx]

  serie_range <- as.cell_limits(paste0(sheetname, "!", dims))
  serie_range
}

series_wb_data <- function(dataset, idx) {
  dims <- attr(dataset, "dims")
  sheetname <- attr(dataset, "sheet")

  dims <- dims[, idx]
  dims <- paste0(dims[2], ":", dims[length(dims)])

  serie_range <- as.cell_limits(paste0(sheetname, "!", dims))
  serie_range
}

as_series <- function(
  x,
  x_class,
  y_class,
  sheetname = "sheet1",
  secondary_y = 0
) {
  dataset <- x$data_series

  # Optional offset when the data is written to a non-default cell
  # position in the host sheet (see sheet_add_drawing.ms_chart).
  start_col <- x$start_col %||% 1L
  start_row <- x$start_row %||% 1L
  col_offset <- start_col - 1L
  header_row <- start_row
  first_val_row <- start_row + 1L
  last_val_row <- start_row + nrow(dataset)

  w_x <- which(names(dataset) %in% x$xvar)

  x_serie_range <- cell_limits(
    ul = c(first_val_row, w_x + col_offset),
    lr = c(last_val_row,  w_x + col_offset),
    sheet = sheetname
  )

  if (inherits(dataset, "wb_data")) {
    x_serie_range <- series_wb_data(dataset, w_x)
  }

  x_serie_range <- as.range(
    x_serie_range,
    fo = "A1",
    strict = TRUE,
    sheet = TRUE
  )
  x_serie <- update(x_class, region = x_serie_range, values = dataset[[x$x]])

  label_columns <- get_label_names(x)

  if (inherits(dataset, "wb_data")) {
    label_columns <- x$label_cols
  }

  series <- list()

  series_nams <- get_series_names(x)
  if (x$asis) {
    series_nams <- x$yvar
  }

  w_y_values <- which(names(dataset) %in% series_nams)
  w_l_values <- which(names(dataset) %in% label_columns)

  for (w_y_index in seq_along(w_y_values)) {
    w_y <- w_y_values[w_y_index]
    w_l <- w_l_values[w_y_index]
    y_colname <- names(dataset)[w_y]
    l_colname <- names(dataset)[w_l]

    serie_name_range <- ra_ref(row_ref = header_row,
                               col_ref = w_y + col_offset,
                               sheet = sheetname)
    serie_name_range <- to_string(serie_name_range, fo = "A1")
    if (inherits(dataset, "wb_data")) {
      serie_name_range <- series_wb_name(dataset, w_y)
      serie_name_range <- as.range(
        serie_name_range,
        fo = "A1",
        strict = TRUE,
        sheet = TRUE
      )
    }
    serie_name <- str_ref(values = y_colname, region = serie_name_range)

    y_serie_range <- cell_limits(
      ul = c(first_val_row, w_y + col_offset),
      lr = c(last_val_row,  w_y + col_offset),
      sheet = sheetname
    )

    if (inherits(dataset, "wb_data")) {
      y_serie_range <- series_wb_data(dataset, w_y)
    }
    y_serie_range <- as.range(
      y_serie_range,
      fo = "A1",
      strict = TRUE,
      sheet = TRUE
    )

    y_serie <- update(
      y_class,
      region = y_serie_range,
      values = dataset[[y_colname]]
    )

    if (length(label_columns) > 0) {
      label_serie_range <- cell_limits(
        ul = c(first_val_row, w_l + col_offset),
        lr = c(last_val_row,  w_l + col_offset),
        sheet = sheetname
      )
      label_serie_range <- as.range(
        label_serie_range,
        fo = "A1",
        strict = TRUE,
        sheet = TRUE
      )

      if (inherits(dataset, "wb_data")) {
        label_serie_range <- series_wb_data(dataset, w_l)
        label_serie_range <- as.range(
          label_serie_range,
          fo = "A1",
          strict = TRUE,
          sheet = TRUE
        )
      }
      label_serie <- label_ref(
        values = dataset[[l_colname]],
        region = label_serie_range
      )
    } else {
      label_serie <- NULL
    }

    # bubble size series (for bubble charts)
    bubble_size_serie <- NULL
    if (!is.null(x$size_cols)) {
      sz_col <- x$size_cols[1]
      if (!is.null(x$group)) {
        # transposed name: "sz-groupname"
        grp_name <- sub("^.*-", "", y_colname)
        sz_colname <- paste0(sz_col, "-", grp_name)
      } else {
        sz_colname <- sz_col
      }
      if (sz_colname %in% names(dataset)) {
        w_sz <- which(names(dataset) == sz_colname)
        sz_range <- cell_limits(
          ul = c(first_val_row, w_sz + col_offset),
          lr = c(last_val_row,  w_sz + col_offset),
          sheet = sheetname
        )
        sz_range <- as.range(sz_range, fo = "A1", strict = TRUE, sheet = TRUE)
        bubble_size_serie <- update(
          num_ref(values = numeric(0), region = ""),
          region = sz_range,
          values = dataset[[sz_colname]]
        )
      }
    }

    ser <- list(
      idx = length(series) + secondary_y,
      order = length(series) + secondary_y,
      tx = serie_name,
      x = x_serie,
      y = y_serie,
      bubble_size = bubble_size_serie,
      label = label_serie,
      stroke = x$series_settings$colour[y_colname],
      fill = x$series_settings$fill[y_colname],
      symbol = x$series_settings$symbol[y_colname],
      line_style = x$series_settings$line_style[y_colname],
      size = x$series_settings$size[y_colname],
      line_width = x$series_settings$line_width[y_colname],
      labels_fp = x$series_settings$labels_fp[[y_colname]],
      smooth = x$series_settings$smooth[y_colname]
    )
    series <- append(series, list(ser))
  }
  series
}

`%||%` <- function(x, y) if (is.null(x)) y else x

fmt_name <- function(x) {
  if (inherits(x, "Date")) {
    "date_fmt"
  } else if (is.factor(x) || is.character(x)) {
    "str_fmt"
  } else if (is.integer(x)) {
    "integer_fmt"
  } else if (is.double(x)) {
    "double_fmt"
  } else {
    stop("unknown type of data")
  }
}

serie_builtin_class <- function(x) {
  if (inherits(x, "Date")) {
    date_ref(x)
  } else if (is.factor(x) || is.character(x)) {
    str_ref(x)
  } else if (is.integer(x)) {
    num_ref(x)
  } else if (is.double(x)) {
    num_ref(x)
  } else {
    stop("unknown type of data")
  }
}

is_valid_color = function(x) {
  sapply(x, function(x) {
    tryCatch(is.matrix(col2rgb(x)), error = function(e) FALSE)
  })
}

pretty_num_axes <- function(x, data_x, data_y) {
  range_x <- pretty(data_x)
  range_y <- pretty(data_y)
  x <- chart_ax_x(
    x,
    limit_min = range_x[1],
    limit_max = range_x[length(range_x)]
  )
  x <- chart_ax_y(
    x,
    limit_min = range_y[1],
    limit_max = range_y[length(range_y)]
  )
  x
}

#' @importFrom grDevices col2rgb rgb
#' @noRd
stock_fill_xml <- function(color) {
  elts <- col2rgb(color, alpha = TRUE)[, 1]
  hex <- sprintf("%02X%02X%02X", elts[1], elts[2], elts[3])
  alpha <- elts[4] / 255.0 * 100000
  sprintf(
    "<a:solidFill><a:srgbClr val=\"%s\"><a:alpha val=\"%.0f\"/></a:srgbClr></a:solidFill>",
    hex, alpha
  )
}

#' @importFrom grDevices rgb
ooxml_fp_border <- function(x, in_tags = NULL) {
  stopifnot(inherits(x, "fp_border"))
  colspecs <- as.list(col2rgb(x$color, alpha = TRUE)[, 1] / 255)

  alpha <- colspecs$alpha
  is_transparent <- alpha < 0.0001

  no_line <- is_transparent || x$width < 0.001 || x$style %in% "none"
  if (no_line && is.null(in_tags)) {
    return("")
  }
  if (no_line) {
    out <- "<a:ln><a:noFill/></a:ln>"
    begin <- paste0("<", in_tags, ">", collapse = "")
    end <- paste0("</", rev(in_tags), ">", collapse = "")
    return(paste0(begin, out, end))
  }

  colspecs$alpha <- NULL
  hexcol <- do.call(rgb, colspecs)
  hexcol <- substring(hexcol, 2)

  solidfill <- "<a:solidFill><a:srgbClr val=\"%s\"><a:alpha val=\"%.0f\"/></a:srgbClr></a:solidFill>"
  solidfill <- sprintf(solidfill, hexcol, alpha * 100000)

  presetdash <- "<a:prstDash val="
  if (x$style %in% "solid") {
    presetdash <- paste0(presetdash, "\"solid\"/>")
  } else if (x$style %in% "dotted") {
    presetdash <- paste0(presetdash, "\"sysDot\"/>")
  } else {
    presetdash <- paste0(presetdash, "\"sysDash\"/>")
  }

  out <- sprintf("<a:ln algn=\"ctr\" w=\"%.0f\">", x$width * 12700)
  out <- paste0(out, solidfill, presetdash, "</a:ln>")

  if (!is.null(in_tags)) {
    begin <- paste0("<", in_tags, ">", collapse = "")
    end <- paste0("</", rev(in_tags), ">", collapse = "")
    out <- paste0(begin, out, end)
  }
  out
}


ooxml_txpr <- function(fptext) {
  out <- format(fptext, type = "pml")
  out <- gsub("a:rPr", "a:defRPr", out, fixed = TRUE)
  rpr <- "<a:p><a:pPr>%s</a:pPr></a:p>"
  rpr <- sprintf(rpr, out)
  paste0("<c:txPr><a:bodyPr/><a:lstStyle/>", rpr, "</c:txPr>")
}

#' @export
#' @title Add an ms_chart to an Excel sheet
#' @description Add an `ms_chart` object to a sheet in an xlsx workbook
#' created with [officer::read_xlsx()]. The chart data is written into
#' the sheet and the chart is displayed at the specified position.
#' @param x an rxlsx object (created by [officer::read_xlsx()])
#' @param value an `ms_chart` object
#' @param sheet sheet name where the chart and its data will be placed.
#' The sheet must already exist (see [officer::add_sheet()]).
#' @param start_col column index where chart data will be written
#' (default 1, i.e. column A). When `write_data = FALSE`, this is still
#' the cell position that the chart XML will point at, but no data is
#' written.
#' @param start_row row index where chart data will be written
#' (default 1). Same semantics with `write_data = FALSE` as for
#' `start_col`.
#' @param write_data if `TRUE` (the default), the chart's
#' `data_series` is written into the sheet at
#' `(start_col, start_row)` before the chart is added.
#' Pass `FALSE` when the data is already present in the sheet (for
#' example to avoid rewriting it when several charts share the same
#' dataset, or when inserting a chart that references data written
#' independently via [officer::sheet_write_data()]).
#' @param left,top top-left anchor of the chart, in inches.
#'   Defaults to `(1, 1)`. Same convention as
#'   [officer::sheet_add_drawing()] and `rvg::sheet_add_drawing.dml()`.
#' @param width,height size of the chart, in inches. Defaults
#'   to `6 x 4`.
#' @param ... unused
#' @return the rxlsx object (invisibly)
#' @importFrom officer sheet_write_data sheet_add_drawing xlsx_drawing
#' @importFrom xml2 read_xml xml_find_all xml_attr as_xml_document
#' @method sheet_add_drawing ms_chart
#' @examples
#' library(officer)
#' library(mschart)
#'
#' my_chart <- ms_barchart(
#'   data = data.frame(
#'     x = c("A", "B", "C"),
#'     y = c(1, 3, 2),
#'     group = rep("serie1", 3)
#'   ),
#'   x = "x", y = "y", group = "group"
#' )
#'
#' x <- read_xlsx()
#' x <- add_sheet(x, label = "chart_sheet")
#' x <- sheet_add_drawing(x, value = my_chart, sheet = "chart_sheet")
#' print(x, target = tempfile(fileext = ".xlsx"))
#'
#' # Sharing one dataset between several charts on the same sheet:
#' # write the data once, then add each chart with write_data = FALSE.
#' shared <- data.frame(
#'   x = c("A", "B", "C"),
#'   y = c(1, 3, 2),
#'   group = rep("serie1", 3)
#' )
#' chart_a <- ms_barchart(shared, x = "x", y = "y", group = "group")
#' chart_b <- ms_linechart(shared, x = "x", y = "y", group = "group")
#'
#' x <- read_xlsx()
#' x <- add_sheet(x, label = "multi")
#' x <- sheet_write_data(x, value = chart_a$data_series, sheet = "multi")
#' x <- sheet_add_drawing(x, value = chart_a, sheet = "multi",
#'                        write_data = FALSE,
#'                        left = 3, top = 0.5, width = 5, height = 3.5)
#' x <- sheet_add_drawing(x, value = chart_b, sheet = "multi",
#'                        write_data = FALSE,
#'                        left = 9, top = 0.5, width = 5, height = 3.5)
#' print(x, target = tempfile(fileext = ".xlsx"))
sheet_add_drawing.ms_chart <- function(
  x,
  value,
  sheet,
  start_col = 1L,
  start_row = 1L,
  write_data = TRUE,
  left = 1,
  top = 1,
  width = 6,
  height = 4,
  ...
) {
  stopifnot(inherits(x, "rxlsx"))

  # write chart data into the sheet (can be skipped when the data
  # has already been put in place by the caller)
  if (isTRUE(write_data)) {
    x <- sheet_write_data(
      x,
      value = value$data_series,
      sheet = sheet,
      start_row = start_row,
      start_col = start_col
    )
  }

  # as_series() reads these offsets to shift <c:cat>/<c:val>/<c:xVal>/
  # <c:yVal>/<c:bubbleSize> ranges accordingly
  value$start_col <- as.integer(start_col)
  value$start_row <- as.integer(start_row)

  # generate chart XML for xlsx context
  chart_xml <- format(
    value,
    id_x = "64451712",
    id_y = "64453248",
    sheetname = sheet,
    drop_ext_data = TRUE
  )

  package_dir <- x$package_dir

  # write chart XML to xl/charts/
  charts_dir <- file.path(package_dir, "xl", "charts")
  dir.create(charts_dir, recursive = TRUE, showWarnings = FALSE)
  dir.create(
    file.path(charts_dir, "_rels"),
    recursive = TRUE,
    showWarnings = FALSE
  )

  existing_charts <- list.files(charts_dir, pattern = "^chart[0-9]+\\.xml$")
  if (length(existing_charts) == 0L) {
    chart_name <- "chart1.xml"
  } else {
    nums <- as.integer(gsub("\\D", "", existing_charts))
    chart_name <- sprintf("chart%d.xml", max(nums) + 1L)
  }

  chart_file <- file.path(charts_dir, chart_name)
  writeLines(chart_xml, chart_file, useBytes = TRUE)

  # empty rels for the chart
  chart_rels <- paste0(
    "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>",
    "<Relationships xmlns=\"http://schemas.openxmlformats.org/package/2006/relationships\"/>"
  )
  writeLines(
    chart_rels,
    file.path(charts_dir, "_rels", paste0(chart_name, ".rels")),
    useBytes = TRUE
  )

  # content type
  partname <- paste0("/xl/charts/", chart_name)
  x$content_type$add_override(setNames(
    "application/vnd.openxmlformats-officedocument.drawingml.chart+xml",
    partname
  ))

  # get or create drawing for the sheet
  sheet_idx <- which(x$worksheets$sheet_names() == sheet)
  sheet_obj <- x$sheets$get_sheet(sheet_idx)
  drw <- xlsx_drawing$new(package_dir, sheet_obj, x$content_type)

  # add chart relationship and anchor in drawing
  chart_rid <- drw$add_chart_rel(chart_name)
  drw$add_chart_anchor(
    chart_rid,
    left = left, top = top,
    width = width, height = height
  )

  invisible(x)
}

#' @title Treemap chart object
#' @description Creation of a treemap chart object that can be inserted
#' in a 'Microsoft' document. Treemap charts use the chartEx pipeline
#' (Office 2016+); older versions of 'Microsoft Office' will display a
#' fallback placeholder.
#'
#' Data is hierarchical: one column per level (parent to leaf, left to
#' right) and one numeric column for the leaf values.
#' @param data a data.frame.
#' @param path character vector of column names defining the hierarchy,
#' from outermost (root) to innermost (leaf).
#' @param value column name for the numeric leaf values.
#' @param labels unused for now; reserved for future custom data label
#' columns.
#' @section Per-leaf coloring with `chart_data_fill()`:
#' Per-leaf coloring via [chart_data_fill()] with a named vector works
#' correctly on flat treemaps (`length(path) == 1`). With a hierarchy
#' (`length(path) >= 2`), PowerPoint silently re-maps `<cx:dataPt>`
#' indices in a way that drops `idx="0"` and applies the last specified
#' color to remaining leaves. This is a PowerPoint rendering limitation
#' (also reproducible from Excel-generated chartEx files); the mschart
#' XML output is conformant. For hierarchical treemaps, prefer a single
#' fill color via `chart_data_fill(x, "#HEX")`.
#' @return An `ms_chart` object (subclass `ms_treemapchart`).
#' @family 'Office' chart objects
#' @seealso [chart_labels()], [chart_theme()]
#' @export
#' @examples
#' library(officer)
#'
#' dat <- data.frame(
#'   region  = c("EU", "EU", "EU", "AM", "AM"),
#'   country = c("FR", "FR", "DE", "US", "US"),
#'   city    = c("Paris", "Lyon", "Berlin", "NYC", "LA"),
#'   value   = c(10, 5, 12, 20, 8),
#'   stringsAsFactors = FALSE
#' )
#' tm <- ms_treemapchart(
#'   data = dat, path = c("region", "country", "city"), value = "value"
#' )
#' tm <- chart_labels(tm, title = "Sales by region")
#'
#' doc <- read_pptx()
#' doc <- add_slide(doc)
#' doc <- ph_with(doc, tm, location = ph_location_fullsize())
#' print(doc, target = tempfile(fileext = ".pptx"))
ms_treemapchart <- function(data, path, value, labels = NULL) {
  stopifnot(is.data.frame(data))
  stopifnot(is.character(path), length(path) >= 1L)
  stopifnot(is.character(value), length(value) == 1L)

  missing_path <- setdiff(path, names(data))
  if (length(missing_path)) {
    stop(
      "column(s) ",
      paste(shQuote(missing_path), collapse = ", "),
      " not found in data",
      call. = FALSE
    )
  }
  if (!value %in% names(data)) {
    stop("column ", shQuote(value), " not found in data", call. = FALSE)
  }
  if (!is.numeric(data[[value]])) {
    stop("column ", shQuote(value), " must be numeric", call. = FALSE)
  }

  if (
    inherits(data, "data.table") ||
      inherits(data, "tbl_df") ||
      inherits(data, "tbl")
  ) {
    data <- as.data.frame(data, stringsAsFactors = FALSE)
  }

  # Normalize path columns to character and ensure data_series order is
  # path columns first (one per level, root..leaf) then the value column.
  for (p in path) {
    if (is.factor(data[[p]])) {
      data[[p]] <- as.character(data[[p]])
    } else {
      data[[p]] <- as.character(data[[p]])
    }
  }

  data_series <- data[, c(path, value), drop = FALSE]
  rownames(data_series) <- NULL

  out <- list(
    data = data,
    data_series = data_series,
    path = path,
    value = value,
    # ms_chart compat fields used by setters / print method
    x = path[1],
    y = value,
    xvar = path[1],
    yvar = value,
    group = NULL,
    label_cols = labels,
    asis = FALSE,
    theme = mschart_theme(),
    labels = list(title = NULL, x = NULL, y = NULL),
    layout_id = "treemap"
  )
  class(out) <- c("ms_treemapchart", "ms_chart_ex", "ms_chart")
  out
}

#' @export
#' @method format ms_treemapchart
format.ms_treemapchart <- function(
  x,
  id_x = NULL,
  id_y = NULL,
  sheetname = "sheet1",
  drop_ext_data = FALSE,
  ...
) {
  cx_format_hierarchical(
    x,
    layout_id = "treemap",
    sheetname = sheetname,
    drop_ext_data = drop_ext_data,
    series_layout_pr = "<cx:layoutPr><cx:parentLabelLayout val=\"overlapping\"/></cx:layoutPr>"
  )
}

#' @export
#' @method print ms_treemapchart
print.ms_treemapchart <- function(x, preview = FALSE, ...) {
  if (preview && interactive()) {
    doc <- officer::read_pptx()
    doc <- officer::add_slide(
      doc,
      layout = "Title and Content",
      master = "Office Theme"
    )
    doc <- officer::ph_with(doc, x, location = officer::ph_location_fullsize())
    file_out <- print(doc, target = tempfile(fileext = ".pptx"))
    utils::browseURL(file_out)
    return(invisible())
  }
  cat(sprintf("* %s object (chartEx)\n\n", shQuote("ms_treemapchart")))
  cat(sprintf("* path: %s\n", paste(x$path, collapse = " > ")))
  cat(sprintf("* value: %s\n\n", x$value))
  cat(sprintf(
    "* original data [%.0f,%.0f] (sample):\n",
    nrow(x$data),
    ncol(x$data)
  ))
  print(x$data[seq_len(min(c(nrow(x$data), 5))), ])
}

cx_ns <- c(
  cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
  a = "http://schemas.openxmlformats.org/drawingml/2006/main"
)

bar_data <- function() {
  data.frame(
    step = c("Start", "Q1", "Q2", "Q3", "End"),
    amount = c(100, 30, -20, 40, 150),
    stringsAsFactors = FALSE
  )
}
hier_data <- function() {
  data.frame(
    region = c("EU", "EU", "AM", "AM"),
    city = c("Paris", "Lyon", "NYC", "LA"),
    value = c(10, 5, 20, 8),
    stringsAsFactors = FALSE
  )
}

test_that("chart_data_fill: single color emits one <cx:spPr> on the series", {
  wf <- ms_waterfallchart(bar_data(), x = "step", y = "amount")
  wf <- chart_data_fill(wf, values = "#3366CC")
  doc <- xml2::read_xml(format(wf))
  spPr_nodes <- xml2::xml_find_all(
    doc,
    "//cx:series/cx:spPr/a:solidFill/a:srgbClr",
    cx_ns
  )
  expect_length(spPr_nodes, 1L)
  expect_equal(xml2::xml_attr(spPr_nodes, "val"), "3366CC")
  # No per-point dataPt emitted in single-color mode
  expect_length(xml2::xml_find_all(doc, "//cx:series/cx:dataPt", cx_ns), 0L)
})

test_that("chart_data_fill: named vector emits per-point <cx:dataPt> with correct idx", {
  wf <- ms_waterfallchart(bar_data(), x = "step", y = "amount")
  wf <- chart_data_fill(
    wf,
    values = c(Start = "gray", End = "gray", Q2 = "red")
  )
  doc <- xml2::read_xml(format(wf))
  dpt <- xml2::xml_find_all(doc, "//cx:series/cx:dataPt", cx_ns)
  expect_length(dpt, 3L)
  # 0-based idx: Start=0, Q2=2, End=4
  expect_setequal(xml2::xml_attr(dpt, "idx"), c("0", "2", "4"))
  # Q2 -> red FF0000
  q2_color <- xml2::xml_attr(
    xml2::xml_find_first(
      dpt[xml2::xml_attr(dpt, "idx") == "2"],
      ".//a:srgbClr",
      cx_ns
    ),
    "val"
  )
  expect_equal(q2_color, "FF0000")
})

test_that("chart_data_fill: full per-row vector applied positionally", {
  wf <- ms_waterfallchart(bar_data(), x = "step", y = "amount")
  cols <- c("#111111", "#222222", "#333333", "#444444", "#555555")
  wf <- chart_data_fill(wf, values = cols)
  doc <- xml2::read_xml(format(wf))
  dpt <- xml2::xml_find_all(doc, "//cx:series/cx:dataPt", cx_ns)
  expect_length(dpt, 5L)
  vals <- xml2::xml_attr(
    xml2::xml_find_all(
      doc,
      "//cx:series/cx:dataPt/cx:spPr/a:solidFill/a:srgbClr",
      cx_ns
    ),
    "val"
  )
  expect_equal(vals, c("111111", "222222", "333333", "444444", "555555"))
})

test_that("chart_data_fill: hierarchical chart matches against leaf column", {
  tm <- ms_treemapchart(
    hier_data(),
    path = c("region", "city"),
    value = "value"
  )
  tm <- chart_data_fill(tm, values = c(Paris = "blue", LA = "orange"))
  doc <- xml2::read_xml(format(tm))
  dpt <- xml2::xml_find_all(doc, "//cx:series/cx:dataPt", cx_ns)
  expect_length(dpt, 2L)
  expect_setequal(xml2::xml_attr(dpt, "idx"), c("0", "3"))
})

test_that("chart_data_fill: histogram rejects named-vector fill (no cat column)", {
  set.seed(1)
  hi <- ms_histogramchart(data.frame(x = rnorm(50)), value = "x")
  expect_error(
    chart_data_fill(hi, values = c(A = "red")),
    "named-vector fill is not supported"
  )
  # but single color works
  hi2 <- chart_data_fill(hi, values = "purple")
  doc <- xml2::read_xml(format(hi2))
  fill <- xml2::xml_find_first(
    doc,
    "//cx:series/cx:spPr/a:solidFill/a:srgbClr",
    cx_ns
  )
  expect_equal(xml2::xml_attr(fill, "val"), "A020F0")
})

test_that("chart_data_fill: pareto with aggregate=TRUE rejects named-vector fill", {
  d <- data.frame(defect = sample(c("A", "B", "C"), 30, replace = TRUE))
  pa <- ms_paretochart(d, x = "defect")
  expect_error(
    chart_data_fill(pa, values = c(A = "red")),
    "named-vector fill is not supported"
  )
  # single color is fine and applies to the column series only
  pa2 <- chart_data_fill(pa, values = "#0080FF")
  doc <- xml2::read_xml(format(pa2))
  fills <- xml2::xml_find_all(
    doc,
    "//cx:series[@layoutId='clusteredColumn']/cx:spPr/a:solidFill/a:srgbClr",
    cx_ns
  )
  expect_length(fills, 1L)
})

test_that("chart_data_fill: invalid color rejected", {
  wf <- ms_waterfallchart(bar_data(), x = "step", y = "amount")
  expect_error(chart_data_fill(wf, values = "not-a-color"), "invalid color")
  expect_error(chart_data_fill(wf, values = NULL), "non-empty")
  expect_error(chart_data_fill(wf, values = 1L), "character vector")
})

test_that("chart_data_fill: unknown category name in named vector errors", {
  wf <- ms_waterfallchart(bar_data(), x = "step", y = "amount")
  expect_error(
    chart_data_fill(wf, values = c(Bogus = "red")),
    "not found in 'step'"
  )
})

test_that("chart_data_fill: round-trip pptx still opens (single color, treemap)", {
  skip_if_not_installed("officer")
  tm <- ms_treemapchart(
    hier_data(),
    path = c("region", "city"),
    value = "value"
  )
  tm <- chart_data_fill(tm, values = "#FF8800")
  doc <- officer::read_pptx()
  doc <- officer::add_slide(doc, "Title and Content", "Office Theme")
  doc <- officer::ph_with(doc, tm, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

test_that("chart_data_fill: round-trip pptx still opens (per-point, waterfall)", {
  skip_if_not_installed("officer")
  wf <- ms_waterfallchart(
    bar_data(),
    x = "step",
    y = "amount",
    subtotals = c(1, 5)
  )
  wf <- chart_data_fill(
    wf,
    values = c(Start = "gray", End = "gray", Q2 = "red")
  )
  doc <- officer::read_pptx()
  doc <- officer::add_slide(doc, "Title and Content", "Office Theme")
  doc <- officer::ph_with(doc, wf, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

cx_ns <- c(
  cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
  a = "http://schemas.openxmlformats.org/drawingml/2006/main"
)

# --- defaults preserved per layout (back-compat with Steps 0..4) ---------

test_that("default visibility: treemap shows category, hides val", {
  d <- data.frame(city = c("A", "B"), value = c(10, 20))
  tm <- ms_treemapchart(d, path = "city", value = "value")
  doc <- xml2::read_xml(format(tm))
  vis <- xml2::xml_find_first(doc, "//cx:dataLabels/cx:visibility", cx_ns)
  expect_equal(xml2::xml_attr(vis, "categoryName"), "1")
  expect_equal(xml2::xml_attr(vis, "value"), "0")
})

test_that("default visibility: waterfall shows val, hides cat", {
  d <- data.frame(step = c("A", "B"), amount = c(1, 2))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  doc <- xml2::read_xml(format(wf))
  vis <- xml2::xml_find_first(doc, "//cx:dataLabels/cx:visibility", cx_ns)
  expect_equal(xml2::xml_attr(vis, "value"), "1")
  expect_equal(xml2::xml_attr(vis, "categoryName"), "0")
})

test_that("default visibility: boxplot/histogram/pareto emit no <cx:dataLabels>", {
  set.seed(1)
  bp <- ms_boxplotchart(
    data.frame(g = rep(c("A", "B"), each = 5), v = rnorm(10)),
    x = "g",
    y = "v"
  )
  hi <- ms_histogramchart(data.frame(x = rnorm(20)), value = "x")
  pa <- ms_paretochart(
    data.frame(d = c("A", "B"), n = c(5, 3)),
    x = "d",
    y = "n",
    aggregate = FALSE
  )
  for (chart in list(bp, hi, pa)) {
    doc <- xml2::read_xml(format(chart))
    expect_length(xml2::xml_find_all(doc, "//cx:dataLabels", cx_ns), 0L)
  }
})

# --- chart_data_labels overrides -----------------------------------------

test_that("chart_data_labels overrides treemap defaults (val on, cat off)", {
  d <- data.frame(city = c("A", "B"), value = c(10, 20))
  tm <- ms_treemapchart(d, path = "city", value = "value")
  tm <- chart_data_labels(tm, show_val = TRUE, show_cat = FALSE)
  doc <- xml2::read_xml(format(tm))
  vis <- xml2::xml_find_first(doc, "//cx:dataLabels/cx:visibility", cx_ns)
  expect_equal(xml2::xml_attr(vis, "value"), "1")
  expect_equal(xml2::xml_attr(vis, "categoryName"), "0")
})

test_that("chart_data_labels enables labels on histogram (none by default)", {
  set.seed(1)
  hi <- ms_histogramchart(data.frame(x = rnorm(20)), value = "x")
  hi <- chart_data_labels(hi, show_val = TRUE, position = "outEnd")
  doc <- xml2::read_xml(format(hi))
  dl <- xml2::xml_find_first(doc, "//cx:dataLabels", cx_ns)
  expect_equal(xml2::xml_attr(dl, "pos"), "outEnd")
  vis <- xml2::xml_find_first(dl, "./cx:visibility", cx_ns)
  expect_equal(xml2::xml_attr(vis, "value"), "1")
})

test_that("chart_data_labels(num_fmt=) emits <cx:numFmt> inside dataLabels", {
  d <- data.frame(step = c("A", "B"), amount = c(1.5, 2.5))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  wf <- chart_data_labels(wf, num_fmt = "0.00")
  doc <- xml2::read_xml(format(wf))
  nf <- xml2::xml_find_first(doc, "//cx:dataLabels/cx:numFmt", cx_ns)
  expect_equal(xml2::xml_attr(nf, "formatCode"), "0.00")
})

test_that("chart_data_labels(position=) overrides default position", {
  d <- data.frame(step = c("A", "B"), amount = c(1, 2))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  wf <- chart_data_labels(wf, position = "ctr")
  doc <- xml2::read_xml(format(wf))
  dl <- xml2::xml_find_first(doc, "//cx:dataLabels", cx_ns)
  expect_equal(xml2::xml_attr(dl, "pos"), "ctr")
})

# --- chart_labels_text font ----------------------------------------------

test_that("chart_labels_text adds <cx:txPr> inside dataLabels with sz/color", {
  d <- data.frame(step = c("A", "B"), amount = c(1, 2))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  wf <- chart_labels_text(
    wf,
    officer::fp_text(font.size = 9, bold = TRUE, color = "purple")
  )
  doc <- xml2::read_xml(format(wf))
  defRPr <- xml2::xml_find_first(
    doc,
    "//cx:dataLabels/cx:txPr//a:defRPr",
    cx_ns
  )
  expect_equal(xml2::xml_attr(defRPr, "sz"), "900")
  expect_equal(xml2::xml_attr(defRPr, "b"), "1")
  expect_equal(
    xml2::xml_attr(xml2::xml_find_first(defRPr, ".//a:srgbClr", cx_ns), "val"),
    "A020F0"
  )
})

test_that("chart_labels_text errors on non-fp_text input", {
  d <- data.frame(step = c("A", "B"), amount = c(1, 2))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  expect_error(
    chart_labels_text(wf, list(font.size = 10)),
    "single fp_text"
  )
})

# --- round-trip ---------------------------------------------------------

test_that("round-trip pptx still opens with chart_data_labels + chart_labels_text", {
  skip_if_not_installed("officer")
  d <- data.frame(step = c("A", "B", "C"), amount = c(10, -5, 8))
  wf <- ms_waterfallchart(d, x = "step", y = "amount")
  wf <- chart_data_labels(
    wf,
    show_val = TRUE,
    num_fmt = "0",
    position = "outEnd"
  )
  wf <- chart_labels_text(wf, officer::fp_text(font.size = 8, bold = TRUE))
  doc <- officer::read_pptx()
  doc <- officer::add_slide(doc, "Title and Content", "Office Theme")
  doc <- officer::ph_with(doc, wf, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

clustered_pos <- c("ctr", "inBase", "inEnd", "outEnd")
stacked_pos <- c("ctr", "inBase", "inEnd")

#' @export
#' @method to_pml ms_barchart
to_pml.ms_barchart <- function(
  x,
  add_ns = FALSE,
  id_x,
  id_y,
  sheetname = "sheet1",
  secondary_y = 0,
  ...
) {
  if ("clustered" %in% x$options$grouping) {
    if (!x$label_settings$position %in% clustered_pos) {
      stop(
        "label position issue with grouping 'clustered'.",
        "Arg. position in chart_data_labels() should match one of ",
        paste(shQuote(clustered_pos), collapse = ", "),
        ".",
        call. = FALSE
      )
    }
  }
  if ("stacked" %in% x$options$grouping) {
    if (!x$label_settings$position %in% stacked_pos) {
      stop(
        "label position issue with grouping 'clustered'.",
        "Arg. position in chart_data_labels() should match one of ",
        paste(shQuote(stacked_pos), collapse = ", "),
        ".",
        call. = FALSE
      )
    }
  }

  series <- as_series(
    x,
    x_class = serie_builtin_class(x$data[[x$x]]),
    y_class = serie_builtin_class(x$data[[x$y]]),
    sheetname = sheetname,
    secondary_y = secondary_y
  )

  str_series_ <- sapply(series, function(serie, template) {
    marker_str <- get_sppr_xml(serie$fill, serie$stroke, serie$line_width)

    label_settings <- x$label_settings
    label_settings$labels_fp <- serie$labels_fp

    if (!is.null(x$label_cols)) {
      label_pml <- to_pml(serie$label)
    } else {
      label_pml <- ""
    }

    paste0(
      "<c:ser>",
      sprintf("<c:idx val=\"%.0f\"/>", max(0, serie$idx)),
      sprintf("<c:order val=\"%.0f\"/>", max(0, serie$order)),
      sprintf("<c:tx>%s</c:tx>", to_pml(serie$tx)),
      marker_str,
      "<c:invertIfNegative val=\"0\"/>",
      to_pml(label_settings, show_label = !is.null(x$label_cols)),
      "<c:cat>",
      to_pml(serie$x),
      "</c:cat>",
      "<c:val>",
      to_pml(serie$y),
      "</c:val>",
      label_pml,
      "</c:ser>"
    )
  })
  str_series_ <- paste(str_series_, collapse = "")

  x_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_x)
  y_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_y)

  dir_ <- structure(c("bar", "col"), .Names = c("horizontal", "vertical"))
  dir_ <- dir_[x$options$dir]

  paste0(
    "<c:barChart>",
    sprintf("<c:barDir val=\"%s\"/>", dir_),
    sprintf("<c:grouping val=\"%s\"/>", x$options$grouping),
    sprintf("<c:varyColors val=\"%.0f\"/>", x$options$vary_colors),
    str_series_,
    to_pml(x$label_settings, !is.null(x$label_cols)),
    sprintf("<c:gapWidth val=\"%.0f\"/>", x$options$gap_width),
    sprintf("<c:overlap val=\"%.0f\"/>", x$options$overlap),
    x_ax_id,
    y_ax_id,
    "</c:barChart>"
  )
}

standard_pos <- c("b", "ctr", "l", "r", "t")

#' @export
#' @method to_pml ms_linechart
to_pml.ms_linechart <- function(
  x,
  add_ns = FALSE,
  id_x,
  id_y,
  sheetname = "sheet1",
  secondary_y = 0,
  ...
) {
  if (!x$label_settings$position %in% standard_pos) {
    stop(
      "label position issue.",
      "Arg. position in chart_data_labels() should match one of ",
      paste(shQuote(standard_pos), collapse = ", "),
      ".",
      call. = FALSE
    )
  }

  series <- as_series(
    x,
    x_class = serie_builtin_class(x$data[[x$x]]),
    y_class = serie_builtin_class(x$data[[x$y]]),
    sheetname = sheetname,
    secondary_y = secondary_y
  )

  # sapply linec-----
  str_series_ <- sapply(
    series,
    function(serie, has_line, has_marker) {
      if (!has_line) {
        line_str <- "<c:spPr><a:ln><a:noFill/></a:ln></c:spPr>"
      } else {
        line_properties <- fp_border(
          color = serie$stroke,
          style = serie$line_style,
          width = serie$line_width
        )
        line_str <- ooxml_fp_border(line_properties, in_tags = c("c:spPr"))
      }
      if (!has_marker) {
        marker_str <- "<c:marker><c:symbol val=\"none\"/></c:marker>"
      } else {
        marker_str <- get_marker_xml(
          serie$fill,
          serie$stroke,
          serie$symbol,
          serie$size
        )
      }

      label_settings <- x$label_settings
      label_settings$labels_fp <- serie$labels_fp

      if (!is.null(x$label_cols)) {
        label_pml <- to_pml(serie$label)
      } else {
        label_pml <- ""
      }

      paste0(
        "<c:ser>",
        sprintf("<c:idx val=\"%.0f\"/>", serie$idx),
        sprintf("<c:order val=\"%.0f\"/>", serie$order),
        sprintf("<c:tx>%s</c:tx>", to_pml(serie$tx)),
        line_str,
        marker_str,
        to_pml(label_settings, show_label = !is.null(x$label_cols)),
        "<c:cat>",
        to_pml(serie$x),
        "</c:cat>",
        "<c:val>",
        to_pml(serie$y),
        "</c:val>",
        label_pml,
        sprintf("<c:smooth val=\"%.0f\"/>", serie$smooth),
        "</c:ser>"
      )
    },
    has_line = has_lines[x$options$style],
    has_marker = has_markers[x$options$style]
  )

  str_series_ <- paste(str_series_, collapse = "")

  x_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_x)
  y_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_y)

  paste0(
    "<c:lineChart>",
    sprintf("<c:grouping val=\"%s\"/>", x$options$grouping),
    sprintf("<c:varyColors val=\"%.0f\"/>", x$options$vary_colors),
    str_series_,
    to_pml(x$label_settings),
    x_ax_id,
    y_ax_id,
    "</c:lineChart>"
  )
}


#' @export
#' @method to_pml ms_stockchart
to_pml.ms_stockchart <- function(
  x,
  add_ns = FALSE,
  id_x,
  id_y,
  sheetname = "sheet1",
  secondary_y = 0,
  ...
) {
  series <- as_series(
    x,
    x_class = serie_builtin_class(x$data[[x$x]]),
    y_class = serie_builtin_class(x$data[["y_val"]]),
    sheetname = sheetname,
    secondary_y = secondary_y
  )

  str_series_ <- sapply(series, function(serie) {
    line_properties <- fp_border(
      color = serie$stroke,
      style = serie$line_style,
      width = serie$line_width
    )
    line_str <- ooxml_fp_border(line_properties, in_tags = c("c:spPr"))
    marker_str <- get_marker_xml(
      serie$fill, serie$stroke,
      serie$symbol, serie$size
    )

    paste0(
      "<c:ser>",
      sprintf("<c:idx val=\"%.0f\"/>", serie$idx),
      sprintf("<c:order val=\"%.0f\"/>", serie$order),
      sprintf("<c:tx>%s</c:tx>", to_pml(serie$tx)),
      line_str,
      marker_str,
      "<c:cat>", to_pml(serie$x), "</c:cat>",
      "<c:val>", to_pml(serie$y), "</c:val>",
      "<c:smooth val=\"0\"/>",
      "</c:ser>"
    )
  })

  str_series_ <- paste(str_series_, collapse = "")

  # hiLowLines
  hi_low_str <- ""
  if (!isFALSE(x$options$hi_low_lines)) {
    hi_low_str <- ooxml_fp_border(
      x$options$hi_low_lines,
      in_tags = c("c:hiLowLines", "c:spPr")
    )
  }

  # upDownBars (OHLC only)
  up_down_str <- ""
  has_open <- !is.null(x$stock_cols$open)
  if (has_open) {
    up_fill <- stock_fill_xml(x$options$up_bars_fill)
    up_border <- ooxml_fp_border(x$options$up_bars_border)
    down_fill <- stock_fill_xml(x$options$down_bars_fill)
    down_border <- ooxml_fp_border(x$options$down_bars_border)

    up_down_str <- paste0(
      "<c:upDownBars>",
      "<c:gapWidth val=\"150\"/>",
      "<c:upBars><c:spPr>", up_fill, up_border, "</c:spPr></c:upBars>",
      "<c:downBars><c:spPr>", down_fill, down_border, "</c:spPr></c:downBars>",
      "</c:upDownBars>"
    )
  }

  x_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_x)
  y_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_y)

  paste0(
    "<c:stockChart>",
    str_series_,
    "<c:dLbls>",
    "<c:showLegendKey val=\"0\"/>",
    "<c:showVal val=\"0\"/>",
    "<c:showCatName val=\"0\"/>",
    "<c:showSerName val=\"0\"/>",
    "<c:showPercent val=\"0\"/>",
    "<c:showBubbleSize val=\"0\"/>",
    "</c:dLbls>",
    hi_low_str,
    up_down_str,
    x_ax_id,
    y_ax_id,
    "</c:stockChart>"
  )
}

#' @export
#' @method to_pml ms_radarchart
to_pml.ms_radarchart <- function(
  x,
  add_ns = FALSE,
  id_x,
  id_y,
  sheetname = "sheet1",
  secondary_y = 0,
  ...
) {
  series <- as_series(
    x,
    x_class = serie_builtin_class(x$data[[x$x]]),
    y_class = serie_builtin_class(x$data[[x$y]]),
    sheetname = sheetname,
    secondary_y = secondary_y
  )

  has_lines <- c(
    standard = TRUE, marker = TRUE, filled = TRUE
  )
  has_markers <- c(
    standard = FALSE, marker = TRUE, filled = FALSE
  )

  str_series_ <- sapply(
    series,
    function(serie, has_line, has_marker) {
      if (!has_line) {
        line_str <- "<c:spPr><a:ln><a:noFill/></a:ln></c:spPr>"
      } else {
        line_properties <- fp_border(
          color = serie$stroke,
          style = serie$line_style,
          width = serie$line_width
        )
        line_str <- ooxml_fp_border(line_properties, in_tags = c("c:spPr"))
      }
      if (!has_marker) {
        marker_str <- "<c:marker><c:symbol val=\"none\"/></c:marker>"
      } else {
        marker_str <- get_marker_xml(
          serie$fill, serie$stroke,
          serie$symbol, serie$size
        )
      }

      label_settings <- x$label_settings
      label_settings$labels_fp <- serie$labels_fp

      label_pml <- ""
      if (!is.null(x$label_cols)) {
        label_pml <- to_pml(serie$label)
      }

      paste0(
        "<c:ser>",
        sprintf("<c:idx val=\"%.0f\"/>", serie$idx),
        sprintf("<c:order val=\"%.0f\"/>", serie$order),
        sprintf("<c:tx>%s</c:tx>", to_pml(serie$tx)),
        line_str,
        marker_str,
        to_pml(label_settings,
               with_position = FALSE,
               show_label = !is.null(x$label_cols)),
        "<c:cat>", to_pml(serie$x), "</c:cat>",
        "<c:val>", to_pml(serie$y), "</c:val>",
        "</c:ser>"
      )
    },
    has_line = has_lines[x$options$radarstyle],
    has_marker = has_markers[x$options$radarstyle]
  )

  str_series_ <- paste(str_series_, collapse = "")

  x_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_x)
  y_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_y)

  paste0(
    "<c:radarChart>",
    sprintf("<c:radarStyle val=\"%s\"/>", x$options$radarstyle),
    sprintf("<c:varyColors val=\"%.0f\"/>", x$options$vary_colors),
    str_series_,
    to_pml(x$label_settings,
           with_position = FALSE,
           show_label = !is.null(x$label_cols)),
    x_ax_id,
    y_ax_id,
    "</c:radarChart>"
  )
}

#' @export
#' @method to_pml ms_bubblechart
to_pml.ms_bubblechart <- function(
  x,
  add_ns = FALSE,
  id_x,
  id_y,
  sheetname = "sheet1",
  secondary_y = 0,
  ...
) {
  x_class <- serie_builtin_class(x$data[[x$x]])
  y_class <- serie_builtin_class(x$data[[x$y]])

  series <- as_series(
    x,
    x_class = x_class,
    y_class = y_class,
    sheetname = sheetname,
    secondary_y = secondary_y
  )

  str_series_ <- sapply(series, function(serie) {
    sppr_str <- get_sppr_xml(serie$fill, serie$stroke, serie$line_width)

    label_settings <- x$label_settings
    label_settings$labels_fp <- serie$labels_fp

    label_pml <- ""
    if (!is.null(x$label_cols) && !is.null(serie$label)) {
      label_pml <- to_pml(serie$label)
    }

    bubble_size_str <- ""
    if (!is.null(serie$bubble_size)) {
      bubble_size_str <- paste0(
        "<c:bubbleSize>", to_pml(serie$bubble_size), "</c:bubbleSize>"
      )
    }

    paste0(
      "<c:ser>",
      sprintf("<c:idx val=\"%.0f\"/>", serie$idx),
      sprintf("<c:order val=\"%.0f\"/>", serie$order),
      sprintf("<c:tx>%s</c:tx>", to_pml(serie$tx)),
      sppr_str,
      to_pml(label_settings, show_label = !is.null(x$label_cols)),
      "<c:invertIfNegative val=\"0\"/>",
      "<c:xVal>", to_pml(serie$x), "</c:xVal>",
      "<c:yVal>", to_pml(serie$y), "</c:yVal>",
      bubble_size_str,
      sprintf(
        "<c:bubble3D val=\"%.0f\"/>",
        x$options$bubble3D %||% 0
      ),
      label_pml,
      "</c:ser>"
    )
  })

  str_series_ <- paste(str_series_, collapse = "")

  x_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_x)
  y_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_y)

  paste0(
    "<c:bubbleChart>",
    sprintf("<c:varyColors val=\"%.0f\"/>", x$options$vary_colors),
    str_series_,
    to_pml(x$label_settings, !is.null(x$label_cols)),
    x_ax_id,
    y_ax_id,
    "</c:bubbleChart>"
  )
}

#' @export
#' @method to_pml ms_areachart
to_pml.ms_areachart <- function(
  x,
  add_ns = FALSE,
  id_x,
  id_y,
  sheetname = "sheet1",
  secondary_y = 0,
  ...
) {
  series <- as_series(
    x,
    x_class = serie_builtin_class(x$data[[x$x]]),
    y_class = serie_builtin_class(x$data[[x$y]]),
    sheetname = sheetname,
    secondary_y = secondary_y
  )

  str_series_ <- sapply(series, function(serie) {
    marker_str <- get_sppr_xml(serie$fill, serie$stroke, serie$line_width)

    label_settings <- x$label_settings
    label_settings$labels_fp <- serie$labels_fp

    if (!is.null(x$label_cols)) {
      label_pml <- to_pml(serie$label)
    } else {
      label_pml <- ""
    }

    paste0(
      "<c:ser>",
      sprintf("<c:idx val=\"%.0f\"/>", serie$idx),
      sprintf("<c:order val=\"%.0f\"/>", serie$order),
      "<c:tx>",
      to_pml(serie$tx),
      "</c:tx>",
      marker_str,
      to_pml(
        label_settings,
        with_position = FALSE,
        show_label = !is.null(x$label_cols)
      ),
      "<c:cat>",
      to_pml(serie$x),
      "</c:cat>",
      "<c:val>",
      to_pml(serie$y),
      "</c:val>",
      label_pml,
      "</c:ser>"
    )
  })
  str_series_ <- paste(str_series_, collapse = "")

  x_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_x)
  y_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_y)

  paste0(
    "<c:areaChart>",
    sprintf("<c:grouping val=\"%s\"/>", x$options$grouping),
    sprintf("<c:varyColors val=\"%.0f\"/>", x$options$vary_colors),
    str_series_,
    to_pml(
      x$label_settings,
      with_position = FALSE,
      show_label = !is.null(x$label_cols)
    ),
    x_ax_id,
    y_ax_id,
    "</c:areaChart>"
  )
}

scatterstyles <- c(
  'none',
  'line',
  'lineMarker',
  'marker',
  'smooth',
  'smoothMarker'
)
has_markers <- c(FALSE, FALSE, TRUE, TRUE, FALSE, TRUE)
names(has_markers) <- scatterstyles
has_lines <- c(FALSE, TRUE, TRUE, FALSE, TRUE, TRUE)
names(has_lines) <- scatterstyles

#' @export
#' @method to_pml ms_scatterchart
to_pml.ms_scatterchart <- function(
  x,
  add_ns = FALSE,
  id_x,
  id_y,
  sheetname = "sheet1",
  secondary_y = 0,
  ...
) {
  if (!x$label_settings$position %in% standard_pos) {
    stop(
      "label position issue.",
      "Arg. position in chart_data_labels() should match one of ",
      paste(shQuote(standard_pos), collapse = ", "),
      ".",
      call. = FALSE
    )
  }

  if (isTRUE(x$asis)) {
    series <- as_series(
      x,
      x_class = serie_builtin_class(sort(unname(unlist(x$data_series[
        x$xvar
      ])))),
      y_class = serie_builtin_class(sort(unname(unlist(x$data_series[
        x$yvar
      ])))),
      sheetname = sheetname,
      secondary_y = secondary_y
    )
  } else {
    series <- as_series(
      x,
      x_class = serie_builtin_class(x$data[[x$x]]),
      y_class = serie_builtin_class(x$data[[x$y]]),
      sheetname = sheetname
    )
  }

  str_series_ <- sapply(
    series,
    function(serie, has_line, has_marker) {
      if (!has_line || serie$line_style %in% "none") {
        line_str <- "<c:spPr><a:ln><a:noFill/></a:ln></c:spPr>"
      } else {
        line_properties <- fp_border(
          color = serie$stroke,
          style = serie$line_style,
          width = serie$line_width
        )
        line_str <- ooxml_fp_border(line_properties, in_tags = c("c:spPr"))
      }
      if (!has_marker) {
        marker_str <- "<c:marker><c:symbol val=\"none\"/></c:marker>"
      } else {
        marker_str <- get_marker_xml(
          serie$fill,
          serie$stroke,
          serie$symbol,
          serie$size
        )
      }

      label_settings <- x$label_settings
      label_settings$labels_fp <- serie$labels_fp

      if (!is.null(x$label_cols)) {
        label_pml <- to_pml(serie$label)
      } else {
        label_pml <- ""
      }

      paste0(
        "<c:ser>",
        sprintf("<c:idx val=\"%.0f\"/>", serie$idx),
        sprintf("<c:order val=\"%.0f\"/>", serie$order),
        sprintf("<c:tx>%s</c:tx>", to_pml(serie$tx)),
        line_str,
        marker_str,
        to_pml(label_settings, show_label = !is.null(x$label_cols)),
        "<c:xVal>",
        to_pml(serie$x),
        "</c:xVal>",
        "<c:yVal>",
        to_pml(serie$y),
        "</c:yVal>",
        label_pml,
        sprintf("<c:smooth val=\"%.0f\"/>", serie$smooth),
        "</c:ser>"
      )
    },
    has_line = has_lines[x$options$style],
    has_marker = has_markers[x$options$style]
  )

  str_series_ <- paste(str_series_, collapse = "")

  x_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_x)
  y_ax_id <- sprintf("<c:axId val=\"%s\"/>", id_y)

  paste0(
    "<c:scatterChart>",
    sprintf("<c:scatterStyle val=\"%s\"/>", x$options$style),
    sprintf("<c:varyColors val=\"%.0f\"/>", x$options$vary_colors),
    str_series_,
    to_pml(x$label_settings, !is.null(x$label_cols)),
    x_ax_id,
    y_ax_id,
    "</c:scatterChart>"
  )
}


#' @export
#' @method to_pml ms_piechart
to_pml.ms_piechart <- function(
  x,
  add_ns = FALSE,
  id_x,
  id_y,
  sheetname = "sheet1",
  secondary_y = 0,
  ...
) {
  series <- as_series(
    x,
    x_class = serie_builtin_class(x$data[[x$x]]),
    y_class = serie_builtin_class(x$data[[x$y]]),
    sheetname = sheetname
  )

  str_series_ <- sapply(series, function(serie) {
    cats <- as.character(x$data[[x$x]])
    n <- length(cats)

    dpt_str <- vapply(
      seq_len(n),
      function(i) {
        cat_i <- cats[i]
        paste0(
          sprintf("<c:dPt><c:idx val=\"%s\"/><c:bubble3D val=\"0\"/>", i - 1),
          get_sppr_xml(
            x$series_settings$fill[[cat_i]],
            x$series_settings$colour[[cat_i]],
            x$series_settings$line_width[[cat_i]]
          ),
          "</c:dPt>"
        )
      },
      character(1)
    )
    dpt_str <- paste0(dpt_str, collapse = "")

    label_settings <- x$label_settings
    label_settings$labels_fp <- serie$labels_fp

    label_pml <- if (!is.null(x$label_cols)) to_pml(serie$label) else ""

    paste0(
      "<c:ser>",
      sprintf("<c:idx val=\"%.0f\"/>", serie$idx),
      sprintf("<c:order val=\"%.0f\"/>", serie$order),
      "<c:tx>",
      to_pml(serie$tx),
      "</c:tx>",
      dpt_str,
      to_pml(
        label_settings,
        with_position = FALSE,
        show_label = !is.null(x$label_cols)
      ),
      "<c:cat>",
      to_pml(serie$x),
      "</c:cat>",
      "<c:val>",
      to_pml(serie$y),
      "</c:val>",
      label_pml,
      "</c:ser>"
    )
  })
  str_series_ <- paste(str_series_, collapse = "")

  is_donut <- x$options$hole_size > 0
  tag <- if (is_donut) "c:doughnutChart" else "c:pieChart"
  hole_str <- if (is_donut) {
    sprintf("<c:holeSize val=\"%.0f\"/>", x$options$hole_size)
  } else {
    ""
  }

  paste0(
    "<",
    tag,
    ">",
    sprintf("<c:varyColors val=\"%.0f\"/>", x$options$vary_colors),
    str_series_,
    to_pml(
      x$label_settings,
      with_position = FALSE,
      show_label = !is.null(x$label_cols)
    ),
    hole_str,
    "</",
    tag,
    ">"
  )
}


#' @importFrom grDevices col2rgb
get_marker_xml <- function(fill, stroke, symbol = NULL, size = NULL) {
  fill_elts <- col2rgb(fill, alpha = TRUE)[, 1]
  fill_hex <- sprintf("%02X%02X%02X", fill_elts[1], fill_elts[2], fill_elts[3])
  stroke_elts <- col2rgb(stroke, alpha = TRUE)[, 1]
  stroke_hex <- sprintf(
    "%02X%02X%02X",
    stroke_elts[1],
    stroke_elts[2],
    stroke_elts[3]
  )

  if (!is.null(size) && size < 1) {
    symbol <- "none"
    size <- 2L
  }

  str_sym <- ""
  if (!is.null(symbol)) {
    str_sym <- sprintf("<c:symbol val=\"%s\"/>", symbol)
  }

  str_size <- ""
  if (!is.null(size)) {
    str_size <- sprintf("<c:size val=\"%.0f\"/>", size)
  }

  paste0(
    "<c:marker>",
    str_sym,
    str_size,
    get_sppr_xml(fill, stroke),
    "</c:marker>"
  )
}


get_sppr_xml <- function(fill, stroke, line_width = NULL) {
  fill_elts <- col2rgb(fill, alpha = TRUE)[, 1]
  fill_hex <- sprintf("%02X%02X%02X", fill_elts[1], fill_elts[2], fill_elts[3])
  stroke_elts <- col2rgb(stroke, alpha = TRUE)[, 1]
  stroke_hex <- sprintf(
    "%02X%02X%02X",
    stroke_elts[1],
    stroke_elts[2],
    stroke_elts[3]
  )
  stroke_width <- ""
  if (!is.null(line_width)) {
    stroke_width <- sprintf(" w=\"%.0f\"", 12700 * line_width)
  }
  paste0(
    "<c:spPr>",
    sprintf(
      "<a:solidFill><a:srgbClr val=\"%s\"><a:alpha val=\"%.0f\"/></a:srgbClr></a:solidFill>",
      fill_hex,
      fill_elts[4] / 255.0 * 100000
    ),
    sprintf(
      "<a:ln%s><a:solidFill><a:srgbClr val=\"%s\"><a:alpha val=\"%.0f\"/></a:srgbClr></a:solidFill></a:ln>",
      stroke_width,
      stroke_hex,
      stroke_elts[4] / 255.0 * 100000
    ),
    "<a:effectLst/></c:spPr>"
  )
}

#' @export
#' @title Set a barchart as a stacked barchart
#' @description Apply settings to an \code{ms_barchart} object to
#' produce a stacked barchart. Options are available to use percentage
#' instead of values and to choose if bars should be vertically or horizontally drawn.
#' @param x an \code{\link{ms_barchart}} object
#' @param dir the direction of the bars in the chart, value must be one of "horizontal" or "vertical".
#' @param percent should bars be displayed as percentages.
#' @param gap_width gap width between bars for each category on a bar chart, as a percentage
#' of the bar width. It can be set between 0 and 500.
#' @return An `ms_chart` object.
#' @seealso [chart_settings()], [ms_barchart()]
#' @examples
#' library(officer)
#'
#' my_bar_stack_01 <- ms_barchart(data = browser_data, x = "browser",
#'   y = "value", group = "serie")
#' my_bar_stack_01 <- as_bar_stack( my_bar_stack_01 )
#'
#' my_bar_stack_02 <- ms_barchart(data = browser_data, x = "browser",
#'   y = "value", group = "serie")
#' my_bar_stack_02 <- as_bar_stack( my_bar_stack_02, percent = TRUE,
#'   dir = "horizontal" )
#'
#' doc <- read_pptx()
#' doc <- add_slide(doc, layout = "Title and Content", master = "Office Theme")
#' doc <- ph_with(doc, my_bar_stack_02, location = ph_location_fullsize())
#'
#' fileout <- tempfile(fileext = ".pptx")
#' print(doc, target = fileout)
as_bar_stack <- function(x, dir = "vertical", percent = FALSE, gap_width = 50) {
  stopifnot(inherits(x, "ms_barchart"))

  grouping <- "stacked"
  if (percent) {
    grouping <- "percentStacked"
  }

  x <- chart_settings(
    x,
    grouping = grouping,
    dir = dir,
    gap_width = gap_width,
    overlap = 100
  )
  x <- chart_data_stroke(x, values = "transparent")
  if (dir == "horizontal") {
    x <- chart_theme(x = x, title_x_rot = 270, title_y_rot = 0)
  }

  x
}

# Properties supported by each chart type
# (verified against to_pml methods)
.series_property_support <- list(
  fill = c(
    "ms_barchart", "ms_linechart", "ms_areachart",
    "ms_scatterchart", "ms_stockchart",
    "ms_radarchart", "ms_bubblechart", "ms_piechart"
  ),
  colour = c(
    "ms_barchart", "ms_linechart", "ms_areachart",
    "ms_scatterchart", "ms_stockchart",
    "ms_radarchart", "ms_bubblechart", "ms_piechart"
  ),
  symbol = c(
    "ms_linechart", "ms_scatterchart",
    "ms_stockchart", "ms_radarchart"
  ),
  size = c(
    "ms_linechart", "ms_scatterchart",
    "ms_stockchart", "ms_radarchart"
  ),
  line_width = c(
    "ms_barchart", "ms_linechart", "ms_areachart",
    "ms_scatterchart", "ms_stockchart",
    "ms_radarchart", "ms_bubblechart", "ms_piechart"
  ),
  line_style = c(
    "ms_linechart", "ms_scatterchart",
    "ms_stockchart", "ms_radarchart"
  ),
  smooth = c(
    "ms_linechart", "ms_scatterchart"
  ),
  labels_fp = c(
    "ms_barchart", "ms_linechart", "ms_areachart",
    "ms_scatterchart", "ms_radarchart", "ms_piechart"
  )
)

check_series_property <- function(x, property) {
  chart_type <- class(x)[1]
  supported <- .series_property_support[[property]]
  if (!is.null(supported) &&
    !chart_type %in% supported) {
    warning(
      sprintf(
        "'%s' property has no effect on '%s' charts.",
        property, chart_type
      ),
      call. = FALSE
    )
  }
}

#' @export
#' @title Modify labels font settings
#' @description Specify mappings from levels in the data to displayed text font settings.
#' @param x an `ms_chart` object.
#' @param values a named list of [officer::fp_text()] objects to map data labels to.
#' It is a named list, the values will be matched based on the names.
#' If it contains only one [officer::fp_text()] object, it will be associated to all existing series.
#' @examples
#' library(officer)
#'
#' fp_text_settings <- list(
#'   serie1 = fp_text(font.size = 7, color = "red"),
#'   serie2 = fp_text(font.size = 0, color = "purple"),
#'   serie3 = fp_text(font.size = 19, color = "wheat")
#' )
#'
#' barchart <- ms_barchart(
#'   data = browser_data,
#'   x = "browser", y = "value", group = "serie")
#' barchart <- chart_data_labels(barchart, show_val = TRUE)
#' barchart <- chart_labels_text( barchart,
#'   values = fp_text_settings )
#' @return An `ms_chart` object.
#' @family Series customization functions
chart_labels_text <- function(x, values) {
  check_series_property(x, "labels_fp")
  serie_names <- names(x$series_settings$labels_fp)

  if (inherits(values, "fp_text")) {
    values <- rep(list(values), length(serie_names))
    values <- setNames(values, serie_names)
  }
  if (is.null(values) || !is.list(values) || length(values) < 1) {
    stop("values must be a list of fp_text objects", call. = FALSE)
  }

  if (!all(sapply(values, inherits, what = "fp_text"))) {
    stop("values must be a list of fp_text objects", call. = FALSE)
  }

  if (!all(names(values) %in% serie_names)) {
    stop(
      "values's names do not match series' names: ",
      paste0(shQuote(serie_names), collapse = ", ")
    )
  }

  x$series_settings$labels_fp[names(values)] <- values
  x
}


#' @export
#' @title Modify fill colour
#' @description Specify mappings from levels in the data to displayed fill colours.
#' @param x an `ms_chart` object.
#' @param values `character(num of series|1)`: a set of colour values to map data values to.
#' It is a named vector, the values will be matched based on the names.
#' If it contains only one colour, this colour will be associated to all existing series.
#' @param update_stroke if `TRUE` (the default), the series stroke
#' colour is also updated to `values`, so that the filled shape
#' and its border match. Series whose stroke was set to
#' `"transparent"` (for example by the `ms_areachart()` and
#' `ms_piechart()` constructors) are left untouched to preserve that
#' deliberate "no border" default. Set to `FALSE` to keep the current
#' stroke colours untouched and manage them independently via
#' [chart_data_stroke()].
#' @examples
#' my_scatter <- ms_scatterchart(data = iris, x = "Sepal.Length",
#'   y = "Sepal.Width",  group = "Species")
#' my_scatter <- chart_data_fill(my_scatter,
#'   values = c(virginica = "#6FA2FF", versicolor = "#FF6161", setosa = "#81FF5B") )
#' @return An `ms_chart` object.
#' @family Series customization functions
chart_data_fill <- function(x, values, update_stroke = TRUE) {
  check_series_property(x, "fill")
  valid_cols <- is_valid_color(values)
  if (!all(valid_cols)) {
    stop("invalid color(s) in argument values")
  }

  serie_names <- names(x$series_settings$fill)

  if (length(values) == 1) {
    values <- rep(values, length(serie_names))
    names(values) <- serie_names
  }

  if (!all(names(values) %in% serie_names)) {
    stop(
      "values's names do not match series' names: ",
      paste0(shQuote(serie_names), collapse = ", ")
    )
  }

  x$series_settings$fill[names(values)] <- values

  if (isTRUE(update_stroke)) {
    current <- x$series_settings$colour[names(values)]
    to_update <- names(values)[current != "transparent"]
    if (length(to_update) > 0) {
      x$series_settings$colour[to_update] <- values[to_update]
    }
  }

  x
}

#' @export
#' @title Modify marker stroke colour
#' @description Specify mappings from levels in the data to displayed marker stroke colours.
#' @param x an `ms_chart` object.
#' @param values `character(num of series)`: a set of colour values to map data values to.
#' It is a named vector, the values will be matched based on the names.
#' If it contains only one colour, this colour will be associated to all existing series.
#' @examples
#' my_scatter <- ms_scatterchart(data = iris, x = "Sepal.Length",
#'   y = "Sepal.Width",  group = "Species")
#' my_scatter <- chart_data_fill(my_scatter,
#'   values = c(virginica = "#6FA2FF", versicolor = "#FF6161", setosa = "#81FF5B") )
#' my_scatter <- chart_data_stroke(my_scatter,
#'   values = c(virginica = "black", versicolor = "black", setosa = "black") )
#' @return An `ms_chart` object.
#' @family Series customization functions
chart_data_stroke <- function(x, values) {
  check_series_property(x, "colour")
  valid_cols <- is_valid_color(values)
  if (!all(valid_cols)) {
    stop("invalid color(s) in argument values")
  }

  serie_names <- names(x$series_settings$colour)

  if (length(values) == 1) {
    values <- rep(values, length(serie_names))
    names(values) <- serie_names
  }

  if (!all(names(values) %in% serie_names)) {
    stop(
      "values's names do not match series' names: ",
      paste0(shQuote(serie_names), collapse = ", ")
    )
  }

  x$series_settings$colour[names(values)] <- values
  x
}

#' @export
#' @title Modify symbol
#' @description Specify mappings from levels in the data to displayed symbols.
#' @param x an `ms_chart` object.
#' @param values `character(num of series)`: a set of symbol values to map data values to.
#' It is a named vector, the values will be matched based on the names.
#' Possible values are: 'circle', 'dash', 'diamond', 'dot', 'none', 'plus',
#' 'square', 'star', 'triangle', 'x', 'auto'.
#' If it contains only one symbol, this symbol will be associated to all existing series.
#' @examples
#' my_scatter <- ms_scatterchart(data = iris, x = "Sepal.Length",
#'   y = "Sepal.Width",  group = "Species")
#' my_scatter <- chart_data_fill(my_scatter,
#'   values = c(virginica = "#6FA2FF", versicolor = "#FF6161", setosa = "#81FF5B") )
#' my_scatter <- chart_data_stroke(my_scatter,
#'   values = c(virginica = "black", versicolor = "black", setosa = "black") )
#' my_scatter <- chart_data_symbol(my_scatter,
#'   values = c(virginica = "circle", versicolor = "diamond", setosa = "circle") )
#' @return An `ms_chart` object.
#' @family Series customization functions
chart_data_symbol <- function(x, values) {
  check_series_property(x, "symbol")
  if (!all(values %in% st_markerstyle)) {
    stop(
      "values should have values matching ",
      paste0(shQuote(st_markerstyle), collapse = ", ")
    )
  }

  serie_names <- names(x$series_settings$symbol)

  if (length(values) == 1) {
    values <- rep(values, length(serie_names))
    names(values) <- serie_names
  }

  if (!all(names(values) %in% serie_names)) {
    stop(
      "values's names do not match series' names: ",
      paste0(shQuote(serie_names), collapse = ", ")
    )
  }

  x$series_settings$symbol[names(values)] <- values
  x
}

#' @export
#' @title Modify symbol size
#' @description Specify mappings from levels in the data to displayed size of symbols.
#' @param x an `ms_chart` object.
#' @param values `double(num of series)`: a set of size values to map data values to.
#' It is a named vector, the values will be matched based on the names.
#' If it contains only one size, this size will be associated to all existing series.
#' @examples
#' my_scatter <- ms_scatterchart(data = iris, x = "Sepal.Length",
#'   y = "Sepal.Width",  group = "Species")
#' my_scatter <- chart_data_fill(my_scatter,
#'   values = c(virginica = "#6FA2FF", versicolor = "#FF6161", setosa = "#81FF5B") )
#' my_scatter <- chart_data_stroke(my_scatter,
#'   values = c(virginica = "black", versicolor = "black", setosa = "black") )
#' my_scatter <- chart_data_symbol(my_scatter,
#'   values = c(virginica = "circle", versicolor = "diamond", setosa = "circle") )
#' my_scatter <- chart_data_size(my_scatter,
#'   values = c(virginica = 20, versicolor = 16, setosa = 20) )
#' @return An `ms_chart` object.
#' @family Series customization functions
chart_data_size <- function(x, values) {
  check_series_property(x, "size")
  if (!is.numeric(values)) {
    stop("values should be numeric values")
  }
  if (any(sign(values) < 0)) {
    stop("values should not contain negative values")
  }

  serie_names <- names(x$series_settings$size)

  if (length(values) == 1) {
    values <- rep(values, length(serie_names))
    names(values) <- serie_names
  }

  if (!all(names(values) %in% serie_names)) {
    stop(
      "values's names do not match series' names: ",
      paste0(shQuote(serie_names), collapse = ", ")
    )
  }

  x$series_settings$size[names(values)] <- values
  x
}

#' @export
#' @title Modify line width
#' @description Specify mappings from levels in the data to the displayed line width.
#' @param x an `ms_chart` object.
#' @param values `double(num of series)`: a set of size values to map data values to.
#' It is a named vector, the values will be matched based on the names.
#' If it contains only one size, this size will be associated to all existing series.
#' @examples
#' my_scatter <- ms_scatterchart(data = iris, x = "Sepal.Length",
#'   y = "Sepal.Width",  group = "Species")
#' my_scatter <- chart_settings(my_scatter, style = "lineMarker")
#' my_scatter <- chart_data_fill(my_scatter,
#'   values = c(virginica = "#6FA2FF", versicolor = "#FF6161", setosa = "#81FF5B") )
#' my_scatter <- chart_data_stroke(my_scatter,
#'   values = c(virginica = "black", versicolor = "black", setosa = "black") )
#' my_scatter <- chart_data_symbol(my_scatter,
#'   values = c(virginica = "circle", versicolor = "diamond", setosa = "circle") )
#' my_scatter <- chart_data_size(my_scatter,
#'   values = c(virginica = 20, versicolor = 16, setosa = 20) )
#' my_scatter <- chart_data_line_width(my_scatter,
#'   values = c(virginica = 2, versicolor = 3, setosa = 6) )
#' @return An `ms_chart` object.
#' @family Series customization functions
chart_data_line_width <- function(x, values) {
  check_series_property(x, "line_width")
  if (!is.numeric(values)) {
    stop("values should be numeric values")
  }
  if (any(sign(values) < 0)) {
    stop("values should not contain negative values")
  }

  serie_names <- names(x$series_settings$line_width)

  if (length(values) == 1) {
    values <- rep(values, length(serie_names))
    names(values) <- serie_names
  }

  if (!all(names(values) %in% serie_names)) {
    stop(
      "values's names do not match series' names: ",
      paste0(shQuote(serie_names), collapse = ", ")
    )
  }

  x$series_settings$line_width[names(values)] <- values
  x
}

#' @export
#' @title Modify line style
#' @description Specify mappings from levels in the data to displayed line style.
#' @param x an `ms_chart` object.
#' @param values `character(num of series)`: a set of line style values to map data values to.
#' It is a named vector, the values will be matched based on the names.
#' Possible values are: 'none', 'solid', 'dashed', 'dotted'.
#' If it contains only one line style, this style will be associated to all existing series.
#' @examples
#' my_scatter <- ms_scatterchart(data = iris, x = "Sepal.Length",
#'   y = "Sepal.Width",  group = "Species")
#' my_scatter <- chart_data_fill(my_scatter,
#'   values = c(virginica = "#6FA2FF", versicolor = "#FF6161", setosa = "#81FF5B") )
#' my_scatter <- chart_data_stroke(my_scatter,
#'   values = c(virginica = "black", versicolor = "black", setosa = "black") )
#' my_scatter <- chart_data_symbol(my_scatter,
#'   values = c(virginica = "circle", versicolor = "diamond", setosa = "circle") )
#' my_scatter <- chart_data_line_style(my_scatter,
#'   values = c(virginica = "solid", versicolor = "dotted", setosa = "dashed") )
#' @return An `ms_chart` object.
#' @family Series customization functions
chart_data_line_style <- function(x, values) {
  check_series_property(x, "line_style")
  if (!all(values %in% st_linestyle)) {
    stop(
      "values should have values matching ",
      paste0(shQuote(st_linestyle), collapse = ", ")
    )
  }

  serie_names <- names(x$series_settings$line_style)

  if (length(values) == 1) {
    values <- rep(values, length(serie_names))
    names(values) <- serie_names
  }

  if (!all(names(values) %in% serie_names)) {
    stop(
      "values's names do not match series' names: ",
      paste0(shQuote(serie_names), collapse = ", ")
    )
  }

  x$series_settings$line_style[names(values)] <- values
  x
}

#' @export
#' @title Smooth series
#' @description Specify whether lines should be smoothed, per series. This
#' feature only applies to [ms_linechart()].
#' @param x an `ms_chart` object.
#' @param values  `integer(num of series)`: a set of smooth values to map data values to.
#' It is a named vector, the values will be matched based on the names.
#' Use `0` to disable smoothing and `1` to enable it.
#' If it contains only one integer it will be associated to all existing series.
#' @examples
#' linec <- ms_linechart(data = iris, x = "Sepal.Length",
#'   y = "Sepal.Width", group = "Species")
#'linec <- chart_data_smooth(linec,
#'   values = c(virginica = 0, versicolor = 0, setosa = 0) )
#' @return An `ms_chart` object.
#' @family Series customization functions
chart_data_smooth <- function(x, values) {
  check_series_property(x, "smooth")
  as_bool <- c(1, 0)

  if (!all(values %in% as_bool)) {
    stop("smooth can only take values of 0 or 1")
  }

  serie_names <- names(x$series_settings$smooth)

  if (length(values) == 1) {
    values <- rep(values, length(serie_names))
    names(values) <- serie_names
  }

  if (!all(names(values) %in% serie_names)) {
    stop(
      "values's names do not match series' names: ",
      paste0(shQuote(serie_names), collapse = ", ")
    )
  }

  x$series_settings$smooth[names(values)] <- values
  x
}

test_that("linechart smooth settings are correctly applied", {
  dat <- data.frame(
    color = c(rep("green", 3), rep("unclear", 3), rep("gray", 3)),
    musician = c(rep(c("Robert Wyatt", "John Zorn", "Damon Albarn"), 3)),
    count = c(120, 101, 131, 200, 154, 187, 122, 197, 159),
    stringsAsFactors = FALSE
  )

  chart_01 <- ms_linechart(
    data = dat,
    x = "musician",
    y = "count",
    group = "color"
  )

  settings <- c(green = 1L, unclear = 0L, gray = 0L)
  settings <- settings[order(names(settings))]
  chart_01 <- chart_data_smooth(chart_01, values = settings)

  xml <- format(
    chart_01,
    sheetname = "sheet1",
    id_x = "64451212",
    id_y = "64453248"
  )

  chart <- xml2::read_xml(xml)
  serie_names <- xml2::xml_find_all(
    chart,
    "//c:ser/c:tx/c:strRef/c:strCache/c:pt/c:v"
  )
  serie_names <- xml2::xml_text(serie_names)
  smooth_data <- xml2::xml_find_all(chart, "//c:smooth")
  smooth_data <- as.integer(xml2::xml_attr(smooth_data, "val"))
  names(smooth_data) <- serie_names
  smooth_data <- smooth_data[order(names(smooth_data))]

  expect_equal(settings, smooth_data, ignore_attr = TRUE)
})

test_that("linechart style=none produces noFill XML", {
  dat <- data.frame(
    color = c(rep("green", 3), rep("unclear", 3), rep("gray", 3)),
    musician = c(rep(c("Robert Wyatt", "John Zorn", "Damon Albarn"), 3)),
    count = c(120, 101, 131, 200, 154, 187, 122, 197, 159),
    stringsAsFactors = FALSE
  )

  lty <- c(green = "none", unclear = "solid", gray = "dotted")
  lty <- lty[order(names(lty))]

  chart_02 <- ms_linechart(
    data = dat,
    x = "musician",
    y = "count",
    group = "color"
  )
  chart_02 <- chart_data_line_style(chart_02, values = lty)

  xml <- format(
    chart_02,
    sheetname = "sheet1",
    id_x = "64451212",
    id_y = "64453248"
  )

  chart <- xml2::read_xml(xml)

  # "none" → noFill, others → solidFill
  lookup_fill <- c(none = "noFill", solid = "solidFill", dotted = "solidFill")
  node_fill <- xml2::xml_find_all(
    chart,
    "//c:ser/c:spPr/a:ln/a:noFill|//c:ser/c:spPr/a:ln/a:solidFill"
  )
  node_fill <- xml2::xml_name(node_fill)
  expect_equal(node_fill, unname(lookup_fill[lty]))

  # prstDash: absent for "none", correct val for others
  lines <- xml2::xml_find_all(chart, "//c:ser/c:spPr/a:ln")
  lookup_lty <- c(none = NA_character_, solid = "solid", dotted = "sysDot")
  attr_lty <- vapply(
    lines,
    function(x) xml2::xml_attr(xml2::xml_find_first(x, "a:prstDash"), "val"),
    FUN.VALUE = character(1)
  )
  expect_equal(attr_lty, unname(lookup_lty[lty]))
})

boxplot_data <- function() {
  set.seed(1)
  data.frame(
    group = rep(c("A", "B", "C"), each = 10),
    value = c(rnorm(10, 0, 5), rnorm(10, 3, 7), rnorm(10, -2, 4)),
    stringsAsFactors = FALSE
  )
}

test_that("ms_boxplotchart constructor validates inputs", {
  d <- boxplot_data()
  expect_error(ms_boxplotchart(d, x = "missing", y = "value"), "not found")
  expect_error(ms_boxplotchart(d, x = "group", y = "missing"), "not found")
  expect_error(ms_boxplotchart(d, x = "group", y = "group"), "must be numeric")
  expect_error(
    ms_boxplotchart(d, x = "group", y = "value", quartile_method = "bogus")
  )
})

test_that("ms_boxplotchart format produces well-formed cx XML", {
  bp <- ms_boxplotchart(
    boxplot_data(),
    x = "group",
    y = "value",
    quartile_method = "inclusive",
    show_mean_marker = TRUE,
    show_outliers = FALSE
  )
  xml_str <- format(bp, sheetname = "sheet1")
  doc <- xml2::read_xml(xml_str)
  ns <- c(cx = "http://schemas.microsoft.com/office/drawing/2014/chartex")

  series <- xml2::xml_find_all(doc, "//cx:series", ns)
  expect_length(series, 1L)
  expect_equal(xml2::xml_attr(series[[1]], "layoutId"), "boxWhisker")

  vis <- xml2::xml_find_first(doc, "//cx:layoutPr/cx:visibility", ns)
  expect_equal(xml2::xml_attr(vis, "meanMarker"), "1")
  expect_equal(xml2::xml_attr(vis, "outliers"), "0")

  stats <- xml2::xml_find_first(doc, "//cx:layoutPr/cx:statistics", ns)
  expect_equal(xml2::xml_attr(stats, "quartileMethod"), "inclusive")

  axes <- xml2::xml_find_all(doc, "//cx:plotArea/cx:axis", ns)
  expect_length(axes, 2L)
})

test_that("ms_boxplotchart end-to-end pptx", {
  skip_if_not_installed("officer")
  skip_if_not_installed("writexl")

  bp <- ms_boxplotchart(boxplot_data(), x = "group", y = "value")
  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(doc, bp, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

test_that("boxplot series carries an <a:ln> by default (accent2)", {
  set.seed(1)
  bd <- data.frame(g = rep(c("A","B"), each = 10), v = rnorm(20))
  ms <- ms_boxplotchart(bd, x = "g", y = "v")
  doc <- xml2::read_xml(format(ms))
  ns <- c(
    cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
    a  = "http://schemas.openxmlformats.org/drawingml/2006/main"
  )
  ln <- xml2::xml_find_first(
    doc, "//cx:series[@layoutId='boxWhisker']/cx:spPr/a:ln", ns
  )
  expect_false(inherits(ln, "xml_missing"))
  expect_equal(xml2::xml_attr(ln, "w"), "12700")
  scheme <- xml2::xml_find_first(ln, ".//a:schemeClr", ns)
  expect_equal(xml2::xml_attr(scheme, "val"), "accent2")
})

test_that("chart_settings(line = fp_border) overrides the boxplot stroke", {
  set.seed(1)
  bd <- data.frame(g = rep(c("A","B"), each = 10), v = rnorm(20))
  ms <- ms_boxplotchart(bd, x = "g", y = "v")
  ms <- chart_settings(ms, line = officer::fp_border(color = "red", width = 1.5))
  doc <- xml2::read_xml(format(ms))
  ns <- c(
    cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
    a  = "http://schemas.openxmlformats.org/drawingml/2006/main"
  )
  ln <- xml2::xml_find_first(
    doc, "//cx:series[@layoutId='boxWhisker']/cx:spPr/a:ln", ns
  )
  expect_equal(xml2::xml_attr(ln, "w"), "19050")  # 1.5 pt * 12700
  srgb <- xml2::xml_find_first(ln, ".//a:srgbClr", ns)
  expect_equal(xml2::xml_attr(srgb, "val"), "FF0000")
})

test_that("chart_data_stroke wins over the default boxplot stroke (single spPr)", {
  set.seed(1)
  bd <- data.frame(g = rep(c("A","B"), each = 10), v = rnorm(20))
  ms <- ms_boxplotchart(bd, x = "g", y = "v")
  ms <- chart_data_stroke(ms, values = "blue", width = 1)
  doc <- xml2::read_xml(format(ms))
  ns <- c(
    cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
    a  = "http://schemas.openxmlformats.org/drawingml/2006/main"
  )
  # Excel rejects two <cx:spPr> in a row -- the default must not be
  # injected when chart_data_stroke already produced one.
  spprs <- xml2::xml_find_all(
    doc, "//cx:series[@layoutId='boxWhisker']/cx:spPr", ns
  )
  expect_length(spprs, 1L)
  srgb <- xml2::xml_find_first(spprs, ".//a:ln//a:srgbClr", ns)
  expect_equal(xml2::xml_attr(srgb, "val"), "0000FF")
})

test_that("chart_settings(line = FALSE) drops the default boxplot stroke", {
  set.seed(1)
  bd <- data.frame(g = rep(c("A","B"), each = 10), v = rnorm(20))
  ms <- ms_boxplotchart(bd, x = "g", y = "v")
  ms <- chart_settings(ms, line = FALSE)
  doc <- xml2::read_xml(format(ms))
  ns <- c(cx = "http://schemas.microsoft.com/office/drawing/2014/chartex")
  expect_true(inherits(
    xml2::xml_find_first(
      doc, "//cx:series[@layoutId='boxWhisker']/cx:spPr", ns
    ),
    "xml_missing"
  ))
})

test_that("chart_settings(line = ...) rejects bad values on boxplot", {
  bd <- data.frame(g = c("A","B"), v = c(1, 2))
  ms <- ms_boxplotchart(bd, x = "g", y = "v")
  expect_error(chart_settings(ms, line = "blue"), "fp_border")
  expect_error(chart_settings(ms, line = 1), "fp_border")
})

cx_ns <- c(
  cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
  a = "http://schemas.openxmlformats.org/drawingml/2006/main"
)

bar_data <- function() {
  data.frame(
    step = c("Start", "Q1", "Q2", "Q3", "End"),
    amount = c(100, 30, -20, 40, 150)
  )
}

test_that("chart_data_stroke: single color emits per-point <a:ln> on waterfall", {
  # Waterfall keeps stroke per-point: Excel reuses series-level spPr/ln
  # for the connector lines between bars, so a uniform color is still
  # written as one <cx:dataPt> per row.
  wf <- ms_waterfallchart(bar_data(), x = "step", y = "amount")
  wf <- chart_data_stroke(wf, values = "black", width = 1.5)
  doc <- xml2::read_xml(format(wf))
  expect_length(
    xml2::xml_find_all(doc, "//cx:series/cx:spPr/a:ln", cx_ns),
    0L
  )
  lns <- xml2::xml_find_all(doc, "//cx:series/cx:dataPt/cx:spPr/a:ln", cx_ns)
  expect_length(lns, 5L)
  expect_true(all(
    xml2::xml_attr(lns, "w") == as.character(round(1.5 * 12700))
  ))
  fills <- xml2::xml_find_all(lns, ".//a:srgbClr", cx_ns)
  expect_true(all(xml2::xml_attr(fills, "val") == "000000"))
})

test_that("chart_data_stroke: named vector emits per-point dataPt with <a:ln>", {
  wf <- ms_waterfallchart(bar_data(), x = "step", y = "amount")
  wf <- chart_data_stroke(wf, values = c(Start = "red", End = "blue"))
  doc <- xml2::read_xml(format(wf))
  dpt <- xml2::xml_find_all(doc, "//cx:series/cx:dataPt", cx_ns)
  expect_length(dpt, 2L)
  expect_setequal(xml2::xml_attr(dpt, "idx"), c("0", "4"))
  end_color <- xml2::xml_attr(
    xml2::xml_find_first(
      dpt[xml2::xml_attr(dpt, "idx") == "4"],
      ".//a:ln//a:srgbClr",
      cx_ns
    ),
    "val"
  )
  expect_equal(end_color, "0000FF")
})

test_that("chart_data_fill + chart_data_stroke combine in same dataPt", {
  wf <- ms_waterfallchart(bar_data(), x = "step", y = "amount")
  wf <- chart_data_fill(wf, values = c(Q2 = "red"))
  wf <- chart_data_stroke(wf, values = c(Q2 = "black"))
  doc <- xml2::read_xml(format(wf))
  dpt_q2 <- xml2::xml_find_first(doc, "//cx:series/cx:dataPt[@idx='2']", cx_ns)
  fill <- xml2::xml_find_first(dpt_q2, ".//a:solidFill/a:srgbClr", cx_ns)
  expect_equal(xml2::xml_attr(fill, "val"), "FF0000")
  ln <- xml2::xml_find_first(dpt_q2, ".//a:ln//a:srgbClr", cx_ns)
  expect_equal(xml2::xml_attr(ln, "val"), "000000")
})

test_that("chart_data_stroke: invalid color rejected", {
  wf <- ms_waterfallchart(bar_data(), x = "step", y = "amount")
  expect_error(chart_data_stroke(wf, values = "not-a-color"), "invalid color")
  expect_error(chart_data_stroke(wf, values = NULL), "non-empty")
})

test_that("chart_data_stroke: histogram rejects named-vector (no cat column)", {
  set.seed(1)
  hi <- ms_histogramchart(data.frame(x = rnorm(20)), value = "x")
  expect_error(
    chart_data_stroke(hi, values = c(A = "red")),
    "named-vector stroke"
  )
  hi2 <- chart_data_stroke(hi, values = "black")
  doc <- xml2::read_xml(format(hi2))
  expect_length(
    xml2::xml_find_all(doc, "//cx:series/cx:spPr/a:ln", cx_ns),
    1L
  )
})

test_that("round-trip pptx still opens with fill+stroke combo", {
  skip_if_not_installed("officer")
  wf <- ms_waterfallchart(
    bar_data(),
    x = "step",
    y = "amount",
    subtotals = c(1, 5)
  )
  wf <- chart_data_fill(
    wf,
    values = c(Start = "gray", End = "gray", Q2 = "red")
  )
  wf <- chart_data_stroke(wf, values = "black", width = 1)
  doc <- officer::read_pptx()
  doc <- officer::add_slide(doc, "Title and Content", "Office Theme")
  doc <- officer::ph_with(doc, wf, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

#' @export
#' @title Set chart options
#' @description Set chart properties.
#' @param x an \code{ms_chart} object.
#' @param ... unused parameter
#' @return An `ms_chart` object.
#' @seealso [ms_barchart()], [ms_areachart()], [ms_scatterchart()], [ms_linechart()]
#' @section Illustrations:
#'
#' \if{html}{\figure{fig_chart_settings_1.png}{options: width="500"}}
#'
#' \if{html}{\figure{fig_chart_settings_2.png}{options: width="500"}}
#'
#' \if{html}{\figure{fig_chart_settings_3.png}{options: width="500"}}
#' @examples
#' library(mschart)
#' library(officer)
#'
#' chart_01 <- ms_barchart(
#'   data = browser_data, x = "browser",
#'   y = "value", group = "serie"
#' )
#' chart_01 <- chart_theme(chart_01,
#'   grid_major_line_x = fp_border(width = 0),
#'   grid_minor_line_x = fp_border(width = 0)
#' )
#'
#' chart_02 <- chart_settings(
#'   x = chart_01,
#'   grouping = "stacked", overlap = 100
#' )
#'
#'
#' chart_03 <- ms_areachart(
#'   data = browser_ts, x = "date",
#'   y = "freq", group = "browser"
#' )
#' chart_03 <- chart_settings(chart_03,
#'   grouping = "percentStacked"
#' )
chart_settings <- function(x, ...) {
  UseMethod("chart_settings")
}


# Warn when a user passes `table = ...` to a chart_settings method
# that does not support data tables. Excel only renders <c:dTable>
# for bar / line / area / stock charts, so the arg would otherwise
# be silently absorbed by `...` on the other types.
warn_unsupported_table <- function(x, ...) {
  if ("table" %in% names(list(...))) {
    warning(
      "`table` is not supported on '", class(x)[1],
      "' charts and was ignored. Data tables are only available on ",
      "'ms_barchart', 'ms_linechart', 'ms_areachart' and 'ms_stockchart'.",
      call. = FALSE
    )
  }
}


barchart_options <- function(
  vary_colors = FALSE,
  gap_width = 150,
  dir = "vertical",
  grouping = "clustered",
  overlap = 0,
  table = FALSE
) {
  # bardir <- structure(c("bar", "col"), .Names = c("horizontal", "vertical"))
  bardir <- c("horizontal", "vertical")
  if (!dir %in% bardir) {
    stop("dir should be one of ", paste0(shQuote(bardir), collapse = ", "))
  }

  if (!(gap_width >= 0 && gap_width <= 500)) {
    stop("gap_width should be between 0 and 500")
  }
  if (!grouping %in% st_bargrouping) {
    stop(
      "grouping should be one of ",
      paste0(shQuote(st_bargrouping), collapse = ", ")
    )
  }
  if (!(overlap >= -100 && overlap <= 100)) {
    stop("overlap should be between -100 and 100")
  }

  out <- list(
    vary_colors = vary_colors,
    gap_width = gap_width,
    dir = dir,
    grouping = grouping,
    overlap = overlap,
    table = table
  )
  class(out) <- "barchart_options"
  out
}


#' @export
#' @describeIn chart_settings barchart settings
#' @param vary_colors if \code{TRUE}, each data point in a single series is displayed in a different color.
#' @param gap_width A gap appears between the bar or clustered bars for each category on a bar chart.
#' The default width for this gap is 150 percent of the bar width. It can be set
#' between 0 and 500 percent of the bar width.
#' @param dir the direction of the bars in the chart, value must be one of "horizontal" or "vertical".
#' @param grouping grouping of the series. For a barchart one of
#' "percentStacked", "clustered", "standard" or "stacked". For a
#' linechart or an areachart one of "percentStacked", "standard" or
#' "stacked" ("clustered" is bar-only).
#' @param overlap In a bar chart having two or more series, the bars for each
#' category are clustered together. By default, these bars are directly
#' adjacent to each other. The bars can be made to overlap each other or
#' have a space between them using the overlap property. Its values range
#' between -100 and 100, representing the percentage of the bar width by
#' which to overlap adjacent bars. A setting of -100 creates a gap of a
#' full bar width and a setting of 100 causes all the bars in a category
#' to be superimposed. The default value is 0.
#' @param table if \code{TRUE} set a table below the barchart.
chart_settings.ms_barchart <- function(
  x,
  vary_colors,
  gap_width,
  dir,
  grouping,
  overlap,
  table,
  ...
) {
  options <- barchart_options(
    vary_colors = if (missing(vary_colors)) {
      x$options$vary_colors
    } else {
      vary_colors
    },
    gap_width = if (missing(gap_width)) x$options$gap_width else gap_width,
    dir = if (missing(dir)) x$options$dir else dir,
    grouping = if (missing(grouping)) x$options$grouping else grouping,
    overlap = if (missing(overlap)) x$options$overlap else overlap,
    table = if (missing(table)) x$options$table else table
  )
  x$options <- options
  x
}


linechart_options <- function(vary_colors = FALSE,
                              grouping = "standard",
                              table = FALSE) {
  if (!grouping %in% st_grouping) {
    stop(
      "grouping should be one of ",
      paste0(shQuote(st_grouping), collapse = ", ")
    )
  }
  out <- list(vary_colors = vary_colors, grouping = grouping, table = table)
  class(out) <- "linechart_options"
  out
}

#' @export
#' @describeIn chart_settings linechart settings
#' @param style Style for the linechart or scatterchart type of markers. One
#' of 'none', 'line', 'lineMarker', 'marker', 'smooth', 'smoothMarker'.
chart_settings.ms_linechart <- function(x, vary_colors, style, grouping,
                                        table, ...) {
  options <- linechart_options(
    vary_colors = if (missing(vary_colors)) {
      x$options$vary_colors
    } else {
      vary_colors
    },
    grouping = if (missing(grouping)) {
      x$options$grouping %||% "standard"
    } else {
      grouping
    },
    table = if (missing(table)) x$options$table else table
  )

  style <- if (missing(style)) x$options$style %||% "lineMarker" else style
  if (!style %in% st_scatterstyle) {
    stop(
      "style should be one of ",
      paste0(shQuote(st_scatterstyle), collapse = ", ")
    )
  }

  options$style <- style
  x$options <- options
  x
}


#' @export
#' @describeIn chart_settings areachart settings
chart_settings.ms_areachart <- function(x, vary_colors, grouping, table, ...) {
  vary_colors <- if (missing(vary_colors)) {
    x$options$vary_colors %||% FALSE
  } else {
    vary_colors
  }
  grouping <- if (missing(grouping)) {
    x$options$grouping %||% "standard"
  } else {
    grouping
  }
  table <- if (missing(table)) x$options$table %||% FALSE else table

  if (!grouping %in% st_grouping) {
    stop(
      "grouping should be one of ",
      paste0(shQuote(st_grouping), collapse = ", ")
    )
  }
  options <- list(vary_colors = vary_colors, grouping = grouping, table = table)
  class(options) <- "areachart_options"

  x$options <- options
  x
}

#' @export
#' @describeIn chart_settings scatterchart settings
chart_settings.ms_scatterchart <- function(x, vary_colors, style, ...) {
  warn_unsupported_table(x, ...)
  vary_colors <- if (missing(vary_colors)) {
    x$options$vary_colors %||% FALSE
  } else {
    vary_colors
  }
  style <- if (missing(style)) x$options$style %||% "marker" else style

  if (!style %in% st_scatterstyle) {
    stop(
      "style should be one of ",
      paste0(shQuote(st_scatterstyle), collapse = ", ")
    )
  }

  if (grepl("smooth", style)) {
    x <- chart_data_smooth(x, values = 1)
  } else {
    x <- chart_data_smooth(x, values = 0)
  }

  options <- list(
    vary_colors = vary_colors,
    style = style,
    table = FALSE
  )
  class(options) <- "scatterchart_options"

  x$options <- options
  x
}


#' @export
#' @describeIn chart_settings stockchart settings
#' @param hi_low_lines an [officer::fp_border()] for the high-low lines.
#' Set to `FALSE` to hide them.
#' @param up_bars_fill fill colour for up bars (OHLC only, close > open).
#' @param up_bars_border an [officer::fp_border()] for up bar borders.
#' @param down_bars_fill fill colour for down bars (OHLC only, close < open).
#' @param down_bars_border an [officer::fp_border()] for down bar borders.
chart_settings.ms_stockchart <- function(
  x,
  vary_colors,
  table,
  hi_low_lines,
  up_bars_fill,
  up_bars_border,
  down_bars_fill,
  down_bars_border,
  ...
) {
  vary_colors <- if (missing(vary_colors)) {
    x$options$vary_colors %||% FALSE
  } else {
    vary_colors
  }
  table <- if (missing(table)) x$options$table %||% FALSE else table
  hi_low_lines <- if (missing(hi_low_lines)) {
    x$options$hi_low_lines %||% fp_border(color = "#404040", width = 0.75)
  } else {
    hi_low_lines
  }
  up_bars_fill <- if (missing(up_bars_fill)) {
    x$options$up_bars_fill %||% "white"
  } else {
    up_bars_fill
  }
  up_bars_border <- if (missing(up_bars_border)) {
    x$options$up_bars_border %||% fp_border(color = "#404040", width = 0.75)
  } else {
    up_bars_border
  }
  down_bars_fill <- if (missing(down_bars_fill)) {
    x$options$down_bars_fill %||% "#404040"
  } else {
    down_bars_fill
  }
  down_bars_border <- if (missing(down_bars_border)) {
    x$options$down_bars_border %||% fp_border(color = "#404040", width = 0.75)
  } else {
    down_bars_border
  }

  options <- list(
    vary_colors = vary_colors,
    table = table,
    hi_low_lines = hi_low_lines,
    up_bars_fill = up_bars_fill,
    up_bars_border = up_bars_border,
    down_bars_fill = down_bars_fill,
    down_bars_border = down_bars_border
  )
  class(options) <- "stockchart_options"
  x$options <- options
  x
}

#' @export
#' @describeIn chart_settings radarchart settings
chart_settings.ms_radarchart <- function(x, vary_colors, style, ...) {
  warn_unsupported_table(x, ...)
  vary_colors <- if (missing(vary_colors)) {
    x$options$vary_colors %||% FALSE
  } else {
    vary_colors
  }
  style <- if (missing(style)) {
    x$options$radarstyle %||% "marker"
  } else {
    style
  }

  if (!style %in% st_radarstyle) {
    stop(
      "style should be one of ",
      paste0(shQuote(st_radarstyle), collapse = ", ")
    )
  }

  options <- list(
    vary_colors = vary_colors,
    radarstyle = style,
    table = FALSE
  )
  class(options) <- "radarchart_options"

  x$options <- options
  x
}

#' @export
#' @describeIn chart_settings bubblechart settings
#' @param bubble3D logical, use 3D effect for bubbles.
chart_settings.ms_bubblechart <- function(x, vary_colors,
                                          bubble3D = FALSE, ...) {
  warn_unsupported_table(x, ...)
  vary_colors <- if (missing(vary_colors)) {
    x$options$vary_colors %||% FALSE
  } else {
    vary_colors
  }

  options <- list(
    vary_colors = vary_colors,
    bubble3D = bubble3D,
    table = FALSE
  )
  class(options) <- "bubblechart_options"

  x$options <- options
  x
}

piechart_options <- function(vary_colors = TRUE, hole_size = 0) {
  if (!(hole_size >= 0 && hole_size <= 90)) {
    stop("hole_size should be between 0 and 90")
  }
  out <- list(vary_colors = vary_colors, hole_size = hole_size)
  class(out) <- "piechart_options"
  out
}

#' @export
#' @describeIn chart_settings piechart settings
#' @param hole_size size of the hole in a doughnut chart, between 0 and 90
#' (percent of the radius). Default 0 produces a pie chart;
#' values above 0 produce a doughnut chart.
chart_settings.ms_piechart <- function(x, vary_colors, hole_size, ...) {
  warn_unsupported_table(x, ...)
  options <- piechart_options(
    vary_colors = if (missing(vary_colors)) {
      x$options$vary_colors
    } else {
      vary_colors
    },
    hole_size = if (missing(hole_size)) x$options$hole_size else hole_size
  )
  x$options <- options
  x
}

pareto_data <- function() {
  set.seed(1)
  data.frame(
    defect = sample(
      c("A", "B", "C", "D"),
      50,
      replace = TRUE,
      prob = c(0.5, 0.25, 0.15, 0.1)
    ),
    stringsAsFactors = FALSE
  )
}

test_that("ms_paretochart constructor validates inputs", {
  d <- pareto_data()
  expect_error(ms_paretochart(d, x = "missing"), "not found")
  expect_error(ms_paretochart(d, x = "defect", y = "missing"), "not found")
  expect_error(ms_paretochart(d, x = "defect", y = "defect"), "must be numeric")
})

test_that("ms_paretochart format produces well-formed cx XML with two series and three axes", {
  pa <- ms_paretochart(pareto_data(), x = "defect")
  xml_str <- format(pa, sheetname = "sheet1")
  doc <- xml2::read_xml(xml_str)
  ns <- c(cx = "http://schemas.microsoft.com/office/drawing/2014/chartex")

  series <- xml2::xml_find_all(doc, "//cx:series", ns)
  expect_length(series, 2L)
  expect_equal(xml2::xml_attr(series[[1]], "layoutId"), "clusteredColumn")
  expect_equal(xml2::xml_attr(series[[2]], "layoutId"), "paretoLine")
  expect_equal(xml2::xml_attr(series[[2]], "ownerIdx"), "0")

  # aggregation marker on the column series
  agg <- xml2::xml_find_all(
    doc,
    "//cx:series[@layoutId='clusteredColumn']/cx:layoutPr/cx:aggregation",
    ns
  )
  expect_length(agg, 1L)

  # 3 axes
  axes <- xml2::xml_find_all(doc, "//cx:plotArea/cx:axis", ns)
  expect_length(axes, 3L)
  pct_axis <- axes[[3]]
  unit_node <- xml2::xml_find_first(pct_axis, "./cx:units", ns)
  expect_equal(xml2::xml_attr(unit_node, "unit"), "percentage")
})

test_that("ms_paretochart aggregate=FALSE omits aggregation element", {
  d <- data.frame(defect = c("A", "B", "C"), n = c(20, 12, 8))
  pa <- ms_paretochart(d, x = "defect", y = "n", aggregate = FALSE)
  xml_str <- format(pa, sheetname = "sheet1")
  doc <- xml2::read_xml(xml_str)
  ns <- c(cx = "http://schemas.microsoft.com/office/drawing/2014/chartex")
  agg <- xml2::xml_find_all(doc, "//cx:aggregation", ns)
  expect_length(agg, 0L)
})

test_that("ms_paretochart end-to-end pptx", {
  skip_if_not_installed("officer")
  skip_if_not_installed("writexl")

  pa <- ms_paretochart(pareto_data(), x = "defect")
  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(doc, pa, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

test_that("paretoLine series carries an <a:ln> by default (accent2)", {
  pa <- ms_paretochart(
    data.frame(x = c("A", "B"), y = c(2, 1)),
    x = "x", y = "y", aggregate = FALSE
  )
  doc <- xml2::read_xml(format(pa))
  ns <- c(
    cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
    a  = "http://schemas.openxmlformats.org/drawingml/2006/main"
  )
  ln <- xml2::xml_find_first(
    doc, "//cx:series[@layoutId='paretoLine']/cx:spPr/a:ln", ns
  )
  expect_false(inherits(ln, "xml_missing"))
  expect_equal(xml2::xml_attr(ln, "w"), "12700")
  scheme <- xml2::xml_find_first(ln, ".//a:schemeClr", ns)
  expect_equal(xml2::xml_attr(scheme, "val"), "accent2")
})

test_that("chart_settings(line = fp_border) overrides the pareto line", {
  pa <- ms_paretochart(
    data.frame(x = c("A", "B"), y = c(2, 1)),
    x = "x", y = "y", aggregate = FALSE
  )
  pa <- chart_settings(pa, line = officer::fp_border(color = "orange", width = 2))
  doc <- xml2::read_xml(format(pa))
  ns <- c(
    cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
    a  = "http://schemas.openxmlformats.org/drawingml/2006/main"
  )
  ln <- xml2::xml_find_first(
    doc, "//cx:series[@layoutId='paretoLine']/cx:spPr/a:ln", ns
  )
  expect_equal(xml2::xml_attr(ln, "w"), "25400")  # 2 pt * 12700
  srgb <- xml2::xml_find_first(ln, ".//a:srgbClr", ns)
  expect_equal(xml2::xml_attr(srgb, "val"), "FFA500")
})

test_that("chart_settings(line = FALSE) suppresses the pareto line spPr", {
  pa <- ms_paretochart(
    data.frame(x = c("A", "B"), y = c(2, 1)),
    x = "x", y = "y", aggregate = FALSE
  )
  pa <- chart_settings(pa, line = FALSE)
  doc <- xml2::read_xml(format(pa))
  ns <- c(cx = "http://schemas.microsoft.com/office/drawing/2014/chartex")
  expect_true(inherits(
    xml2::xml_find_first(
      doc, "//cx:series[@layoutId='paretoLine']/cx:spPr", ns
    ),
    "xml_missing"
  ))
})

test_that("chart_settings rejects bad `line` values", {
  pa <- ms_paretochart(
    data.frame(x = c("A", "B"), y = c(2, 1)),
    x = "x", y = "y", aggregate = FALSE
  )
  expect_error(chart_settings(pa, line = "blue"), "fp_border")
  expect_error(chart_settings(pa, line = 1), "fp_border")
})

st_layouttarget <- c('inner', 'outer')

st_layoutmode <- c('edge', 'factor')

st_sizerepresents <- c('area', 'w')

st_splittype <- c('auto', 'cust', 'percent', 'pos', 'val')

st_lblalgn <- c('ctr', 'l', 'r')

st_dlblpos <- c(
  'bestFit',
  'b',
  'ctr',
  'inBase',
  'inEnd',
  'l',
  'outEnd',
  'r',
  't'
)

st_markerstyle <- c(
  'circle',
  'dash',
  'diamond',
  'dot',
  'none',
  'plus',
  'square',
  'star',
  'triangle',
  'x',
  'auto'
)

st_linestyle <- c('none', 'solid', 'dotted', 'dashed')

st_trendlinetype <- c('exp', 'linear', 'log', 'movingAvg', 'poly', 'power')

st_errdir <- c('x', 'y')

st_errbartype <- c('both', 'minus', 'plus')

st_errvaltype <- c('cust', 'fixedVal', 'percentage', 'stdDev', 'stdErr')

st_grouping <- c('percentStacked', 'standard', 'stacked')

st_scatterstyle <- c(
  'none',
  'line',
  'lineMarker',
  'marker',
  'smooth',
  'smoothMarker'
)

st_radarstyle <- c('standard', 'marker', 'filled')

st_bargrouping <- c('percentStacked', 'clustered', 'standard', 'stacked')

st_bardir <- c('bar', 'col')

st_shape <- c('cone', 'coneToMax', 'box', 'cylinder', 'pyramid', 'pyramidToMax')

st_ofpietype <- c('pie', 'bar')

st_axpos <- c('b', 'l', 'r', 't')

st_crosses <- c('autoZero', 'max', 'min')

st_crossbetween <- c('between', 'midCat')

st_tickmark <- c('cross', 'in', 'none', 'out')

st_ticklblpos <- c('high', 'low', 'nextTo', 'none')

st_timeunit <- c('days', 'months', 'years')

st_builtinunit <- c(
  'hundreds',
  'thousands',
  'tenThousands',
  'hundredThousands',
  'millions',
  'tenMillions',
  'hundredMillions',
  'billions',
  'trillions'
)

st_pictureformat <- c('stretch', 'stack', 'stackScale')

st_orientation <- c('maxMin', 'minMax')

st_legendpos <- c('b', 'tr', 'l', 'r', 't', 'n')

st_dispblanksas <- c('span', 'gap', 'zero')

st_pagesetuporientation <- c('default', 'portrait', 'landscape')

## -- assert functions

assert_area <- function(data_x, data_y) {
  if (!is.numeric(data_y)) {
    stop("y column should be numeric.")
  }
  check_x <- inherits(data_x, "Date") ||
    is.character(data_x) ||
    is.factor(data_x)
  if (!check_x) {
    stop("x column should be a date or a categorical column.")
  }
}

assert_scatter <- function(data_x, data_y) {
  if (!is.numeric(data_y)) {
    stop("y column should be numeric.")
  }
  if (!is.numeric(data_x)) {
    stop("x column should be numeric.")
  }
}

assert_line <- function(data_y) {
  if (!is.numeric(data_y)) {
    stop("y column should be numeric.")
  }
}

assert_pie <- function(data_x, data_y) {
  if (!is.numeric(data_y)) {
    stop("y column should be numeric.")
  }
  if (is.numeric(data_x)) {
    stop("x column should be a categorical column (character or factor).")
  }
}

#' @title Linechart object
#' @description Creation of a linechart object that can be
#' inserted in a 'Microsoft' document.
#'
#' In a line chart, category data is distributed evenly along the horizontal axis, and
#' all value data is distributed evenly along the vertical axis. Line charts can show
#' continuous data over time on an evenly scaled axis, so they're ideal for showing
#' trends in data at equal intervals, like months and quarters.
#' @param data a data.frame
#' @param x column name for x values.
#' @param y column name for y values.
#' @param group grouping column name used to split data into series. Optional.
#' @param labels column names of columns to be used as custom data labels
#' displayed next to data points (not axis labels). Optional.
#' If more than one name is provided, only the first one will be used as a label, but all
#' labels (transposed if a group is used) will be available in the Excel file
#' associated with the chart.
#' @param asis logical parameter defaulting to FALSE. When FALSE, the data is
#' reshaped internally so that each series becomes a separate column. When TRUE,
#' the data is used as-is and must already have one column for categories and
#' one column per series.
#' @return An `ms_chart` object.
#' @export
#' @family 'Office' chart objects
#' @seealso [chart_settings()], [chart_ax_x()], [chart_ax_y()],
#' [chart_data_labels()], [chart_theme()], [chart_labels()]
#' @section Illustrations:
#'
#' \if{html}{\figure{fig_ms_linechart_1.png}{options: width="500"}}
#'
#' \if{html}{\figure{fig_ms_linechart_2.png}{options: width="500"}}
#'
#' \if{html}{\figure{fig_ms_linechart_3.png}{options: width="500"}}
#' @examples
#' library(officer)
#' @example examples/02_linechart.R
ms_linechart <- function(
  data,
  x,
  y,
  group = NULL,
  labels = NULL,
  asis = FALSE
) {
  out <- ms_chart(
    data = data,
    x = x,
    y = y,
    group = group,
    labels = labels,
    type = "lineplot",
    asis = asis
  )
  out$options <- linechart_options()
  class(out) <- c("ms_linechart", "ms_chart")
  out <- chart_settings(out)
  out
}

#' @title Barchart object
#' @description Creation of a barchart object that can be
#' inserted in a 'Microsoft' document.
#'
#' Bar charts illustrate comparisons among individual items. In a bar chart, the
#' categories are typically organized along the vertical axis, and the values
#' along the horizontal axis.
#'
#' Consider using a bar chart when:
#'
#' * The axis labels are long.
#' * The values that are shown are durations.
#' @inheritParams ms_linechart
#' @return An `ms_chart` object.
#' @family 'Office' chart objects
#' @seealso [chart_settings()], [chart_ax_x()], [chart_ax_y()],
#' [chart_data_labels()], [chart_theme()], [chart_labels()]
#' @export
#' @section Illustrations:
#'
#' \if{html}{\figure{fig_ms_barchart_1.png}{options: width="500"}}
#'
#' \if{html}{\figure{fig_ms_barchart_2.png}{options: width="500"}}
#'
#' \if{html}{\figure{fig_ms_barchart_3.png}{options: width="500"}}
#'
#' \if{html}{\figure{fig_ms_barchart_4.png}{options: width="500"}}
#'
#' \if{html}{\figure{fig_ms_barchart_5.png}{options: width="500"}}
#'
#' \if{html}{\figure{fig_ms_barchart_6.png}{options: width="500"}}
#' @examples
#' library(officer)
#' @example examples/01_barchart.R
ms_barchart <- function(data, x, y, group = NULL, labels = NULL, asis = FALSE) {
  out <- ms_chart(
    data = data,
    x = x,
    y = y,
    group = group,
    labels = labels,
    type = "barplot",
    asis = asis
  )
  out$options <- barchart_options()
  class(out) <- c("ms_barchart", "ms_chart")
  out
}

#' @title Areachart object
#' @description Creation of an areachart object that can be
#' inserted in a 'Microsoft' document.
#'
#' Area charts can be used to plot change over time and draw attention to the
#' total value across a trend. By showing the sum of the plotted values, an area
#' chart also shows the relationship of parts to a whole.
#' @inheritParams ms_linechart
#' @return An `ms_chart` object.
#' @family 'Office' chart objects
#' @seealso [chart_settings()], [chart_ax_x()], [chart_ax_y()],
#' [chart_data_labels()], [chart_theme()], [chart_labels()]
#' @export
#' @examples
#' library(officer)
#' @example examples/03_areachart.R
ms_areachart <- function(
  data,
  x,
  y,
  group = NULL,
  labels = NULL,
  asis = FALSE
) {
  out <- ms_chart(
    data = data,
    x = x,
    y = y,
    group = group,
    labels = labels,
    type = "areaplot",
    asis = asis
  )
  class(out) <- c("ms_areachart", "ms_chart")
  out <- chart_settings(out)

  serie_names <- names(out$series_settings$colour)
  values <- setNames(rep("transparent", length(serie_names)), serie_names)
  out <- chart_data_stroke(out, values = values)

  out
}

#' @title Scatterchart object
#' @description Creation of a scatterchart object that can be
#' inserted in a 'Microsoft' document.
#' @inheritParams ms_linechart
#' @return An `ms_chart` object.
#' @family 'Office' chart objects
#' @seealso [chart_settings()], [chart_ax_x()], [chart_ax_y()],
#' [chart_data_labels()], [chart_theme()], [chart_labels()]
#' @export
#' @section Illustrations:
#'
#' \if{html}{\figure{fig_ms_scatterchart_1.png}{options: width="500"}}
#'
#' \if{html}{\figure{fig_ms_scatterchart_2.png}{options: width="500"}}
#' @examples
#' library(officer)
#' @example examples/04_scatterchart.R
ms_scatterchart <- function(
  data,
  x,
  y,
  group = NULL,
  labels = NULL,
  asis = FALSE
) {
  out <- ms_chart(
    data = data,
    x = x,
    y = y,
    group = group,
    labels = labels,
    excel_data_setup = transpose_series_bysplit,
    type = "scatterplot",
    asis = asis
  )
  class(out) <- c("ms_scatterchart", "ms_chart")

  out <- chart_settings(out)

  out
}

#' @title Stockchart object
#' @description Creation of a stock chart object that can be inserted
#' in a 'Microsoft' document. When `open` is omitted the chart is
#' a High-Low-Close chart. When `open` is provided it becomes an
#' Open-High-Low-Close chart with up/down bars (candlestick).
#' @param data a data.frame
#' @param x column name for categories (typically dates)
#' @param open column name for open values (optional, enables OHLC mode)
#' @param high column name for high values
#' @param low column name for low values
#' @param close column name for close values
#' @return An `ms_chart` object.
#' @family 'Office' chart objects
#' @seealso [chart_settings()], [chart_ax_x()], [chart_ax_y()],
#' [chart_theme()], [chart_labels()]
#' @export
#' @examples
#' library(officer)
#'
#' dat <- data.frame(
#'   date = as.Date("2024-01-01") + 0:4,
#'   open = c(44, 25, 38, 50, 34),
#'   high = c(55, 57, 57, 58, 58),
#'   low = c(11, 12, 13, 11, 25),
#'   close = c(32, 35, 34, 35, 43)
#' )
#'
#' # HLC chart
#' stock_hlc <- ms_stockchart(
#'   data = dat, x = "date",
#'   high = "high", low = "low", close = "close"
#' )
#' stock_hlc
#'
#' # OHLC chart (candlestick)
#' stock_ohlc <- ms_stockchart(
#'   data = dat, x = "date",
#'   open = "open", high = "high",
#'   low = "low", close = "close"
#' )
#' stock_ohlc
ms_stockchart <- function(data, x, open = NULL, high, low, close) {
  stopifnot(is.data.frame(data))
  all_cols <- c(x, open, high, low, close)
  for (col in all_cols) {
    if (!col %in% names(data)) {
      stop("column ", shQuote(col), " not found in data", call. = FALSE)
    }
  }
  num_cols <- c(open, high, low, close)
  for (col in num_cols) {
    if (!is.numeric(data[[col]])) {
      stop("column ", shQuote(col), " must be numeric", call. = FALSE)
    }
  }

  has_open <- !is.null(open)

  if (has_open) {
    series_names <- c(open, high, low, close)
  } else {
    series_names <- c(high, low, close)
  }

  # reshape to long format with fixed series order
  data_long <- data.frame(
    x_val = rep(data[[x]], length(series_names)),
    y_val = unlist(lapply(series_names, function(s) data[[s]]),
      use.names = FALSE
    ),
    group = factor(
      rep(series_names, each = nrow(data)),
      levels = series_names
    ),
    stringsAsFactors = FALSE
  )
  names(data_long)[1] <- x

  out <- ms_chart(
    data = data_long,
    x = x,
    y = "y_val",
    group = "group",
    type = "stockplot"
  )
  out$stock_cols <- if (has_open) {
    list(open = open, high = high, low = low, close = close)
  } else {
    list(high = high, low = low, close = close)
  }
  out$axis_x_xml <- axis_content_xml
  out$axis_y_xml <- axis_content_xml
  class(out) <- c("ms_stockchart", "ms_chart")
  out <- chart_settings(out)
  out
}

#' @title Radarchart object
#' @description Creation of a radar (spider) chart object that can be
#' inserted in a 'Microsoft' document.
#' @inheritParams ms_linechart
#' @return An `ms_chart` object.
#' @family 'Office' chart objects
#' @seealso [chart_settings()], [chart_ax_x()], [chart_ax_y()],
#' [chart_data_labels()], [chart_theme()], [chart_labels()]
#' @export
#' @examples
#' library(officer)
#'
#' dat <- data.frame(
#'   axis = c("Sales", "Marketing", "Dev", "Support", "HR"),
#'   s1 = c(4, 3, 5, 2, 4),
#'   s2 = c(3, 5, 2, 4, 3)
#' )
#' dat_long <- data.frame(
#'   axis = rep(dat$axis, 2),
#'   value = c(dat$s1, dat$s2),
#'   group = rep(c("Team A", "Team B"), each = 5)
#' )
#'
#' radar <- ms_radarchart(
#'   data = dat_long, x = "axis",
#'   y = "value", group = "group"
#' )
#' radar
ms_radarchart <- function(
  data, x, y, group = NULL,
  labels = NULL, asis = FALSE
) {
  out <- ms_chart(
    data = data, x = x, y = y,
    group = group, labels = labels,
    type = "radarplot", asis = asis
  )
  out$axis_x_xml <- axis_content_xml_radar
  out$axis_y_xml <- axis_content_xml_radar
  class(out) <- c("ms_radarchart", "ms_chart")
  out <- chart_settings(out)
  out
}

#' @title Bubblechart object
#' @description Creation of a bubblechart object that can be
#' inserted in a 'Microsoft' document. A bubble chart is a scatter
#' chart where each point has a third numeric dimension controlling
#' its size.
#' @inheritParams ms_scatterchart
#' @param size column name for bubble size values (must be numeric)
#' @return An `ms_chart` object.
#' @family 'Office' chart objects
#' @seealso [chart_settings()], [chart_ax_x()], [chart_ax_y()],
#' [chart_data_labels()], [chart_theme()], [chart_labels()]
#' @export
#' @examples
#' library(officer)
#'
#' dat <- data.frame(
#'   x = c(1, 2, 3, 4, 5),
#'   y = c(10, 20, 15, 25, 30),
#'   sz = c(5, 10, 7, 15, 12),
#'   grp = rep("s1", 5)
#' )
#'
#' bubble <- ms_bubblechart(
#'   data = dat, x = "x", y = "y",
#'   size = "sz", group = "grp"
#' )
#'
#' # adjust axes to avoid clipping extreme bubbles
#' bubble <- chart_ax_x(bubble, limit_min = 0, limit_max = 6)
#' bubble <- chart_ax_y(bubble, limit_min = 5, limit_max = 35)
#' bubble
ms_bubblechart <- function(
  data,
  x,
  y,
  size,
  group = NULL,
  labels = NULL,
  asis = FALSE
) {
  if (!size %in% names(data)) {
    stop("column ", shQuote(size), " not found in data", call. = FALSE)
  }
  if (!is.numeric(data[[size]])) {
    stop("column ", shQuote(size), " must be numeric", call. = FALSE)
  }

  out <- ms_chart(
    data = data,
    x = x,
    y = y,
    group = group,
    labels = labels,
    excel_data_setup = transpose_series_bysplit,
    type = "bubbleplot",
    asis = asis
  )
  out$size_cols <- size
  out$size <- size
  # rebuild data_series to include size column
  out$data_series <- transpose_series_bysplit(out)
  class(out) <- c("ms_bubblechart", "ms_chart")

  out <- chart_settings(out)

  out
}

#' @title Combined chart object
#' @description Combine several chart objects into a single chart
#' with shared axes. Each chart must be a named argument.
#'
#' The title and x-axis label are taken from the first chart.
#' The y-axis label of the first chart on the secondary axis is
#' used as the secondary y-axis label.
#'
#' Only one secondary y-axis (right) and one secondary x-axis (top)
#' are supported.
#' @param ... named `ms_chart` objects.
#' @param secondary_y character vector of chart names to plot on
#' the secondary (right) y-axis.
#' @param secondary_x character vector of chart names to plot on
#' the secondary (top) x-axis.
#' @family 'Office' chart objects
#' @seealso [chart_settings()], [chart_ax_x()], [chart_ax_y()],
#' [chart_data_labels()], [chart_theme()], [chart_labels()]
#' @export
#' @example examples/05_combochart.R
ms_chart_combine <- function(..., secondary_y = NULL, secondary_x = NULL) {
  inputs <- list(...)

  if (is.null(names(inputs)) || any(names(inputs) == "")) {
    stop("All charts must be named arguments.", call. = FALSE)
  }
  for (i in seq_along(inputs)) {
    if (!inherits(inputs[[i]], "ms_chart")) {
      stop(
        "Argument ", shQuote(names(inputs)[i]),
        " is not an ms_chart object.",
        call. = FALSE
      )
    }
  }

  bad_y <- setdiff(secondary_y, names(inputs))
  if (length(bad_y)) {
    stop(
      "secondary_y names not found: ",
      paste(shQuote(bad_y), collapse = ", "),
      call. = FALSE
    )
  }
  bad_x <- setdiff(secondary_x, names(inputs))
  if (length(bad_x)) {
    stop(
      "secondary_x names not found: ",
      paste(shQuote(bad_x), collapse = ", "),
      call. = FALSE
    )
  }

  out <- inputs[[1]]

  sec_y_done <- FALSE
  sec_x_done <- FALSE

  for (i in seq_along(inputs)[-1]) {
    nm <- names(inputs)[i]
    chart_i <- inputs[[i]]

    is_sec_y <- nm %in% secondary_y
    is_sec_x <- nm %in% secondary_x

    lbl <- chart_i$labels
    xlab <- NULL
    ylab <- NULL

    if (!sec_y_done && is_sec_y && !is_sec_x) {
      chart_i$x_axis$delete <- 1L
      chart_i$x_axis$axis_position <- "b"
      chart_i$x_axis$crosses <- "autoZero"

      chart_i$y_axis$delete <- 0L
      chart_i$y_axis$axis_position <- "r"
      chart_i$y_axis$crosses <- "max"

      ylab <- lbl$y
      sec_y_done <- TRUE
    } else if (!sec_x_done && is_sec_x && !is_sec_y) {
      chart_i$x_axis$delete <- 0L
      chart_i$x_axis$axis_position <- "t"
      chart_i$x_axis$crosses <- "max"

      chart_i$y_axis$delete <- 1L
      chart_i$y_axis$axis_position <- "l"
      chart_i$y_axis$crosses <- "autoZero"

      xlab <- lbl$x
      sec_x_done <- TRUE
    } else {
      chart_i$y_axis <- axis_options(axis_position = "l", delete = 1L)
      chart_i$x_axis <- axis_options(axis_position = "b", delete = 1L)
    }

    attr(chart_i, "secondary_y") <- is_sec_y
    attr(chart_i, "secondary_x") <- is_sec_x
    chart_i$labels$title <- list(title = NULL, x = xlab, y = ylab)

    out$secondary <- append(out$secondary, list(chart_i))
  }

  out
}

#' @title Piechart object
#' @description Creation of a piechart object that can be
#' inserted in a 'Microsoft' document.
#'
#' Pie charts show the proportion of each category as a slice
#' of a circle. Doughnut charts are similar but have a hole
#' in the centre. Use `chart_settings(x, hole_size = ...)` to
#' control the hole size: 0 produces a pie chart, values
#' above 0 produce a doughnut chart.
#'
#' Data must be pre-aggregated: one row per slice, no grouping
#' column.
#' @param data a data.frame
#' @param x column name for categories (slices).
#' @param y column name for values (slice sizes).
#' @param labels column names of columns to be used as custom data labels
#' displayed next to data points (not axis labels). Optional.
#' If more than one name is provided, only the first one will be used as a label, but all
#' labels (transposed if a group is used) will be available in the Excel file
#' associated with the chart.
#' @return An `ms_chart` object.
#' @export
#' @family 'Office' chart objects
#' @seealso [chart_settings()], [chart_data_labels()], [chart_theme()], [chart_labels()]
#' @examples
#' library(officer)
#' library(mschart)
#'
#' dat <- data.frame(
#'   browser = c("Chrome", "Firefox", "Safari", "Edge", "Other"),
#'   value = c(64, 12, 8, 5, 11)
#' )
#'
#' # Pie chart
#' pie <- ms_piechart(data = dat, x = "browser", y = "value")
#' pie <- chart_labels(pie, title = "Browser share")
#'
#' # Doughnut chart
#' donut <- ms_piechart(data = dat, x = "browser", y = "value")
#' donut <- chart_settings(donut, hole_size = 50)
#' donut <- chart_labels(donut, title = "Browser share (donut)")
ms_piechart <- function(data, x, y, labels = NULL) {
  out <- ms_chart(
    data = data,
    x = x,
    y = y,
    group = NULL,
    labels = labels,
    type = "pieplot"
  )
  out$options <- piechart_options()
  class(out) <- c("ms_piechart", "ms_chart")
  out <- chart_settings(out)

  # pie charts style slices, not series -- replace the series-level
  # fill/colour/line_width with per-category vectors so chart_data_fill(),
  # chart_data_stroke() and chart_data_line_width() target individual
  # slices.
  cat_values <- data[[x]]
  cat_names <- if (is.factor(cat_values)) {
    levels(cat_values)
  } else {
    as.character(unique(cat_values))
  }
  n <- length(cat_names)
  if (n <= length(colour_list)) {
    pal <- colour_list[[n]]
  } else {
    pal <- rep_len(colour_list[[length(colour_list)]], n)
  }
  out$series_settings$fill <- setNames(pal, cat_names)
  out$series_settings$colour <- setNames(rep("transparent", n), cat_names)
  out$series_settings$line_width <- setNames(rep(2, n), cat_names)

  out
}


# ms_chart -----

#' @importFrom grDevices colors
ms_chart <- function(
  data,
  x,
  y,
  group = NULL,
  labels = NULL,
  excel_data_setup = shape_as_series,
  type = NULL,
  asis = FALSE
) {
  stopifnot(is.data.frame(data))
  stopifnot(x %in% names(data))
  stopifnot(y %in% names(data))

  # if wb_data is passed, only create asis mschart output
  if (inherits(data, "wb_data")) {
    asis <- TRUE
  }

  xvar <- x
  yvar <- y

  if (
    inherits(data, "data.table") ||
      inherits(data, "tbl_df") ||
      inherits(data, "tbl")
  ) {
    data <- as.data.frame(data, stringsAsFactors = FALSE)
  }

  if (!is.null(group) && !(group %in% names(data))) {
    stop(
      "column ",
      shQuote(group),
      " could not be found in data.",
      call. = FALSE
    )
  }
  if (!is.null(labels)) {
    labs <- labels[!labels %in% names(data)]
    if (!(all(labs))) {
      stop(
        "column(s) ",
        paste(shQuote(labs), collapse = ", "),
        " could not be found in data.",
        call. = FALSE
      )
    }
  }

  theme_ <- mschart_theme()

  if (asis) {
    data_x <- sort(unname(unlist(data[[x]])))
  } else {
    data_x <- data[[x]]
  }

  if (asis) {
    data_y <- sort(unname(unlist(data[y])))
  } else {
    data_y <- data[[y]]
  }

  if (type == "areaplot" || type == "barplot") {
    assert_area(data_x, data_y)
  }

  if (type == "scatterplot" || type == "bubbleplot") {
    assert_scatter(data_x, data_y)
  }

  if (type == "lineplot" || type == "radarplot" || type == "stockplot") {
    assert_line(data_y)
  }

  if (type == "pieplot") {
    assert_pie(data_x, data_y)
  }

  tryCatch(
    {
      x_axis_tag <- get_axis_tag(data_x)
    },
    error = function(e) {
      stop(
        "column ",
        shQuote(x),
        ": ",
        e$message,
        " [",
        paste(class(data_x), collapse = ","),
        "]",
        call. = FALSE
      )
    }
  )
  tryCatch(
    {
      y_axis_tag <- get_axis_tag(data_y)
    },
    error = function(e) {
      stop(
        "column ",
        shQuote(y),
        ": ",
        e$message,
        " [",
        paste(class(data_y), collapse = ","),
        "]",
        call. = FALSE
      )
    }
  )

  x_axis_ <- axis_options(axis_position = "b")
  y_axis_ <- axis_options(axis_position = "l")

  x <- x[1]
  y <- y[1]

  lbls <- list(title = NULL, x = x, y = y)

  out <- list(
    data = data,
    x = x,
    y = y,
    group = group,
    label_cols = labels,
    theme = theme_,
    options = list(),
    x_axis = x_axis_,
    y_axis = y_axis_,
    axis_tag = list(
      x = x_axis_tag,
      y = y_axis_tag
    ),
    fmt_names = list(
      x = fmt_name(data_x),
      y = fmt_name(data_y)
    ),
    labels = lbls,
    asis = asis,
    xvar = xvar,
    yvar = yvar,
    axis_x_xml = axis_content_xml,
    axis_y_xml = axis_content_xml
  )
  class(out) <- c("ms_chart")
  out <- chart_data_labels(out)

  if (type == "areaplot" || type == "lineplot") {
    xtag <- if (inherits(data_x, "Date")) {
      "c:dateAx"
    } else if (is.character(data_x) || is.factor(data_x)) {
      "c:catAx"
    } else {
      "c:valAx"
    }
    out$axis_tag <- list(x = xtag, y = "c:valAx")
  }

  if (type == "scatterplot" || type == "bubbleplot") {
    out <- pretty_num_axes(out, data_x, data_y)
  }

  if (!asis) {
    out$data_series <- excel_data_setup(out)
    series_names <- get_series_names(out)
  } else {
    out$data_series <- out$data
    series_names <- names(out$data)[-1]
  }

  if (length(series_names) <= length(colour_list)) {
    palette_ <- colour_list[[length(series_names)]]
  } else {
    palette_ <- sample(colors(), size = length(series_names), replace = TRUE)
  }

  series_symbols <- rep("circle", length(series_names))
  series_lstyle <- rep("solid", length(series_names))
  series_size <- rep(12, length(series_names))
  series_lwidth <- rep(2, length(series_names))
  labels_fp <- rep(list(fp_text(font.size = 0)), length(series_names))
  series_smooth <- rep(1, length(series_names))
  out$series_settings <- list(
    fill = setNames(palette_, series_names),
    colour = setNames(palette_, series_names),
    symbol = setNames(series_symbols, series_names),
    line_style = setNames(series_lstyle, series_names),
    size = setNames(series_size, series_names),
    line_width = setNames(series_lwidth, series_names),
    labels_fp = setNames(labels_fp, series_names),
    smooth = setNames(series_smooth, series_names)
  )

  out
}

#' @title Print method for ms_chart
#' @description An \code{ms_chart} object cannot be rendered
#' in R. The default printing method will only display
#' simple information about the object.
#' If argument \code{preview} is set to TRUE, a \code{pptx} file
#' will be produced and opened with function \code{browseURL}.
#'
#' @param x an \code{ms_chart} object.
#' @param preview preview the chart in a PowerPoint document
#' @param ... unused
#' @return No return value, called for side effects.
#' @export
#' @importFrom officer read_pptx add_slide ph_location_fullsize ph_with
#' @importFrom utils browseURL
print.ms_chart <- function(x, preview = FALSE, ...) {
  if (preview && interactive()) {
    doc <- read_pptx()
    doc <- add_slide(doc, layout = "Title and Content", master = "Office Theme")
    doc <- ph_with(doc, x, location = ph_location_fullsize())
    file_out <- print(doc, target = tempfile(fileext = ".pptx"))
    browseURL(file_out)
    return(invisible())
  }
  class_val <- setdiff(class(x), "ms_chart")
  cat(sprintf("* %s object\n\n", shQuote(class_val)))

  cat(sprintf(
    "* original data [%.0f,%.0f] (sample):\n",
    nrow(x$data),
    ncol(x$data)
  ))
  print(x$data[seq_len(min(c(nrow(x$data), 5))), ])
  cat(sprintf(
    "\n* series data [%.0f,%.0f] (sample):\n",
    nrow(x$data_series),
    ncol(x$data_series)
  ))
  print(x$data_series[seq_len(min(c(nrow(x$data_series), 5))), ])
}

colour_list <- list(
  c("#4477AA"),
  c("#4477AA", "#CC6677"),
  c("#4477AA", "#DDCC77", "#CC6677"),
  c("#4477AA", "#117733", "#DDCC77", "#CC6677"),
  c("#332288", "#88CCEE", "#117733", "#DDCC77", "#CC6677"),
  c("#332288", "#88CCEE", "#117733", "#DDCC77", "#CC6677", "#AA4499"),
  c(
    "#332288",
    "#88CCEE",
    "#44AA99",
    "#117733",
    "#DDCC77",
    "#CC6677",
    "#AA4499"
  ),
  c(
    "#332288",
    "#88CCEE",
    "#44AA99",
    "#117733",
    "#999933",
    "#DDCC77",
    "#CC6677",
    "#AA4499"
  ),
  c(
    "#332288",
    "#88CCEE",
    "#44AA99",
    "#117733",
    "#999933",
    "#DDCC77",
    "#CC6677",
    "#882255",
    "#AA4499"
  ),
  c(
    "#332288",
    "#88CCEE",
    "#44AA99",
    "#117733",
    "#999933",
    "#DDCC77",
    "#661100",
    "#CC6677",
    "#882255",
    "#AA4499"
  ),
  c(
    "#332288",
    "#6699CC",
    "#88CCEE",
    "#44AA99",
    "#117733",
    "#999933",
    "#DDCC77",
    "#661100",
    "#CC6677",
    "#882255",
    "#AA4499"
  ),
  c(
    "#332288",
    "#6699CC",
    "#88CCEE",
    "#44AA99",
    "#117733",
    "#999933",
    "#DDCC77",
    "#661100",
    "#CC6677",
    "#AA4466",
    "#882255",
    "#AA4499"
  )
)


#' @importFrom htmltools htmlEscape
#' @importFrom xml2 xml_attr<- xml_remove
#' @method format ms_chart
#' @export
format.ms_chart <- function(
  x,
  id_x,
  id_y,
  sheetname = "sheet1",
  drop_ext_data = FALSE,
  ...
) {
  str_ <- to_pml(
    x,
    id_x = id_x,
    id_y = id_y,
    sheetname = sheetname,
    secondary_y = 0
  )

  if (is.null(x$x_axis$num_fmt)) {
    x$x_axis$num_fmt <- x$theme[[x$fmt_names$x]]
  }
  if (is.null(x$y_axis$num_fmt)) {
    x$y_axis$num_fmt <- x$theme[[x$fmt_names$y]]
  }

  x_axis_str <- x$axis_x_xml(
    x$x_axis,
    id = id_x,
    theme = x$theme,
    cross_id = id_y,
    is_x = TRUE,
    lab = htmlEscape(x$labels$x),
    rot = x$theme$title_x_rot
  )

  x_axis_str <- sprintf("<%s>%s</%s>", x$axis_tag$x, x_axis_str, x$axis_tag$x)

  y_axis_str <- x$axis_y_xml(
    x$y_axis,
    id = id_y,
    theme = x$theme,
    cross_id = id_x,
    is_x = FALSE,
    lab = htmlEscape(x$labels$y),
    rot = x$theme$title_y_rot
  )

  y_axis_str <- sprintf("<%s>%s</%s>", x$axis_tag$y, y_axis_str, x$axis_tag$y)

  secondary <- TRUE # logical will become FALSE if secondary axis are created

  ids <- as.integer(id_x) + c(1111L, 2222L, 3333L, 4444L)

  axis_str <- paste0(x_axis_str, y_axis_str)

  if (length(x$secondary)) {
    ser_id <- length(x$yvar) + 1L

    for (sec in seq_along(x$secondary)) {
      is_sec_x <- isTRUE(attr(x$secondary[[sec]], "secondary_x"))
      is_sec_y <- isTRUE(attr(x$secondary[[sec]], "secondary_y"))

      # charts reference their axis via this id
      if (is_sec_y || is_sec_x) {
        x_id <- as.character(ids[1])
        y_id <- as.character(ids[2])
      } else {
        x_id <- id_x
        y_id <- id_y
      }

      xlab <- if (is_sec_x && !is_sec_y) {
        htmlEscape(x$secondary[[sec]]$labels$x)
      } else {
        NULL
      }
      ylab <- if (is_sec_y && !is_sec_x) {
        htmlEscape(x$secondary[[sec]]$labels$y)
      } else {
        NULL
      }

      # add only one secondary x and y axis if required
      if (secondary && (is_sec_x || is_sec_y)) {
        axis_l_str <- axis_content_xml(
          x$secondary[[sec]]$y_axis,
          id = y_id,
          theme = x$secondary[[sec]]$theme,
          cross_id = x_id,
          is_x = FALSE,
          lab = ylab,
          rot = x$secondary[[sec]]$theme$title_y_rot
        )

        x_axis_str <- sprintf(
          "<%s>%s</%s>",
          x$secondary[[sec]]$axis_tag$y,
          axis_l_str,
          x$secondary[[sec]]$axis_tag$y
        )

        axis_r_str <- axis_content_xml(
          x$secondary[[sec]]$x_axis,
          id = x_id,
          theme = x$secondary[[sec]]$theme,
          cross_id = y_id,
          is_x = TRUE,
          lab = xlab
        )
        y_axis_str <- sprintf(
          "<%s>%s</%s>",
          x$secondary[[sec]]$axis_tag$x,
          axis_r_str,
          x$secondary[[sec]]$axis_tag$x
        )

        secondary <- FALSE

        axis_str <- paste0(axis_str, x_axis_str, y_axis_str)
      }

      # all secondary charts
      str_ <- paste0(
        str_,
        to_pml(
          x$secondary[[sec]],
          id_y = y_id,
          id_x = x_id,
          sheetname = sheetname,
          secondary_y = ser_id
        )
      )

      ser_id <- ser_id + length(x$secondary[[sec]]$yvar)
    }
  }

  if (inherits(x, "ms_piechart")) {
    axis_str <- ""
    table_str <- ""
  } else {
    table_str <- table_content_xml(x)
  }

  sppr_str <- sppr_content_xml(x$theme, "plot")

  ns <- "xmlns:c=\"http://schemas.openxmlformats.org/drawingml/2006/chart\" xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\" xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\""

  xml_elt <- paste0(
    "<c:plotArea ",
    ns,
    "><c:layout/>",
    str_,
    axis_str,
    table_str,
    sppr_str,
    "</c:plotArea>"
  )
  xml_doc <- read_xml(system.file(package = "mschart", "template", "chart.xml"))

  node <- xml_find_first(xml_doc, "//c:plotArea")
  xml_replace(node, as_xml_document(xml_elt))

  if (!is.null(x$labels[["title"]])) {
    chartnode <- xml_find_first(xml_doc, "//c:chart")
    title_ <- "<c:title %s><c:tx><c:rich><a:bodyPr/><a:lstStyle/><a:p><a:pPr><a:defRPr/></a:pPr><a:r>%s<a:t>%s</a:t></a:r></a:p></c:rich></c:tx><c:layout/><c:overlay val=\"0\"/></c:title>"
    title_ <- sprintf(
      title_,
      ns,
      format(x$theme[["main_title"]], type = "pml"),
      htmlEscape(x$labels[["title"]])
    )
    xml_add_child(chartnode, as_xml_document(title_), .where = 0)
  } else {
    # null is not enough
    atd_node <- xml_find_first(xml_doc, "//c:chart/c:autoTitleDeleted")
    xml_attr(atd_node, "val") <- "1"
  }

  if (x$theme[["legend_position"]] %in% "n") {
    legend_pos <- xml_find_first(xml_doc, "//c:chart/c:legend")
    xml_remove(legend_pos)
  } else {
    legend_pos <- xml_find_first(xml_doc, "//c:chart/c:legend/c:legendPos")
    xml_attr(legend_pos, "val") <- x$theme[["legend_position"]]

    # manual layout for the legend box (x / y / w / h, all fractional)
    lx <- x$theme[["legend_x"]]
    ly <- x$theme[["legend_y"]]
    lw <- x$theme[["legend_w"]]
    lh <- x$theme[["legend_h"]]
    if (!is.null(lx) || !is.null(ly) || !is.null(lw) || !is.null(lh)) {
      ns_layout <- paste(
        "xmlns:c=\"http://schemas.openxmlformats.org/drawingml/2006/chart\"",
        "xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\""
      )
      manual <- paste0(
        "<c:layout ", ns_layout, "><c:manualLayout>",
        "<c:xMode val=\"edge\"/>",
        "<c:yMode val=\"edge\"/>",
        if (!is.null(lx)) sprintf("<c:x val=\"%g\"/>", lx),
        if (!is.null(ly)) sprintf("<c:y val=\"%g\"/>", ly),
        if (!is.null(lw)) sprintf("<c:w val=\"%g\"/>", lw),
        if (!is.null(lh)) sprintf("<c:h val=\"%g\"/>", lh),
        "</c:manualLayout></c:layout>"
      )
      layout_node <- xml_find_first(xml_doc, "//c:chart/c:legend/c:layout")
      xml_replace(layout_node, as_xml_document(manual))
    }

    rpr <- format(x$theme[["legend_text"]], type = "pml")
    rpr <- gsub("a:rPr", "a:defRPr", rpr)
    labels_text_pr <- "<c:txPr xmlns:c=\"http://schemas.openxmlformats.org/drawingml/2006/chart\" xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\"><a:bodyPr/><a:lstStyle/><a:p><a:pPr>%s</a:pPr></a:p></c:txPr>"
    labels_text_pr <- sprintf(labels_text_pr, rpr)
    legend_ <- xml_find_first(xml_doc, "//c:chart/c:legend")
    xml_add_child(legend_, as_xml_document(labels_text_pr))
  }

  chart_area_node <- xml_find_first(xml_doc, "//c:chartSpace")
  chart_area_properties <- sppr_content_xml(x$theme, what = "chart", ns = ns)
  xml_add_child(chart_area_node, as_xml_document(chart_area_properties))

  if (drop_ext_data) {
    xml_remove(xml_find_first(xml_doc, "//c:externalData"))
  }

  as.character(xml_doc)
}

skip_on_cran()

test_that("barchart visual snapshot", {
  skip_if_not_installed("doconv")
  skip_if_not(doconv::msoffice_available())
  require(doconv)
  local_edition(3L)

  chart <- ms_barchart(
    data = browser_data,
    x = "browser",
    y = "value",
    group = "serie"
  )

  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(
    doc,
    chart,
    location = officer::ph_location_fullsize()
  )
  expect_snapshot_doc(x = doc, name = "visual-barchart", engine = "testthat")
})

test_that("linechart visual snapshot", {
  skip_if_not_installed("doconv")
  skip_if_not(doconv::msoffice_available())
  require(doconv)
  local_edition(3L)

  chart <- ms_linechart(
    data = browser_ts,
    x = "date",
    y = "freq",
    group = "browser"
  )

  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(
    doc,
    chart,
    location = officer::ph_location_fullsize()
  )
  expect_snapshot_doc(x = doc, name = "visual-linechart", engine = "testthat")
})

test_that("areachart visual snapshot", {
  skip_if_not_installed("doconv")
  skip_if_not(doconv::msoffice_available())
  require(doconv)
  local_edition(3L)

  chart <- ms_areachart(
    data = browser_ts,
    x = "date",
    y = "freq",
    group = "browser"
  )

  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(
    doc,
    chart,
    location = officer::ph_location_fullsize()
  )
  expect_snapshot_doc(x = doc, name = "visual-areachart", engine = "testthat")
})

test_that("scatterchart visual snapshot", {
  skip_if_not_installed("doconv")
  skip_if_not(doconv::msoffice_available())
  require(doconv)
  local_edition(3L)

  chart <- ms_scatterchart(
    data = iris,
    x = "Sepal.Length",
    y = "Sepal.Width",
    group = "Species"
  )

  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(
    doc,
    chart,
    location = officer::ph_location_fullsize()
  )
  expect_snapshot_doc(
    x = doc,
    name = "visual-scatterchart",
    engine = "testthat"
  )
})

test_that("pie visual snapshot", {
  skip_if_not_installed("doconv")
  skip_if_not(doconv::msoffice_available())
  require(doconv)
  local_edition(3L)

  dat <- data.frame(
    browser = c("Chrome", "Firefox", "Safari", "Edge", "Other"),
    value = c(64, 12, 8, 5, 11)
  )

  # Pie chart
  pie <- ms_piechart(data = dat, x = "browser", y = "value")
  pie <- chart_labels(pie, title = "Browser share")

  # Doughnut chart
  donut <- ms_piechart(data = dat, x = "browser", y = "value")
  donut <- chart_settings(donut, hole_size = 50)
  donut <- chart_labels(donut, title = "Browser share (donut)")

  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(doc, pie, location = officer::ph_location_fullsize())
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(
    doc,
    donut,
    location = officer::ph_location_fullsize()
  )
  expect_snapshot_doc(x = doc, name = "visual-pie", engine = "testthat")
})

test_that("ms_stockchart accepts an x column literally named 'group'", {
  df <- data.frame(
    group = c("A", "B", "C", "D"),
    high = c(11, 15, 10, 18),
    low = c(9, 13, 8, 16),
    close = c(10, 14, 9, 17)
  )

  expect_no_error({
    chart <- ms_stockchart(
      df,
      x = "group",
      high = "high",
      low = "low",
      close = "close"
    )
    format(
      chart,
      sheetname = "sheet1",
      id_x = "64451212",
      id_y = "64453248"
    )
  })
})

set.seed(1)
weather <- data.frame(
  temperature_c = runif(20, 5, 30),
  pressure_hpa = runif(20, 990, 1030),
  humidity_t = runif(20, 30, 90),
  humidity_p = runif(20, 30, 90)
)

build_combo <- function() {
  sc_temp <- ms_scatterchart(
    data = weather,
    x = "temperature_c",
    y = "humidity_t"
  )
  sc_pres <- ms_scatterchart(
    data = weather,
    x = "pressure_hpa",
    y = "humidity_p"
  )
  ms_chart_combine(
    temp = sc_temp,
    pres = sc_pres,
    secondary_x = "pres"
  )
}

format_combo_xml <- function(combo) {
  xml <- format(
    combo,
    sheetname = "sheet1",
    id_x = "64451712",
    id_y = "64453248"
  )
  xml2::read_xml(xml)
}

test_that("independent_x keeps both x columns in data_series", {
  cb <- build_combo()
  expect_true(all(
    c("temperature_c", "pressure_hpa") %in%
      names(cb$data_series)
  ))
  expect_equal(cb$xvar, "temperature_c")
  expect_equal(cb$secondary[[1]]$xvar, "pressure_hpa")
})

test_that("independent_x produces unique series idx/order", {
  doc <- format_combo_xml(build_combo())
  idx <- as.integer(xml2::xml_attr(
    xml2::xml_find_all(doc, "//c:ser/c:idx"),
    "val"
  ))
  order <- as.integer(xml2::xml_attr(
    xml2::xml_find_all(doc, "//c:ser/c:order"),
    "val"
  ))
  expect_length(idx, 2L)
  expect_equal(anyDuplicated(idx), 0L)
  expect_equal(anyDuplicated(order), 0L)
})

test_that("independent_x emits a top secondary x axis and 4 distinct axIds", {
  doc <- format_combo_xml(build_combo())
  ax_ids <- xml2::xml_attr(
    xml2::xml_find_all(doc, "//c:plotArea/*/c:axId"),
    "val"
  )
  expect_equal(length(unique(ax_ids)), 4L)
  axpos <- xml2::xml_attr(
    xml2::xml_find_all(doc, "//c:plotArea/*/c:axPos"),
    "val"
  )
  expect_true("t" %in% axpos)
  expect_true("b" %in% axpos)
})

test_that("independent_x: primary and secondary reference disjoint columns", {
  doc <- format_combo_xml(build_combo())
  refs <- xml2::xml_text(xml2::xml_find_all(doc, "//c:f"))
  cols <- unique(sub("\\$.*$", "", sub("^.*!\\$", "", refs)))
  expect_true(all(c("A", "B", "C", "D") %in% cols))
})

test_that("ms_chart_combine errors on y column collisions in independent_x", {
  weather2 <- weather
  names(weather2)[names(weather2) == "humidity_p"] <- "humidity_t"
  sc_temp <- ms_scatterchart(weather, x = "temperature_c", y = "humidity_t")
  sc_pres <- ms_scatterchart(weather2, x = "pressure_hpa", y = "humidity_t")
  expect_error(
    ms_chart_combine(temp = sc_temp, pres = sc_pres, secondary_x = "pres"),
    "same column name"
  )
})

test_that("ms_chart_combine errors when a name is in both secondary_x and _y", {
  sc_temp <- ms_scatterchart(weather, x = "temperature_c", y = "humidity_t")
  sc_pres <- ms_scatterchart(weather, x = "pressure_hpa", y = "humidity_p")
  expect_error(
    ms_chart_combine(
      temp = sc_temp,
      pres = sc_pres,
      secondary_x = "pres",
      secondary_y = "pres"
    ),
    "cannot be on both"
  )
})

test_that("ms_chart_combine errors when mixing secondary_y and secondary_x", {
  sc_a <- ms_scatterchart(weather, x = "temperature_c", y = "humidity_t")
  sc_b <- ms_scatterchart(weather, x = "temperature_c", y = "humidity_p")
  sc_c <- ms_scatterchart(weather, x = "pressure_hpa", y = "humidity_p")
  expect_error(
    ms_chart_combine(
      a = sc_a,
      b = sc_b,
      c = sc_c,
      secondary_y = "b",
      secondary_x = "c"
    ),
    "not supported"
  )
})

test_that("ms_chart_combine errors on multiple secondary_x charts", {
  sc1 <- ms_scatterchart(weather, x = "temperature_c", y = "humidity_t")
  sc2 <- ms_scatterchart(weather, x = "pressure_hpa", y = "humidity_p")
  sc3 <- ms_scatterchart(weather, x = "pressure_hpa", y = "humidity_t")
  expect_error(
    ms_chart_combine(a = sc1, b = sc2, c = sc3, secondary_x = c("b", "c")),
    "Only one chart can be placed on secondary_x"
  )
})

test_that("custom labels are correctly set in barchart", {
  dat_no_group <- data.frame(
    stringsAsFactors = FALSE,
    cut = c("Fair", "Good", "Very Good", "Premium", "Ideal"),
    carat = c(1, 0.82, 0.71, 0.86, 0.54),
    n = c(1610L, 4906L, 12082L, 13791L, 21551L),
    label = c("\U0001f603", "\U0001f525", "", "Hello \U0001f47d", "\U0001f44e"),
    group = c("OK", "KO", "KO", "KO", "KO")
  )

  chrt <- ms_barchart(
    data = dat_no_group,
    group = "group",
    x = "cut",
    labels = "label",
    y = "n"
  )

  xml <- format(
    chrt,
    sheetname = "sheet1",
    id_x = "64451212",
    id_y = "64453248"
  )

  chart <- xml2::read_xml(xml)
  showDataLabelsRange <- xml2::xml_find_all(
    chart,
    "//c:ser/c:dLbls/c:extLst/c:ext/c15:showDataLabelsRange"
  )

  expect_equal(
    xml2::xml_attr(showDataLabelsRange, "val"),
    c("1", "1"),
    ignore_attr = TRUE
  )

  label_pt <- xml2::xml_find_all(
    chart,
    "//c:ser/c:extLst/c:ext/c15:datalabelsRange/c15:dlblRangeCache/c:pt"
  )
  expect_equal(
    xml2::xml_text(label_pt),
    c(
      "",
      "\U0001f525",
      "\U0001f44e",
      "Hello \U0001f47d",
      "",
      "\U0001f603",
      "",
      "",
      "",
      ""
    ),
    ignore_attr = TRUE
  )
})

browser_data <- data.frame(
  browser = rep(
    c("Android", "Chrome", "IE", "Firefox", "Opera", "Safari"),
    each = 3
  ),
  serie = rep(c("v1", "v2", "v3"), times = 6),
  value = c(
    80,
    90,
    77,
    150,
    140,
    131,
    60,
    70,
    62,
    50,
    60,
    52,
    40,
    50,
    42,
    55,
    65,
    55
  )
)
target <- data.frame(
  browser = unique(browser_data$browser),
  cible = 220
)

format_combo_xml <- function(combo) {
  xml <- format(
    combo,
    sheetname = "sheet1",
    id_x = "64451712",
    id_y = "64453248"
  )
  xml2::read_xml(xml)
}

test_that("ms_chart_combine assigns unique series idx/order across all charts", {
  bars <- ms_barchart(
    browser_data,
    x = "browser",
    y = "value",
    group = "serie"
  ) |>
    as_bar_stack()
  line <- ms_linechart(target, x = "browser", y = "cible")

  combo <- ms_chart_combine(bars = bars, target = line)
  doc <- format_combo_xml(combo)

  idx <- as.integer(xml2::xml_attr(
    xml2::xml_find_all(doc, "//c:ser/c:idx"),
    "val"
  ))
  order <- as.integer(xml2::xml_attr(
    xml2::xml_find_all(doc, "//c:ser/c:order"),
    "val"
  ))

  expect_length(idx, 4L)
  expect_equal(anyDuplicated(idx), 0L)
  expect_equal(anyDuplicated(order), 0L)
})

test_that("ms_chart_combine merges secondary data into the embedded sheet", {
  bars <- ms_barchart(
    browser_data,
    x = "browser",
    y = "value",
    group = "serie"
  ) |>
    as_bar_stack()
  line <- ms_linechart(target, x = "browser", y = "cible")

  combo <- ms_chart_combine(bars = bars, target = line)

  expect_true("cible" %in% names(combo$data_series))
  expect_equal(nrow(combo$data_series), nrow(target))
  expect_equal(combo$data_series$cible, rep(220, nrow(target)))

  doc <- format_combo_xml(combo)
  line_val <- xml2::xml_text(
    xml2::xml_find_first(doc, "//c:lineChart/c:ser/c:val/c:numRef/c:f")
  )
  bar_vals <- xml2::xml_text(
    xml2::xml_find_all(doc, "//c:barChart/c:ser/c:val/c:numRef/c:f")
  )
  expect_false(line_val %in% bar_vals)
})

test_that("ms_chart_combine errors on column-name collision between charts", {
  d1 <- data.frame(x = letters[1:3], y = 1:3)
  d2 <- data.frame(x = letters[1:3], y = 4:6)
  c1 <- ms_barchart(d1, x = "x", y = "y")
  c2 <- ms_linechart(d2, x = "x", y = "y")

  expect_error(ms_chart_combine(bars = c1, line = c2), "column name")
})

test_that("ms_chart_combine errors on x-value mismatch", {
  bars <- ms_barchart(
    browser_data,
    x = "browser",
    y = "value",
    group = "serie"
  ) |>
    as_bar_stack()
  line_bad_x <- ms_linechart(
    data.frame(browser = c("Android", "Chrome", "Other"), cible = 220),
    x = "browser",
    y = "cible"
  )

  expect_error(ms_chart_combine(bars = bars, target = line_bad_x), "x values")
})

xdr_ns <- c(
  xdr = "http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing"
)

bar_chart <- function() {
  ms_barchart(
    data = data.frame(
      x = c("A", "B", "C"),
      y = c(1, 3, 2),
      g = rep("s", 3)
    ),
    x = "x",
    y = "y",
    group = "g"
  )
}

# Build a one-shot xlsx with a chart anchored as requested, then return the
# parsed drawing1.xml so tests can inspect anchor shape and cell coordinates.
write_and_read_drawing <- function(...) {
  x <- officer::read_xlsx()
  x <- officer::add_sheet(x, label = "s")
  x <- officer::sheet_add_drawing(
    x,
    value = bar_chart(),
    sheet = "s",
    ...
  )
  out <- tempfile(fileext = ".xlsx")
  print(x, target = out)
  pkg <- tempfile()
  dir.create(pkg)
  officer::unpack_folder(out, folder = pkg)
  drw <- list.files(
    file.path(pkg, "xl", "drawings"),
    pattern = "\\.xml$",
    full.names = TRUE
  )[1]
  xml2::read_xml(drw)
}

test_that("anchor = \"B2:H20\" produces a twoCellAnchor", {
  doc <- write_and_read_drawing(anchor = "B2:H20")
  anchor <- xml2::xml_find_first(doc, "//xdr:twoCellAnchor", xdr_ns)
  expect_false(inherits(anchor, "xml_missing"))
  expect_equal(xml2::xml_attr(anchor, "editAs"), "twoCell")
  expect_equal(
    xml2::xml_text(
      xml2::xml_find_first(anchor, "xdr:from/xdr:col", xdr_ns)
    ),
    "1"
  )
  expect_equal(
    xml2::xml_text(
      xml2::xml_find_first(anchor, "xdr:to/xdr:col", xdr_ns)
    ),
    "7"
  )
  expect_equal(
    xml2::xml_text(
      xml2::xml_find_first(anchor, "xdr:to/xdr:row", xdr_ns)
    ),
    "19"
  )
})

test_that("anchor single cell produces a oneCellAnchor with width/height", {
  doc <- write_and_read_drawing(anchor = "C4", width = 5, height = 3)
  anchor <- xml2::xml_find_first(doc, "//xdr:oneCellAnchor", xdr_ns)
  expect_false(inherits(anchor, "xml_missing"))
  expect_equal(
    xml2::xml_text(
      xml2::xml_find_first(anchor, "xdr:from/xdr:col", xdr_ns)
    ),
    "2"
  )
  ext <- xml2::xml_find_first(anchor, "xdr:ext", xdr_ns)
  expect_equal(xml2::xml_attr(ext, "cx"), as.character(5 * 914400))
  expect_equal(xml2::xml_attr(ext, "cy"), as.character(3 * 914400))
})

test_that("no anchor keeps absoluteAnchor (legacy behaviour)", {
  doc <- write_and_read_drawing(left = 1, top = 1, width = 6, height = 4)
  expect_false(inherits(
    xml2::xml_find_first(doc, "//xdr:absoluteAnchor", xdr_ns),
    "xml_missing"
  ))
})

test_that("edit_as is passed through to the twoCellAnchor", {
  doc <- write_and_read_drawing(anchor = "B2:F10", edit_as = "oneCell")
  expect_equal(
    xml2::xml_attr(
      xml2::xml_find_first(doc, "//xdr:twoCellAnchor", xdr_ns),
      "editAs"
    ),
    "oneCell"
  )
})

test_that("invalid anchor values are rejected with a clear message", {
  x <- officer::read_xlsx()
  x <- officer::add_sheet(x, label = "s")
  expect_error(
    officer::sheet_add_drawing(
      x,
      value = bar_chart(),
      sheet = "s",
      anchor = 42
    ),
    "anchor.*single string"
  )
  expect_error(
    officer::sheet_add_drawing(
      x,
      value = bar_chart(),
      sheet = "s",
      anchor = "B2:"
    ),
    "FROM:TO"
  )
  expect_error(
    officer::sheet_add_drawing(
      x,
      value = bar_chart(),
      sheet = "s",
      anchor = "ZZ"
    ),
    "invalid cell reference"
  )
})

#' @export
#' @title Modify data labels settings
#' @description Data labels show details about data series. This function indicates that
#' data labels should be displayed. See [chart_labels_text()] for modifying
#' text settings associated with labels.
#' @param x an `ms_chart` object.
#' @param num_fmt `character(1)`: number formatting specifies number format properties which
#' indicate how to format and render the numeric values. It can be "General", "0.00", "#,##0",
#' "#,##0.00", "mm-dd-yy", "m/d/yy h:mm", etc.
#' @param position `character(1)`: it specifies the position of the data label.
#' It should be one of 'b', 'ctr', 'inBase', 'inEnd', 'l',
#' 'outEnd', 'r', 't'.
#' When grouping is 'clustered', it should be one of 'ctr','inBase','inEnd','outEnd'.
#' When grouping is 'stacked', it should be one of 'ctr','inBase','inEnd'.
#' When grouping is 'standard', it should be one of 'b','ctr','l','r','t'.
#' @param show_legend_key show legend key if TRUE.
#' @param show_val show values if TRUE.
#' @param show_cat_name show categories if TRUE.
#' @param show_serie_name show names of series if TRUE.
#' @param show_percent show percentages if TRUE.
#' @param separator separator between the label components (value, category
#' name, series name, etc.) when multiple components are displayed.
#' Default is `", "`.
#' @return An `ms_chart` object.
#' @seealso [chart_labels_text()], [chart_labels()]
#' @examples
#' my_bc <- ms_barchart(
#'   data = browser_data, x = "browser",
#'   y = "value", group = "serie"
#' )
#' my_bc <- chart_data_labels(my_bc, show_val = TRUE, position = "outEnd")
chart_data_labels <- function(
  x,
  num_fmt = "General",
  position = "ctr",
  show_legend_key = FALSE,
  show_val = FALSE,
  show_cat_name = FALSE,
  show_serie_name = FALSE,
  show_percent = FALSE,
  separator = ", "
) {
  if (!position %in% st_dlblpos) {
    stop(
      "position should be one of ",
      paste0(shQuote(st_dlblpos), collapse = ", ")
    )
  }

  out <- list(
    num_fmt = num_fmt,
    position = position,
    show_legend_key = show_legend_key,
    show_val = show_val,
    show_cat_name = show_cat_name,
    show_serie_name = show_serie_name,
    show_percent = show_percent,
    separator = separator
  )
  class(out) <- "labels_options"
  x$label_settings <- out
  x
}

#' @export
#' @method to_pml labels_options
to_pml.labels_options <- function(
  x,
  add_ns = FALSE,
  with_position = TRUE,
  show_label = FALSE,
  ...
) {
  txpr <- ""
  if (!is.null(x$labels_fp)) {
    txpr <- ooxml_txpr(x$labels_fp)
  }

  str_ <- paste0(
    "<c:dLbls>",
    sprintf("<c:numFmt formatCode=\"%s\" sourceLinked=\"0\"/>", x$num_fmt),
    txpr,
    if (with_position) sprintf("<c:dLblPos val=\"%s\"/>", x$position),
    sprintf("<c:showLegendKey val=\"%.0f\"/>", x$show_legend_key),
    sprintf("<c:showVal val=\"%.0f\"/>", as.integer(x$show_val)),
    sprintf("<c:showCatName val=\"%.0f\"/>", as.integer(x$show_cat_name)),
    sprintf("<c:showSerName val=\"%.0f\"/>", as.integer(x$show_serie_name)),
    sprintf("<c:showPercent val=\"%.0f\"/>", x$show_percent),
    sprintf("<c:showBubbleSize val=\"%.0f\"/>", FALSE),
    sprintf("<c:separator val=\"%s\"/>", x$separator),
    if (show_label) {
      paste0(
        "<c:extLst>",
        "<c:ext uri=\"{CE6537A1-D6FC-4f65-9D91-7224C49458BB}\" xmlns:c15=\"http://schemas.microsoft.com/office/drawing/2012/chart\">",
        "<c15:dlblFieldTable/>",
        "<c15:showDataLabelsRange val=\"1\"/>",
        "</c:ext>",
        "</c:extLst>"
      )
    },
    "</c:dLbls>"
  )

  str_
}

#' @title Set chart theme
#' @description Modify chart theme with function \code{set_theme}.
#' @param x an `ms_chart` object.
#' @param value a [mschart_theme()] object.
#' @examples
#' library(officer)
#' mytheme <- mschart_theme(
#'   axis_title = fp_text(color = "red", font.size = 24, bold = TRUE),
#'   grid_major_line_y = fp_border(width = 1, color = "orange"),
#'   axis_ticks_y = fp_border(width = 0.4, color = "gray")
#' )
#'
#'
#' my_bc <- ms_barchart(
#'   data = browser_data, x = "browser",
#'   y = "value", group = "serie"
#' )
#' my_bc <- chart_settings(my_bc,
#'   dir = "horizontal", grouping = "stacked",
#'   gap_width = 150, overlap = 100
#' )
#' my_bc <- set_theme(my_bc, mytheme)
#'
#' my_bc_2 <- ms_barchart(
#'   data = browser_data, x = "browser",
#'   y = "value", group = "serie"
#' )
#' my_bc_2 <- chart_theme(my_bc_2,
#'   grid_major_line_y = fp_border(width = 0.5, color = "cyan")
#' )
#'
#' # Manual legend layout: place the legend in the top-right corner
#' # using fractions of the chart area (0 to 1).
#' my_bc_3 <- ms_barchart(
#'   data = browser_data, x = "browser",
#'   y = "value", group = "serie"
#' )
#' my_bc_3 <- chart_theme(my_bc_3,
#'   legend_position = "r",
#'   legend_x = 0.80, legend_y = 0.15,
#'   legend_w = 0.18, legend_h = 0.30
#' )
#' @return An `ms_chart` object.
#' @export
set_theme <- function(x, value) {
  if (!inherits(value, "mschart_theme")) {
    stop("value must be an mschart_theme object.", call. = FALSE)
  }
  x$theme <- value
  x
}

#' helper class to check function input
#' @param x input argument to check
#' @param class a class to check against
#' @keywords internal
#' @noRd
assert_class <- function(x, class) {
  if (!is.logical(x) && !inherits(x, class)) {
    stop("input must be logical or ", class, " not ", x)
  }
}

#' @importFrom officer fp_text fp_border
#' @description Use \code{mschart_theme()} to create a chart theme.
#' @param axis_title,axis_title_x,axis_title_y axis title formatting properties (see [officer::fp_text()])
#' @param main_title title formatting properties (see [officer::fp_text()])
#' @param legend_text legend text formatting properties (see [officer::fp_text()])
#' @param table_text table text formatting properties (see [officer::fp_text()])
#' @param axis_text,axis_text_x,axis_text_y axis text formatting properties (see [officer::fp_text()])
#' @param title_rot,title_x_rot,title_y_rot rotation angle
#' @param axis_ticks,axis_ticks_x,axis_ticks_y axis ticks formatting properties (see [officer::fp_border()])
#' @param grid_major_line,grid_major_line_x,grid_major_line_y major grid lines formatting properties (see [officer::fp_border()])
#' @param grid_minor_line,grid_minor_line_x,grid_minor_line_y minor grid lines formatting properties (see [officer::fp_border()])
#' @param plot_border plot area border lines formatting properties (see [officer::fp_border()])
#' @param chart_border chart area border lines formatting properties (see [officer::fp_border()])
#' @param plot_background plot area background fill color - single character value (e.g. "#000000" or "black")
#' @param chart_background chart area background fill color - single character value (e.g. "#000000" or "black")
#' @param date_fmt date format
#' @param str_fmt string or factor format
#' @param double_fmt double format
#' @param integer_fmt integer format
#' @param legend_position it specifies the position of the legend. It should be
#' one of 'b', 'tr', 'l', 'r', 't', 'n' (for 'none').
#' @param legend_x,legend_y,legend_w,legend_h optional fractions between
#' 0 and 1 to manually position and size the legend box within the chart
#' area. Each value is a fraction of the chart width (`legend_x`,
#' `legend_w`) or height (`legend_y`, `legend_h`). Any `NULL` value
#' keeps the default automatic layout; setting at least one triggers
#' manual placement via `<c:manualLayout>`.
#' @rdname set_theme
#' @return An `mschart_theme` object (for `mschart_theme()`).
#' @seealso [ms_barchart()], [ms_areachart()], [ms_scatterchart()], [ms_linechart()]
#' @export
mschart_theme <- function(
  axis_title = fp_text(bold = TRUE, font.size = 16),
  axis_title_x = axis_title,
  axis_title_y = axis_title,
  main_title = fp_text(bold = TRUE, font.size = 20),
  legend_text = fp_text(font.size = 14),
  table_text = fp_text(bold = FALSE, font.size = 9),
  axis_text = fp_text(),
  axis_text_x = axis_text,
  axis_text_y = axis_text,
  title_rot = 0,
  title_x_rot = 0,
  title_y_rot = 270,
  axis_ticks = fp_border(color = "#99999999"),
  axis_ticks_x = axis_ticks,
  axis_ticks_y = axis_ticks,
  grid_major_line = fp_border(color = "#99999999", style = "dashed"),
  grid_major_line_x = grid_major_line,
  grid_major_line_y = grid_major_line,
  grid_minor_line = fp_border(width = 0),
  grid_minor_line_x = grid_minor_line,
  grid_minor_line_y = grid_minor_line,
  chart_background = NULL,
  chart_border = fp_border(color = "transparent"),
  plot_background = NULL,
  plot_border = fp_border(color = "transparent"),
  date_fmt = "yyyy/mm/dd",
  str_fmt = "General",
  double_fmt = "#,##0.00",
  integer_fmt = "0",
  legend_position = "b",
  legend_x = NULL,
  legend_y = NULL,
  legend_w = NULL,
  legend_h = NULL
) {
  # strict check: these must be fp_text objects
  stopifnot(inherits(main_title, "fp_text"))
  stopifnot(inherits(table_text, "fp_text"))
  stopifnot(inherits(legend_text, "fp_text"))
  stopifnot(inherits(axis_title, "fp_text"))
  stopifnot(inherits(axis_title_x, "fp_text"))
  stopifnot(inherits(axis_title_y, "fp_text"))
  stopifnot(inherits(axis_text, "fp_text"))
  stopifnot(inherits(axis_text_x, "fp_text"))
  stopifnot(inherits(axis_text_y, "fp_text"))
  # strict check: these must be fp_border objects
  stopifnot(inherits(axis_ticks, "fp_border"))
  stopifnot(inherits(axis_ticks_x, "fp_border"))
  stopifnot(inherits(axis_ticks_y, "fp_border"))
  # assert_class: accept fp_border or logical (FALSE to disable the grid line)
  assert_class(grid_major_line, "fp_border")
  assert_class(grid_major_line_x, "fp_border")
  assert_class(grid_major_line_y, "fp_border")
  assert_class(grid_minor_line, "fp_border")
  assert_class(grid_minor_line_x, "fp_border")
  assert_class(grid_minor_line_y, "fp_border")
  # strict check: these must be fp_border objects
  stopifnot(inherits(chart_border, "fp_border"))
  stopifnot(inherits(plot_border, "fp_border"))

  if (title_rot < 0 || title_rot > 359) {
    stop("title_rot must be between 0 and 359")
  }

  if (title_x_rot < 0 || title_x_rot > 359) {
    stop("title_x_rot must be between 0 and 359")
  }

  if (title_y_rot < 0 || title_y_rot > 359) {
    stop("title_y_rot must be between 0 and 359")
  }

  if (!legend_position %in% st_legendpos) {
    stop(
      "legend_position should be one of ",
      paste0(shQuote(st_legendpos), collapse = ", ")
    )
  }

  validate_legend_fraction(legend_x, "legend_x")
  validate_legend_fraction(legend_y, "legend_y")
  validate_legend_fraction(legend_w, "legend_w")
  validate_legend_fraction(legend_h, "legend_h")

  out <- list(
    main_title = main_title,
    legend_text = legend_text,
    table_text = table_text,
    axis_title_x = axis_title_x,
    axis_title_y = axis_title_y,
    title_rot = title_rot,
    title_x_rot = title_x_rot,
    title_y_rot = title_y_rot,
    axis_text_x = axis_text_x,
    axis_text_y = axis_text_y,
    axis_ticks_x = axis_ticks_x,
    axis_ticks_y = axis_ticks_y,
    grid_major_line_x = grid_major_line_x,
    grid_major_line_y = grid_major_line_y,
    grid_minor_line_x = grid_minor_line_x,
    grid_minor_line_y = grid_minor_line_y,
    chart_background = chart_background,
    chart_border = chart_border,
    plot_background = plot_background,
    plot_border = plot_border,
    legend_position = legend_position,
    date_fmt = date_fmt,
    str_fmt = str_fmt,
    double_fmt = double_fmt,
    integer_fmt = integer_fmt,
    legend_x = legend_x,
    legend_y = legend_y,
    legend_w = legend_w,
    legend_h = legend_h
  )
  class(out) <- "mschart_theme"
  out
}

validate_legend_fraction <- function(value, name) {
  if (is.null(value)) {
    return(invisible(NULL))
  }
  if (!is.numeric(value) || length(value) != 1 || is.na(value) ||
      value < 0 || value > 1) {
    stop(
      "`", name,
      "` must be NULL or a single numeric value between 0 and 1.",
      call. = FALSE
    )
  }
}

#' @rdname set_theme
#' @return An `ms_chart` object (for `set_theme()` and `chart_theme()`).
#' @export
#' @description Use \code{chart_theme()} to modify components of the theme of a chart.
chart_theme <- function(
  x,
  axis_title_x,
  axis_title_y,
  main_title,
  legend_text,
  title_rot,
  title_x_rot,
  title_y_rot,
  axis_text_x,
  axis_text_y,
  axis_ticks_x,
  axis_ticks_y,
  grid_major_line_x,
  grid_major_line_y,
  grid_minor_line_x,
  grid_minor_line_y,
  chart_background,
  chart_border,
  plot_background,
  plot_border,
  date_fmt,
  str_fmt,
  double_fmt,
  integer_fmt,
  legend_position,
  legend_x,
  legend_y,
  legend_w,
  legend_h
) {
  if (!missing(axis_title_x)) {
    if (!all(class(axis_title_x) %in% class(x$theme$axis_title_x))) {
      stop("axis_title_x should be of class ", class(x$theme$axis_title_x))
    }
    x$theme$axis_title_x <- axis_title_x
  }

  if (!missing(axis_title_y)) {
    if (!all(class(axis_title_y) %in% class(x$theme$axis_title_y))) {
      stop("axis_title_y should be of class ", class(x$theme$axis_title_y))
    }
    x$theme$axis_title_y <- axis_title_y
  }

  if (!missing(legend_text)) {
    if (!all(class(legend_text) %in% class(x$theme$legend_text))) {
      stop("legend_text should be of class ", class(x$theme$legend_text))
    }
    x$theme$legend_text <- legend_text
  }

  if (!missing(main_title)) {
    if (!all(class(main_title) %in% class(x$theme$main_title))) {
      stop("main_title should be of class ", class(x$theme$main_title))
    }
    x$theme$main_title <- main_title
  }

  if (!missing(axis_text_x)) {
    if (!all(class(axis_text_x) %in% class(x$theme$axis_text_x))) {
      stop("axis_text_x should be of class ", class(x$theme$axis_text_x))
    }
    x$theme$axis_text_x <- axis_text_x
  }

  if (!missing(axis_text_y)) {
    if (!all(class(axis_text_y) %in% class(x$theme$axis_text_y))) {
      stop("axis_text_y should be of class ", class(x$theme$axis_text_y))
    }
    x$theme$axis_text_y <- axis_text_y
  }

  if (!missing(axis_ticks_x)) {
    if (!all(class(axis_ticks_x) %in% class(x$theme$axis_ticks_x))) {
      stop("axis_ticks_x should be of class ", class(x$theme$axis_ticks_x))
    }
    x$theme$axis_ticks_x <- axis_ticks_x
  }

  if (!missing(axis_ticks_y)) {
    if (!all(class(axis_ticks_y) %in% class(x$theme$axis_ticks_y))) {
      stop("axis_ticks_y should be of class ", class(x$theme$axis_ticks_y))
    }
    x$theme$axis_ticks_y <- axis_ticks_y
  }

  if (!missing(grid_major_line_x)) {
    assert_class(grid_major_line_x, class(x$theme$grid_major_line_x))
    x$theme$grid_major_line_x <- grid_major_line_x
  }

  if (!missing(grid_major_line_y)) {
    assert_class(grid_major_line_y, class(x$theme$grid_major_line_y))
    x$theme$grid_major_line_y <- grid_major_line_y
  }

  if (!missing(grid_minor_line_x)) {
    if (!all(class(grid_minor_line_x) %in% class(x$theme$grid_minor_line_x))) {
      stop(
        "grid_minor_line_x should be of class ",
        class(x$theme$grid_minor_line_x)
      )
    }
    x$theme$grid_minor_line_x <- grid_minor_line_x
  }

  if (!missing(grid_minor_line_y)) {
    if (!all(class(grid_minor_line_y) %in% class(x$theme$grid_minor_line_y))) {
      stop(
        "grid_minor_line_y should be of class ",
        class(x$theme$grid_minor_line_y)
      )
    }
    x$theme$grid_minor_line_y <- grid_minor_line_y
  }

  if (!missing(chart_border)) {
    if (!all(class(chart_border) %in% class(x$theme$chart_border))) {
      stop("chart_border should be of class ", class(x$theme$chart_border))
    }
    x$theme$chart_border <- chart_border
  }

  if (!missing(plot_border)) {
    if (!all(class(plot_border) %in% class(x$theme$plot_border))) {
      stop("plot_border should be of class ", class(x$theme$plot_border))
    }
    x$theme$plot_border <- plot_border
  }

  if (!missing(date_fmt)) {
    if (!all(class(date_fmt) %in% class(x$theme$date_fmt))) {
      stop("date_fmt should be of class ", class(x$theme$date_fmt))
    }
    x$theme$date_fmt <- date_fmt
  }

  if (!missing(str_fmt)) {
    if (!all(class(str_fmt) %in% class(x$theme$str_fmt))) {
      stop("str_fmt should be of class ", class(x$theme$str_fmt))
    }
    x$theme$str_fmt <- str_fmt
  }

  if (!missing(double_fmt)) {
    if (!all(class(double_fmt) %in% class(x$theme$double_fmt))) {
      stop("double_fmt should be of class ", class(x$theme$double_fmt))
    }
    x$theme$double_fmt <- double_fmt
  }

  if (!missing(integer_fmt)) {
    if (!all(class(integer_fmt) %in% class(x$theme$integer_fmt))) {
      stop("integer_fmt should be of class ", class(x$theme$integer_fmt))
    }
    x$theme$integer_fmt <- integer_fmt
  }

  if (!missing(legend_position)) {
    if (!legend_position %in% st_legendpos) {
      stop(
        "legend_position should be one of ",
        paste0(shQuote(st_legendpos), collapse = ", ")
      )
    }
    x$theme$legend_position <- legend_position
  }

  if (!missing(legend_x)) {
    validate_legend_fraction(legend_x, "legend_x")
    x$theme$legend_x <- legend_x
  }
  if (!missing(legend_y)) {
    validate_legend_fraction(legend_y, "legend_y")
    x$theme$legend_y <- legend_y
  }
  if (!missing(legend_w)) {
    validate_legend_fraction(legend_w, "legend_w")
    x$theme$legend_w <- legend_w
  }
  if (!missing(legend_h)) {
    validate_legend_fraction(legend_h, "legend_h")
    x$theme$legend_h <- legend_h
  }

  if (!missing(title_rot)) {
    if (title_rot < 0 && title_rot > 359) {
      stop("title_rot must be between 0 and 359")
    }
    x$theme$title_rot <- title_rot
  }

  if (!missing(title_x_rot)) {
    if (title_x_rot < 0 && title_x_rot > 359) {
      stop("title_x_rot must be between 0 and 359")
    }
    x$theme$title_x_rot <- title_x_rot
  }

  if (!missing(title_y_rot)) {
    if (title_y_rot < 0 && title_y_rot > 359) {
      stop("title_y_rot must be between 0 and 359")
    }
    x$theme$title_y_rot <- title_y_rot
  }

  x
}

cx_ns <- c(
  cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
  a = "http://schemas.openxmlformats.org/drawingml/2006/main"
)

test_that("axis title: xlab/ylab from chart_labels emit <cx:title> inside <cx:axis>", {
  dat <- data.frame(step = c("A", "B", "C"), amount = c(1, 2, 3))
  wf <- ms_waterfallchart(dat, x = "step", y = "amount")
  wf <- chart_labels(wf, xlab = "Stages", ylab = "Amount (USD)")
  doc <- xml2::read_xml(format(wf))

  x_title <- xml2::xml_find_first(
    doc,
    "//cx:axis[@id='0']/cx:title//a:t",
    cx_ns
  )
  expect_equal(xml2::xml_text(x_title), "Stages")

  y_title <- xml2::xml_find_first(
    doc,
    "//cx:axis[@id='1']/cx:title//a:t",
    cx_ns
  )
  expect_equal(xml2::xml_text(y_title), "Amount (USD)")
})

test_that("axis title: theme axis_title_x and _y are consumed", {
  dat <- data.frame(step = c("A", "B", "C"), amount = c(1, 2, 3))
  wf <- ms_waterfallchart(dat, x = "step", y = "amount")
  wf <- chart_labels(wf, xlab = "Stages", ylab = "Amount")
  th <- mschart_theme(
    axis_title_x = officer::fp_text(font.size = 14, bold = TRUE, color = "red"),
    axis_title_y = officer::fp_text(
      font.size = 12,
      italic = TRUE,
      color = "blue"
    )
  )
  wf <- set_theme(wf, th)
  doc <- xml2::read_xml(format(wf))

  x_rPr <- xml2::xml_find_first(
    doc,
    "//cx:axis[@id='0']/cx:title//a:r/a:rPr",
    cx_ns
  )
  expect_equal(xml2::xml_attr(x_rPr, "sz"), "1400")
  expect_equal(xml2::xml_attr(x_rPr, "b"), "1")
  expect_equal(
    xml2::xml_attr(xml2::xml_find_first(x_rPr, ".//a:srgbClr", cx_ns), "val"),
    "FF0000"
  )

  y_rPr <- xml2::xml_find_first(
    doc,
    "//cx:axis[@id='1']/cx:title//a:r/a:rPr",
    cx_ns
  )
  expect_equal(xml2::xml_attr(y_rPr, "sz"), "1200")
  expect_equal(xml2::xml_attr(y_rPr, "i"), "1")
})

test_that("axis title: NULL xlab/ylab emits no <cx:title> inside axis", {
  dat <- data.frame(step = c("A", "B"), amount = c(1, 2))
  wf <- ms_waterfallchart(dat, x = "step", y = "amount")
  doc <- xml2::read_xml(format(wf))
  axis_titles <- xml2::xml_find_all(doc, "//cx:axis/cx:title", cx_ns)
  expect_length(axis_titles, 0L)
})

test_that("axis title: funnel single axis honours xlab", {
  d <- data.frame(stage = c("A", "B"), count = c(100, 50))
  fn <- ms_funnelchart(d, x = "stage", y = "count")
  fn <- chart_labels(fn, xlab = "Funnel stage")
  doc <- xml2::read_xml(format(fn))
  t <- xml2::xml_find_first(doc, "//cx:axis[@id='1']/cx:title//a:t", cx_ns)
  expect_equal(xml2::xml_text(t), "Funnel stage")
})

test_that("axis title: pareto axes 0 and 1 honour xlab/ylab; axis 2 ignores", {
  d <- data.frame(defect = c("A", "B", "C"), n = c(10, 5, 3))
  pa <- ms_paretochart(d, x = "defect", y = "n", aggregate = FALSE)
  pa <- chart_labels(pa, xlab = "Defect type", ylab = "Count")
  doc <- xml2::read_xml(format(pa))
  expect_equal(
    xml2::xml_text(xml2::xml_find_first(
      doc,
      "//cx:axis[@id='0']/cx:title//a:t",
      cx_ns
    )),
    "Defect type"
  )
  expect_equal(
    xml2::xml_text(xml2::xml_find_first(
      doc,
      "//cx:axis[@id='1']/cx:title//a:t",
      cx_ns
    )),
    "Count"
  )
  # axis id=2 (percentage) has no title
  ax2_titles <- xml2::xml_find_all(doc, "//cx:axis[@id='2']/cx:title", cx_ns)
  expect_length(ax2_titles, 0L)
})

test_that("axis title: round-trip pptx still opens with axis titles", {
  skip_if_not_installed("officer")
  dat <- data.frame(step = c("A", "B", "C"), amount = c(10, -5, 8))
  wf <- ms_waterfallchart(dat, x = "step", y = "amount")
  wf <- chart_labels(wf, title = "T", xlab = "X axis", ylab = "Y axis")
  doc <- officer::read_pptx()
  doc <- officer::add_slide(doc, "Title and Content", "Office Theme")
  doc <- officer::ph_with(doc, wf, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

cx_ns <- c(
  cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
  a = "http://schemas.openxmlformats.org/drawingml/2006/main"
)

# --- title is styled by theme$main_title ----------------------------------

test_that("chart_theme: main_title font/color is consumed by treemap title", {
  dat <- data.frame(city = c("A", "B"), value = c(10, 20))
  tm <- ms_treemapchart(dat, path = "city", value = "value")
  tm <- chart_labels(tm, title = "My Title")
  th <- mschart_theme(
    main_title = officer::fp_text(font.size = 30, bold = TRUE, color = "red")
  )
  tm <- set_theme(tm, th)
  doc <- xml2::read_xml(format(tm))
  rPr <- xml2::xml_find_first(doc, "//cx:title//a:r/a:rPr", cx_ns)
  expect_equal(xml2::xml_attr(rPr, "sz"), "3000")
  expect_equal(xml2::xml_attr(rPr, "b"), "1")
  fill <- xml2::xml_find_first(rPr, ".//a:srgbClr", cx_ns)
  expect_equal(xml2::xml_attr(fill, "val"), "FF0000")
})

test_that("chart_theme: title default theme produces sz=2000, b=1 (mschart_theme default)", {
  dat <- data.frame(city = c("A", "B"), value = c(10, 20))
  tm <- ms_treemapchart(dat, path = "city", value = "value")
  tm <- chart_labels(tm, title = "Default")
  doc <- xml2::read_xml(format(tm))
  rPr <- xml2::xml_find_first(doc, "//cx:title//a:r/a:rPr", cx_ns)
  expect_equal(xml2::xml_attr(rPr, "sz"), "2000")
  expect_equal(xml2::xml_attr(rPr, "b"), "1")
})

test_that("chart_theme: title is omitted entirely when no title is set", {
  dat <- data.frame(city = c("A", "B"), value = c(10, 20))
  tm <- ms_treemapchart(dat, path = "city", value = "value")
  doc <- xml2::read_xml(format(tm))
  expect_length(xml2::xml_find_all(doc, "//cx:title", cx_ns), 0L)
})

# --- axis tick labels are styled by theme$axis_text_x / _y ---------------

test_that("chart_theme: axis_text_x font is consumed by waterfall x axis", {
  dat <- data.frame(step = c("A", "B", "C"), amount = c(1, 2, 3))
  wf <- ms_waterfallchart(dat, x = "step", y = "amount")
  th <- mschart_theme(
    axis_text_x = officer::fp_text(font.size = 9, italic = TRUE, color = "blue")
  )
  wf <- set_theme(wf, th)
  doc <- xml2::read_xml(format(wf))
  defRPr <- xml2::xml_find_first(
    doc,
    "//cx:axis[@id='0']/cx:txPr//a:defRPr",
    cx_ns
  )
  expect_equal(xml2::xml_attr(defRPr, "sz"), "900")
  expect_equal(xml2::xml_attr(defRPr, "i"), "1")
  fill <- xml2::xml_find_first(defRPr, ".//a:srgbClr", cx_ns)
  expect_equal(xml2::xml_attr(fill, "val"), "0000FF")
})

test_that("chart_theme: axis_text_y font is consumed by waterfall y axis", {
  dat <- data.frame(step = c("A", "B", "C"), amount = c(1, 2, 3))
  wf <- ms_waterfallchart(dat, x = "step", y = "amount")
  th <- mschart_theme(
    axis_text_y = officer::fp_text(font.size = 11, bold = TRUE)
  )
  wf <- set_theme(wf, th)
  doc <- xml2::read_xml(format(wf))
  defRPr <- xml2::xml_find_first(
    doc,
    "//cx:axis[@id='1']/cx:txPr//a:defRPr",
    cx_ns
  )
  expect_equal(xml2::xml_attr(defRPr, "sz"), "1100")
  expect_equal(xml2::xml_attr(defRPr, "b"), "1")
})

test_that("chart_theme: pareto axis 2 (percentage) inherits axis_text_y", {
  d <- data.frame(defect = c("A", "B", "C"), n = c(10, 5, 3))
  pa <- ms_paretochart(d, x = "defect", y = "n", aggregate = FALSE)
  th <- mschart_theme(
    axis_text_y = officer::fp_text(font.size = 8, color = "purple")
  )
  pa <- set_theme(pa, th)
  doc <- xml2::read_xml(format(pa))
  defRPr2 <- xml2::xml_find_first(
    doc,
    "//cx:axis[@id='2']/cx:txPr//a:defRPr",
    cx_ns
  )
  expect_equal(xml2::xml_attr(defRPr2, "sz"), "800")
  fill <- xml2::xml_find_first(defRPr2, ".//a:srgbClr", cx_ns)
  expect_equal(xml2::xml_attr(fill, "val"), "A020F0")
})

test_that("chart_theme: funnel single axis id=1 uses axis_text_x", {
  d <- data.frame(stage = c("A", "B"), count = c(100, 50))
  fn <- ms_funnelchart(d, x = "stage", y = "count")
  th <- mschart_theme(axis_text_x = officer::fp_text(font.size = 7))
  fn <- set_theme(fn, th)
  doc <- xml2::read_xml(format(fn))
  defRPr <- xml2::xml_find_first(
    doc,
    "//cx:axis[@id='1']/cx:txPr//a:defRPr",
    cx_ns
  )
  expect_equal(xml2::xml_attr(defRPr, "sz"), "700")
})

# --- round-trip pptx ------------------------------------------------------

test_that("chart_theme: round-trip pptx still opens with custom title+axes theme", {
  skip_if_not_installed("officer")
  dat <- data.frame(step = c("A", "B", "C"), amount = c(10, -5, 8))
  wf <- ms_waterfallchart(dat, x = "step", y = "amount")
  wf <- chart_labels(wf, title = "Themed")
  th <- mschart_theme(
    main_title = officer::fp_text(
      font.size = 28,
      bold = TRUE,
      color = "darkgreen"
    ),
    axis_text_x = officer::fp_text(font.size = 9, italic = TRUE),
    axis_text_y = officer::fp_text(font.size = 9, color = "gray40")
  )
  wf <- set_theme(wf, th)
  doc <- officer::read_pptx()
  doc <- officer::add_slide(doc, "Title and Content", "Office Theme")
  doc <- officer::ph_with(doc, wf, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

#' Apply ggplot2 theme
#'
#' A theme that approximates the style of ggplot2::theme_grey.
#'
#' @param x a mschart object
#' @param base_size base font size
#' @param base_family font family
#'
#' @return a mschart object
#'
#' @export
#'
#' @section theme_ggplot2():
#'
#' \if{html}{\figure{fig_theme_ggplot2.png}{options: width="500"}}
#'
#' @examples
#' p <- ms_scatterchart(
#'   data = iris, x = "Sepal.Length",
#'   y = "Sepal.Width", group = "Species"
#' )
#'
#' p <- theme_ggplot2(p)
#' p <- chart_fill_ggplot2(p)
theme_ggplot2 <- function(x, base_size = 11, base_family = "Arial") {
  t <- mschart_theme(
    main_title = fp_text(
      color = "black",
      font.size = 1.2 * base_size,
      font.family = base_family
    ),
    axis_title = fp_text(
      color = "black",
      font.size = base_size,
      font.family = base_family
    ),
    axis_text = fp_text(
      color = "grey30",
      font.size = 0.8 * base_size,
      font.family = base_family
    ),
    axis_ticks = fp_border(color = "grey20", width = 1, style = "solid"),
    grid_major_line_x = fp_border(color = "white", width = 1, style = "solid"),
    grid_major_line_y = fp_border(color = "white", width = 1, style = "solid"),
    grid_minor_line_x = fp_border(
      color = "white",
      width = 0.5,
      style = "solid"
    ),
    grid_minor_line_y = fp_border(
      color = "white",
      width = 0.5,
      style = "solid"
    ),
    chart_background = "white",
    plot_background = "grey92",
    legend_text = fp_text(
      color = "black",
      font.size = base_size,
      font.family = base_family
    ),
    legend_position = "r"
  )
  set_theme(x, t)
}

#' Apply ggplot2 color scale
#'
#' The default hue color scale from ggplot2.
#'
#' @param x a mschart object
#' @param stroke a boolean. Apply the color scale to stroke? Defaults to `TRUE`.
#'
#' @return a mschart object
#'
#' @export
#'
#' @section chart_fill_ggplot2():
#'
#' \if{html}{\figure{fig_theme_ggplot2.png}{options: width="500"}}
#'
#' @examples
#' p <- ms_scatterchart(
#'   data = iris, x = "Sepal.Length",
#'   y = "Sepal.Width", group = "Species"
#' )
#'
#' p <- theme_ggplot2(p)
#' p <- chart_fill_ggplot2(p)
#' @importFrom scales hue_pal
chart_fill_ggplot2 <- function(x, stroke = TRUE) {
  if (!is.null(x$group)) {
    groups <- unique(x$data[[x$group]])
    ngroups <- length(groups)
    pal <- hue_pal()(ngroups)
    names(pal) <- groups
  } else {
    pal <- hue_pal()(1)
  }

  x <- chart_data_fill(x, values = pal)
  if (stroke) {
    x <- chart_data_stroke(x, values = pal)
  }
  x
}

pml_chart <- function(x, value, id_x, id_y) {
  charts_dir <- file.path(x$package_dir, "ppt/charts")
  xlsx_dir <- file.path(x$package_dir, "ppt/embeddings")

  dir.create(charts_dir, recursive = TRUE, showWarnings = FALSE)
  dir.create(
    file.path(charts_dir, "_rels"),
    recursive = TRUE,
    showWarnings = FALSE
  )
  dir.create(xlsx_dir, recursive = TRUE, showWarnings = FALSE)

  chart_file <- tempfile(
    tmpdir = charts_dir,
    pattern = "chart",
    fileext = ".xml"
  )
  xlsx_file <- tempfile(tmpdir = xlsx_dir, pattern = "data", fileext = ".xlsx")
  rel_filename <- file.path(
    charts_dir,
    "_rels",
    paste0(basename(chart_file), ".rels")
  )

  rel_str <- paste0(
    "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>",
    "<Relationships  xmlns=\"http://schemas.openxmlformats.org/package/2006/relationships\"><Relationship Id=\"rId1\" Type=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships/package\" Target=\"../embeddings/%s\"/></Relationships>"
  )
  rel_str <- sprintf(rel_str, basename(xlsx_file))
  writeLines(rel_str, rel_filename, useBytes = TRUE)

  write_xlsx(x = list("sheet1" = value$data_series), path = xlsx_file)
  xml_elt <- format(value, id_x = id_x, id_y = id_y)
  writeLines(xml_elt, chart_file, useBytes = TRUE)

  slide <- x$slide$get_slide(x$cursor)
  next_id <- slide$relationship()$get_next_id()
  slide$relationship()$add(
    paste0("rId", next_id),
    type = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/chart",
    target = paste0("../charts/", basename(chart_file))
  )

  reference_ <- "<c:chart xmlns:c=\"http://schemas.openxmlformats.org/drawingml/2006/chart\" xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\" r:id=\"rId%.0f\"/>"
  reference_ <- sprintf(reference_, next_id)
  graphic_frame <- paste0(
    "<p:graphicFrame ",
    "xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\" ",
    "xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\" ",
    "xmlns:p=\"http://schemas.openxmlformats.org/presentationml/2006/main\">",
    "<p:nvGraphicFramePr>",
    "<p:cNvPr id=\"\" name=\"\"/>",
    "<p:cNvGraphicFramePr><a:graphicFrameLocks noGrp=\"true\"/></p:cNvGraphicFramePr>",
    "<p:nvPr/>",
    "</p:nvGraphicFramePr>",
    "<p:xfrm rot=\"0\">",
    "<a:off x=\"0\" y=\"0\"/>",
    "<a:ext cx=\"0\" cy=\"0\"/>",
    "</p:xfrm>",
    "<a:graphic>",
    "<a:graphicData uri=\"http://schemas.openxmlformats.org/drawingml/2006/chart\">",
    reference_,
    "</a:graphicData>",
    "</a:graphic>",
    "</p:graphicFrame>"
  )

  partname <- file.path("/ppt/charts", basename(chart_file))
  override <- setNames(
    "application/vnd.openxmlformats-officedocument.drawingml.chart+xml",
    partname
  )
  x$content_type$add_override(value = override)
  graphic_frame
}

#' @importFrom stats setNames
#' @importFrom writexl write_xlsx
#' @importFrom xml2 read_xml xml_find_first xml_replace as_xml_document xml_add_child write_xml
#' @importFrom officer ph_with
#' @importFrom xml2 as_xml_document
#' @export
#' @method ph_with ms_chart
#' @title Add a MS Chart output into a PowerPoint object
#' @description Produces a Microsoft Chart graphics output from R instructions
#' and adds the result in a PowerPoint document object produced
#' by [officer::read_pptx()].
#' @param x a pptx device
#' @param value chart object
#' @param location a location for a placeholder.
#' @param ... Arguments to be passed to methods.
#' @return An rpptx object.
#' @seealso [body_add_chart()]
#' @examples
#' my_barchart <- ms_barchart(data = browser_data,
#'   x = "browser", y = "value", group = "serie")
#' my_barchart <- chart_settings( x = my_barchart,
#'   dir="vertical", grouping="clustered", gap_width = 50 )
#' my_barchart <- chart_ax_x( x= my_barchart,
#'   cross_between = 'between', major_tick_mark="out")
#' my_barchart <- chart_ax_y( x= my_barchart,
#'   cross_between = "midCat", major_tick_mark="in")
#'
#' library(officer)
#' doc <- read_pptx()
#' doc <- add_slide(doc, "Title and Content", "Office Theme")
#' doc <- ph_with(doc, my_barchart, location = ph_location_fullsize())
#'
#' fileout <- tempfile(fileext = ".pptx")
#' print(doc, target = fileout)
ph_with.ms_chart <- function(x, value, location, ...) {
  stopifnot(inherits(x, "rpptx"))
  graphic_frame <- pml_chart(x, value, id_x = "64451712", id_y = "64453248")
  x$content_type$add_ext(
    extension = "xlsx",
    type = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
  )
  ph_with(
    x = x,
    value = as_xml_document(graphic_frame),
    location = location,
    ...
  )
}

# utils/tools ----
get_series_names <- function(x) {
  names_ <- character(0)
  if (!is.null(x$group)) {
    data_grps <- x$data[[x$group]]
    if (is.factor(data_grps)) {
      names_ <- levels(data_grps)
    } else if (is.character(data_grps)) {
      names_ <- sort(unique(data_grps))
    } else {
      names_ <- as.character(sort(unique(data_grps)))
    }
  } else {
    names_ <- x$y
  }
  names_
}

get_label_names <- function(x) {
  if (is.null(x$label_cols)) {
    return(character(0))
  }

  names_ <- character(0)
  if (!is.null(x$group)) {
    data_grps <- x$data[[x$group]]
    if (is.factor(data_grps)) {
      names_ <- levels(data_grps)
    } else if (is.character(data_grps)) {
      names_ <- sort(unique(data_grps))
    } else {
      names_ <- as.character(sort(unique(data_grps)))
    }
    names_ <- paste0(x$label_cols[1], "-", names_)
  } else {
    names_ <- x$label_cols[1]
  }
}

# 2 different transpose ops -----
#' @importFrom data.table setorderv rbindlist setDF
transpose_data <- function(data, vars, group) {
  list_data <- split(data[vars], data[[group]])
  group_names <- names(list_data)
  list_data <- mapply(
    function(name, x) {
      colnames(x)[2] <- name
      x
    },
    name = group_names,
    x = list_data,
    SIMPLIFY = FALSE
  )
  out <- rbindlist(list_data, use.names = TRUE, fill = TRUE)
  setDF(out)
  out
}


#' @importFrom data.table := .N
dcast_data <- function(data, x, y, group) {
  dataset <- as.data.table(data)
  dataset <- dataset[, c(".fake_id.") := list(seq_len(.N)), by = c(x, group)]
  form_str <- sprintf("%s + .fake_id. ~ %s", x, group)
  out <- dcast.data.table(
    dataset,
    formula = as.formula(form_str),
    fill = NA,
    value.var = y
  )
  out$.fake_id. <- NULL
  setDF(out)
  out
}

# main functions -----
transpose_series_bysplit <- function(x) {
  if (!is.null(x$group)) {
    vars <- c(x$x, x$y)
    out <- transpose_data(x$data, vars, x$group)
    group_names <- setdiff(colnames(out), x$x)
    if (!is.null(x$size_cols)) {
      for (sz in x$size_cols) {
        data_sz <- transpose_data(x$data, c(x$x, sz), x$group)
        data_sz[[1]] <- NULL
        names(data_sz) <- paste0(sz, "-", names(data_sz))
        out <- cbind(out, data_sz)
      }
    }
    if (!is.null(x$label_cols)) {
      for (lab in x$label_cols) {
        data_label <- transpose_data(x$data, c(x$x, lab), x$group)
        data_label[[1]] <- NULL
        names(data_label) <- paste0(lab, "-", names(data_label))
        out <- cbind(out, data_label)
      }
    }
  } else {
    vars <- c(x$x, x$y, x$size_cols, x$label_cols)
    out <- x$data[, vars]
  }
  out
}


#' @importFrom stats as.formula
#' @importFrom data.table as.data.table dcast.data.table setorderv setnames
shape_as_series <- function(x) {
  if (!is.null(x$group)) {
    out <- dcast_data(data = x$data, x = x$x, y = x$y, group = x$group)
    if (!is.null(x$label_cols)) {
      for (lab in x$label_cols) {
        data_label <- dcast_data(
          data = x$data,
          x = x$x,
          y = lab,
          group = x$group
        )
        data_label[[1]] <- NULL
        names(data_label) <- paste0(lab, "-", names(data_label))
        out <- cbind(out, data_label)
      }
    }
  } else {
    out <- x$data[, c(x$x, x$y, x$label_cols)]
    # if( !is.null(x$label_cols)){
    #   wlabs <- which(names(out) %in% x$label_cols)
    #   names(out)[wlabs] <- paste0(label_cols, "-", names(out)[wlabs])
    # }
  }
  as.data.frame(out)
}

#' @title xml code for table in chart
#' @param x an object of class table_options
#' @noRd
table_content_xml <- function(x) {
  x_horizontal_id <- ifelse(x$x_table$horizontal, 1, 0)
  x_vertical_id <- ifelse(x$x_table$vertical, 1, 0)
  x_outline_id <- ifelse(x$x_table$outline, 1, 0)
  x_show_keys_id <- ifelse(x$x_table$show_keys, 1, 0)

  txpr <- ""
  if (!is.null(x$labels_fp)) {
    txpr <- ooxml_txpr(x$labels_fp)
  } else {
    txpr <- ooxml_txpr(x$theme$table_text)
  }

  if (x$options$table) {
    table_str <- paste0(
      "<c:dTable>",
      sprintf("<c:showHorzBorder val=\"%s\"/>", x_horizontal_id),
      sprintf("<c:showVertBorder val=\"%s\"/>", x_vertical_id),
      sprintf("<c:showOutline val=\"%s\"/>", x_outline_id),
      sprintf("<c:showKeys val=\"%s\"/>", x_show_keys_id),
      txpr,
      "</c:dTable>"
    )
  } else {
    table_str <- NULL
  }
  table_str
}

# axes xml utils ----

#' @noRd
#' @title find the tags corresponding to the type of x
#' that will be used as axis tag in the chart
#' @param x a vector used as axis
get_axis_tag <- function(x) {
  if (inherits(x, c("POSIXt", "Date"))) {
    axis_tag <- "c:dateAx"
  } else if (is.factor(x) || is.character(x)) {
    axis_tag <- "c:catAx"
  } else if (is.numeric(x)) {
    axis_tag <- "c:valAx"
  } else {
    stop("unknow type of data")
  }
  axis_tag
}


#' @noRd
#' @title xml code for an axis
#' @param x an object of class axis_options
#' @param id to be doc
#' @param cross_id to be doc
#' @param theme an object of class mschart_theme
#' @param is_x if TRUE, generate xml for x axis, else for y axis
#' @param lab label for the axis
#' @param rot rotation of title
axis_content_xml <- function(
  x,
  id,
  cross_id,
  theme,
  is_x = TRUE,
  lab = NULL,
  rot = 0
) {
  x_title_id <- paste0("axis_title_", ifelse(is_x, "x", "y"))

  if (is.null(lab)) {
    title_ <- ""
  } else {
    title_ <- "<c:title><c:tx><c:rich><a:bodyPr rot=\"%.0f\" vert=\"horz\" anchor=\"ctr\"/><a:lstStyle/><a:p><a:pPr><a:defRPr/></a:pPr><a:r>%s<a:t>%s</a:t></a:r></a:p></c:rich></c:tx><c:layout/><c:overlay val=\"0\"/></c:title>"
    title_ <- sprintf(
      title_,
      rot * 60000,
      format(theme[[x_title_id]], type = "pml"),
      lab
    )
  }

  major_tm <- "<c:majorTickMark val=\"%s\"/>"
  major_tm <- sprintf(major_tm, x$major_tick_mark)

  minor_tm <- "<c:minorTickMark val=\"%s\"/>"
  minor_tm <- sprintf(minor_tm, x$minor_tick_mark)

  grid_major_id <- paste0("grid_major_line_", ifelse(is_x, "x", "y"))
  major_gl <- ""
  if (!isFALSE(theme[[grid_major_id]])) {
    major_gl <- ooxml_fp_border(
      theme[[grid_major_id]],
      in_tags = c("c:majorGridlines", "c:spPr")
    )
  }

  grid_minor_id <- paste0("grid_minor_line_", ifelse(is_x, "x", "y"))
  minor_gl <- ""
  if (!isFALSE(theme[[grid_minor_id]])) {
    minor_gl <- ooxml_fp_border(
      theme[[grid_minor_id]],
      in_tags = c("c:minorGridlines", "c:spPr")
    )
  }

  lim_max <- ""
  if (!is.null(x$limit_max)) {
    lim_max <- sprintf("<c:max val=\"%.02f\"/>", x$limit_max)
  }
  lim_min <- ""
  if (!is.null(x$limit_min)) {
    lim_min <- sprintf("<c:min val=\"%.02f\"/>", x$limit_min)
  }

  scaling_str <- sprintf(
    "<c:scaling><c:orientation val=\"%s\"/>%s%s</c:scaling>",
    x$orientation,
    lim_max,
    lim_min
  )
  delete <- sprintf("<c:delete val=\"%.0f\"/>", x$delete)
  position <- sprintf("<c:axPos val=\"%s\"/>", x$axis_position)
  crosses <- sprintf("<c:crosses val=\"%s\"/>", x$crosses)

  lim_max <- ""
  if (!is.null(x$limit_max)) {
    lim_max <- sprintf("<c:max val=\"%.02f\"/>", x$limit_max)
  }
  lim_min <- ""
  if (!is.null(x$limit_min)) {
    lim_min <- sprintf("<c:min val=\"%.02f\"/>", x$limit_min)
  }
  cross_at <- ""
  if (!is.null(x$position)) {
    cross_at <- sprintf("<c:crossesAt val=\"%.02f\"/>", x$position)
    crosses <- ""
  }

  num_fmt <- ""
  if (!is.null(x$num_fmt)) {
    num_fmt <- sprintf(
      "<c:numFmt formatCode=\"%s\" sourceLinked=\"0\"/>",
      x$num_fmt
    )
  }

  tl_pos <- ""
  tl_pos <- "<c:tickLblPos val=\"%s\"/>"
  tl_pos <- sprintf(tl_pos, x$tick_label_pos)

  axis_major_ticks_id <- paste0("axis_ticks_", ifelse(is_x, "x", "y"))
  axis_ticks <- ooxml_fp_border(
    theme[[axis_major_ticks_id]],
    in_tags = c("c:spPr")
  )

  labels_text_id <- paste0("axis_text_", ifelse(is_x, "x", "y"))
  rpr <- format(theme[[labels_text_id]], type = "pml")
  rpr <- gsub("a:rPr", "a:defRPr", rpr)
  labels_text_pr <- "<c:txPr><a:bodyPr rot=\"%.0f\" vert=\"horz\"/><a:lstStyle/><a:p><a:pPr>%s</a:pPr></a:p></c:txPr>"
  labels_text_pr <- sprintf(labels_text_pr, x$rotation * 60000, rpr)

  major_unit_str <- ""
  if (!is.null(x$major_unit)) {
    major_unit_str <- sprintf("<c:majorUnit val=\"%s\"/>", x$major_unit)
  }
  minor_unit_str <- ""
  if (!is.null(x$minor_unit)) {
    minor_unit_str <- sprintf("<c:minorUnit val=\"%s\"/>", x$minor_unit)
  }
  major_time_unit_str <- ""
  if (!is.null(x$major_time_unit)) {
    major_time_unit_str <- sprintf(
      "<c:majorTimeUnit val=\"%s\"/>",
      x$major_time_unit
    )
  }
  minor_time_unit_str <- ""
  if (!is.null(x$minor_time_unit)) {
    minor_time_unit_str <- sprintf(
      "<c:minorTimeUnit val=\"%s\"/>",
      x$minor_time_unit
    )
  }

  str_ <- paste0(
    sprintf("<c:axId val=\"%s\"/>", id),
    scaling_str,
    delete,
    position,
    major_gl,
    minor_gl,
    title_,
    major_tm,
    minor_tm,
    tl_pos,
    labels_text_pr,
    axis_ticks,
    num_fmt,
    sprintf("<c:crossAx val=\"%s\"/>", cross_id),
    cross_at,
    crosses,
    major_unit_str,
    minor_unit_str,
    major_time_unit_str,
    minor_time_unit_str
  )
  str_
}

#' @noRd
axis_content_xml_radar <- function(x, id, cross_id, is_x = TRUE, ...) {
  cross_between <- ""
  if (!is_x) {
    cross_between <- "<c:crossBetween val=\"between\"/>"
  }
  paste0(
    sprintf("<c:axId val=\"%s\"/>", id),
    sprintf(
      "<c:scaling><c:orientation val=\"%s\"/></c:scaling>",
      x$orientation
    ),
    sprintf("<c:delete val=\"%.0f\"/>", x$delete),
    sprintf("<c:axPos val=\"%s\"/>", x$axis_position),
    "<c:majorTickMark val=\"none\"/>",
    "<c:minorTickMark val=\"none\"/>",
    "<c:tickLblPos val=\"nextTo\"/>",
    sprintf("<c:crossAx val=\"%s\"/>", cross_id),
    "<c:crosses val=\"autoZero\"/>",
    cross_between
  )
}

#' @importFrom officer to_pml
str_ref <- function(values, region = NULL, num_fmt = NULL) {
  x <- list(
    region = region,
    values = values,
    num_fmt = num_fmt
  )
  class(x) <- c("col_ref", "str_ref")
  x
}

#' @importFrom stats update
update.col_ref <- function(
  object,
  values = NULL,
  region = NULL,
  num_fmt = NULL,
  ...
) {
  if (!is.null(values)) {
    object$values <- values
  }
  if (!is.null(region)) {
    object$region <- region
  }
  if (!is.null(num_fmt)) {
    object$num_fmt <- num_fmt
  }
  object
}


#' @export
#' @method to_pml str_ref
to_pml.str_ref <- function(x, add_ns = FALSE, ...) {
  pt_ <- "<c:pt idx=\"%.0f\"><c:v>%s</c:v></c:pt>"
  values <- character(length(x$values))

  if (is.factor(x$values)) {
    values <- htmlEscape(as.character(x$values))
  } else if (is.numeric(x$values)) {
    values <- as.character(x$values)
  } else if (is.character(x$values)) {
    values <- htmlEscape(x$values)
  }

  pt_ <- sprintf(pt_, seq_along(values) - 1, values)
  pt_ <- paste0(pt_, collapse = "")

  num_fmt <- ""
  if (!is.null(x$num_fmt)) {
    num_fmt <- sprintf("<c:formatCode>%s</c:formatCode>", x$num_fmt)
  }
  pml_ <- "<c:strRef><c:f>%s</c:f><c:strCache>%s<c:ptCount val=\"%.0f\"/>%s</c:strCache></c:strRef>"

  sprintf(pml_, x$region, num_fmt, length(values), pt_)
}


num_ref <- function(values, region = NULL, num_fmt = NULL) {
  x <- list(
    region = region,
    values = values,
    num_fmt = num_fmt
  )
  class(x) <- c("col_ref", "num_ref")
  x
}

#' @export
#' @method to_pml num_ref
to_pml.num_ref <- function(x, add_ns = FALSE, ...) {
  pt_ <- "<c:pt idx=\"%.0f\"><c:v>%s</c:v></c:pt>"
  values <- character(length(x$values))
  if (inherits(x$values, "Date")) {
    values <- as.integer(x$values - as.Date("1900-01-01") - 2)
  } else {
    values <- format(
      x$values,
      trim = TRUE,
      scientific = FALSE,
      big.mark = "",
      decimal.mark = "."
    )
  }

  pt_ <- sprintf(pt_, seq_along(values) - 1, values)
  pt_ <- paste0(pt_[!is.na(x$values)], collapse = "")
  num_fmt <- ""
  if (!is.null(x$num_fmt)) {
    num_fmt <- sprintf("<c:formatCode>%s</c:formatCode>", x$num_fmt)
  }
  pml_ <- "<c:numRef><c:f>%s</c:f><c:numCache>%s<c:ptCount val=\"%.0f\"/>%s</c:numCache></c:numRef>"
  sprintf(pml_, x$region, num_fmt, length(values), pt_)
}

date_ref <- function(values, region = NULL, num_fmt = NULL) {
  x <- list(
    region = region,
    values = values,
    num_fmt = num_fmt
  )
  class(x) <- c("col_ref", "date_ref")
  x
}

#' @export
#' @method to_pml date_ref
to_pml.date_ref <- function(x, add_ns = FALSE, ...) {
  pt_ <- "<c:pt idx=\"%.0f\"><c:v>%.0f</c:v></c:pt>"
  values <- as.integer(x$values - as.Date("1899-12-30"))
  pt_ <- sprintf(pt_, seq_along(values) - 1, values)
  pt_ <- paste0(pt_[!is.na(values)], collapse = "")
  num_fmt <- sprintf("<c:formatCode>%s</c:formatCode>", "yyyy\\-mm\\-dd")
  pml_ <- "<c:numRef><c:f>%s</c:f><c:numCache>%s<c:ptCount val=\"%.0f\"/>%s</c:numCache></c:numRef>"
  sprintf(pml_, x$region, num_fmt, length(values), pt_)
}

label_ref <- function(values, region = NULL, num_fmt = NULL) {
  x <- list(
    region = region,
    values = values,
    num_fmt = num_fmt
  )
  class(x) <- c("col_ref", "label_ref")
  x
}

#' @export
#' @method to_pml label_ref
to_pml.label_ref <- function(x, add_ns = FALSE, ...) {
  values <- character(length(x$values))

  if (is.factor(x$values)) {
    values <- htmlEscape(as.character(x$values))
  } else if (is.numeric(x$values)) {
    values <- as.character(x$values)
  } else if (is.character(x$values)) {
    values <- htmlEscape(x$values)
  } else {
    values <- htmlEscape(format(x$values))
  }

  values[is.na(x$values)] <- ""

  pt_ <- sprintf(
    "<c:pt idx=\"%.0f\"><c:v>%s</c:v></c:pt>",
    seq_along(values) - 1,
    values
  )
  pt_ <- paste0(pt_, collapse = "")

  num_fmt <- ""
  pml_ <- "<c15:datalabelsRange><c15:f>%s</c15:f><c15:dlblRangeCache>%s<c:ptCount val=\"%.0f\"/>%s</c15:dlblRangeCache></c15:datalabelsRange>"
  pml_ <- paste0(
    "<c:extLst>",
    "<c:ext uri=\"{02D57815-91ED-43cb-92C2-25804820EDAC}\" xmlns:c15=\"http://schemas.microsoft.com/office/drawing/2012/chart\">",
    pml_,
    "</c:ext></c:extLst>"
  )

  sprintf(pml_, x$region, num_fmt, length(values), pt_)
}

# Base helpers for chartEx (cx:) charts -- a parallel pipeline to the
# classic c: charts. ms_chart_ex inherits from ms_chart so user-facing
# styling setters that operate on $theme / $labels keep working.

CX_NS <- paste(
  "xmlns:cx=\"http://schemas.microsoft.com/office/drawing/2014/chartex\"",
  "xmlns:mc=\"http://schemas.openxmlformats.org/markup-compatibility/2006\"",
  "xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\""
)

# Per-class plumbing for ph_with.ms_chart and body_add_chart.

chart_part_content_type <- function(x) UseMethod("chart_part_content_type")
#' @exportS3Method
chart_part_content_type.ms_chart <- function(x) {
  "application/vnd.openxmlformats-officedocument.drawingml.chart+xml"
}
#' @exportS3Method
chart_part_content_type.ms_chart_ex <- function(x) {
  "application/vnd.ms-office.chartex+xml"
}

chart_part_rel_type <- function(x) UseMethod("chart_part_rel_type")
#' @exportS3Method
chart_part_rel_type.ms_chart <- function(x) {
  "http://schemas.openxmlformats.org/officeDocument/2006/relationships/chart"
}
#' @exportS3Method
chart_part_rel_type.ms_chart_ex <- function(x) {
  "http://schemas.microsoft.com/office/2014/relationships/chartEx"
}

chart_graphicdata_uri <- function(x) UseMethod("chart_graphicdata_uri")
#' @exportS3Method
chart_graphicdata_uri.ms_chart <- function(x) {
  "http://schemas.openxmlformats.org/drawingml/2006/chart"
}
#' @exportS3Method
chart_graphicdata_uri.ms_chart_ex <- function(x) {
  "http://schemas.microsoft.com/office/drawing/2014/chartex"
}

chart_reference_xml <- function(x, rel_id) UseMethod("chart_reference_xml")
#' @exportS3Method
chart_reference_xml.ms_chart <- function(x, rel_id) {
  sprintf(
    paste0(
      "<c:chart xmlns:c=\"http://schemas.openxmlformats.org/drawingml/2006/chart\"",
      " xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\"",
      " r:id=\"rId%.0f\"/>"
    ),
    rel_id
  )
}
# Sidecar parts (chart style + colors). chartEx requires both:
# without them PowerPoint cannot render the new layouts (treemap, sunburst,
# funnel, ...) and flags the file as corrupt. Classic c: charts are fine
# without sidecars so the default method writes nothing.
chart_extra_parts <- function(
  value,
  charts_dir,
  content_type,
  part_root = "/ppt/charts"
) {
  UseMethod("chart_extra_parts")
}
#' @exportS3Method
chart_extra_parts.ms_chart <- function(
  value,
  charts_dir,
  content_type,
  part_root = "/ppt/charts"
) {
  ""
}
#' @exportS3Method
chart_extra_parts.ms_chart_ex <- function(
  value,
  charts_dir,
  content_type,
  part_root = "/ppt/charts"
) {
  style_file <- tempfile(
    tmpdir = charts_dir,
    pattern = "style",
    fileext = ".xml"
  )
  colors_file <- tempfile(
    tmpdir = charts_dir,
    pattern = "colors",
    fileext = ".xml"
  )
  file.copy(
    system.file(package = "mschart", "template", "chartex_style.xml"),
    style_file,
    overwrite = TRUE
  )
  file.copy(
    system.file(package = "mschart", "template", "chartex_colors.xml"),
    colors_file,
    overwrite = TRUE
  )
  content_type$add_override(
    value = setNames(
      "application/vnd.ms-office.chartstyle+xml",
      file.path(part_root, basename(style_file))
    )
  )
  content_type$add_override(
    value = setNames(
      "application/vnd.ms-office.chartcolorstyle+xml",
      file.path(part_root, basename(colors_file))
    )
  )
  paste0(
    sprintf(
      "<Relationship Id=\"rId2\" Type=\"http://schemas.microsoft.com/office/2011/relationships/chartStyle\" Target=\"%s\"/>",
      basename(style_file)
    ),
    sprintf(
      "<Relationship Id=\"rId3\" Type=\"http://schemas.microsoft.com/office/2011/relationships/chartColorStyle\" Target=\"%s\"/>",
      basename(colors_file)
    )
  )
}

#' @exportS3Method
chart_reference_xml.ms_chart_ex <- function(x, rel_id) {
  sprintf(
    paste0(
      "<cx:chart xmlns:cx=\"http://schemas.microsoft.com/office/drawing/2014/chartex\"",
      " xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\"",
      " r:id=\"rId%.0f\"/>"
    ),
    rel_id
  )
}

# Build the full chartEx XML for a hierarchical chart (treemap, sunburst).
# Both layouts use the same data shape: multi-level <cx:strDim type="cat">
# (leaf-first) + <cx:numDim type="size">. Differences are layoutId and
# layout-specific <cx:layoutPr> children.
cx_format_hierarchical <- function(
  x,
  layout_id,
  sheetname,
  drop_ext_data,
  series_layout_pr = ""
) {
  ds <- x$data_series
  n <- nrow(ds)
  path <- x$path
  value <- x$value

  path_cols <- xl_col_letter(seq_along(path))
  value_col <- xl_col_letter(length(path) + 1L)
  row1 <- 2L
  row2 <- n + 1L

  # Office orders levels leaf-first (lvl[0] = leaf, lvl[n-1] = root).
  cat_levels_xml <- paste0(
    vapply(
      rev(seq_along(path)),
      function(i) {
        cx_str_lvl(ds[[path[i]]])
      },
      character(1)
    ),
    collapse = ""
  )
  cat_range <- if (length(path) == 1L) {
    cx_range(sheetname, path_cols[1], row1, row2)
  } else {
    sprintf(
      "%s!$%s$%d:$%s$%d",
      sheetname,
      path_cols[1],
      row1,
      path_cols[length(path)],
      row2
    )
  }
  cat_dim <- paste0(
    "<cx:strDim type=\"cat\">",
    sprintf("<cx:f>%s</cx:f>", cat_range),
    cat_levels_xml,
    "</cx:strDim>"
  )

  val_dim <- paste0(
    "<cx:numDim type=\"size\">",
    sprintf("<cx:f>%s</cx:f>", cx_range(sheetname, value_col, row1, row2)),
    cx_num_lvl(ds[[value]], format_code = "Standard"),
    "</cx:numDim>"
  )

  unique_id <- x$unique_id %||% uuid_generate()
  fill <- cx_render_series_fill(x)
  series_xml <- paste0(
    sprintf(
      "<cx:series layoutId=\"%s\" uniqueId=\"%s\">",
      layout_id,
      unique_id
    ),
    "<cx:tx><cx:txData>",
    sprintf("<cx:f>%s</cx:f>", cx_cell(sheetname, value_col, 1L)),
    sprintf("<cx:v>%s</cx:v>", htmltools::htmlEscape(value)),
    "</cx:txData></cx:tx>",
    fill$spPr,
    fill$dataPts,
    cx_data_labels_xml(
      defaults = list(
        position = "ctr",
        show_cat = TRUE,
        show_val = FALSE,
        show_serie = FALSE
      ),
      opts = x$cx_data_labels,
      fp = x$cx_data_labels$fp
    ),
    "<cx:dataId val=\"0\"/>",
    series_layout_pr,
    "</cx:series>"
  )

  title_xml <- cx_chart_title(x$labels[["title"]], fp = x$theme$main_title)

  ext_data_xml <- if (drop_ext_data) {
    ""
  } else {
    "<cx:externalData r:id=\"rId1\" cx:autoUpdate=\"0\"/>"
  }

  paste0(
    "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>",
    "<cx:chartSpace",
    " xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\"",
    " xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\"",
    " xmlns:cx=\"http://schemas.microsoft.com/office/drawing/2014/chartex\">",
    "<cx:chartData>",
    ext_data_xml,
    "<cx:data id=\"0\">",
    cat_dim,
    val_dim,
    "</cx:data>",
    "</cx:chartData>",
    "<cx:chart>",
    title_xml,
    "<cx:plotArea><cx:plotAreaRegion>",
    series_xml,
    "</cx:plotAreaRegion></cx:plotArea>",
    "</cx:chart>",
    "</cx:chartSpace>"
  )
}

# Excel A1 column letters (A, B, ..., Z, AA, ...). Path columns + value
# column live in the first columns of the embedded sheet, in the order
# given by data_series. We only need a small, safe range generator.
xl_col_letter <- function(i) {
  out <- character(length(i))
  for (k in seq_along(i)) {
    n <- i[k]
    s <- ""
    while (n > 0) {
      r <- (n - 1L) %% 26L
      s <- paste0(LETTERS[r + 1L], s)
      n <- (n - 1L) %/% 26L
    }
    out[k] <- s
  }
  out
}

cx_range <- function(sheet, col, row1, row2) {
  sprintf("%s!$%s$%d:$%s$%d", sheet, col, row1, col, row2)
}

cx_cell <- function(sheet, col, row) {
  sprintf("%s!$%s$%d", sheet, col, row)
}

# Build a <cx:lvl> with inline string points. No formatCode (matches
# Excel-generated chartEx for string dimensions).
cx_str_lvl <- function(values) {
  pts <- paste0(
    sprintf(
      "<cx:pt idx=\"%d\">%s</cx:pt>",
      seq_along(values) - 1L,
      htmltools::htmlEscape(values)
    ),
    collapse = ""
  )
  sprintf("<cx:lvl ptCount=\"%d\">%s</cx:lvl>", length(values), pts)
}

# Build a <cx:lvl> with inline numeric points.
cx_num_lvl <- function(values, format_code = "General") {
  vals <- format(values, scientific = FALSE, trim = TRUE)
  vals[is.na(values)] <- ""
  pts <- paste0(
    sprintf("<cx:pt idx=\"%d\">%s</cx:pt>", seq_along(values) - 1L, vals),
    collapse = ""
  )
  sprintf(
    "<cx:lvl ptCount=\"%d\" formatCode=\"%s\">%s</cx:lvl>",
    length(values),
    format_code,
    pts
  )
}

#' @title Chart Generation for 'Microsoft Word', 'Microsoft Excel' and 'Microsoft PowerPoint' Documents
#'
#' @description It lets R users create Microsoft Office charts from data, and
#' then add title, legends, and annotations to the chart object.
#'
#' The graph produced is a Microsoft graph, which means that it can be edited in
#' your Microsoft software and that the underlying data are available.
#'
#' The package will not allow you to make the same charts as with ggplot2. It
#' allows only a subset of the charts possible with 'Office Chart'. The package is
#' often used to industrialize graphs that are then consumed and annotated by non-R
#' users.
#'
#' The following chart types are available.
#'
#' Classical charts (Office 2007+ DrawingML pipeline):
#'
#' * bar charts: [ms_barchart()]
#' * line charts: [ms_linechart()]
#' * scatter plots: [ms_scatterchart()]
#' * area charts: [ms_areachart()]
#' * pie and doughnut charts: [ms_piechart()]
#' * bubble charts: [ms_bubblechart()]
#' * radar (spider) charts: [ms_radarchart()]
#' * stock charts (HLC and OHLC): [ms_stockchart()]
#'
#' chartEx charts (Office 2016+ pipeline; older viewers show a
#' placeholder):
#'
#' * box-and-whisker: [ms_boxplotchart()]
#' * funnel: [ms_funnelchart()]
#' * histogram: [ms_histogramchart()]
#' * pareto: [ms_paretochart()]
#' * sunburst: [ms_sunburstchart()]
#' * treemap: [ms_treemapchart()]
#' * waterfall: [ms_waterfallchart()]
#'
#' Several classical chart types can be combined on a single chart,
#' with an optional secondary axis (y or x), using [ms_chart_combine()].
#'
#' These functions create a 'chart' object that can be customized:
#'
#' * by using options specific to the chart (with [chart_settings()]),
#' * by changing the options related to the axes (with [chart_ax_x()] and [chart_ax_y()]),
#' * by changing the options related to the labels (with [chart_data_labels()]),
#' * by changing the colors, line widths, ... with functions
#'   * [chart_labels_text()]
#'   * [chart_data_fill()]
#'   * [chart_data_line_style()]
#'   * [chart_data_line_width()]
#'   * [chart_data_size()]
#'   * [chart_data_smooth()]
#'   * [chart_data_stroke()]
#'   * [chart_data_symbol()]
#' * by changing the general theme with function [chart_theme()],
#' * by changing the title labels with function [chart_labels()].
#'
#' You can add a chart into a slide in PowerPoint with function [ph_with.ms_chart()].
#'
#' You can add a chart into a Word document with function [body_add_chart()].
#'
#' You can add a chart into an Excel sheet with function [sheet_add_drawing.ms_chart()].
#'
#'
#' ## Series styling properties by chart type
#'
#' Not all series styling properties have an effect on every
#' chart type. A warning is emitted when a property is set on
#' a chart type that does not support it.
#'
#' Classical charts:
#'
#' | Property | bar | line | area | scatter | stock | radar | bubble | pie |
#' |:-----------|:---:|:----:|:----:|:-------:|:-----:|:-----:|:------:|:---:|
#' | fill       | x   | x    | x    | x       | x     | x     | x      | x   |
#' | colour     | x   | x    | x    | x       | x     | x     | x      | x   |
#' | symbol     |     | x    |      | x       | x     | x     |        |     |
#' | size       |     | x    |      | x       | x     | x     |        |     |
#' | line_width | x   | x    | x    | x       | x     | x     | x      | x   |
#' | line_style |     | x    |      | x       | x     | x     |        |     |
#' | smooth     |     | x    |      | x       |       |       |        |     |
#' | labels_fp  | x   | x    | x    | x       |       | x     |        | x   |
#'
#' chartEx charts expose a narrower set of styling knobs, since most
#' visual aspects are computed by Office from the data (bins, levels,
#' connectors, ...). For these, prefer the chart-type-specific
#' `chart_settings()` method when one is available (e.g. for pareto
#' and boxplot: `chart_settings(x, line = fp_border(...))`).
#'
#' | Property | boxplot | funnel | histogram | pareto | sunburst | treemap | waterfall |
#' |:-----------|:-------:|:------:|:---------:|:------:|:--------:|:-------:|:---------:|
#' | fill       | x       | x      | x         | x      | x        | x       | x         |
#' | colour     | x       | x      | x         | x      | x        | x       | x         |
#' | labels_fp  | x       | x      | x         | x      | x        | x       | x         |
#'
#' ## Two arguments often confused: `asis` and `write_data`
#'
#' `asis` is a **constructor argument** (on [ms_barchart()],
#' [ms_linechart()] and most other classical constructors). It
#' describes the *input shape* of the data frame:
#'
#' * `asis = FALSE` (default): long format, with a `group` column that
#'   splits the rows into series. `mschart` reshapes the data
#'   internally.
#' * `asis = TRUE`: wide format, with one column per series. `y`
#'   accepts a vector of series column names.
#'
#' `write_data` is an **embed-time argument** of
#' [sheet_add_drawing.ms_chart()] for the Excel pipeline. It decides
#' whether `mschart` writes the chart's data into the target sheet
#' (`TRUE`, the default) or leaves you in charge of placing it via
#' [officer::sheet_write_data()] (`FALSE`, recommended for non-trivial
#' workbooks).
#'
#' The two arguments are independent: a chart built with `asis = TRUE`
#' can still be embedded with either `write_data = TRUE` or
#' `write_data = FALSE`.
#'
#' @seealso \url{https://ardata-fr.github.io/officeverse/}
#' @name mschart
"_PACKAGE"

#' @title Sunburst chart object
#' @description Creation of a sunburst chart object that can be inserted
#' in a 'Microsoft' document. Sunburst charts use the chartEx pipeline
#' (Office 2016+); older versions of 'Microsoft Office' will display a
#' fallback placeholder.
#'
#' Data is hierarchical: one column per level (parent to leaf, left to
#' right) and one numeric column for the leaf values. Leaf values
#' propagate to parent rings as sums.
#' @param data a data.frame.
#' @param path character vector of column names defining the hierarchy,
#' from outermost (root) to innermost (leaf).
#' @param value column name for the numeric leaf values.
#' @param labels unused for now; reserved for future custom data label
#' columns.
#' @section Per-leaf coloring:
#' Same caveat as [ms_treemapchart()]: per-leaf colors via
#' [chart_data_fill()] only work reliably with `length(path) == 1`.
#' @return An `ms_chart` object (subclass `ms_sunburstchart`).
#' @family 'Office' chart objects
#' @seealso [chart_labels()], [ms_treemapchart()]
#' @export
#' @examples
#' library(officer)
#'
#' dat <- data.frame(
#'   region  = c("EU", "EU", "EU", "AM", "AM"),
#'   country = c("FR", "FR", "DE", "US", "US"),
#'   city    = c("Paris", "Lyon", "Berlin", "NYC", "LA"),
#'   value   = c(10, 5, 12, 20, 8),
#'   stringsAsFactors = FALSE
#' )
#' sb <- ms_sunburstchart(
#'   data = dat, path = c("region", "country", "city"), value = "value"
#' )
#'
#' doc <- read_pptx()
#' doc <- add_slide(doc)
#' doc <- ph_with(doc, sb, location = ph_location_fullsize())
#' print(doc, target = tempfile(fileext = ".pptx"))
ms_sunburstchart <- function(data, path, value, labels = NULL) {
  out <- ms_treemapchart(
    data = data,
    path = path,
    value = value,
    labels = labels
  )
  class(out) <- c("ms_sunburstchart", "ms_chart_ex", "ms_chart")
  out
}

#' @export
#' @method format ms_sunburstchart
format.ms_sunburstchart <- function(
  x,
  id_x = NULL,
  id_y = NULL,
  sheetname = "sheet1",
  drop_ext_data = FALSE,
  ...
) {
  cx_format_hierarchical(
    x,
    layout_id = "sunburst",
    sheetname = sheetname,
    drop_ext_data = drop_ext_data,
    series_layout_pr = ""
  )
}

#' @export
#' @method print ms_sunburstchart
print.ms_sunburstchart <- function(x, preview = FALSE, ...) {
  if (preview && interactive()) {
    doc <- officer::read_pptx()
    doc <- officer::add_slide(
      doc,
      layout = "Title and Content",
      master = "Office Theme"
    )
    doc <- officer::ph_with(doc, x, location = officer::ph_location_fullsize())
    file_out <- print(doc, target = tempfile(fileext = ".pptx"))
    utils::browseURL(file_out)
    return(invisible())
  }
  cat(sprintf("* %s object (chartEx)\n\n", shQuote("ms_sunburstchart")))
  cat(sprintf("* path: %s\n", paste(x$path, collapse = " > ")))
  cat(sprintf("* value: %s\n\n", x$value))
  cat(sprintf(
    "* original data [%.0f,%.0f] (sample):\n",
    nrow(x$data),
    ncol(x$data)
  ))
  print(x$data[seq_len(min(c(nrow(x$data), 5))), ])
}

sunburst_data <- function() {
  data.frame(
    region = c("EU", "EU", "EU", "AM", "AM"),
    country = c("FR", "FR", "DE", "US", "US"),
    city = c("Paris", "Lyon", "Berlin", "NYC", "LA"),
    value = c(10, 5, 12, 20, 8),
    stringsAsFactors = FALSE
  )
}

test_that("ms_sunburstchart constructor validates inputs", {
  d <- sunburst_data()
  expect_error(
    ms_sunburstchart(d, path = c("region", "missing"), value = "value"),
    "not found in data"
  )
  expect_error(
    ms_sunburstchart(d, path = "region", value = "missing"),
    "not found in data"
  )
  expect_error(
    ms_sunburstchart(d, path = "region", value = "city"),
    "must be numeric"
  )
})

test_that("ms_sunburstchart format produces well-formed cx XML with layoutId=sunburst", {
  sb <- ms_sunburstchart(
    sunburst_data(),
    path = c("region", "country", "city"),
    value = "value"
  )

  xml_str <- format(sb, sheetname = "sheet1")
  doc <- xml2::read_xml(xml_str)
  ns <- c(cx = "http://schemas.microsoft.com/office/drawing/2014/chartex")

  cat_lvls <- xml2::xml_find_all(doc, "//cx:strDim[@type='cat']/cx:lvl", ns)
  expect_length(cat_lvls, 3L)
  val_lvls <- xml2::xml_find_all(doc, "//cx:numDim[@type='size']/cx:lvl", ns)
  expect_length(val_lvls, 1L)

  series <- xml2::xml_find_all(doc, "//cx:series", ns)
  expect_length(series, 1L)
  expect_equal(xml2::xml_attr(series[[1]], "layoutId"), "sunburst")

  # sunburst has no <cx:layoutPr> (unlike treemap)
  layout_pr <- xml2::xml_find_all(doc, "//cx:series/cx:layoutPr", ns)
  expect_length(layout_pr, 0L)
})

test_that("ms_sunburstchart can be inserted in a pptx end-to-end", {
  skip_if_not_installed("officer")
  skip_if_not_installed("writexl")

  sb <- ms_sunburstchart(
    sunburst_data(),
    path = c("region", "country", "city"),
    value = "value"
  )
  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(doc, sb, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

#' @title Waterfall chart object
#' @description Creation of a waterfall chart object that can be inserted
#' in a 'Microsoft' document. Waterfall charts use the chartEx pipeline
#' (Office 2016+); older versions of 'Microsoft Office' will display a
#' fallback placeholder.
#'
#' Each row is one bar. Positive values rise, negative values fall.
#' Categories listed in `subtotals` are rendered as absolute totals
#' (typical for "Start", intermediate totals, and "End" bars).
#' @param data a data.frame.
#' @param x category column name.
#' @param y numeric value column name. Signed values: positive = up,
#' negative = down. For subtotal rows, the value should be the running
#' total at that point.
#' @param subtotals integer vector of 1-based row indices that should
#' be rendered as subtotal/total bars. Defaults to none.
#' @return An `ms_chart` object (subclass `ms_waterfallchart`).
#' @family 'Office' chart objects
#' @seealso [chart_labels()]
#' @export
#' @examples
#' library(officer)
#'
#' dat <- data.frame(
#'   step = c("Start", "Q1", "Q2", "Q3", "End"),
#'   amount = c(100, 30, -20, 40, 150),
#'   stringsAsFactors = FALSE
#' )
#' wf <- ms_waterfallchart(
#'   data = dat, x = "step", y = "amount",
#'   subtotals = c(1, 5)
#' )
#'
#' doc <- read_pptx()
#' doc <- add_slide(doc)
#' doc <- ph_with(doc, wf, location = ph_location_fullsize())
#' print(doc, target = tempfile(fileext = ".pptx"))
ms_waterfallchart <- function(data, x, y, subtotals = NULL) {
  stopifnot(is.data.frame(data))
  stopifnot(is.character(x), length(x) == 1L)
  stopifnot(is.character(y), length(y) == 1L)

  if (!x %in% names(data)) {
    stop("column ", shQuote(x), " not found in data", call. = FALSE)
  }
  if (!y %in% names(data)) {
    stop("column ", shQuote(y), " not found in data", call. = FALSE)
  }
  if (!is.numeric(data[[y]])) {
    stop("column ", shQuote(y), " must be numeric", call. = FALSE)
  }
  if (
    inherits(data, "data.table") ||
      inherits(data, "tbl_df") ||
      inherits(data, "tbl")
  ) {
    data <- as.data.frame(data, stringsAsFactors = FALSE)
  }
  if (is.factor(data[[x]])) {
    data[[x]] <- as.character(data[[x]])
  }
  data[[x]] <- as.character(data[[x]])

  if (!is.null(subtotals)) {
    if (!is.numeric(subtotals)) {
      stop("'subtotals' must be a numeric vector of row indices", call. = FALSE)
    }
    if (any(subtotals < 1L | subtotals > nrow(data))) {
      stop(
        "'subtotals' indices must be between 1 and ",
        nrow(data),
        call. = FALSE
      )
    }
    subtotals <- as.integer(subtotals)
  }

  data_series <- data[, c(x, y), drop = FALSE]
  rownames(data_series) <- NULL

  out <- list(
    data = data,
    data_series = data_series,
    x = x,
    y = y,
    subtotals = subtotals,
    xvar = x,
    yvar = y,
    group = NULL,
    asis = FALSE,
    theme = mschart_theme(),
    labels = list(title = NULL, x = NULL, y = NULL),
    layout_id = "waterfall"
  )
  class(out) <- c("ms_waterfallchart", "ms_chart_ex", "ms_chart")
  out
}

#' @export
#' @method format ms_waterfallchart
format.ms_waterfallchart <- function(
  x,
  id_x = NULL,
  id_y = NULL,
  sheetname = "sheet1",
  drop_ext_data = FALSE,
  ...
) {
  ds <- x$data_series
  n <- nrow(ds)
  cat_col <- x$x
  val_col <- x$y
  row1 <- 2L
  row2 <- n + 1L

  cat_dim <- paste0(
    "<cx:strDim type=\"cat\">",
    sprintf("<cx:f>%s</cx:f>", cx_range(sheetname, "A", row1, row2)),
    cx_str_lvl(ds[[cat_col]]),
    "</cx:strDim>"
  )
  val_dim <- paste0(
    "<cx:numDim type=\"val\">",
    sprintf("<cx:f>%s</cx:f>", cx_range(sheetname, "B", row1, row2)),
    cx_num_lvl(ds[[val_col]], format_code = "Standard"),
    "</cx:numDim>"
  )

  subtotals_xml <- if (length(x$subtotals)) {
    paste0(
      "<cx:layoutPr><cx:subtotals>",
      paste0(
        sprintf("<cx:idx val=\"%d\"/>", x$subtotals - 1L),
        collapse = ""
      ),
      "</cx:subtotals></cx:layoutPr>"
    )
  } else {
    ""
  }

  unique_id <- x$unique_id %||% uuid_generate()
  fill <- cx_render_series_fill(x)
  series_xml <- paste0(
    sprintf(
      "<cx:series layoutId=\"waterfall\" uniqueId=\"%s\">",
      unique_id
    ),
    "<cx:tx><cx:txData>",
    sprintf("<cx:f>%s</cx:f>", cx_cell(sheetname, "B", 1L)),
    sprintf("<cx:v>%s</cx:v>", htmltools::htmlEscape(val_col)),
    "</cx:txData></cx:tx>",
    fill$spPr,
    fill$dataPts,
    cx_data_labels_xml(
      defaults = list(
        position = "outEnd",
        show_val = TRUE,
        show_cat = FALSE,
        show_serie = FALSE
      ),
      opts = x$cx_data_labels,
      fp = x$cx_data_labels$fp
    ),
    "<cx:dataId val=\"0\"/>",
    subtotals_xml,
    "</cx:series>"
  )

  axx <- x$cx_axis_x %||% list()
  axy <- x$cx_axis_y %||% list()
  axes_xml <- paste0(
    cx_axis(
      "0",
      "<cx:catScaling gapWidth=\"0.5\"/>",
      fp = x$theme$axis_text_x,
      title = x$labels[["x"]],
      title_fp = x$theme$axis_title_x,
      num_fmt = axx$num_fmt
    ),
    cx_axis(
      "1",
      cx_val_scaling(min = axy$limit_min, max = axy$limit_max),
      fp = x$theme$axis_text_y,
      title = x$labels[["y"]],
      title_fp = x$theme$axis_title_y,
      major_grid = axy$major_grid %||% TRUE,
      minor_grid = axy$minor_grid %||% FALSE,
      num_fmt = axy$num_fmt
    )
  )

  title_xml <- cx_chart_title(x$labels[["title"]], fp = x$theme$main_title)

  ext_data_xml <- if (drop_ext_data) {
    ""
  } else {
    "<cx:externalData r:id=\"rId1\" cx:autoUpdate=\"0\"/>"
  }

  paste0(
    "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>",
    "<cx:chartSpace",
    " xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\"",
    " xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\"",
    " xmlns:cx=\"http://schemas.microsoft.com/office/drawing/2014/chartex\">",
    "<cx:chartData>",
    ext_data_xml,
    "<cx:data id=\"0\">",
    cat_dim,
    val_dim,
    "</cx:data>",
    "</cx:chartData>",
    "<cx:chart>",
    title_xml,
    "<cx:plotArea><cx:plotAreaRegion>",
    series_xml,
    "</cx:plotAreaRegion>",
    axes_xml,
    "</cx:plotArea>",
    "</cx:chart>",
    "</cx:chartSpace>"
  )
}

#' @export
#' @method print ms_waterfallchart
print.ms_waterfallchart <- function(x, preview = FALSE, ...) {
  if (preview && interactive()) {
    doc <- officer::read_pptx()
    doc <- officer::add_slide(
      doc,
      layout = "Title and Content",
      master = "Office Theme"
    )
    doc <- officer::ph_with(doc, x, location = officer::ph_location_fullsize())
    file_out <- print(doc, target = tempfile(fileext = ".pptx"))
    utils::browseURL(file_out)
    return(invisible())
  }
  cat(sprintf("* %s object (chartEx)\n\n", shQuote("ms_waterfallchart")))
  cat(sprintf("* x: %s\n", x$x))
  cat(sprintf("* y: %s\n", x$y))
  if (length(x$subtotals)) {
    cat(sprintf(
      "* subtotals (rows): %s\n",
      paste(x$subtotals, collapse = ", ")
    ))
  }
  cat(sprintf(
    "\n* original data [%.0f,%.0f] (sample):\n",
    nrow(x$data),
    ncol(x$data)
  ))
  print(x$data[seq_len(min(c(nrow(x$data), 5))), ])
}

waterfall_data <- function() {
  data.frame(
    step = c("Start", "Q1", "Q2", "Q3", "End"),
    amount = c(100, 30, -20, 40, 150),
    stringsAsFactors = FALSE
  )
}

test_that("ms_waterfallchart constructor validates inputs", {
  d <- waterfall_data()
  expect_error(ms_waterfallchart(d, x = "missing", y = "amount"), "not found")
  expect_error(ms_waterfallchart(d, x = "step", y = "missing"), "not found")
  expect_error(ms_waterfallchart(d, x = "step", y = "step"), "must be numeric")
  expect_error(
    ms_waterfallchart(d, x = "step", y = "amount", subtotals = c(1, 99)),
    "between 1 and"
  )
})

test_that("ms_waterfallchart format produces well-formed cx XML", {
  wf <- ms_waterfallchart(
    waterfall_data(),
    x = "step",
    y = "amount",
    subtotals = c(1, 5)
  )
  xml_str <- format(wf, sheetname = "sheet1")
  doc <- xml2::read_xml(xml_str)
  ns <- c(cx = "http://schemas.microsoft.com/office/drawing/2014/chartex")

  expect_equal(xml2::xml_name(xml2::xml_root(doc)), "chartSpace")

  cat_lvls <- xml2::xml_find_all(doc, "//cx:strDim[@type='cat']/cx:lvl", ns)
  expect_length(cat_lvls, 1L)
  val_lvls <- xml2::xml_find_all(doc, "//cx:numDim[@type='val']/cx:lvl", ns)
  expect_length(val_lvls, 1L)

  series <- xml2::xml_find_all(doc, "//cx:series", ns)
  expect_length(series, 1L)
  expect_equal(xml2::xml_attr(series[[1]], "layoutId"), "waterfall")

  # subtotals: 1-based 1 and 5 -> 0-based 0 and 4
  idx_nodes <- xml2::xml_find_all(doc, "//cx:subtotals/cx:idx", ns)
  expect_length(idx_nodes, 2L)
  expect_equal(xml2::xml_attr(idx_nodes, "val"), c("0", "4"))

  # axes
  axes <- xml2::xml_find_all(doc, "//cx:plotArea/cx:axis", ns)
  expect_length(axes, 2L)
})

test_that("ms_waterfallchart end-to-end pptx", {
  skip_if_not_installed("officer")
  skip_if_not_installed("writexl")

  wf <- ms_waterfallchart(
    waterfall_data(),
    x = "step",
    y = "amount",
    subtotals = c(1, 5)
  )
  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(doc, wf, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

#' @title Histogram chart object
#' @description Creation of a histogram chart object that can be inserted
#' in a 'Microsoft' document. Histogram charts use the chartEx pipeline
#' (Office 2016+); older versions of 'Microsoft Office' will display a
#' fallback placeholder.
#'
#' Data is a single column of raw numeric observations. Office computes
#' the bins automatically; pass `bin_count` or `bin_width` to override.
#' @param data a data.frame.
#' @param value numeric column name (raw observations).
#' @param bin_count integer, requested number of bins. Mutually exclusive
#' with `bin_width`. NULL = automatic.
#' @param bin_width numeric, requested bin width. Mutually exclusive with
#' `bin_count`. NULL = automatic.
#' @param interval_closed one of `"right"` (default) or `"left"`. Defines
#' which end of each bin is inclusive.
#' @param underflow numeric, values below this go in an "underflow" bin.
#' NULL to disable.
#' @param overflow numeric, values above this go in an "overflow" bin.
#' NULL to disable.
#' @return An `ms_chart` object (subclass `ms_histogramchart`).
#' @family 'Office' chart objects
#' @seealso [chart_labels()]
#' @export
#' @examples
#' library(officer)
#'
#' set.seed(1)
#' dat <- data.frame(x = rnorm(200, mean = 10, sd = 4))
#' hi <- ms_histogramchart(dat, value = "x", bin_count = 12)
#'
#' doc <- read_pptx()
#' doc <- add_slide(doc)
#' doc <- ph_with(doc, hi, location = ph_location_fullsize())
#' print(doc, target = tempfile(fileext = ".pptx"))
ms_histogramchart <- function(
  data,
  value,
  bin_count = NULL,
  bin_width = NULL,
  interval_closed = c("right", "left"),
  underflow = NULL,
  overflow = NULL
) {
  stopifnot(is.data.frame(data))
  stopifnot(is.character(value), length(value) == 1L)
  interval_closed <- match.arg(interval_closed)

  if (!value %in% names(data)) {
    stop("column ", shQuote(value), " not found in data", call. = FALSE)
  }
  if (!is.numeric(data[[value]])) {
    stop("column ", shQuote(value), " must be numeric", call. = FALSE)
  }
  if (!is.null(bin_count) && !is.null(bin_width)) {
    stop("provide either 'bin_count' or 'bin_width', not both", call. = FALSE)
  }
  if (
    inherits(data, "data.table") ||
      inherits(data, "tbl_df") ||
      inherits(data, "tbl")
  ) {
    data <- as.data.frame(data, stringsAsFactors = FALSE)
  }

  data_series <- data[, value, drop = FALSE]
  rownames(data_series) <- NULL

  out <- list(
    data = data,
    data_series = data_series,
    value = value,
    x = value,
    y = value,
    xvar = value,
    yvar = value,
    group = NULL,
    asis = FALSE,
    theme = mschart_theme(),
    labels = list(title = NULL, x = NULL, y = NULL),
    layout_id = "clusteredColumn",
    bin_count = bin_count,
    bin_width = bin_width,
    interval_closed = interval_closed,
    underflow = underflow,
    overflow = overflow
  )
  class(out) <- c("ms_histogramchart", "ms_chart_ex", "ms_chart")
  out
}

#' @export
#' @method format ms_histogramchart
format.ms_histogramchart <- function(
  x,
  id_x = NULL,
  id_y = NULL,
  sheetname = "sheet1",
  drop_ext_data = FALSE,
  ...
) {
  ds <- x$data_series
  n <- nrow(ds)
  val_col <- x$value
  row1 <- 2L
  row2 <- n + 1L

  val_dim <- paste0(
    "<cx:numDim type=\"val\">",
    sprintf("<cx:f>%s</cx:f>", cx_range(sheetname, "A", row1, row2)),
    cx_num_lvl(ds[[val_col]], format_code = "Standard"),
    "</cx:numDim>"
  )

  # Per chartEx schema (CT_Binning): intervalClosed/underflow/overflow are
  # attributes; binCount and binSize are CHILD ELEMENTS with a `val` attr.
  closed_attr <- if (x$interval_closed == "right") "r" else "l"
  binning_attrs <- sprintf("intervalClosed=\"%s\"", closed_attr)
  if (!is.null(x$underflow)) {
    binning_attrs <- paste0(
      binning_attrs,
      sprintf(
        " underflow=\"%s\"",
        format(x$underflow, scientific = FALSE, trim = TRUE)
      )
    )
  }
  if (!is.null(x$overflow)) {
    binning_attrs <- paste0(
      binning_attrs,
      sprintf(
        " overflow=\"%s\"",
        format(x$overflow, scientific = FALSE, trim = TRUE)
      )
    )
  }

  binning_children <- ""
  if (!is.null(x$bin_count)) {
    binning_children <- sprintf(
      "<cx:binCount val=\"%d\"/>",
      as.integer(x$bin_count)
    )
  }
  if (!is.null(x$bin_width)) {
    binning_children <- sprintf(
      "<cx:binSize val=\"%s\"/>",
      format(x$bin_width, scientific = FALSE, trim = TRUE)
    )
  }
  binning_xml <- if (nzchar(binning_children)) {
    sprintf("<cx:binning %s>%s</cx:binning>", binning_attrs, binning_children)
  } else {
    sprintf("<cx:binning %s/>", binning_attrs)
  }

  unique_id <- x$unique_id %||% uuid_generate()
  fill <- cx_render_series_fill(x)
  series_xml <- paste0(
    sprintf(
      "<cx:series layoutId=\"clusteredColumn\" uniqueId=\"%s\">",
      unique_id
    ),
    "<cx:tx><cx:txData>",
    sprintf("<cx:f>%s</cx:f>", cx_cell(sheetname, "A", 1L)),
    sprintf("<cx:v>%s</cx:v>", htmltools::htmlEscape(val_col)),
    "</cx:txData></cx:tx>",
    fill$spPr,
    fill$dataPts,
    cx_data_labels_xml(
      defaults = NULL,
      opts = x$cx_data_labels,
      fp = x$cx_data_labels$fp
    ),
    "<cx:dataId val=\"0\"/>",
    "<cx:layoutPr>",
    binning_xml,
    "</cx:layoutPr>",
    "</cx:series>"
  )

  axx <- x$cx_axis_x %||% list()
  axy <- x$cx_axis_y %||% list()
  axes_xml <- paste0(
    cx_axis(
      "0",
      "<cx:catScaling gapWidth=\"0\"/>",
      fp = x$theme$axis_text_x,
      title = x$labels[["x"]],
      title_fp = x$theme$axis_title_x,
      num_fmt = axx$num_fmt
    ),
    cx_axis(
      "1",
      cx_val_scaling(min = axy$limit_min, max = axy$limit_max),
      fp = x$theme$axis_text_y,
      title = x$labels[["y"]],
      title_fp = x$theme$axis_title_y,
      major_grid = axy$major_grid %||% TRUE,
      minor_grid = axy$minor_grid %||% FALSE,
      num_fmt = axy$num_fmt
    )
  )

  title_xml <- cx_chart_title(x$labels[["title"]], fp = x$theme$main_title)

  ext_data_xml <- if (drop_ext_data) {
    ""
  } else {
    "<cx:externalData r:id=\"rId1\" cx:autoUpdate=\"0\"/>"
  }

  paste0(
    "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>",
    "<cx:chartSpace",
    " xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\"",
    " xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\"",
    " xmlns:cx=\"http://schemas.microsoft.com/office/drawing/2014/chartex\">",
    "<cx:chartData>",
    ext_data_xml,
    "<cx:data id=\"0\">",
    val_dim,
    "</cx:data>",
    "</cx:chartData>",
    "<cx:chart>",
    title_xml,
    "<cx:plotArea><cx:plotAreaRegion>",
    series_xml,
    "</cx:plotAreaRegion>",
    axes_xml,
    "</cx:plotArea>",
    "</cx:chart>",
    "</cx:chartSpace>"
  )
}

#' @export
#' @method print ms_histogramchart
print.ms_histogramchart <- function(x, preview = FALSE, ...) {
  if (preview && interactive()) {
    doc <- officer::read_pptx()
    doc <- officer::add_slide(
      doc,
      layout = "Title and Content",
      master = "Office Theme"
    )
    doc <- officer::ph_with(doc, x, location = officer::ph_location_fullsize())
    file_out <- print(doc, target = tempfile(fileext = ".pptx"))
    utils::browseURL(file_out)
    return(invisible())
  }
  cat(sprintf("* %s object (chartEx)\n\n", shQuote("ms_histogramchart")))
  cat(sprintf("* value: %s\n", x$value))
  if (!is.null(x$bin_count)) {
    cat(sprintf("* bin_count: %d\n", x$bin_count))
  }
  if (!is.null(x$bin_width)) {
    cat(sprintf("* bin_width: %s\n", x$bin_width))
  }
  cat(sprintf("* interval closed: %s\n\n", x$interval_closed))
  cat(sprintf(
    "* original data [%.0f,%.0f] (sample):\n",
    nrow(x$data),
    ncol(x$data)
  ))
  print(x$data[seq_len(min(c(nrow(x$data), 5))), , drop = FALSE])
}

histogram_data <- function() {
  set.seed(1)
  data.frame(x = rnorm(80, mean = 12, sd = 4))
}

test_that("ms_histogramchart constructor validates inputs", {
  d <- histogram_data()
  expect_error(ms_histogramchart(d, value = "missing"), "not found")
  expect_error(
    ms_histogramchart(data.frame(g = letters[1:5]), value = "g"),
    "must be numeric"
  )
  expect_error(
    ms_histogramchart(d, value = "x", bin_count = 10, bin_width = 1),
    "either"
  )
})

test_that("ms_histogramchart format produces well-formed cx XML", {
  hi <- ms_histogramchart(
    histogram_data(),
    value = "x",
    bin_count = 10,
    interval_closed = "left"
  )
  xml_str <- format(hi, sheetname = "sheet1")
  doc <- xml2::read_xml(xml_str)
  ns <- c(cx = "http://schemas.microsoft.com/office/drawing/2014/chartex")

  # No cat dim, only val dim
  cat_dims <- xml2::xml_find_all(doc, "//cx:strDim", ns)
  expect_length(cat_dims, 0L)
  val_dims <- xml2::xml_find_all(doc, "//cx:numDim[@type='val']", ns)
  expect_length(val_dims, 1L)

  series <- xml2::xml_find_all(doc, "//cx:series", ns)
  expect_length(series, 1L)
  expect_equal(xml2::xml_attr(series[[1]], "layoutId"), "clusteredColumn")

  binning <- xml2::xml_find_first(doc, "//cx:layoutPr/cx:binning", ns)
  expect_equal(xml2::xml_attr(binning, "intervalClosed"), "l")
  bin_count <- xml2::xml_find_first(doc, "//cx:binning/cx:binCount", ns)
  expect_equal(xml2::xml_attr(bin_count, "val"), "10")

  axes <- xml2::xml_find_all(doc, "//cx:plotArea/cx:axis", ns)
  expect_length(axes, 2L)
})

test_that("ms_histogramchart end-to-end pptx", {
  skip_if_not_installed("officer")
  skip_if_not_installed("writexl")

  hi <- ms_histogramchart(histogram_data(), value = "x", bin_width = 2)
  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(doc, hi, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

# Style helpers for chartEx (cx:) charts. Wrappers thin enough that
# heavy lifting lives in officer (`solid_fill()`, `to_pml.sp_line()`,
# `format(fp_text, type="pml")`). Shape-property fragments are namespaced
# `a:` (DrawingML) and reused inside chartEx wrappers `<cx:spPr>`,
# `<cx:dataPt>`, `<cx:txPr>`, `<cx:title>`.

# --- Series fill dispatch (business logic, kept here) ---------------------

# Returns the category column name in `data_series` against which a
# named-vector `chart_data_fill(values=c(name=color, ...))` matches.
# Histogram is fill-by-bin and Office computes bins itself, so per-point
# fill is not supported there - the method returns NA and only single
# series-wide color works.
cx_fill_cat_column <- function(x) UseMethod("cx_fill_cat_column")
#' @exportS3Method
cx_fill_cat_column.default <- function(x) x$x
#' @exportS3Method
cx_fill_cat_column.ms_treemapchart <- function(x) x$path[length(x$path)]
#' @exportS3Method
cx_fill_cat_column.ms_sunburstchart <- function(x) x$path[length(x$path)]
#' @exportS3Method
cx_fill_cat_column.ms_histogramchart <- function(x) NA_character_
# Pareto with aggregate=TRUE reorders bars by count -> per-point match
# by name would land on the wrong bar. Only single-color is meaningful.
#' @exportS3Method
cx_fill_cat_column.ms_paretochart <- function(x) {
  if (isTRUE(x$aggregate)) NA_character_ else x$x
}

# Layouts whose series spPr line is reused by Excel/PowerPoint to draw
# the connector lines between bars. For these, we must avoid the
# uniform-stroke shortcut (which sets <cx:spPr><a:ln/></cx:spPr> at the
# series level) and emit per-point <cx:dataPt> strokes instead, so the
# connectors keep the chartstyle default.
cx_uses_connectors <- function(x) UseMethod("cx_uses_connectors")
#' @exportS3Method
cx_uses_connectors.default <- function(x) FALSE
#' @exportS3Method
cx_uses_connectors.ms_waterfallchart <- function(x) TRUE

# Build the per-point fill XML fragment for a chartEx series. Reads
# x$cx_fill_per_point (character vector of length nrow(data_series),
# NA for "no override"). Returns "" if no overrides, an inline
# <cx:spPr> if all rows share one color, or a sequence of <cx:dataPt>
# elements otherwise.
cx_render_series_fill <- function(x) {
  vf <- x$cx_fill_per_point
  vs <- x$cx_stroke_per_point
  width <- x$cx_stroke_width %||% 0.75
  if (is.null(vf) && is.null(vs)) {
    return(list(spPr = "", dataPts = ""))
  }
  n <- max(length(vf), length(vs))
  if (is.null(vf)) {
    vf <- rep(NA_character_, n)
  }
  if (is.null(vs)) {
    vs <- rep(NA_character_, n)
  }
  has_any <- !is.na(vf) | !is.na(vs)
  if (!any(has_any)) {
    return(list(spPr = "", dataPts = ""))
  }

  # Series-wide shortcut: every row has the same fill AND same stroke.
  # Layouts with connector lines (waterfall) must keep stroke per-point,
  # otherwise Excel reuses the series spPr line for the connectors.
  has_connectors <- cx_uses_connectors(x)
  uniform_fill <- all(!is.na(vf)) && length(unique(vf)) == 1L
  uniform_stroke <- !has_connectors &&
    all(!is.na(vs)) &&
    length(unique(vs)) == 1L
  fill_only_uniform <- uniform_fill && all(is.na(vs))
  stroke_only_uniform <- uniform_stroke && all(is.na(vf))
  both_uniform <- uniform_fill && uniform_stroke
  if (fill_only_uniform) {
    return(list(spPr = cx_spPr(fill = vf[1]), dataPts = ""))
  }
  if (stroke_only_uniform) {
    return(list(
      spPr = cx_spPr(line_color = vs[1], line_width = width),
      dataPts = ""
    ))
  }
  if (both_uniform) {
    return(list(
      spPr = cx_spPr(fill = vf[1], line_color = vs[1], line_width = width),
      dataPts = ""
    ))
  }

  pts <- vapply(
    which(has_any),
    function(i) {
      cx_dataPt(
        i - 1L,
        fill = if (is.na(vf[i])) NULL else vf[i],
        line_color = if (is.na(vs[i])) NULL else vs[i],
        line_width = width
      )
    },
    character(1)
  )
  list(spPr = "", dataPts = paste0(pts, collapse = ""))
}

# Local duplicates of officer's internal hex_color() / is_transparent().
hex_color <- function(color) {
  rgba <- as.vector(grDevices::col2rgb(color, alpha = TRUE)) / c(1, 1, 1, 255)
  sprintf("%02X%02X%02X", rgba[1], rgba[2], rgba[3])
}

is_transparent <- function(color) {
  if (identical(color, "transparent")) {
    return(TRUE)
  }
  alpha <- as.vector(grDevices::col2rgb(color, alpha = TRUE))[4] / 255
  alpha <= 0
}

# Normalize a color (named, "#RRGGBB", "RRGGBB", NA, NULL, "transparent")
# to uppercase 6-digit hex without "#". Returns NULL for NULL/NA/empty/
# transparent inputs. Errors on unparseable color strings.
cx_color_hex <- function(color) {
  if (is.null(color) || length(color) == 0L) {
    return(NULL)
  }
  if (is.na(color) || !nzchar(color)) {
    return(NULL)
  }
  if (identical(color, "transparent")) {
    return(NULL)
  }
  if (grepl("^[0-9A-Fa-f]{6}$", color)) {
    color <- paste0("#", color)
  }
  hex <- tryCatch(hex_color(color), error = function(e) NULL)
  if (is.null(hex)) {
    stop("invalid color: ", shQuote(color), call. = FALSE)
  }
  if (is_transparent(color)) {
    return(NULL)
  }
  toupper(hex)
}

# --- Solid fill / line wrappers (delegate to officer) ---------------------

# Officer's solid_fill always emits a non-empty fragment; we layer a
# null-check on top so `cx_solid_fill(NULL)` and `cx_solid_fill(NA)`
# return "" (used by render-fill/-spPr to short-circuit empty inputs).
cx_solid_fill <- function(color) {
  if (is.null(color) || length(color) == 0L) {
    return("")
  }
  if (is.na(color) || !nzchar(color)) {
    return("")
  }
  if (identical(color, "transparent")) {
    return("")
  }
  # officer::solid_fill -> col2rgb requires '#' for hex strings.
  if (grepl("^[0-9A-Fa-f]{6}$", color)) {
    color <- paste0("#", color)
  }
  officer::solid_fill(color)
}

# Build an <a:ln> via officer::sp_line + to_pml. Accepts either a plain
# (color, width) pair or a pre-built sp_line for advanced usage.
cx_line <- function(color = NULL, width = 0.75, sp_line = NULL) {
  if (is.null(sp_line)) {
    if (is.null(color) || identical(color, "transparent")) {
      return("")
    }
    if (grepl("^[0-9A-Fa-f]{6}$", color)) {
      color <- paste0("#", color)
    }
    sp_line <- officer::sp_line(color = color, lwd = width)
  }
  officer::to_pml(sp_line)
}

# --- chartEx-specific wrappers --------------------------------------------

# <cx:spPr>{fill}{line}</cx:spPr>. Either part can be NULL/missing.
cx_spPr <- function(fill = NULL, line_color = NULL, line_width = 0.75) {
  fill_xml <- if (!is.null(fill)) cx_solid_fill(fill) else ""
  line_xml <- if (!is.null(line_color)) cx_line(line_color, line_width) else ""
  if (!nzchar(fill_xml) && !nzchar(line_xml)) {
    return("")
  }
  paste0("<cx:spPr>", fill_xml, line_xml, "</cx:spPr>")
}

# <cx:dataPt idx="N"><cx:spPr>...</cx:spPr></cx:dataPt>. Per-point
# overrides for a chartEx series. Note PowerPoint accepts <cx:dataPt>
# (ECMA-376) and <cx:dPt> identically; we use the
# ECMA name. See `?ms_treemapchart` for the hierarchical-rendering
# limitation in PowerPoint.
cx_dataPt <- function(idx, fill = NULL, line_color = NULL, line_width = 0.75) {
  spPr <- cx_spPr(fill, line_color, line_width)
  if (!nzchar(spPr)) {
    return("")
  }
  sprintf("<cx:dataPt idx=\"%d\">%s</cx:dataPt>", as.integer(idx), spPr)
}

# --- Text properties via officer's fp_text -> PresentationML --------------

# Build an <a:rPr> (or <a:defRPr>) block from an fp_text by reusing
# officer's canonical converter (`format(fp, type="pml")`). The returned
# fragment includes solidFill (with alpha), latin/cs/ea/sym typefaces
# and all sz/b/i/u/strike/cap attributes. `as_def` rewrites the tag to
# <a:defRPr> for use inside <a:pPr> (txPr context).
cx_rPr_from_fp_text <- function(fp, as_def = FALSE) {
  if (is.null(fp)) {
    return("")
  }
  out <- format(fp, type = "pml")
  if (isTRUE(as_def)) {
    out <- gsub("a:rPr", "a:defRPr", out, fixed = TRUE)
  }
  out
}

# <cx:txPr> for axis/legend/dataLabels styling. Style is carried by
# <a:defRPr> inside an empty paragraph (text content itself comes from
# data/category names).
cx_txPr_from_fp_text <- function(fp) {
  if (is.null(fp)) {
    return("")
  }
  defRPr <- cx_rPr_from_fp_text(fp, as_def = TRUE)
  # PowerPoint (notably Mac) reads endParaRPr to color/size auto-rendered
  # text such as data label values. Mirror the styling on both defRPr and
  # endParaRPr for parity.
  endRPr <- gsub(
    "a:rPr",
    "a:endParaRPr",
    cx_rPr_from_fp_text(fp, as_def = FALSE),
    fixed = TRUE
  )
  paste0(
    "<cx:txPr>",
    "<a:bodyPr/>",
    "<a:lstStyle/>",
    "<a:p><a:pPr>",
    defRPr,
    "</a:pPr>",
    endRPr,
    "</a:p>",
    "</cx:txPr>"
  )
}

# <cx:dataLabels [pos="..."]><cx:visibility .../><cx:numFmt .../><cx:txPr .../></cx:dataLabels>
# Per-layout defaults are passed in `defaults` (list with position, show_val,
# show_cat, show_serie). User overrides via x$cx_data_labels (`opts`).
# `fp` is an fp_text for label font (optional).
# Returns "" when both defaults and opts are NULL (chart with no labels).
cx_data_labels_xml <- function(
  defaults = NULL,
  opts = NULL,
  fp = NULL,
  n_points = NULL
) {
  if (is.null(defaults) && is.null(opts)) {
    return("")
  }
  d <- defaults %||% list()
  o <- opts %||% list()
  pos <- o$position %||% d$position
  show_val <- isTRUE(o$show_val %||% d$show_val)
  show_cat <- isTRUE(o$show_cat %||% d$show_cat)
  show_ser <- isTRUE(o$show_serie %||% d$show_serie)
  num_fmt <- o$num_fmt
  pos_attr <- if (is.null(pos) || !nzchar(pos)) {
    ""
  } else {
    sprintf(" pos=\"%s\"", pos)
  }
  vis_xml <- sprintf(
    "<cx:visibility seriesName=\"%d\" categoryName=\"%d\" value=\"%d\"/>",
    as.integer(show_ser),
    as.integer(show_cat),
    as.integer(show_val)
  )
  numfmt_xml <- if (is.null(num_fmt) || !nzchar(num_fmt)) {
    ""
  } else {
    sprintf(
      "<cx:numFmt formatCode=\"%s\" sourceLinked=\"0\"/>",
      htmltools::htmlEscape(num_fmt)
    )
  }
  # The bundled chartstyle (style1.xml) often assigns labels a near-white
  # text color (designed for inEnd labels on dark bars). Without an
  # explicit override the labels become invisible on light backgrounds.
  # Force a sensible default when the caller didn't supply an fp_text.
  if (is.null(fp)) {
    fp <- officer::fp_text(font.size = 10, color = "black")
  }
  txpr_xml <- cx_txPr_from_fp_text(fp)
  # Per-point <cx:dataLabel idx="N"> blocks. Required by some PowerPoint
  # versions for layouts (notably waterfall) that do NOT honor the
  # series-level <cx:visibility> alone.
  perpt_xml <- ""
  if (!is.null(n_points) && n_points > 0L) {
    pp_pos <- if (is.null(pos) || !nzchar(pos)) {
      ""
    } else {
      sprintf(" pos=\"%s\"", pos)
    }
    perpt_xml <- paste0(
      vapply(
        seq_len(n_points) - 1L,
        function(i) {
          sprintf(
            "<cx:dataLabel idx=\"%d\"%s>%s</cx:dataLabel>",
            i,
            pp_pos,
            vis_xml
          )
        },
        character(1)
      ),
      collapse = ""
    )
  }
  paste0(
    sprintf("<cx:dataLabels%s>", pos_attr),
    vis_xml,
    numfmt_xml,
    txpr_xml,
    perpt_xml,
    "</cx:dataLabels>"
  )
}

# <cx:valScaling [min="..."] [max="..."]/>
cx_val_scaling <- function(min = NULL, max = NULL, extra_attrs = "") {
  attrs <- character()
  if (!is.null(min)) {
    attrs <- c(
      attrs,
      sprintf("min=\"%s\"", format(min, scientific = FALSE, trim = TRUE))
    )
  }
  if (!is.null(max)) {
    attrs <- c(
      attrs,
      sprintf("max=\"%s\"", format(max, scientific = FALSE, trim = TRUE))
    )
  }
  if (nzchar(extra_attrs)) {
    attrs <- c(attrs, extra_attrs)
  }
  if (length(attrs)) {
    sprintf("<cx:valScaling %s/>", paste(attrs, collapse = " "))
  } else {
    "<cx:valScaling/>"
  }
}

# <cx:axis id="N">{scaling}{extras}<cx:tickLabels/>{txPr}</cx:axis>.
# `scaling` is the cat/val scaling element (raw XML string). `extras`
# can carry majorGridlines, units, etc. `fp` is an fp_text controlling
# tick label font (typically theme$axis_text_x or theme$axis_text_y).
# Order respects CT_Axis schema: scaling, ..., tickLabels, txPr.
cx_axis <- function(
  id,
  scaling,
  extras = "",
  fp = NULL,
  title = NULL,
  title_fp = NULL,
  major_grid = FALSE,
  minor_grid = FALSE,
  num_fmt = NULL
) {
  # Axis titles use <cx:title> without pos/align/overlay attributes
  # (those would be invalid in axis context per chartEx schema; position
  # is implicit and tied to the parent axis).
  title_xml <- if (is.null(title) || !nzchar(title)) {
    ""
  } else {
    paste0("<cx:title>", cx_rich_text(title, fp = title_fp), "</cx:title>")
  }
  major_xml <- if (isTRUE(major_grid)) "<cx:majorGridlines/>" else ""
  minor_xml <- if (isTRUE(minor_grid)) "<cx:minorGridlines/>" else ""
  numfmt_xml <- if (is.null(num_fmt) || !nzchar(num_fmt)) {
    ""
  } else {
    sprintf(
      "<cx:numFmt formatCode=\"%s\" sourceLinked=\"0\"/>",
      htmltools::htmlEscape(num_fmt)
    )
  }
  # CT_Axis order: scaling, title, units, majorGridlines, minorGridlines,
  # majorTickMarks, minorTickMarks, tickLabels, numFmt, spPr, txPr.
  paste0(
    sprintf("<cx:axis id=\"%s\">", id),
    scaling,
    title_xml,
    extras,
    major_xml,
    minor_xml,
    "<cx:tickLabels/>",
    numfmt_xml,
    cx_txPr_from_fp_text(fp),
    "</cx:axis>"
  )
}

# <cx:title pos="t" align="ctr" overlay="0">...</cx:title>. Returns ""
# when text is NULL/empty. The title text inherits font/color/size from
# the supplied fp_text (typically theme$main_title).
cx_chart_title <- function(text, fp = NULL) {
  if (is.null(text) || !nzchar(text)) {
    return("")
  }
  paste0(
    "<cx:title pos=\"t\" align=\"ctr\" overlay=\"0\">",
    cx_rich_text(text, fp = fp),
    "</cx:title>"
  )
}

# <cx:tx><cx:rich>...</cx:rich></cx:tx> for titles (chart, axis). The
# run carries an explicit <a:rPr> from fp_text. align: "ctr" (default),
# "l" or "r".
cx_rich_text <- function(text, fp = NULL, align = "ctr") {
  rpr <- cx_rPr_from_fp_text(fp, as_def = FALSE)
  if (!nzchar(rpr)) {
    rpr <- "<a:rPr/>"
  }
  paste0(
    "<cx:tx><cx:rich>",
    "<a:bodyPr/>",
    "<a:lstStyle/>",
    sprintf("<a:p><a:pPr algn=\"%s\"/>", align),
    "<a:r>",
    rpr,
    sprintf("<a:t>%s</a:t>", htmltools::htmlEscape(text)),
    "</a:r></a:p>",
    "</cx:rich></cx:tx>"
  )
}

# Build a <cx:spPr><a:ln .../></cx:spPr> fragment used to draw the
# cumulative line of pareto charts and the whisker/median/mean strokes
# of boxplot charts. Without this override Excel resolves the
# chartstyle phClr placeholder to an invisible color.
#   NULL        -> use scheme accent2 (matches Excel-native defaults)
#   FALSE       -> no spPr (lines left to chartstyle, fragile)
#   fp_border() -> convert via officer's ooxml_fp_border
cx_default_line_spPr <- function(line) {
  if (isFALSE(line)) {
    return("")
  }
  if (is.null(line)) {
    return(paste0(
      "<cx:spPr><a:ln w=\"12700\">",
      "<a:solidFill><a:schemeClr val=\"accent2\"/></a:solidFill>",
      "</a:ln></cx:spPr>"
    ))
  }
  ooxml_fp_border(line, in_tags = c("cx:spPr"))
}

cx_ns <- c(
  cx = "http://schemas.microsoft.com/office/drawing/2014/chartex",
  a = "http://schemas.openxmlformats.org/drawingml/2006/main"
)

# --- cx_color_hex (normalization helper, used by chart_data_fill) -------

test_that("cx_color_hex normalizes named, hex and #hex inputs", {
  expect_equal(cx_color_hex("red"), "FF0000")
  expect_equal(cx_color_hex("#00ff00"), "00FF00")
  expect_equal(cx_color_hex("0000FF"), "0000FF")
})

test_that("cx_color_hex returns NULL for empty/NA/transparent", {
  expect_null(cx_color_hex(NULL))
  expect_null(cx_color_hex(NA))
  expect_null(cx_color_hex(""))
  expect_null(cx_color_hex("transparent"))
})

test_that("cx_color_hex errors on invalid color string", {
  expect_error(cx_color_hex("not-a-color"), "invalid color")
})

# --- cx_solid_fill (delegates to officer::solid_fill) -------------------

test_that("cx_solid_fill: delegates to officer::solid_fill, includes alpha", {
  out <- cx_solid_fill("red")
  expect_match(out, "^<a:solidFill>")
  expect_match(out, "<a:srgbClr val=\"FF0000\">")
  expect_match(out, "<a:alpha val=\"100000\"/>")
})

test_that("cx_solid_fill: returns empty for NULL/NA/empty/transparent", {
  expect_equal(cx_solid_fill(NULL), "")
  expect_equal(cx_solid_fill(NA), "")
  expect_equal(cx_solid_fill(""), "")
  expect_equal(cx_solid_fill("transparent"), "")
})

# --- cx_line (delegates to officer::sp_line + to_pml) -------------------

test_that("cx_line: emits <a:ln w=> in EMU and wraps the color via solid_fill", {
  out <- cx_line("black", width = 1)
  expect_match(out, "^<a:ln w=\"12700\"")
  expect_match(out, "<a:srgbClr val=\"000000\">")
  expect_match(cx_line("blue", width = 0.75), "w=\"9525\"")
})

test_that("cx_line: returns empty for NULL/transparent", {
  expect_equal(cx_line(NULL), "")
  expect_equal(cx_line("transparent"), "")
})

test_that("cx_line: accepts a pre-built sp_line for advanced usage", {
  sp <- officer::sp_line(color = "red", lwd = 2, lty = "dash")
  out <- cx_line(sp_line = sp)
  expect_match(out, "w=\"25400\"")
  expect_match(out, "<a:prstDash val=\"dash\"/>")
})

# --- cx_spPr / cx_dataPt -----------------------------------------------

test_that("cx_spPr returns empty when no fill/line provided", {
  expect_equal(cx_spPr(), "")
  expect_equal(cx_spPr(fill = NULL, line_color = NULL), "")
})

test_that("cx_spPr combines fill and line", {
  out <- cx_spPr(fill = "red", line_color = "black", line_width = 1)
  expect_match(out, "^<cx:spPr>")
  expect_match(out, "</cx:spPr>$")
  expect_match(out, "srgbClr val=\"FF0000\"")
  expect_match(out, "<a:ln w=\"12700\"")
})

test_that("cx_dataPt wraps cx:spPr in <cx:dataPt idx=N>", {
  out <- cx_dataPt(2L, fill = "#123456")
  expect_match(out, "^<cx:dataPt idx=\"2\">")
  expect_match(out, "</cx:dataPt>$")
  expect_match(out, "srgbClr val=\"123456\"")
  expect_equal(cx_dataPt(0L), "") # no fill/line -> empty
})

# --- cx_rPr / cx_txPr / cx_rich_text via officer's format(fp, "pml") ---

test_that("cx_rPr_from_fp_text emits a complete <a:rPr> with sz/b/i/solidFill/latin", {
  fp <- officer::fp_text(
    font.size = 14,
    bold = TRUE,
    color = "red",
    font.family = "Calibri"
  )
  out <- cx_rPr_from_fp_text(fp)
  doc <- xml2::read_xml(paste0(
    "<root xmlns:cx=\"",
    cx_ns[["cx"]],
    "\" xmlns:a=\"",
    cx_ns[["a"]],
    "\">",
    out,
    "</root>"
  ))
  rpr <- xml2::xml_find_first(doc, "//a:rPr", cx_ns)
  expect_equal(xml2::xml_attr(rpr, "sz"), "1400")
  expect_equal(xml2::xml_attr(rpr, "b"), "1")
  fill <- xml2::xml_find_first(rpr, ".//a:srgbClr", cx_ns)
  expect_equal(xml2::xml_attr(fill, "val"), "FF0000")
  latin <- xml2::xml_find_first(rpr, ".//a:latin", cx_ns)
  expect_equal(xml2::xml_attr(latin, "typeface"), "Calibri")
})

test_that("cx_rPr_from_fp_text(as_def=TRUE) rewrites tag to <a:defRPr>", {
  fp <- officer::fp_text(font.size = 12)
  out <- cx_rPr_from_fp_text(fp, as_def = TRUE)
  expect_match(out, "<a:defRPr ")
  expect_false(grepl("<a:rPr ", out))
})

test_that("cx_rPr_from_fp_text(NULL) returns empty string", {
  expect_equal(cx_rPr_from_fp_text(NULL), "")
})

test_that("cx_txPr_from_fp_text wraps defRPr in a valid <cx:txPr>", {
  fp <- officer::fp_text(
    font.size = 12,
    bold = TRUE,
    color = "blue",
    font.family = "Calibri"
  )
  out <- cx_txPr_from_fp_text(fp)
  doc <- xml2::read_xml(paste0(
    "<root xmlns:cx=\"",
    cx_ns[["cx"]],
    "\" xmlns:a=\"",
    cx_ns[["a"]],
    "\">",
    out,
    "</root>"
  ))
  txPr <- xml2::xml_find_first(doc, "//cx:txPr", cx_ns)
  expect_false(inherits(txPr, "xml_missing"))
  defRPr <- xml2::xml_find_first(doc, "//a:defRPr", cx_ns)
  expect_equal(xml2::xml_attr(defRPr, "sz"), "1200")
  expect_equal(xml2::xml_attr(defRPr, "b"), "1")
  fill <- xml2::xml_find_first(defRPr, ".//a:srgbClr", cx_ns)
  expect_equal(xml2::xml_attr(fill, "val"), "0000FF")
})

test_that("cx_txPr_from_fp_text(NULL) returns empty string", {
  expect_equal(cx_txPr_from_fp_text(NULL), "")
})

test_that("cx_rich_text builds a valid <cx:tx><cx:rich> with rPr+text", {
  fp <- officer::fp_text(font.size = 18, bold = TRUE, color = "red")
  out <- cx_rich_text("Hello & <world>", fp = fp)
  doc <- xml2::read_xml(paste0(
    "<root xmlns:cx=\"",
    cx_ns[["cx"]],
    "\" xmlns:a=\"",
    cx_ns[["a"]],
    "\">",
    out,
    "</root>"
  ))
  rPr <- xml2::xml_find_first(doc, "//a:r/a:rPr", cx_ns)
  expect_equal(xml2::xml_attr(rPr, "sz"), "1800")
  expect_equal(xml2::xml_attr(rPr, "b"), "1")
  txt <- xml2::xml_find_first(doc, "//a:r/a:t", cx_ns)
  expect_equal(xml2::xml_text(txt), "Hello & <world>")
})

test_that("cx_rich_text without fp still emits a valid title fragment", {
  out <- cx_rich_text("plain title")
  expect_match(out, "<a:t>plain title</a:t>")
  expect_match(out, "<a:rPr/>")
})

test_that("chart_settings on chartEx types without options gives clear error", {
  funnel <- ms_funnelchart(
    data.frame(s = c("A", "B"), n = c(2, 1)),
    x = "s", y = "n"
  )
  expect_error(
    chart_settings(funnel),
    "chart_settings.*no options.*ms_funnelchart"
  )

  treemap <- ms_treemapchart(
    data.frame(g = c("a", "b"), v = c(1, 2)),
    path = "g", value = "v"
  )
  expect_error(
    chart_settings(treemap),
    "ms_treemapchart"
  )

  # pareto and boxplot still dispatch to their dedicated methods
  pa <- ms_paretochart(
    data.frame(x = c("A", "B"), y = c(2, 1)),
    x = "x", y = "y", aggregate = FALSE
  )
  expect_silent(chart_settings(pa, line = officer::fp_border(color = "red")))
})

#' @title Boxplot chart object
#' @description Creation of a box-and-whisker chart object that can be
#' inserted in a 'Microsoft' document. Boxplot charts use the chartEx
#' pipeline (Office 2016+); older versions of 'Microsoft Office' will
#' display a fallback placeholder.
#'
#' Data is in long format: one row per observation. Office computes
#' quartiles and whiskers from the raw values; do not pre-aggregate.
#' @param data a data.frame.
#' @param x category column name. Each unique value becomes one box.
#' @param y numeric value column name (raw observations).
#' @param quartile_method one of `"exclusive"` (default) or `"inclusive"`.
#' Affects how Q1/Q3 are computed when the count is even.
#' @param show_mean_marker logical, draw the mean as a marker. Default TRUE.
#' @param show_mean_line logical, draw a line connecting means across
#' boxes. Default FALSE.
#' @param show_outliers logical, plot outlier points. Default TRUE.
#' @param show_inner_points logical, plot all non-outlier points. Default FALSE.
#' @return An `ms_chart` object (subclass `ms_boxplotchart`).
#' @family 'Office' chart objects
#' @seealso [chart_labels()]
#' @export
#' @examples
#' library(officer)
#'
#' set.seed(1)
#' dat <- data.frame(
#'   group = rep(c("A", "B", "C"), each = 20),
#'   value = c(rnorm(20, 0, 5), rnorm(20, 3, 7), rnorm(20, -2, 4))
#' )
#' bp <- ms_boxplotchart(dat, x = "group", y = "value")
#'
#' doc <- read_pptx()
#' doc <- add_slide(doc)
#' doc <- ph_with(doc, bp, location = ph_location_fullsize())
#' print(doc, target = tempfile(fileext = ".pptx"))
ms_boxplotchart <- function(
  data,
  x,
  y,
  quartile_method = c("exclusive", "inclusive"),
  show_mean_marker = TRUE,
  show_mean_line = FALSE,
  show_outliers = TRUE,
  show_inner_points = FALSE
) {
  stopifnot(is.data.frame(data))
  stopifnot(is.character(x), length(x) == 1L)
  stopifnot(is.character(y), length(y) == 1L)
  quartile_method <- match.arg(quartile_method)

  if (!x %in% names(data)) {
    stop("column ", shQuote(x), " not found in data", call. = FALSE)
  }
  if (!y %in% names(data)) {
    stop("column ", shQuote(y), " not found in data", call. = FALSE)
  }
  if (!is.numeric(data[[y]])) {
    stop("column ", shQuote(y), " must be numeric", call. = FALSE)
  }
  if (
    inherits(data, "data.table") ||
      inherits(data, "tbl_df") ||
      inherits(data, "tbl")
  ) {
    data <- as.data.frame(data, stringsAsFactors = FALSE)
  }
  data[[x]] <- as.character(data[[x]])

  data_series <- data[, c(x, y), drop = FALSE]
  rownames(data_series) <- NULL

  out <- list(
    data = data,
    data_series = data_series,
    x = x,
    y = y,
    xvar = x,
    yvar = y,
    group = NULL,
    asis = FALSE,
    theme = mschart_theme(),
    labels = list(title = NULL, x = NULL, y = NULL),
    layout_id = "boxWhisker",
    quartile_method = quartile_method,
    show_mean_marker = isTRUE(show_mean_marker),
    show_mean_line = isTRUE(show_mean_line),
    show_outliers = isTRUE(show_outliers),
    show_inner_points = isTRUE(show_inner_points),
    options = list(line = NULL)
  )
  class(out) <- c("ms_boxplotchart", "ms_chart_ex", "ms_chart")
  out
}

#' @importFrom officer uuid_generate
#' @export
#' @method format ms_boxplotchart
format.ms_boxplotchart <- function(
  x,
  id_x = NULL,
  id_y = NULL,
  sheetname = "sheet1",
  drop_ext_data = FALSE,
  ...
) {
  ds <- x$data_series
  n <- nrow(ds)
  cat_col <- x$x
  val_col <- x$y
  row1 <- 2L
  row2 <- n + 1L

  cat_dim <- paste0(
    "<cx:strDim type=\"cat\">",
    sprintf("<cx:f>%s</cx:f>", cx_range(sheetname, "A", row1, row2)),
    cx_str_lvl(ds[[cat_col]]),
    "</cx:strDim>"
  )
  val_dim <- paste0(
    "<cx:numDim type=\"val\">",
    sprintf("<cx:f>%s</cx:f>", cx_range(sheetname, "B", row1, row2)),
    cx_num_lvl(ds[[val_col]], format_code = "Standard"),
    "</cx:numDim>"
  )

  unique_id <- x$unique_id %||% uuid_generate()
  fill <- cx_render_series_fill(x)
  # Inject a default stroke override only when chart_data_fill/stroke
  # produced no <cx:spPr>: a series can only carry one <cx:spPr> block
  # so a user override always wins.
  default_line_spPr <- if (nzchar(fill$spPr)) {
    ""
  } else {
    cx_default_line_spPr(x$options$line)
  }
  visibility_attrs <- sprintf(
    "meanLine=\"%d\" meanMarker=\"%d\" nonoutliers=\"%d\" outliers=\"%d\"",
    as.integer(x$show_mean_line),
    as.integer(x$show_mean_marker),
    as.integer(x$show_inner_points),
    as.integer(x$show_outliers)
  )

  series_xml <- paste0(
    sprintf(
      "<cx:series layoutId=\"boxWhisker\" uniqueId=\"%s\">",
      unique_id
    ),
    "<cx:tx><cx:txData>",
    sprintf("<cx:f>%s</cx:f>", cx_cell(sheetname, "B", 1L)),
    sprintf("<cx:v>%s</cx:v>", htmltools::htmlEscape(val_col)),
    "</cx:txData></cx:tx>",
    fill$spPr,
    default_line_spPr,
    fill$dataPts,
    cx_data_labels_xml(
      defaults = NULL,
      opts = x$cx_data_labels,
      fp = x$cx_data_labels$fp
    ),
    "<cx:dataId val=\"0\"/>",
    "<cx:layoutPr>",
    sprintf("<cx:visibility %s/>", visibility_attrs),
    sprintf(
      "<cx:statistics quartileMethod=\"%s\"/>",
      x$quartile_method
    ),
    "</cx:layoutPr>",
    "</cx:series>"
  )

  axx <- x$cx_axis_x %||% list()
  axy <- x$cx_axis_y %||% list()
  axes_xml <- paste0(
    cx_axis(
      "0",
      "<cx:catScaling gapWidth=\"1\"/>",
      fp = x$theme$axis_text_x,
      title = x$labels[["x"]],
      title_fp = x$theme$axis_title_x,
      num_fmt = axx$num_fmt
    ),
    cx_axis(
      "1",
      cx_val_scaling(min = axy$limit_min, max = axy$limit_max),
      fp = x$theme$axis_text_y,
      title = x$labels[["y"]],
      title_fp = x$theme$axis_title_y,
      major_grid = axy$major_grid %||% TRUE,
      minor_grid = axy$minor_grid %||% FALSE,
      num_fmt = axy$num_fmt
    )
  )

  title_xml <- cx_chart_title(x$labels[["title"]], fp = x$theme$main_title)

  ext_data_xml <- if (drop_ext_data) {
    ""
  } else {
    "<cx:externalData r:id=\"rId1\" cx:autoUpdate=\"0\"/>"
  }

  paste0(
    "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>",
    "<cx:chartSpace",
    " xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\"",
    " xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\"",
    " xmlns:cx=\"http://schemas.microsoft.com/office/drawing/2014/chartex\">",
    "<cx:chartData>",
    ext_data_xml,
    "<cx:data id=\"0\">",
    cat_dim,
    val_dim,
    "</cx:data>",
    "</cx:chartData>",
    "<cx:chart>",
    title_xml,
    "<cx:plotArea><cx:plotAreaRegion>",
    series_xml,
    "</cx:plotAreaRegion>",
    axes_xml,
    "</cx:plotArea>",
    "</cx:chart>",
    "</cx:chartSpace>"
  )
}

#' @export
#' @method print ms_boxplotchart
print.ms_boxplotchart <- function(x, preview = FALSE, ...) {
  if (preview && interactive()) {
    doc <- officer::read_pptx()
    doc <- officer::add_slide(
      doc,
      layout = "Title and Content",
      master = "Office Theme"
    )
    doc <- officer::ph_with(doc, x, location = officer::ph_location_fullsize())
    file_out <- print(doc, target = tempfile(fileext = ".pptx"))
    utils::browseURL(file_out)
    return(invisible())
  }
  cat(sprintf("* %s object (chartEx)\n\n", shQuote("ms_boxplotchart")))
  cat(sprintf("* x: %s\n", x$x))
  cat(sprintf("* y: %s\n", x$y))
  cat(sprintf("* quartile method: %s\n\n", x$quartile_method))
  cat(sprintf(
    "* original data [%.0f,%.0f] (sample):\n",
    nrow(x$data),
    ncol(x$data)
  ))
  print(x$data[seq_len(min(c(nrow(x$data), 5))), ])
}

#' @title Pareto chart object
#' @description Creation of a Pareto chart object that can be inserted
#' in a 'Microsoft' document. Pareto charts use the chartEx pipeline
#' (Office 2016+); older versions of 'Microsoft Office' will display a
#' fallback placeholder.
#'
#' Office draws columns sorted by descending count plus a cumulative
#' line on a secondary percentage axis.
#'
#' Two input modes are supported:
#' * `aggregate = TRUE` (default): `data` is in long format (one row per
#'   observation) and Office counts occurrences of each `x` value.
#'   `y` is optional; when supplied, values are summed per category.
#' * `aggregate = FALSE`: `data` is already aggregated (one row per
#'   category) and `y` is the count/value column.
#' @param data a data.frame.
#' @param x category column name.
#' @param y optional numeric column. With `aggregate=TRUE` and `y=NULL`,
#' each row counts as 1.
#' @param aggregate logical, see Description. Default TRUE.
#' @return An `ms_chart` object (subclass `ms_paretochart`).
#' @family 'Office' chart objects
#' @seealso [chart_labels()]
#' @export
#' @examples
#' library(officer)
#'
#' set.seed(1)
#' dat <- data.frame(
#'   defect = sample(c("A", "B", "C", "D"), 50, replace = TRUE,
#'                   prob = c(0.5, 0.25, 0.15, 0.1))
#' )
#' pa <- ms_paretochart(dat, x = "defect")
#'
#' doc <- read_pptx()
#' doc <- add_slide(doc)
#' doc <- ph_with(doc, pa, location = ph_location_fullsize())
#' print(doc, target = tempfile(fileext = ".pptx"))
ms_paretochart <- function(data, x, y = NULL, aggregate = TRUE) {
  stopifnot(is.data.frame(data))
  stopifnot(is.character(x), length(x) == 1L)
  if (!x %in% names(data)) {
    stop("column ", shQuote(x), " not found in data", call. = FALSE)
  }
  if (!is.null(y)) {
    stopifnot(is.character(y), length(y) == 1L)
    if (!y %in% names(data)) {
      stop("column ", shQuote(y), " not found in data", call. = FALSE)
    }
    if (!is.numeric(data[[y]])) {
      stop("column ", shQuote(y), " must be numeric", call. = FALSE)
    }
  }
  if (
    inherits(data, "data.table") ||
      inherits(data, "tbl_df") ||
      inherits(data, "tbl")
  ) {
    data <- as.data.frame(data, stringsAsFactors = FALSE)
  }
  data[[x]] <- as.character(data[[x]])

  # Build the embedded series: when y is omitted, fill column B with 1s
  # (Office aggregates these to a count). When y is provided, keep it.
  # data_series has a fixed positional layout (col 1 = category, col 2 = value);
  # downstream code accesses it by position, so column names only matter as the
  # Excel header. Use make.unique to avoid duplicate headers when the user's
  # x column happens to share its name with the synthesized value column.
  y_label <- y %||% "count"
  if (is.null(y)) {
    data_series <- data.frame(
      data[[x]],
      rep(1L, nrow(data)),
      stringsAsFactors = FALSE
    )
  } else {
    data_series <- data[, c(x, y), drop = FALSE]
  }
  names(data_series) <- make.unique(c(x, y_label))
  rownames(data_series) <- NULL

  out <- list(
    data = data,
    data_series = data_series,
    x = x,
    y = y_label,
    xvar = x,
    yvar = y_label,
    group = NULL,
    asis = FALSE,
    theme = mschart_theme(),
    labels = list(title = NULL, x = NULL, y = NULL),
    layout_id = "paretoLine",
    aggregate = isTRUE(aggregate),
    options = list(line = NULL)
  )
  class(out) <- c("ms_paretochart", "ms_chart_ex", "ms_chart")
  out
}

#' @export
#' @method format ms_paretochart
format.ms_paretochart <- function(
  x,
  id_x = NULL,
  id_y = NULL,
  sheetname = "sheet1",
  drop_ext_data = FALSE,
  ...
) {
  ds <- x$data_series
  n <- nrow(ds)
  # data_series has a fixed layout: col 1 = category, col 2 = value.
  # Access by position so that user column names (e.g. x named "count") cannot
  # collide with the synthesized value column. x$y stays as the display label.
  row1 <- 2L
  row2 <- n + 1L

  cat_dim <- paste0(
    "<cx:strDim type=\"cat\">",
    sprintf("<cx:f>%s</cx:f>", cx_range(sheetname, "A", row1, row2)),
    cx_str_lvl(ds[[1L]]),
    "</cx:strDim>"
  )
  val_dim <- paste0(
    "<cx:numDim type=\"val\">",
    sprintf("<cx:f>%s</cx:f>", cx_range(sheetname, "B", row1, row2)),
    cx_num_lvl(ds[[2L]], format_code = "Standard"),
    "</cx:numDim>"
  )

  unique_id_col <- x$unique_id %||% uuid_generate()
  unique_id_line <- uuid_generate()
  agg_xml <- if (x$aggregate) "<cx:aggregation/>" else ""
  fill <- cx_render_series_fill(x)

  series_col_xml <- paste0(
    sprintf(
      "<cx:series layoutId=\"clusteredColumn\" uniqueId=\"%s\">",
      unique_id_col
    ),
    "<cx:tx><cx:txData>",
    sprintf("<cx:f>%s</cx:f>", cx_cell(sheetname, "B", 1L)),
    sprintf("<cx:v>%s</cx:v>", htmltools::htmlEscape(x$y)),
    "</cx:txData></cx:tx>",
    fill$spPr,
    fill$dataPts,
    cx_data_labels_xml(
      defaults = NULL,
      opts = x$cx_data_labels,
      fp = x$cx_data_labels$fp
    ),
    "<cx:dataId val=\"0\"/>",
    "<cx:layoutPr>",
    agg_xml,
    "</cx:layoutPr>",
    "<cx:axisId val=\"1\"/>",
    "</cx:series>"
  )

  series_line_xml <- paste0(
    sprintf(
      "<cx:series layoutId=\"paretoLine\" ownerIdx=\"0\" uniqueId=\"%s\">",
      unique_id_line
    ),
    cx_default_line_spPr(x$options$line),
    "<cx:axisId val=\"2\"/>",
    "</cx:series>"
  )

  axx <- x$cx_axis_x %||% list()
  axy <- x$cx_axis_y %||% list()
  axes_xml <- paste0(
    cx_axis(
      "0",
      "<cx:catScaling gapWidth=\"0\"/>",
      fp = x$theme$axis_text_x,
      title = x$labels[["x"]],
      title_fp = x$theme$axis_title_x,
      num_fmt = axx$num_fmt
    ),
    cx_axis(
      "1",
      cx_val_scaling(min = axy$limit_min, max = axy$limit_max),
      fp = x$theme$axis_text_y,
      title = x$labels[["y"]],
      title_fp = x$theme$axis_title_y,
      major_grid = axy$major_grid %||% TRUE,
      minor_grid = axy$minor_grid %||% FALSE,
      num_fmt = axy$num_fmt
    ),
    # Axis 2 (cumulative percentage) is fixed by the pareto layout.
    cx_axis(
      "2",
      "<cx:valScaling max=\"1\" min=\"0\"/>",
      "<cx:units unit=\"percentage\"/>",
      fp = x$theme$axis_text_y
    )
  )

  title_xml <- cx_chart_title(x$labels[["title"]], fp = x$theme$main_title)

  ext_data_xml <- if (drop_ext_data) {
    ""
  } else {
    "<cx:externalData r:id=\"rId1\" cx:autoUpdate=\"0\"/>"
  }

  paste0(
    "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>",
    "<cx:chartSpace",
    " xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\"",
    " xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\"",
    " xmlns:cx=\"http://schemas.microsoft.com/office/drawing/2014/chartex\">",
    "<cx:chartData>",
    ext_data_xml,
    "<cx:data id=\"0\">",
    cat_dim,
    val_dim,
    "</cx:data>",
    "</cx:chartData>",
    "<cx:chart>",
    title_xml,
    "<cx:plotArea><cx:plotAreaRegion>",
    series_col_xml,
    series_line_xml,
    "</cx:plotAreaRegion>",
    axes_xml,
    "</cx:plotArea>",
    "</cx:chart>",
    "</cx:chartSpace>"
  )
}

#' @export
#' @method print ms_paretochart
print.ms_paretochart <- function(x, preview = FALSE, ...) {
  if (preview && interactive()) {
    doc <- officer::read_pptx()
    doc <- officer::add_slide(
      doc,
      layout = "Title and Content",
      master = "Office Theme"
    )
    doc <- officer::ph_with(doc, x, location = officer::ph_location_fullsize())
    file_out <- print(doc, target = tempfile(fileext = ".pptx"))
    utils::browseURL(file_out)
    return(invisible())
  }
  cat(sprintf("* %s object (chartEx)\n\n", shQuote("ms_paretochart")))
  cat(sprintf("* x: %s\n", x$x))
  cat(sprintf("* y: %s\n", x$y))
  cat(sprintf("* aggregate: %s\n\n", x$aggregate))
  cat(sprintf(
    "* original data [%.0f,%.0f] (sample):\n",
    nrow(x$data),
    ncol(x$data)
  ))
  print(x$data[seq_len(min(c(nrow(x$data), 5))), , drop = FALSE])
}

funnel_data <- function() {
  data.frame(
    stage = c("Visitors", "Leads", "Opportunities", "Quotes", "Customers"),
    count = c(5000, 4000, 3000, 1000, 250),
    stringsAsFactors = FALSE
  )
}

test_that("ms_funnelchart constructor validates inputs", {
  d <- funnel_data()
  expect_error(ms_funnelchart(d, x = "missing", y = "count"), "not found")
  expect_error(ms_funnelchart(d, x = "stage", y = "missing"), "not found")
  expect_error(ms_funnelchart(d, x = "stage", y = "stage"), "must be numeric")
})

test_that("ms_funnelchart format produces well-formed cx XML", {
  fn <- ms_funnelchart(funnel_data(), x = "stage", y = "count")
  xml_str <- format(fn, sheetname = "sheet1")
  doc <- xml2::read_xml(xml_str)
  ns <- c(cx = "http://schemas.microsoft.com/office/drawing/2014/chartex")

  cat_lvls <- xml2::xml_find_all(doc, "//cx:strDim[@type='cat']/cx:lvl", ns)
  expect_length(cat_lvls, 1L)
  val_lvls <- xml2::xml_find_all(doc, "//cx:numDim[@type='val']/cx:lvl", ns)
  expect_length(val_lvls, 1L)

  series <- xml2::xml_find_all(doc, "//cx:series", ns)
  expect_length(series, 1L)
  expect_equal(xml2::xml_attr(series[[1]], "layoutId"), "funnel")

  # single axis id="1"
  axes <- xml2::xml_find_all(doc, "//cx:plotArea/cx:axis", ns)
  expect_length(axes, 1L)
  expect_equal(xml2::xml_attr(axes[[1]], "id"), "1")
})

test_that("ms_funnelchart end-to-end pptx", {
  skip_if_not_installed("officer")
  skip_if_not_installed("writexl")

  fn <- ms_funnelchart(funnel_data(), x = "stage", y = "count")
  doc <- officer::read_pptx()
  doc <- officer::add_slide(
    doc,
    layout = "Title and Content",
    master = "Office Theme"
  )
  doc <- officer::ph_with(doc, fn, location = officer::ph_location_fullsize())
  out <- tempfile(fileext = ".pptx")
  expect_silent(print(doc, target = out))
  expect_true(file.exists(out))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{mschart}
\alias{mschart-package}
\alias{mschart}
\title{Chart Generation for 'Microsoft Word', 'Microsoft Excel' and 'Microsoft PowerPoint' Documents}
\description{
It lets R users create Microsoft Office charts from data, and
then add title, legends, and annotations to the chart object.

The graph produced is a Microsoft graph, which means that it can be edited in
your Microsoft software and that the underlying data are available.

The package will not allow you to make the same charts as with ggplot2. It
allows only a subset of the charts possible with 'Office Chart'. The package is
often used to industrialize graphs that are then consumed and annotated by non-R
users.

The following chart types are available:
\itemize{
\item bar charts: \code{\link[=ms_barchart]{ms_barchart()}}
\item line charts: \code{\link[=ms_linechart]{ms_linechart()}}
\item scatter plots: \code{\link[=ms_scatterchart]{ms_scatterchart()}}
\item area charts: \code{\link[=ms_areachart]{ms_areachart()}}
\item pie and doughnut charts: \code{\link[=ms_piechart]{ms_piechart()}}
\item bubble charts: \code{\link[=ms_bubblechart]{ms_bubblechart()}}
\item radar (spider) charts: \code{\link[=ms_radarchart]{ms_radarchart()}}
\item stock charts (HLC and OHLC): \code{\link[=ms_stockchart]{ms_stockchart()}}
}

Several chart types can be combined on a single chart, with an
optional secondary axis, using \code{\link[=ms_chart_combine]{ms_chart_combine()}}.

These functions create a 'chart' object that can be customized:
\itemize{
\item by using options specific to the chart (with \code{\link[=chart_settings]{chart_settings()}}),
\item by changing the options related to the axes (with \code{\link[=chart_ax_x]{chart_ax_x()}} and \code{\link[=chart_ax_y]{chart_ax_y()}}),
\item by changing the options related to the labels (with \code{\link[=chart_data_labels]{chart_data_labels()}}),
\item by changing the colors, line widths, ... with functions
\itemize{
\item \code{\link[=chart_labels_text]{chart_labels_text()}}
\item \code{\link[=chart_data_fill]{chart_data_fill()}}
\item \code{\link[=chart_data_line_style]{chart_data_line_style()}}
\item \code{\link[=chart_data_line_width]{chart_data_line_width()}}
\item \code{\link[=chart_data_size]{chart_data_size()}}
\item \code{\link[=chart_data_smooth]{chart_data_smooth()}}
\item \code{\link[=chart_data_stroke]{chart_data_stroke()}}
\item \code{\link[=chart_data_symbol]{chart_data_symbol()}}
}
\item by changing the general theme with function \code{\link[=chart_theme]{chart_theme()}},
\item by changing the title labels with function \code{\link[=chart_labels]{chart_labels()}}.
}

You can add a chart into a slide in PowerPoint with function \code{\link[=ph_with.ms_chart]{ph_with.ms_chart()}}.

You can add a chart into a Word document with function \code{\link[=body_add_chart]{body_add_chart()}}.

You can add a chart into an Excel sheet with function \code{\link[=sheet_add_drawing.ms_chart]{sheet_add_drawing.ms_chart()}}.
\subsection{Series styling properties by chart type}{

Not all series styling properties have an effect on every
chart type. The following table shows which properties are
supported. A warning is emitted when a property is set on
a chart type that does not support it.\tabular{lcccccccc}{
   Property \tab bar \tab line \tab area \tab scatter \tab stock \tab radar \tab bubble \tab pie \cr
   fill \tab x \tab x \tab x \tab x \tab x \tab x \tab x \tab x \cr
   colour \tab x \tab x \tab x \tab x \tab x \tab x \tab x \tab x \cr
   symbol \tab  \tab x \tab  \tab x \tab x \tab x \tab  \tab  \cr
   size \tab  \tab x \tab  \tab x \tab x \tab x \tab  \tab  \cr
   line_width \tab x \tab x \tab x \tab x \tab x \tab x \tab x \tab x \cr
   line_style \tab  \tab x \tab  \tab x \tab x \tab x \tab  \tab  \cr
   smooth \tab  \tab x \tab  \tab x \tab  \tab  \tab  \tab  \cr
   labels_fp \tab x \tab x \tab x \tab x \tab  \tab x \tab  \tab x \cr
}

}
}
\seealso{
\url{https://ardata-fr.github.io/officeverse/}
}
\author{
\strong{Maintainer}: David Gohel \email{david.gohel@ardata.fr}

Other contributors:
\itemize{
  \item ArData [copyright holder]
  \item YouGov [funder]
  \item Jan Marvin Garbuszus (support for openxlsx2 and combo charts) [contributor]
  \item Stefan Moog \email{moogs@gmx.de} (support to set chart and plot area color and border, fix for line style none) [contributor]
  \item Eli Daniels \email{eli.daniels@ardata.fr} [contributor]
  \item Marlon Molina (added table feature) [contributor]
  \item Rokas Klydzia (custom labels) [contributor]
  \item David Camposeco \email{david.camposeco.paulsen@gmail.com} (chart_data_smooth function) [contributor]
  \item Dan Joplin (fix scatter plot data structure) [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_chart.R
\name{ms_bubblechart}
\alias{ms_bubblechart}
\title{Bubblechart object}
\usage{
ms_bubblechart(data, x, y, size, group = NULL, labels = NULL, asis = FALSE)
}
\arguments{
\item{data}{a data.frame}

\item{x}{column name for x values.}

\item{y}{column name for y values.}

\item{size}{column name for bubble size values (must be numeric)}

\item{group}{grouping column name used to split data into series. Optional.}

\item{labels}{column names of columns to be used as custom data labels
displayed next to data points (not axis labels). Optional.
If more than one name is provided, only the first one will be used as a label, but all
labels (transposed if a group is used) will be available in the Excel file
associated with the chart.}

\item{asis}{logical parameter defaulting to FALSE. When FALSE, the data is
reshaped internally so that each series becomes a separate column. When TRUE,
the data is used as-is and must already have one column for categories and
one column per series.}
}
\value{
An \code{ms_chart} object.
}
\description{
Creation of a bubblechart object that can be
inserted in a 'Microsoft' document. A bubble chart is a scatter
chart where each point has a third numeric dimension controlling
its size.
}
\examples{
library(officer)

dat <- data.frame(
  x = c(1, 2, 3, 4, 5),
  y = c(10, 20, 15, 25, 30),
  sz = c(5, 10, 7, 15, 12),
  grp = rep("s1", 5)
)

bubble <- ms_bubblechart(
  data = dat, x = "x", y = "y",
  size = "sz", group = "grp"
)

# adjust axes to avoid clipping extreme bubbles
bubble <- chart_ax_x(bubble, limit_min = 0, limit_max = 6)
bubble <- chart_ax_y(bubble, limit_min = 5, limit_max = 35)
bubble
}
\seealso{
\code{\link[=chart_settings]{chart_settings()}}, \code{\link[=chart_ax_x]{chart_ax_x()}}, \code{\link[=chart_ax_y]{chart_ax_y()}},
\code{\link[=chart_data_labels]{chart_data_labels()}}, \code{\link[=chart_theme]{chart_theme()}}, \code{\link[=chart_labels]{chart_labels()}}

Other 'Office' chart objects: 
\code{\link{ms_areachart}()},
\code{\link{ms_barchart}()},
\code{\link{ms_chart_combine}()},
\code{\link{ms_linechart}()},
\code{\link{ms_piechart}()},
\code{\link{ms_radarchart}()},
\code{\link{ms_scatterchart}()},
\code{\link{ms_stockchart}()}
}
\concept{'Office' chart objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_chart.R
\name{ms_stockchart}
\alias{ms_stockchart}
\title{Stockchart object}
\usage{
ms_stockchart(data, x, open = NULL, high, low, close)
}
\arguments{
\item{data}{a data.frame}

\item{x}{column name for categories (typically dates)}

\item{open}{column name for open values (optional, enables OHLC mode)}

\item{high}{column name for high values}

\item{low}{column name for low values}

\item{close}{column name for close values}
}
\value{
An \code{ms_chart} object.
}
\description{
Creation of a stock chart object that can be inserted
in a 'Microsoft' document. When \code{open} is omitted the chart is
a High-Low-Close chart. When \code{open} is provided it becomes an
Open-High-Low-Close chart with up/down bars (candlestick).
}
\examples{
library(officer)

dat <- data.frame(
  date = as.Date("2024-01-01") + 0:4,
  open = c(44, 25, 38, 50, 34),
  high = c(55, 57, 57, 58, 58),
  low = c(11, 12, 13, 11, 25),
  close = c(32, 35, 34, 35, 43)
)

# HLC chart
stock_hlc <- ms_stockchart(
  data = dat, x = "date",
  high = "high", low = "low", close = "close"
)
stock_hlc

# OHLC chart (candlestick)
stock_ohlc <- ms_stockchart(
  data = dat, x = "date",
  open = "open", high = "high",
  low = "low", close = "close"
)
stock_ohlc
}
\seealso{
\code{\link[=chart_settings]{chart_settings()}}, \code{\link[=chart_ax_x]{chart_ax_x()}}, \code{\link[=chart_ax_y]{chart_ax_y()}},
\code{\link[=chart_theme]{chart_theme()}}, \code{\link[=chart_labels]{chart_labels()}}

Other 'Office' chart objects: 
\code{\link{ms_areachart}()},
\code{\link{ms_barchart}()},
\code{\link{ms_bubblechart}()},
\code{\link{ms_chart_combine}()},
\code{\link{ms_linechart}()},
\code{\link{ms_piechart}()},
\code{\link{ms_radarchart}()},
\code{\link{ms_scatterchart}()}
}
\concept{'Office' chart objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chart_data_styles.R
\name{chart_labels_text}
\alias{chart_labels_text}
\title{Modify labels font settings}
\usage{
chart_labels_text(x, values)
}
\arguments{
\item{x}{an \code{ms_chart} object.}

\item{values}{a named list of \code{\link[officer:fp_text]{officer::fp_text()}} objects to map data labels to.
It is a named list, the values will be matched based on the names.
If it contains only one \code{\link[officer:fp_text]{officer::fp_text()}} object, it will be associated to all existing series.}
}
\value{
An \code{ms_chart} object.
}
\description{
Specify mappings from levels in the data to displayed text font settings.
}
\examples{
library(officer)

fp_text_settings <- list(
  serie1 = fp_text(font.size = 7, color = "red"),
  serie2 = fp_text(font.size = 0, color = "purple"),
  serie3 = fp_text(font.size = 19, color = "wheat")
)

barchart <- ms_barchart(
  data = browser_data,
  x = "browser", y = "value", group = "serie")
barchart <- chart_data_labels(barchart, show_val = TRUE)
barchart <- chart_labels_text( barchart,
  values = fp_text_settings )
}
\seealso{
Other Series customization functions: 
\code{\link{chart_data_fill}()},
\code{\link{chart_data_line_style}()},
\code{\link{chart_data_line_width}()},
\code{\link{chart_data_size}()},
\code{\link{chart_data_smooth}()},
\code{\link{chart_data_stroke}()},
\code{\link{chart_data_symbol}()}
}
\concept{Series customization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_funnelchart.R
\name{ms_funnelchart}
\alias{ms_funnelchart}
\title{Funnel chart object}
\usage{
ms_funnelchart(data, x, y)
}
\arguments{
\item{data}{a data.frame.}

\item{x}{category column name (stage label).}

\item{y}{numeric value column name.}
}
\value{
An \code{ms_chart} object (subclass \code{ms_funnelchart}).
}
\description{
Creation of a funnel chart object that can be inserted
in a 'Microsoft' document. Funnel charts use the chartEx pipeline
(Office 2016+); older versions of 'Microsoft Office' will display a
fallback placeholder.

Each row is one stage of the funnel. Values are typically decreasing
(e.g. visitors -> leads -> customers). Bars are centered horizontally
and width is proportional to the value.
}
\examples{
library(officer)

dat <- data.frame(
  stage = c("Visitors", "Leads", "Opportunities", "Quotes", "Customers"),
  count = c(5000, 4000, 3000, 1000, 250),
  stringsAsFactors = FALSE
)
fn <- ms_funnelchart(data = dat, x = "stage", y = "count")

doc <- read_pptx()
doc <- add_slide(doc)
doc <- ph_with(doc, fn, location = ph_location_fullsize())
print(doc, target = tempfile(fileext = ".pptx"))
}
\seealso{
\code{\link[=chart_labels]{chart_labels()}}

Other 'Office' chart objects:
\code{\link[=ms_areachart]{ms_areachart()}},
\code{\link[=ms_barchart]{ms_barchart()}},
\code{\link[=ms_boxplotchart]{ms_boxplotchart()}},
\code{\link[=ms_bubblechart]{ms_bubblechart()}},
\code{\link[=ms_chart_combine]{ms_chart_combine()}},
\code{\link[=ms_histogramchart]{ms_histogramchart()}},
\code{\link[=ms_linechart]{ms_linechart()}},
\code{\link[=ms_paretochart]{ms_paretochart()}},
\code{\link[=ms_piechart]{ms_piechart()}},
\code{\link[=ms_radarchart]{ms_radarchart()}},
\code{\link[=ms_scatterchart]{ms_scatterchart()}},
\code{\link[=ms_stockchart]{ms_stockchart()}},
\code{\link[=ms_sunburstchart]{ms_sunburstchart()}},
\code{\link[=ms_treemapchart]{ms_treemapchart()}},
\code{\link[=ms_waterfallchart]{ms_waterfallchart()}}
}
\concept{'Office' chart objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\name{browser_data}
\alias{browser_data}
\title{Dummy dataset for barchart}
\format{
A data frame with 18 rows and 3 variables
}
\description{
A dataset containing two categorical variables and one integer variable:
}
\details{
\itemize{
\item browser web browser
\item serie id of series
\item value integer values
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes.R
\name{chart_fill_ggplot2}
\alias{chart_fill_ggplot2}
\title{Apply ggplot2 color scale}
\usage{
chart_fill_ggplot2(x, stroke = TRUE)
}
\arguments{
\item{x}{a mschart object}

\item{stroke}{a boolean. Apply the color scale to stroke? Defaults to \code{TRUE}.}
}
\value{
a mschart object
}
\description{
The default hue color scale from ggplot2.
}
\section{chart_fill_ggplot2()}{


\if{html}{\figure{fig_theme_ggplot2.png}{options: width="500"}}
}

\examples{
p <- ms_scatterchart(
  data = iris, x = "Sepal.Length",
  y = "Sepal.Width", group = "Species"
)

p <- theme_ggplot2(p)
p <- chart_fill_ggplot2(p)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_chart.R
\name{print.ms_chart}
\alias{print.ms_chart}
\title{Print method for ms_chart}
\usage{
\method{print}{ms_chart}(x, preview = FALSE, ...)
}
\arguments{
\item{x}{an \code{ms_chart} object.}

\item{preview}{preview the chart in a PowerPoint document}

\item{...}{unused}
}
\value{
No return value, called for side effects.
}
\description{
An \code{ms_chart} object cannot be rendered
in R. The default printing method will only display
simple information about the object.
If argument \code{preview} is set to TRUE, a \code{pptx} file
will be produced and opened with function \code{browseURL}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chart_data_styles.R
\name{chart_data_line_width}
\alias{chart_data_line_width}
\title{Modify line width}
\usage{
chart_data_line_width(x, values)
}
\arguments{
\item{x}{an \code{ms_chart} object.}

\item{values}{\verb{double(num of series)}: a set of size values to map data values to.
It is a named vector, the values will be matched based on the names.
If it contains only one size, this size will be associated to all existing series.}
}
\value{
An \code{ms_chart} object.
}
\description{
Specify mappings from levels in the data to the displayed line width.
}
\examples{
my_scatter <- ms_scatterchart(data = iris, x = "Sepal.Length",
  y = "Sepal.Width",  group = "Species")
my_scatter <- chart_settings(my_scatter, style = "lineMarker")
my_scatter <- chart_data_fill(my_scatter,
  values = c(virginica = "#6FA2FF", versicolor = "#FF6161", setosa = "#81FF5B") )
my_scatter <- chart_data_stroke(my_scatter,
  values = c(virginica = "black", versicolor = "black", setosa = "black") )
my_scatter <- chart_data_symbol(my_scatter,
  values = c(virginica = "circle", versicolor = "diamond", setosa = "circle") )
my_scatter <- chart_data_size(my_scatter,
  values = c(virginica = 20, versicolor = 16, setosa = 20) )
my_scatter <- chart_data_line_width(my_scatter,
  values = c(virginica = 2, versicolor = 3, setosa = 6) )
}
\seealso{
Other Series customization functions:
\code{\link[=chart_data_fill]{chart_data_fill()}},
\code{\link[=chart_data_line_style]{chart_data_line_style()}},
\code{\link[=chart_data_size]{chart_data_size()}},
\code{\link[=chart_data_smooth]{chart_data_smooth()}},
\code{\link[=chart_data_stroke]{chart_data_stroke()}},
\code{\link[=chart_data_symbol]{chart_data_symbol()}},
\code{\link[=chart_labels_text]{chart_labels_text()}}
}
\concept{Series customization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ph_with_chart.R
\name{ph_with.ms_chart}
\alias{ph_with.ms_chart}
\title{Add a MS Chart output into a PowerPoint object}
\usage{
\method{ph_with}{ms_chart}(x, value, location, ...)
}
\arguments{
\item{x}{a pptx device}

\item{value}{chart object}

\item{location}{a location for a placeholder.}

\item{...}{Arguments to be passed to methods.}
}
\value{
An rpptx object.
}
\description{
Produces a Microsoft Chart graphics output from R instructions
and adds the result in a PowerPoint document object produced
by \code{\link[officer:read_pptx]{officer::read_pptx()}}.
}
\examples{
my_barchart <- ms_barchart(data = browser_data,
  x = "browser", y = "value", group = "serie")
my_barchart <- chart_settings( x = my_barchart,
  dir="vertical", grouping="clustered", gap_width = 50 )
my_barchart <- chart_ax_x( x= my_barchart,
  cross_between = 'between', major_tick_mark="out")
my_barchart <- chart_ax_y( x= my_barchart,
  cross_between = "midCat", major_tick_mark="in")

library(officer)
doc <- read_pptx()
doc <- add_slide(doc, "Title and Content", "Office Theme")
doc <- ph_with(doc, my_barchart, location = ph_location_fullsize())

fileout <- tempfile(fileext = ".pptx")
print(doc, target = fileout)
}
\seealso{
\code{\link[=body_add_chart]{body_add_chart()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/body_add_chart.R
\name{body_add_chart}
\alias{body_add_chart}
\title{Add a chart to a Word document}
\usage{
body_add_chart(x, chart, style = NULL, pos = "after", width = 5, height = 3)
}
\arguments{
\item{x}{an rdocx object}

\item{chart}{an \code{ms_chart} object.}

\item{style}{paragraph style}

\item{pos}{where to add the new element relative to the cursor,
one of "after", "before", "on".}

\item{height, width}{height and width in inches.}
}
\value{
An rdocx object.
}
\description{
Add a \code{ms_chart} to an rdocx object. The graphic will be
inserted in an empty paragraph.
}
\examples{
library(officer)
my_barchart <- ms_barchart(data = browser_data,
  x = "browser", y = "value", group = "serie")
my_barchart <- chart_settings( my_barchart, grouping = "stacked",
  gap_width = 50, overlap = 100 )

doc <- read_docx()
doc <- body_add_chart(doc, chart = my_barchart, style = "centered")
print(doc, target = tempfile(fileext = ".docx"))
}
\seealso{
\code{\link[=ph_with.ms_chart]{ph_with.ms_chart()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_boxplotchart.R
\name{ms_boxplotchart}
\alias{ms_boxplotchart}
\title{Boxplot chart object}
\usage{
ms_boxplotchart(
  data,
  x,
  y,
  quartile_method = c("exclusive", "inclusive"),
  show_mean_marker = TRUE,
  show_mean_line = FALSE,
  show_outliers = TRUE,
  show_inner_points = FALSE
)
}
\arguments{
\item{data}{a data.frame.}

\item{x}{category column name. Each unique value becomes one box.}

\item{y}{numeric value column name (raw observations).}

\item{quartile_method}{one of \code{"exclusive"} (default) or \code{"inclusive"}.
Affects how Q1/Q3 are computed when the count is even.}

\item{show_mean_marker}{logical, draw the mean as a marker. Default TRUE.}

\item{show_mean_line}{logical, draw a line connecting means across
boxes. Default FALSE.}

\item{show_outliers}{logical, plot outlier points. Default TRUE.}

\item{show_inner_points}{logical, plot all non-outlier points. Default FALSE.}
}
\value{
An \code{ms_chart} object (subclass \code{ms_boxplotchart}).
}
\description{
Creation of a box-and-whisker chart object that can be
inserted in a 'Microsoft' document. Boxplot charts use the chartEx
pipeline (Office 2016+); older versions of 'Microsoft Office' will
display a fallback placeholder.

Data is in long format: one row per observation. Office computes
quartiles and whiskers from the raw values; do not pre-aggregate.
}
\examples{
library(officer)

set.seed(1)
dat <- data.frame(
  group = rep(c("A", "B", "C"), each = 20),
  value = c(rnorm(20, 0, 5), rnorm(20, 3, 7), rnorm(20, -2, 4))
)
bp <- ms_boxplotchart(dat, x = "group", y = "value")

doc <- read_pptx()
doc <- add_slide(doc)
doc <- ph_with(doc, bp, location = ph_location_fullsize())
print(doc, target = tempfile(fileext = ".pptx"))
}
\seealso{
\code{\link[=chart_labels]{chart_labels()}}

Other 'Office' chart objects:
\code{\link[=ms_areachart]{ms_areachart()}},
\code{\link[=ms_barchart]{ms_barchart()}},
\code{\link[=ms_bubblechart]{ms_bubblechart()}},
\code{\link[=ms_chart_combine]{ms_chart_combine()}},
\code{\link[=ms_funnelchart]{ms_funnelchart()}},
\code{\link[=ms_histogramchart]{ms_histogramchart()}},
\code{\link[=ms_linechart]{ms_linechart()}},
\code{\link[=ms_paretochart]{ms_paretochart()}},
\code{\link[=ms_piechart]{ms_piechart()}},
\code{\link[=ms_radarchart]{ms_radarchart()}},
\code{\link[=ms_scatterchart]{ms_scatterchart()}},
\code{\link[=ms_stockchart]{ms_stockchart()}},
\code{\link[=ms_sunburstchart]{ms_sunburstchart()}},
\code{\link[=ms_treemapchart]{ms_treemapchart()}},
\code{\link[=ms_waterfallchart]{ms_waterfallchart()}}
}
\concept{'Office' chart objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{set_theme}
\alias{set_theme}
\alias{mschart_theme}
\alias{chart_theme}
\title{Set chart theme}
\usage{
set_theme(x, value)

mschart_theme(
  axis_title = fp_text(bold = TRUE, font.size = 16),
  axis_title_x = axis_title,
  axis_title_y = axis_title,
  main_title = fp_text(bold = TRUE, font.size = 20),
  legend_text = fp_text(font.size = 14),
  table_text = fp_text(bold = FALSE, font.size = 9),
  axis_text = fp_text(),
  axis_text_x = axis_text,
  axis_text_y = axis_text,
  title_rot = 0,
  title_x_rot = 0,
  title_y_rot = 270,
  axis_ticks = fp_border(color = "#99999999"),
  axis_ticks_x = axis_ticks,
  axis_ticks_y = axis_ticks,
  grid_major_line = fp_border(color = "#99999999", style = "dashed"),
  grid_major_line_x = grid_major_line,
  grid_major_line_y = grid_major_line,
  grid_minor_line = fp_border(width = 0),
  grid_minor_line_x = grid_minor_line,
  grid_minor_line_y = grid_minor_line,
  chart_background = NULL,
  chart_border = fp_border(color = "transparent"),
  plot_background = NULL,
  plot_border = fp_border(color = "transparent"),
  date_fmt = "yyyy/mm/dd",
  str_fmt = "General",
  double_fmt = "#,##0.00",
  integer_fmt = "0",
  legend_position = "b",
  legend_x = NULL,
  legend_y = NULL,
  legend_w = NULL,
  legend_h = NULL
)

chart_theme(
  x,
  axis_title_x,
  axis_title_y,
  main_title,
  legend_text,
  title_rot,
  title_x_rot,
  title_y_rot,
  axis_text_x,
  axis_text_y,
  axis_ticks_x,
  axis_ticks_y,
  grid_major_line_x,
  grid_major_line_y,
  grid_minor_line_x,
  grid_minor_line_y,
  chart_background,
  chart_border,
  plot_background,
  plot_border,
  date_fmt,
  str_fmt,
  double_fmt,
  integer_fmt,
  legend_position,
  legend_x,
  legend_y,
  legend_w,
  legend_h
)
}
\arguments{
\item{x}{an \code{ms_chart} object.}

\item{value}{a \code{\link[=mschart_theme]{mschart_theme()}} object.}

\item{axis_title, axis_title_x, axis_title_y}{axis title formatting properties (see \code{\link[officer:fp_text]{officer::fp_text()}})}

\item{main_title}{title formatting properties (see \code{\link[officer:fp_text]{officer::fp_text()}})}

\item{legend_text}{legend text formatting properties (see \code{\link[officer:fp_text]{officer::fp_text()}})}

\item{table_text}{table text formatting properties (see \code{\link[officer:fp_text]{officer::fp_text()}})}

\item{axis_text, axis_text_x, axis_text_y}{axis text formatting properties (see \code{\link[officer:fp_text]{officer::fp_text()}})}

\item{title_rot, title_x_rot, title_y_rot}{rotation angle}

\item{axis_ticks, axis_ticks_x, axis_ticks_y}{axis ticks formatting properties (see \code{\link[officer:fp_border]{officer::fp_border()}})}

\item{grid_major_line, grid_major_line_x, grid_major_line_y}{major grid lines formatting properties (see \code{\link[officer:fp_border]{officer::fp_border()}})}

\item{grid_minor_line, grid_minor_line_x, grid_minor_line_y}{minor grid lines formatting properties (see \code{\link[officer:fp_border]{officer::fp_border()}})}

\item{chart_background}{chart area background fill color - single character value (e.g. "#000000" or "black")}

\item{chart_border}{chart area border lines formatting properties (see \code{\link[officer:fp_border]{officer::fp_border()}})}

\item{plot_background}{plot area background fill color - single character value (e.g. "#000000" or "black")}

\item{plot_border}{plot area border lines formatting properties (see \code{\link[officer:fp_border]{officer::fp_border()}})}

\item{date_fmt}{date format}

\item{str_fmt}{string or factor format}

\item{double_fmt}{double format}

\item{integer_fmt}{integer format}

\item{legend_position}{it specifies the position of the legend. It should be
one of 'b', 'tr', 'l', 'r', 't', 'n' (for 'none').}

\item{legend_x, legend_y, legend_w, legend_h}{optional fractions between
0 and 1 to manually position and size the legend box within the chart
area. Each value is a fraction of the chart width (\code{legend_x},
\code{legend_w}) or height (\code{legend_y}, \code{legend_h}). Any \code{NULL} value
keeps the default automatic layout; setting at least one triggers
manual placement via \verb{<c:manualLayout>}.}
}
\value{
An \code{ms_chart} object.

An \code{mschart_theme} object (for \code{mschart_theme()}).

An \code{ms_chart} object (for \code{set_theme()} and \code{chart_theme()}).
}
\description{
Modify chart theme with function \code{set_theme}.

Use \code{mschart_theme()} to create a chart theme.

Use \code{chart_theme()} to modify components of the theme of a chart.
}
\examples{
library(officer)
mytheme <- mschart_theme(
  axis_title = fp_text(color = "red", font.size = 24, bold = TRUE),
  grid_major_line_y = fp_border(width = 1, color = "orange"),
  axis_ticks_y = fp_border(width = 0.4, color = "gray")
)


my_bc <- ms_barchart(
  data = browser_data, x = "browser",
  y = "value", group = "serie"
)
my_bc <- chart_settings(my_bc,
  dir = "horizontal", grouping = "stacked",
  gap_width = 150, overlap = 100
)
my_bc <- set_theme(my_bc, mytheme)

my_bc_2 <- ms_barchart(
  data = browser_data, x = "browser",
  y = "value", group = "serie"
)
my_bc_2 <- chart_theme(my_bc_2,
  grid_major_line_y = fp_border(width = 0.5, color = "cyan")
)

# Manual legend layout: place the legend in the top-right corner
# using fractions of the chart area (0 to 1).
my_bc_3 <- ms_barchart(
  data = browser_data, x = "browser",
  y = "value", group = "serie"
)
my_bc_3 <- chart_theme(my_bc_3,
  legend_position = "r",
  legend_x = 0.80, legend_y = 0.15,
  legend_w = 0.18, legend_h = 0.30
)
}
\seealso{
\code{\link[=ms_barchart]{ms_barchart()}}, \code{\link[=ms_areachart]{ms_areachart()}}, \code{\link[=ms_scatterchart]{ms_scatterchart()}}, \code{\link[=ms_linechart]{ms_linechart()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_waterfallchart.R
\name{ms_waterfallchart}
\alias{ms_waterfallchart}
\title{Waterfall chart object}
\usage{
ms_waterfallchart(data, x, y, subtotals = NULL)
}
\arguments{
\item{data}{a data.frame.}

\item{x}{category column name.}

\item{y}{numeric value column name. Signed values: positive = up,
negative = down. For subtotal rows, the value should be the running
total at that point.}

\item{subtotals}{integer vector of 1-based row indices that should
be rendered as subtotal/total bars. Defaults to none.}
}
\value{
An \code{ms_chart} object (subclass \code{ms_waterfallchart}).
}
\description{
Creation of a waterfall chart object that can be inserted
in a 'Microsoft' document. Waterfall charts use the chartEx pipeline
(Office 2016+); older versions of 'Microsoft Office' will display a
fallback placeholder.

Each row is one bar. Positive values rise, negative values fall.
Categories listed in \code{subtotals} are rendered as absolute totals
(typical for "Start", intermediate totals, and "End" bars).
}
\examples{
library(officer)

dat <- data.frame(
  step = c("Start", "Q1", "Q2", "Q3", "End"),
  amount = c(100, 30, -20, 40, 150),
  stringsAsFactors = FALSE
)
wf <- ms_waterfallchart(
  data = dat, x = "step", y = "amount",
  subtotals = c(1, 5)
)

doc <- read_pptx()
doc <- add_slide(doc)
doc <- ph_with(doc, wf, location = ph_location_fullsize())
print(doc, target = tempfile(fileext = ".pptx"))
}
\seealso{
\code{\link[=chart_labels]{chart_labels()}}

Other 'Office' chart objects:
\code{\link[=ms_areachart]{ms_areachart()}},
\code{\link[=ms_barchart]{ms_barchart()}},
\code{\link[=ms_boxplotchart]{ms_boxplotchart()}},
\code{\link[=ms_bubblechart]{ms_bubblechart()}},
\code{\link[=ms_chart_combine]{ms_chart_combine()}},
\code{\link[=ms_funnelchart]{ms_funnelchart()}},
\code{\link[=ms_histogramchart]{ms_histogramchart()}},
\code{\link[=ms_linechart]{ms_linechart()}},
\code{\link[=ms_paretochart]{ms_paretochart()}},
\code{\link[=ms_piechart]{ms_piechart()}},
\code{\link[=ms_radarchart]{ms_radarchart()}},
\code{\link[=ms_scatterchart]{ms_scatterchart()}},
\code{\link[=ms_stockchart]{ms_stockchart()}},
\code{\link[=ms_sunburstchart]{ms_sunburstchart()}},
\code{\link[=ms_treemapchart]{ms_treemapchart()}}
}
\concept{'Office' chart objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_chart.R
\name{ms_barchart}
\alias{ms_barchart}
\title{Barchart object}
\usage{
ms_barchart(data, x, y, group = NULL, labels = NULL, asis = FALSE)
}
\arguments{
\item{data}{a data.frame}

\item{x}{column name for x values.}

\item{y}{column name for y values.}

\item{group}{grouping column name used to split data into series. Optional.}

\item{labels}{column names of columns to be used as custom data labels
displayed next to data points (not axis labels). Optional.
If more than one name is provided, only the first one will be used as a label, but all
labels (transposed if a group is used) will be available in the Excel file
associated with the chart.}

\item{asis}{logical parameter defaulting to FALSE. When FALSE, the data is
reshaped internally so that each series becomes a separate column. When TRUE,
the data is used as-is and must already have one column for categories and
one column per series.}
}
\value{
An \code{ms_chart} object.
}
\description{
Creation of a barchart object that can be
inserted in a 'Microsoft' document.

Bar charts illustrate comparisons among individual items. In a bar chart, the
categories are typically organized along the vertical axis, and the values
along the horizontal axis.

Consider using a bar chart when:
\itemize{
\item The axis labels are long.
\item The values that are shown are durations.
}
}
\section{Illustrations}{


\if{html}{\figure{fig_ms_barchart_1.png}{options: width="500"}}

\if{html}{\figure{fig_ms_barchart_2.png}{options: width="500"}}

\if{html}{\figure{fig_ms_barchart_3.png}{options: width="500"}}

\if{html}{\figure{fig_ms_barchart_4.png}{options: width="500"}}

\if{html}{\figure{fig_ms_barchart_5.png}{options: width="500"}}

\if{html}{\figure{fig_ms_barchart_6.png}{options: width="500"}}
}

\examples{
library(officer)
library(mschart)
library(officer)


# example chart 01 -------

chart_01 <- ms_barchart(
  data = browser_data, x = "browser",
  y = "value", group = "serie"
)
chart_01 <- chart_settings(
  x = chart_01, dir = "vertical",
  grouping = "clustered", gap_width = 50
)
chart_01 <- chart_ax_x(
  x = chart_01, cross_between = "between",
  major_tick_mark = "out"
)
chart_01 <- chart_ax_y(
  x = chart_01, cross_between = "midCat",
  major_tick_mark = "in"
)
# print(chart_01, preview = TRUE)


# example chart 02 -------
dat <- data.frame(
  Species = factor(c("setosa", "versicolor", "virginica"),
    levels = c("setosa", "versicolor", "virginica")
  ),
  mean = c(5.006, 5.936, 6.588)
)
chart_02 <- ms_barchart(data = dat, x = "Species", y = "mean")
chart_02 <- chart_settings(x = chart_02, dir = "horizontal")
chart_02 <- chart_theme(x = chart_02, title_x_rot = 270, title_y_rot = 0)
# print(chart_02, preview = TRUE)



# example chart 03 -------

mytheme <- mschart_theme(
  axis_title_x = fp_text(color = "gray", font.size = 20, bold = TRUE),
  axis_title_y = fp_text(color = "gray", font.size = 20, italic = TRUE),
  grid_major_line_y = fp_border(width = 1, color = "wheat"),
  axis_ticks_y = fp_border(width = 1, color = "gray")
)

chart_03 <- ms_barchart(
  data = browser_data, x = "browser",
  y = "value", group = "serie"
)
chart_03 <- chart_settings(chart_03,
  grouping = "stacked",
  gap_width = 150, overlap = 100
)
chart_03 <- chart_ax_x(chart_03,
  cross_between = "between",
  major_tick_mark = "out", minor_tick_mark = "none"
)
chart_03 <- chart_ax_y(chart_03,
  num_fmt = "0.00",
  minor_tick_mark = "none"
)
chart_03 <- set_theme(chart_03, mytheme)
chart_03 <- chart_labels(x = chart_03, title = "Things in percent")
chart_03 <- chart_data_labels(chart_03,
  position = "ctr",
  show_val = TRUE
)
chart_03 <- chart_labels_text(chart_03,
  fp_text(color = "white", bold = TRUE, font.size = 9)
)
chart_03 <- chart_data_fill(chart_03,
  values = c(
    serie1 = "#4477AA",
    serie2 = "#CC6677",
    serie3 = "#DDCC77"
  )
)
chart_03 <- chart_data_stroke(chart_03,
  values = c(
    serie1 = "#223B55",
    serie2 = "#66333C",
    serie3 = "#6F663C"
  )
)
chart_03 <- chart_data_line_width(chart_03,
  values = c(serie1 = 2, serie2 = 2, serie3 = 2)
)
# print(chart_03, preview = TRUE)

# example chart 04 -------

dat_groups <-
  data.frame(
    cut = c(
      "Fair", "Fair", "Fair", "Fair", "Fair",
      "Fair", "Fair", "Fair", "Good", "Good", "Good", "Good", "Good",
      "Good", "Good", "Good", "Very Good", "Very Good", "Very Good",
      "Very Good", "Very Good", "Very Good", "Very Good", "Very Good",
      "Premium", "Premium", "Premium", "Premium", "Premium",
      "Premium", "Premium", "Premium", "Ideal", "Ideal", "Ideal", "Ideal",
      "Ideal", "Ideal", "Ideal", "Ideal"
    ),
    clarity = c(
      "I1", "SI2", "SI1", "VS2", "VS1", "VVS2",
      "VVS1", "IF", "I1", "SI2", "SI1", "VS2", "VS1", "VVS2", "VVS1",
      "IF", "I1", "SI2", "SI1", "VS2", "VS1", "VVS2", "VVS1", "IF",
      "I1", "SI2", "SI1", "VS2", "VS1", "VVS2", "VVS1", "IF", "I1",
      "SI2", "SI1", "VS2", "VS1", "VVS2", "VVS1", "IF"
    ),
    carat = c(
      1.065, 1.01, 0.98, 0.9, 0.77, 0.7, 0.7,
      0.47, 1.07, 1, 0.79, 0.82, 0.7, 0.505, 0.4, 0.46, 1.145, 1.01,
      0.77, 0.71, 0.7, 0.4, 0.36, 0.495, 1.11, 1.04, 0.9, 0.72, 0.7,
      0.455, 0.4, 0.36, 1.13, 1, 0.71, 0.53, 0.53, 0.44, 0.4, 0.34
    ),
    n = c(
      210L, 466L, 408L, 261L, 170L, 69L, 17L, 9L,
      96L, 1081L, 1560L, 978L, 648L, 286L, 186L, 71L, 84L, 2100L,
      3240L, 2591L, 1775L, 1235L, 789L, 268L, 205L, 2949L, 3575L, 3357L,
      1989L, 870L, 616L, 230L, 146L, 2598L, 4282L, 5071L, 3589L,
      2606L, 2047L, 1212L
    )
  )

dat_groups$label <- sprintf(
  "carat median is \%.01f",
  dat_groups$carat
)
dat_groups

text_prop <- fp_text(font.size = 11, color = "gray")

chart_04 <- ms_barchart(
  data = dat_groups, x = "cut",
  labels = "label", y = "n", group = "clarity"
)
chart_04 <- chart_settings(chart_04,
  grouping = "clustered", dir = "horizontal",
  gap_width = 0
)
chart_04 <- chart_data_labels(chart_04, position = "outEnd")
chart_04 <- chart_labels_text(chart_04, text_prop)
chart_04 <- chart_theme(chart_04, title_x_rot = 270, title_y_rot = 0)
# print(chart_04, preview = TRUE)

# example chart 05 -------

dat_no_group <- data.frame(
  stringsAsFactors = FALSE,
  cut = c("Fair", "Good", "Very Good", "Premium", "Ideal"),
  carat = c(1, 0.82, 0.71, 0.86, 0.54),
  n = c(1610L, 4906L, 12082L, 13791L, 21551L),
  label = c(
    "carat median is 1.0",
    "carat median is 0.8", "carat median is 0.7",
    "carat median is 0.9", "carat median is 0.5"
  )
)
chart_05 <- ms_barchart(
  data = dat_no_group,
  x = "cut", labels = "label", y = "n"
)
chart_05 <- chart_settings(chart_05,
  grouping = "clustered"
)
chart_05 <- chart_data_labels(chart_05, position = "outEnd")
chart_05 <- chart_labels_text(chart_05, text_prop)
# print(chart_05, preview = TRUE)

# example chart 06 -------
chart_06 <- ms_barchart(
  data = dat_no_group,
  x = "cut", labels = "label", y = "n"
)
chart_06 <- chart_settings(chart_06,
  grouping = "clustered", table = TRUE
)
chart_06 <- chart_data_labels(chart_06, position = "outEnd")
chart_06 <- chart_labels_text(chart_06, text_prop)
# print(chart_06, preview = TRUE)


# example chart 07 -------
# Series order (and therefore legend order) follows the levels of
# the `group` factor. Convert the column to a factor with the desired
# level order before passing it to ms_barchart().
ordered_data <- browser_data
ordered_data$serie <- factor(ordered_data$serie,
  levels = c("serie3", "serie1", "serie2")
)
chart_07 <- ms_barchart(
  data = ordered_data, x = "browser",
  y = "value", group = "serie"
)
chart_07 <- chart_labels(chart_07,
  title = "Series and legend ordered via factor levels"
)
# print(chart_07, preview = TRUE)
}
\seealso{
\code{\link[=chart_settings]{chart_settings()}}, \code{\link[=chart_ax_x]{chart_ax_x()}}, \code{\link[=chart_ax_y]{chart_ax_y()}},
\code{\link[=chart_data_labels]{chart_data_labels()}}, \code{\link[=chart_theme]{chart_theme()}}, \code{\link[=chart_labels]{chart_labels()}}

Other 'Office' chart objects: 
\code{\link{ms_areachart}()},
\code{\link{ms_bubblechart}()},
\code{\link{ms_chart_combine}()},
\code{\link{ms_linechart}()},
\code{\link{ms_piechart}()},
\code{\link{ms_radarchart}()},
\code{\link{ms_scatterchart}()},
\code{\link{ms_stockchart}()}
}
\concept{'Office' chart objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chart_settings.R
\name{chart_settings}
\alias{chart_settings}
\alias{chart_settings.ms_barchart}
\alias{chart_settings.ms_linechart}
\alias{chart_settings.ms_areachart}
\alias{chart_settings.ms_scatterchart}
\alias{chart_settings.ms_stockchart}
\alias{chart_settings.ms_radarchart}
\alias{chart_settings.ms_bubblechart}
\alias{chart_settings.ms_piechart}
\title{Set chart options}
\usage{
chart_settings(x, ...)

\method{chart_settings}{ms_barchart}(x, vary_colors, gap_width, dir, grouping, overlap, table, ...)

\method{chart_settings}{ms_linechart}(x, vary_colors, style, grouping, table, ...)

\method{chart_settings}{ms_areachart}(x, vary_colors, grouping, table, ...)

\method{chart_settings}{ms_scatterchart}(x, vary_colors, style, ...)

\method{chart_settings}{ms_stockchart}(
  x,
  vary_colors,
  table,
  hi_low_lines,
  up_bars_fill,
  up_bars_border,
  down_bars_fill,
  down_bars_border,
  ...
)

\method{chart_settings}{ms_radarchart}(x, vary_colors, style, ...)

\method{chart_settings}{ms_bubblechart}(x, vary_colors, bubble3D = FALSE, ...)

\method{chart_settings}{ms_piechart}(x, vary_colors, hole_size, ...)
}
\arguments{
\item{x}{an \code{ms_chart} object.}

\item{...}{unused parameter}

\item{vary_colors}{if \code{TRUE}, each data point in a single series is displayed in a different color.}

\item{gap_width}{A gap appears between the bar or clustered bars for each category on a bar chart.
The default width for this gap is 150 percent of the bar width. It can be set
between 0 and 500 percent of the bar width.}

\item{dir}{the direction of the bars in the chart, value must be one of "horizontal" or "vertical".}

\item{grouping}{grouping of the series. For a barchart one of
"percentStacked", "clustered", "standard" or "stacked". For a
linechart or an areachart one of "percentStacked", "standard" or
"stacked" ("clustered" is bar-only).}

\item{overlap}{In a bar chart having two or more series, the bars for each
category are clustered together. By default, these bars are directly
adjacent to each other. The bars can be made to overlap each other or
have a space between them using the overlap property. Its values range
between -100 and 100, representing the percentage of the bar width by
which to overlap adjacent bars. A setting of -100 creates a gap of a
full bar width and a setting of 100 causes all the bars in a category
to be superimposed. The default value is 0.}

\item{table}{if \code{TRUE} set a table below the barchart.}

\item{style}{Style for the linechart or scatterchart type of markers. One
of 'none', 'line', 'lineMarker', 'marker', 'smooth', 'smoothMarker'.}

\item{hi_low_lines}{an \code{\link[officer:fp_border]{officer::fp_border()}} for the high-low lines.
Set to \code{FALSE} to hide them.}

\item{up_bars_fill}{fill colour for up bars (OHLC only, close > open).}

\item{up_bars_border}{an \code{\link[officer:fp_border]{officer::fp_border()}} for up bar borders.}

\item{down_bars_fill}{fill colour for down bars (OHLC only, close < open).}

\item{down_bars_border}{an \code{\link[officer:fp_border]{officer::fp_border()}} for down bar borders.}

\item{bubble3D}{logical, use 3D effect for bubbles.}

\item{hole_size}{size of the hole in a doughnut chart, between 0 and 90
(percent of the radius). Default 0 produces a pie chart;
values above 0 produce a doughnut chart.}
}
\value{
An \code{ms_chart} object.
}
\description{
Set chart properties.
}
\section{Methods (by class)}{
\itemize{
\item \code{chart_settings(ms_barchart)}: barchart settings

\item \code{chart_settings(ms_linechart)}: linechart settings

\item \code{chart_settings(ms_areachart)}: areachart settings

\item \code{chart_settings(ms_scatterchart)}: scatterchart settings

\item \code{chart_settings(ms_stockchart)}: stockchart settings

\item \code{chart_settings(ms_radarchart)}: radarchart settings

\item \code{chart_settings(ms_bubblechart)}: bubblechart settings

\item \code{chart_settings(ms_piechart)}: piechart settings

}}
\section{Illustrations}{


\if{html}{\figure{fig_chart_settings_1.png}{options: width="500"}}

\if{html}{\figure{fig_chart_settings_2.png}{options: width="500"}}

\if{html}{\figure{fig_chart_settings_3.png}{options: width="500"}}
}

\examples{
library(mschart)
library(officer)

chart_01 <- ms_barchart(
  data = browser_data, x = "browser",
  y = "value", group = "serie"
)
chart_01 <- chart_theme(chart_01,
  grid_major_line_x = fp_border(width = 0),
  grid_minor_line_x = fp_border(width = 0)
)

chart_02 <- chart_settings(
  x = chart_01,
  grouping = "stacked", overlap = 100
)


chart_03 <- ms_areachart(
  data = browser_ts, x = "date",
  y = "freq", group = "browser"
)
chart_03 <- chart_settings(chart_03,
  grouping = "percentStacked"
)
}
\seealso{
\code{\link[=ms_barchart]{ms_barchart()}}, \code{\link[=ms_areachart]{ms_areachart()}}, \code{\link[=ms_scatterchart]{ms_scatterchart()}}, \code{\link[=ms_linechart]{ms_linechart()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\name{us_indus_prod}
\alias{us_indus_prod}
\title{Index of US Industrial Production}
\format{
A data frame with 256 rows and 3 variables
}
\description{
Index of US industrial production (1985 = 100).
}
\details{
This is a transformation of the \code{USProdIndex} data
from package 'AER' into a simple data.frame.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\name{browser_ts}
\alias{browser_ts}
\title{Dummy dataset for time series charts}
\format{
A data frame with 36 rows and 3 variables
}
\description{
A dataset containing a date, a categorical, and an integer variable:
}
\details{
\itemize{
\item date date values
\item browser web browser
\item freq values in percent
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chart_data_labels.R
\name{chart_data_labels}
\alias{chart_data_labels}
\title{Modify data labels settings}
\usage{
chart_data_labels(
  x,
  num_fmt = "General",
  position = "ctr",
  show_legend_key = FALSE,
  show_val = FALSE,
  show_cat_name = FALSE,
  show_serie_name = FALSE,
  show_percent = FALSE,
  separator = ", "
)
}
\arguments{
\item{x}{an \code{ms_chart} object.}

\item{num_fmt}{\code{character(1)}: number formatting specifies number format properties which
indicate how to format and render the numeric values. It can be "General", "0.00", "#,##0",
"#,##0.00", "mm-dd-yy", "m/d/yy h:mm", etc.}

\item{position}{\code{character(1)}: it specifies the position of the data label.
It should be one of 'b', 'ctr', 'inBase', 'inEnd', 'l',
'outEnd', 'r', 't'.
When grouping is 'clustered', it should be one of 'ctr','inBase','inEnd','outEnd'.
When grouping is 'stacked', it should be one of 'ctr','inBase','inEnd'.
When grouping is 'standard', it should be one of 'b','ctr','l','r','t'.}

\item{show_legend_key}{show legend key if TRUE.}

\item{show_val}{show values if TRUE.}

\item{show_cat_name}{show categories if TRUE.}

\item{show_serie_name}{show names of series if TRUE.}

\item{show_percent}{show percentages if TRUE.}

\item{separator}{separator between the label components (value, category
name, series name, etc.) when multiple components are displayed.
Default is \code{", "}.}
}
\value{
An \code{ms_chart} object.
}
\description{
Data labels show details about data series. This function indicates that
data labels should be displayed. See \code{\link[=chart_labels_text]{chart_labels_text()}} for modifying
text settings associated with labels.
}
\examples{
my_bc <- ms_barchart(
  data = browser_data, x = "browser",
  y = "value", group = "serie"
)
my_bc <- chart_data_labels(my_bc, show_val = TRUE, position = "outEnd")
}
\seealso{
\code{\link[=chart_labels_text]{chart_labels_text()}}, \code{\link[=chart_labels]{chart_labels()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_paretochart.R
\name{ms_paretochart}
\alias{ms_paretochart}
\title{Pareto chart object}
\usage{
ms_paretochart(data, x, y = NULL, aggregate = TRUE)
}
\arguments{
\item{data}{a data.frame.}

\item{x}{category column name.}

\item{y}{optional numeric column. With \code{aggregate=TRUE} and \code{y=NULL},
each row counts as 1.}

\item{aggregate}{logical, see Description. Default TRUE.}
}
\value{
An \code{ms_chart} object (subclass \code{ms_paretochart}).
}
\description{
Creation of a Pareto chart object that can be inserted
in a 'Microsoft' document. Pareto charts use the chartEx pipeline
(Office 2016+); older versions of 'Microsoft Office' will display a
fallback placeholder.

Office draws columns sorted by descending count plus a cumulative
line on a secondary percentage axis.

Two input modes are supported:
\itemize{
\item \code{aggregate = TRUE} (default): \code{data} is in long format (one row per
observation) and Office counts occurrences of each \code{x} value.
\code{y} is optional; when supplied, values are summed per category.
\item \code{aggregate = FALSE}: \code{data} is already aggregated (one row per
category) and \code{y} is the count/value column.
}
}
\examples{
library(officer)

set.seed(1)
dat <- data.frame(
  defect = sample(c("A", "B", "C", "D"), 50, replace = TRUE,
                  prob = c(0.5, 0.25, 0.15, 0.1))
)
pa <- ms_paretochart(dat, x = "defect")

doc <- read_pptx()
doc <- add_slide(doc)
doc <- ph_with(doc, pa, location = ph_location_fullsize())
print(doc, target = tempfile(fileext = ".pptx"))
}
\seealso{
\code{\link[=chart_labels]{chart_labels()}}

Other 'Office' chart objects:
\code{\link[=ms_areachart]{ms_areachart()}},
\code{\link[=ms_barchart]{ms_barchart()}},
\code{\link[=ms_boxplotchart]{ms_boxplotchart()}},
\code{\link[=ms_bubblechart]{ms_bubblechart()}},
\code{\link[=ms_chart_combine]{ms_chart_combine()}},
\code{\link[=ms_funnelchart]{ms_funnelchart()}},
\code{\link[=ms_histogramchart]{ms_histogramchart()}},
\code{\link[=ms_linechart]{ms_linechart()}},
\code{\link[=ms_piechart]{ms_piechart()}},
\code{\link[=ms_radarchart]{ms_radarchart()}},
\code{\link[=ms_scatterchart]{ms_scatterchart()}},
\code{\link[=ms_stockchart]{ms_stockchart()}},
\code{\link[=ms_sunburstchart]{ms_sunburstchart()}},
\code{\link[=ms_treemapchart]{ms_treemapchart()}},
\code{\link[=ms_waterfallchart]{ms_waterfallchart()}}
}
\concept{'Office' chart objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chart_labels.R
\name{chart_labels}
\alias{chart_labels}
\title{Modify axis and plot labels}
\usage{
chart_labels(x, title = NULL, xlab = NULL, ylab = NULL)
}
\arguments{
\item{x}{an \code{ms_chart} object.}

\item{title}{title of the chart (displayed above the plot area). Use NULL to remove it.}

\item{xlab}{label for the x axis. Use NULL to remove it.}

\item{ylab}{label for the y axis. Use NULL to remove it.}
}
\value{
An \code{ms_chart} object.
}
\description{
Add labels to a chart, labels can be specified for
x axis, y axis and plot.
}
\examples{
mylc <- ms_linechart(
  data = browser_ts, x = "date", y = "freq",
  group = "browser"
)
mylc <- chart_labels(mylc,
  title = "my title", xlab = "my x label",
  ylab = "my y label"
)
}
\seealso{
\code{\link[=chart_data_labels]{chart_data_labels()}}, \code{\link[=chart_ax_x]{chart_ax_x()}}, \code{\link[=chart_ax_y]{chart_ax_y()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{as_bar_stack}
\alias{as_bar_stack}
\title{Set a barchart as a stacked barchart}
\usage{
as_bar_stack(x, dir = "vertical", percent = FALSE, gap_width = 50)
}
\arguments{
\item{x}{an \code{\link{ms_barchart}} object}

\item{dir}{the direction of the bars in the chart, value must be one of "horizontal" or "vertical".}

\item{percent}{should bars be displayed as percentages.}

\item{gap_width}{gap width between bars for each category on a bar chart, as a percentage
of the bar width. It can be set between 0 and 500.}
}
\value{
An \code{ms_chart} object.
}
\description{
Apply settings to an \code{ms_barchart} object to
produce a stacked barchart. Options are available to use percentage
instead of values and to choose if bars should be vertically or horizontally drawn.
}
\examples{
library(officer)

my_bar_stack_01 <- ms_barchart(data = browser_data, x = "browser",
  y = "value", group = "serie")
my_bar_stack_01 <- as_bar_stack( my_bar_stack_01 )

my_bar_stack_02 <- ms_barchart(data = browser_data, x = "browser",
  y = "value", group = "serie")
my_bar_stack_02 <- as_bar_stack( my_bar_stack_02, percent = TRUE,
  dir = "horizontal" )

doc <- read_pptx()
doc <- add_slide(doc, layout = "Title and Content", master = "Office Theme")
doc <- ph_with(doc, my_bar_stack_02, location = ph_location_fullsize())

fileout <- tempfile(fileext = ".pptx")
print(doc, target = fileout)
}
\seealso{
\code{\link[=chart_settings]{chart_settings()}}, \code{\link[=ms_barchart]{ms_barchart()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/axis_setting.R
\name{chart_ax_x}
\alias{chart_ax_x}
\title{X axis settings}
\usage{
chart_ax_x(
  x,
  orientation,
  crosses,
  cross_between,
  major_tick_mark,
  minor_tick_mark,
  tick_label_pos,
  display,
  num_fmt,
  rotation,
  limit_min,
  limit_max,
  position,
  major_unit,
  minor_unit,
  major_time_unit,
  minor_time_unit
)
}
\arguments{
\item{x}{an \code{ms_chart} object.}

\item{orientation}{axis orientation, one of 'maxMin', 'minMax'.}

\item{crosses}{specifies how the axis crosses the perpendicular
axis, one of 'autoZero', 'max', 'min'.}

\item{cross_between}{specifies how the value axis crosses the
category axis between categories, one of 'between', 'midCat'.}

\item{major_tick_mark, minor_tick_mark}{tick marks position,
one of 'cross', 'in', 'none', 'out'.}

\item{tick_label_pos}{ticks labels position, one of 'high',
'low', 'nextTo', 'none'.}

\item{display}{should the axis be displayed (a logical of
length 1).}

\item{num_fmt}{number formatting. See the num_fmt section for more details.}

\item{rotation}{rotation angle. Value should be between \code{-360}
and \code{360}.}

\item{limit_min}{minimum value on the axis. Date objects are also accepted
and will be converted automatically.}

\item{limit_max}{maximum value on the axis. Date objects are also accepted
and will be converted automatically.}

\item{position}{the value at which this axis crosses the perpendicular axis.}

\item{major_unit}{numeric, interval between major ticks and gridlines.}

\item{minor_unit}{numeric, interval between minor ticks and gridlines.}

\item{major_time_unit}{time unit for major ticks on date axes,
one of \code{"days"}, \code{"months"}, \code{"years"}.}

\item{minor_time_unit}{time unit for minor ticks on date axes,
one of \code{"days"}, \code{"months"}, \code{"years"}.}
}
\value{
An \code{ms_chart} object.
}
\description{
Define settings for an x axis.
}
\section{num_fmt}{

All \verb{\%} need to be doubled, \verb{0\%\%} means "a number
and percent symbol".

To my current knowledge, depending on the chart type
and options, the following values are not systematically
used by office chart engine; i.e. when chart pre-compute
percentages, it seems using \verb{0\%\%} will have no
effect.
\itemize{
\item \code{General}: default value
\item \code{0}: display the number with no decimal
\item \code{0.00}: display the number with two decimals
\item \verb{0\%\%}: display as percentages
\item \verb{0.00\%\%}: display as percentages with two decimal places
\item \verb{#,##0}
\item \verb{#,##0.00}
\item \code{0.00E+00}
\item \verb{# ?/?}
\item \verb{# ??/??}
\item \code{mm-dd-yy}
\item \code{d-mmm-yy}
\item \code{d-mmm}
\item \code{mmm-yy}
\item \verb{h:mm AM/PM}
\item \verb{h:mm:ss AM/PM}
\item \code{h:mm}
\item \code{h:mm:ss}
\item \verb{m/d/yy h:mm}
\item \verb{#,##0 ;(#,##0)}
\item \verb{#,##0 ;[Red](#,##0)}
\item \verb{#,##0.00;(#,##0.00)}
\item \verb{#,##0.00;[Red](#,##0.00)}
\item \code{mm:ss}
\item \verb{[h]:mm:ss}
\item \code{mmss.0}
\item \verb{##0.0E+0}
\item \code{@}
}
}

\section{Illustrations}{


\if{html}{\figure{fig_chart_ax_x_1.png}{options: width="500"}}
}

\examples{
library(mschart)

chart_01 <- ms_linechart(
  data = us_indus_prod,
  x = "date", y = "value",
  group = "type"
)

chart_01 <- chart_ax_y(x = chart_01, limit_min = 20, limit_max = 120)
chart_01

# control axis intervals
chart_01 <- chart_ax_x(chart_01,
  major_unit = 10, major_time_unit = "years"
)
chart_01 <- chart_ax_y(chart_01, major_unit = 20)
chart_01
}
\seealso{
\code{\link[=chart_ax_y]{chart_ax_y()}}, \code{\link[=ms_areachart]{ms_areachart()}}, \code{\link[=ms_barchart]{ms_barchart()}}, \code{\link[=ms_scatterchart]{ms_scatterchart()}},
\code{\link[=ms_linechart]{ms_linechart()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_setting.R
\name{chart_table}
\alias{chart_table}
\title{Data table settings}
\usage{
chart_table(x, horizontal, vertical, outline, show_keys)
}
\arguments{
\item{x}{an \code{ms_chart} object.}

\item{horizontal}{write horizontal lines in the table}

\item{vertical}{write vertical lines in the table}

\item{outline}{write an outline in the table}

\item{show_keys}{show keys in the table}
}
\value{
An \code{ms_chart} object.
}
\description{
Define visual settings for the data table displayed below the
chart. Requires \code{chart_settings(x, table = TRUE)} to be called first.
}
\examples{
data <- data.frame(
  supp = factor(rep(c("OJ", "VC"), each = 3),
                levels = c("OJ", "VC")),
  dose = factor(rep(c("low", "medium", "high"), 2),
                levels = c("low", "medium", "high")),
  length = c(13.23, 22.7, 24.06, 7.98, 16.77, 26.14),
  label = LETTERS[1:6],
  stringsAsFactors = FALSE
)

# example chart 03 -------
chart <- ms_linechart(
  data = data, x = "dose", y = "length",
  group = "supp", labels = "label"
)
chart <- chart_settings(
  x = chart, table = TRUE
)

chart <- chart_table(chart,
  horizontal = TRUE, vertical = FALSE,
  outline = TRUE, show_keys = FALSE
)
}
\seealso{
\code{\link[=chart_settings]{chart_settings()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chart_data_styles.R
\name{chart_data_size}
\alias{chart_data_size}
\title{Modify symbol size}
\usage{
chart_data_size(x, values)
}
\arguments{
\item{x}{an \code{ms_chart} object.}

\item{values}{\verb{double(num of series)}: a set of size values to map data values to.
It is a named vector, the values will be matched based on the names.
If it contains only one size, this size will be associated to all existing series.}
}
\value{
An \code{ms_chart} object.
}
\description{
Specify mappings from levels in the data to displayed size of symbols.
}
\examples{
my_scatter <- ms_scatterchart(data = iris, x = "Sepal.Length",
  y = "Sepal.Width",  group = "Species")
my_scatter <- chart_data_fill(my_scatter,
  values = c(virginica = "#6FA2FF", versicolor = "#FF6161", setosa = "#81FF5B") )
my_scatter <- chart_data_stroke(my_scatter,
  values = c(virginica = "black", versicolor = "black", setosa = "black") )
my_scatter <- chart_data_symbol(my_scatter,
  values = c(virginica = "circle", versicolor = "diamond", setosa = "circle") )
my_scatter <- chart_data_size(my_scatter,
  values = c(virginica = 20, versicolor = 16, setosa = 20) )
}
\seealso{
Other Series customization functions: 
\code{\link{chart_data_fill}()},
\code{\link{chart_data_line_style}()},
\code{\link{chart_data_line_width}()},
\code{\link{chart_data_smooth}()},
\code{\link{chart_data_stroke}()},
\code{\link{chart_data_symbol}()},
\code{\link{chart_labels_text}()}
}
\concept{Series customization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chart_data_styles.R
\name{chart_data_smooth}
\alias{chart_data_smooth}
\title{Smooth series}
\usage{
chart_data_smooth(x, values)
}
\arguments{
\item{x}{an \code{ms_chart} object.}

\item{values}{\verb{integer(num of series)}: a set of smooth values to map data values to.
It is a named vector, the values will be matched based on the names.
Use \code{0} to disable smoothing and \code{1} to enable it.
If it contains only one integer it will be associated to all existing series.}
}
\value{
An \code{ms_chart} object.
}
\description{
Specify whether lines should be smoothed, per series. This
feature only applies to \code{\link[=ms_linechart]{ms_linechart()}}.
}
\examples{
linec <- ms_linechart(data = iris, x = "Sepal.Length",
  y = "Sepal.Width", group = "Species")
linec <- chart_data_smooth(linec,
  values = c(virginica = 0, versicolor = 0, setosa = 0) )
}
\seealso{
Other Series customization functions: 
\code{\link{chart_data_fill}()},
\code{\link{chart_data_line_style}()},
\code{\link{chart_data_line_width}()},
\code{\link{chart_data_size}()},
\code{\link{chart_data_stroke}()},
\code{\link{chart_data_symbol}()},
\code{\link{chart_labels_text}()}
}
\concept{Series customization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_histogramchart.R
\name{ms_histogramchart}
\alias{ms_histogramchart}
\title{Histogram chart object}
\usage{
ms_histogramchart(
  data,
  value,
  bin_count = NULL,
  bin_width = NULL,
  interval_closed = c("right", "left"),
  underflow = NULL,
  overflow = NULL
)
}
\arguments{
\item{data}{a data.frame.}

\item{value}{numeric column name (raw observations).}

\item{bin_count}{integer, requested number of bins. Mutually exclusive
with \code{bin_width}. NULL = automatic.}

\item{bin_width}{numeric, requested bin width. Mutually exclusive with
\code{bin_count}. NULL = automatic.}

\item{interval_closed}{one of \code{"right"} (default) or \code{"left"}. Defines
which end of each bin is inclusive.}

\item{underflow}{numeric, values below this go in an "underflow" bin.
NULL to disable.}

\item{overflow}{numeric, values above this go in an "overflow" bin.
NULL to disable.}
}
\value{
An \code{ms_chart} object (subclass \code{ms_histogramchart}).
}
\description{
Creation of a histogram chart object that can be inserted
in a 'Microsoft' document. Histogram charts use the chartEx pipeline
(Office 2016+); older versions of 'Microsoft Office' will display a
fallback placeholder.

Data is a single column of raw numeric observations. Office computes
the bins automatically; pass \code{bin_count} or \code{bin_width} to override.
}
\examples{
library(officer)

set.seed(1)
dat <- data.frame(x = rnorm(200, mean = 10, sd = 4))
hi <- ms_histogramchart(dat, value = "x", bin_count = 12)

doc <- read_pptx()
doc <- add_slide(doc)
doc <- ph_with(doc, hi, location = ph_location_fullsize())
print(doc, target = tempfile(fileext = ".pptx"))
}
\seealso{
\code{\link[=chart_labels]{chart_labels()}}

Other 'Office' chart objects:
\code{\link[=ms_areachart]{ms_areachart()}},
\code{\link[=ms_barchart]{ms_barchart()}},
\code{\link[=ms_boxplotchart]{ms_boxplotchart()}},
\code{\link[=ms_bubblechart]{ms_bubblechart()}},
\code{\link[=ms_chart_combine]{ms_chart_combine()}},
\code{\link[=ms_funnelchart]{ms_funnelchart()}},
\code{\link[=ms_linechart]{ms_linechart()}},
\code{\link[=ms_paretochart]{ms_paretochart()}},
\code{\link[=ms_piechart]{ms_piechart()}},
\code{\link[=ms_radarchart]{ms_radarchart()}},
\code{\link[=ms_scatterchart]{ms_scatterchart()}},
\code{\link[=ms_stockchart]{ms_stockchart()}},
\code{\link[=ms_sunburstchart]{ms_sunburstchart()}},
\code{\link[=ms_treemapchart]{ms_treemapchart()}},
\code{\link[=ms_waterfallchart]{ms_waterfallchart()}}
}
\concept{'Office' chart objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xlsx_add_chart.R
\name{sheet_add_drawing.ms_chart}
\alias{sheet_add_drawing.ms_chart}
\title{Add an ms_chart to an Excel sheet}
\usage{
\method{sheet_add_drawing}{ms_chart}(
  x,
  value,
  sheet,
  start_col = 1L,
  start_row = 1L,
  write_data = TRUE,
  left = 1,
  top = 1,
  width = 6,
  height = 4,
  ...
)
}
\arguments{
\item{x}{an rxlsx object (created by \code{\link[officer:read_xlsx]{officer::read_xlsx()}})}

\item{value}{an \code{ms_chart} object}

\item{sheet}{sheet name where the chart and its data will be placed.
The sheet must already exist (see \code{\link[officer:add_sheet]{officer::add_sheet()}}).}

\item{start_col}{column index where chart data will be written
(default 1, i.e. column A). When \code{write_data = FALSE}, this is still
the cell position that the chart XML will point at, but no data is
written.}

\item{start_row}{row index where chart data will be written
(default 1). Same semantics with \code{write_data = FALSE} as for
\code{start_col}.}

\item{write_data}{if \code{TRUE} (the default), the chart's
\code{data_series} is written into the sheet at
\verb{(start_col, start_row)} before the chart is added.
Pass \code{FALSE} when the data is already present in the sheet (for
example to avoid rewriting it when several charts share the same
dataset, or when inserting a chart that references data written
independently via \code{\link[officer:sheet_write_data]{officer::sheet_write_data()}}).}

\item{left, top}{top-left anchor of the chart, in inches.
Defaults to \verb{(1, 1)}. Same convention as
\code{\link[officer:sheet_add_drawing]{officer::sheet_add_drawing()}} and \code{rvg::sheet_add_drawing.dml()}.}

\item{width, height}{size of the chart, in inches. Defaults
to \verb{6 x 4}.}

\item{...}{unused}
}
\value{
the rxlsx object (invisibly)
}
\description{
Add an \code{ms_chart} object to a sheet in an xlsx workbook
created with \code{\link[officer:read_xlsx]{officer::read_xlsx()}}. The chart data is written into
the sheet and the chart is displayed at the specified position.
}
\examples{
library(officer)
library(mschart)

my_chart <- ms_barchart(
  data = data.frame(
    x = c("A", "B", "C"),
    y = c(1, 3, 2),
    group = rep("serie1", 3)
  ),
  x = "x", y = "y", group = "group"
)

x <- read_xlsx()
x <- add_sheet(x, label = "chart_sheet")
x <- sheet_add_drawing(x, value = my_chart, sheet = "chart_sheet")
print(x, target = tempfile(fileext = ".xlsx"))

# Sharing one dataset between several charts on the same sheet:
# write the data once, then add each chart with write_data = FALSE.
shared <- data.frame(
  x = c("A", "B", "C"),
  y = c(1, 3, 2),
  group = rep("serie1", 3)
)
chart_a <- ms_barchart(shared, x = "x", y = "y", group = "group")
chart_b <- ms_linechart(shared, x = "x", y = "y", group = "group")

x <- read_xlsx()
x <- add_sheet(x, label = "multi")
x <- sheet_write_data(x, value = chart_a$data_series, sheet = "multi")
x <- sheet_add_drawing(x, value = chart_a, sheet = "multi",
                       write_data = FALSE,
                       left = 3, top = 0.5, width = 5, height = 3.5)
x <- sheet_add_drawing(x, value = chart_b, sheet = "multi",
                       write_data = FALSE,
                       left = 9, top = 0.5, width = 5, height = 3.5)
print(x, target = tempfile(fileext = ".xlsx"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_chart.R
\name{ms_piechart}
\alias{ms_piechart}
\title{Piechart object}
\usage{
ms_piechart(data, x, y, labels = NULL)
}
\arguments{
\item{data}{a data.frame}

\item{x}{column name for categories (slices).}

\item{y}{column name for values (slice sizes).}

\item{labels}{column names of columns to be used as custom data labels
displayed next to data points (not axis labels). Optional.
If more than one name is provided, only the first one will be used as a label, but all
labels (transposed if a group is used) will be available in the Excel file
associated with the chart.}
}
\value{
An \code{ms_chart} object.
}
\description{
Creation of a piechart object that can be
inserted in a 'Microsoft' document.

Pie charts show the proportion of each category as a slice
of a circle. Doughnut charts are similar but have a hole
in the centre. Use \code{chart_settings(x, hole_size = ...)} to
control the hole size: 0 produces a pie chart, values
above 0 produce a doughnut chart.

Data must be pre-aggregated: one row per slice, no grouping
column.
}
\examples{
library(officer)
library(mschart)

dat <- data.frame(
  browser = c("Chrome", "Firefox", "Safari", "Edge", "Other"),
  value = c(64, 12, 8, 5, 11)
)

# Pie chart
pie <- ms_piechart(data = dat, x = "browser", y = "value")
pie <- chart_labels(pie, title = "Browser share")

# Doughnut chart
donut <- ms_piechart(data = dat, x = "browser", y = "value")
donut <- chart_settings(donut, hole_size = 50)
donut <- chart_labels(donut, title = "Browser share (donut)")
}
\seealso{
\code{\link[=chart_settings]{chart_settings()}}, \code{\link[=chart_data_labels]{chart_data_labels()}}, \code{\link[=chart_theme]{chart_theme()}}, \code{\link[=chart_labels]{chart_labels()}}

Other 'Office' chart objects: 
\code{\link{ms_areachart}()},
\code{\link{ms_barchart}()},
\code{\link{ms_bubblechart}()},
\code{\link{ms_chart_combine}()},
\code{\link{ms_linechart}()},
\code{\link{ms_radarchart}()},
\code{\link{ms_scatterchart}()},
\code{\link{ms_stockchart}()}
}
\concept{'Office' chart objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_chart.R
\name{ms_chart_combine}
\alias{ms_chart_combine}
\title{Combined chart object}
\usage{
ms_chart_combine(..., secondary_y = NULL, secondary_x = NULL)
}
\arguments{
\item{...}{named \code{ms_chart} objects.}

\item{secondary_y}{character vector of chart names to plot on
the secondary (right) y-axis.}

\item{secondary_x}{character vector of chart names to plot on
the secondary (top) x-axis.}
}
\description{
Combine several chart objects into a single chart
with shared axes. Each chart must be a named argument.

The title and x-axis label are taken from the first chart.
The y-axis label of the first chart on the secondary axis is
used as the secondary y-axis label.

Only one secondary y-axis (right) and one secondary x-axis (top)
are supported.
}
\examples{
# example chart 01 -------

dat <- longley
dat$Year <- as.Date(paste0(dat$Year, "-01-01"))

dat_empl <- data.frame(
  Year = rep(dat$Year, 2),
  value = c(dat$Employed, dat$Unemployed),
  serie = rep(c("Employed", "Unemployed"), each = nrow(dat)),
  stringsAsFactors = FALSE
)

empl <- ms_barchart(
  data = dat_empl, x = "Year",
  y = "value", group = "serie"
)
empl <- chart_labels(
  x = empl, title = "Longley's Economic Regression Data",
  xlab = "1947 to 1962", ylab = "Employment"
)
empl <- chart_ax_x(x = empl, num_fmt = "yyyy")
empl <- as_bar_stack(x = empl, dir = "vertical")

gdp <- ms_linechart(data = dat, x = "Year", y = "GNP.deflator")
gdp <- chart_labels(x = gdp, ylab = "GNP implicit price deflator (1954 = 100)")

cb <- ms_chart_combine(empl = empl, gdp = gdp, secondary_y = "gdp")

# example chart 02 -------
# stacked bars + target line on a shared axis (no secondary axis).

browser_data <- data.frame(
  browser = rep(c("Android", "Chrome", "IE", "Firefox", "Opera", "Safari"),
                each = 3),
  serie   = rep(c("v1", "v2", "v3"), times = 6),
  value   = c(80, 90, 77, 150, 140, 131, 60, 70, 62,
              50, 60, 52, 40, 50, 42, 55, 65, 55)
)

target <- data.frame(
  browser = unique(browser_data$browser),
  cible   = 220
)

bars <- ms_barchart(browser_data, x = "browser",
                    y = "value", group = "serie")
bars <- as_bar_stack(bars)

line <- ms_linechart(target, x = "browser", y = "cible")
line <- chart_data_stroke(line, values = "#D62728")
line <- chart_data_line_width(line, values = 2)
line <- chart_data_symbol(line, values = "none")

cb2 <- ms_chart_combine(bars = bars, target = line)

# example chart 03 -------
# two scatter charts on a shared y axis with independent x ranges.
# bottom (primary) chart uses temperature in Celsius;
# top (secondary_x) chart uses pressure in hPa.
# y series must have distinct column names (no collision in the
# embedded sheet) — here humidity_t vs humidity_p.

set.seed(1)
n <- 20
weather <- data.frame(
  temperature_c = runif(n, 5, 30),
  pressure_hpa  = runif(n, 990, 1030),
  humidity_t    = runif(n, 30, 90),
  humidity_p    = runif(n, 30, 90)
)

sc_temp <- ms_scatterchart(
  data = weather, x = "temperature_c", y = "humidity_t"
)
sc_temp <- chart_labels(
  sc_temp, title = "Humidity vs temperature and pressure",
  xlab = "Temperature (°C)", ylab = "Humidity (\%)"
)

sc_pres <- ms_scatterchart(
  data = weather, x = "pressure_hpa", y = "humidity_p"
)
sc_pres <- chart_data_symbol(sc_pres, values = "triangle")
sc_pres <- chart_labels(sc_pres, xlab = "Pressure (hPa)")

cb3 <- ms_chart_combine(
  temp = sc_temp, pres = sc_pres,
  secondary_x = "pres"
)
}
\seealso{
\code{\link[=chart_settings]{chart_settings()}}, \code{\link[=chart_ax_x]{chart_ax_x()}}, \code{\link[=chart_ax_y]{chart_ax_y()}},
\code{\link[=chart_data_labels]{chart_data_labels()}}, \code{\link[=chart_theme]{chart_theme()}}, \code{\link[=chart_labels]{chart_labels()}}

Other 'Office' chart objects:
\code{\link[=ms_areachart]{ms_areachart()}},
\code{\link[=ms_barchart]{ms_barchart()}},
\code{\link[=ms_boxplotchart]{ms_boxplotchart()}},
\code{\link[=ms_bubblechart]{ms_bubblechart()}},
\code{\link[=ms_funnelchart]{ms_funnelchart()}},
\code{\link[=ms_histogramchart]{ms_histogramchart()}},
\code{\link[=ms_linechart]{ms_linechart()}},
\code{\link[=ms_paretochart]{ms_paretochart()}},
\code{\link[=ms_piechart]{ms_piechart()}},
\code{\link[=ms_radarchart]{ms_radarchart()}},
\code{\link[=ms_scatterchart]{ms_scatterchart()}},
\code{\link[=ms_stockchart]{ms_stockchart()}},
\code{\link[=ms_sunburstchart]{ms_sunburstchart()}},
\code{\link[=ms_treemapchart]{ms_treemapchart()}},
\code{\link[=ms_waterfallchart]{ms_waterfallchart()}}
}
\concept{'Office' chart objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_chart.R
\name{ms_areachart}
\alias{ms_areachart}
\title{Areachart object}
\usage{
ms_areachart(data, x, y, group = NULL, labels = NULL, asis = FALSE)
}
\arguments{
\item{data}{a data.frame}

\item{x}{column name for x values.}

\item{y}{column name for y values.}

\item{group}{grouping column name used to split data into series. Optional.}

\item{labels}{column names of columns to be used as custom data labels
displayed next to data points (not axis labels). Optional.
If more than one name is provided, only the first one will be used as a label, but all
labels (transposed if a group is used) will be available in the Excel file
associated with the chart.}

\item{asis}{logical parameter defaulting to FALSE. When FALSE, the data is
reshaped internally so that each series becomes a separate column. When TRUE,
the data is used as-is and must already have one column for categories and
one column per series, and \code{y} accepts a vector of series column names.
\code{asis} describes the \emph{input shape} read by the constructor. Not to be
confused with the \code{write_data} argument of \code{\link[=sheet_add_drawing.ms_chart]{sheet_add_drawing.ms_chart()}},
which controls whether \code{mschart} writes the chart's data into an Excel
sheet at embed time. The two are independent.}
}
\value{
An \code{ms_chart} object.
}
\description{
Creation of an areachart object that can be
inserted in a 'Microsoft' document.

Area charts can be used to plot change over time and draw attention to the
total value across a trend. By showing the sum of the plotted values, an area
chart also shows the relationship of parts to a whole.
}
\examples{
library(officer)
mytheme <- mschart_theme(
  axis_title_x = fp_text(color = "red", font.size = 24, bold = TRUE),
  axis_title_y = fp_text(color = "green", font.size = 12, italic = TRUE),
  grid_major_line_y = fp_border(width = 1, color = "orange"),
  axis_ticks_y = fp_border(width = 1, color = "orange")
)

# example ac_01 -------
ac_01 <- ms_areachart(
  data = browser_ts, x = "date",
  y = "freq", group = "browser"
)
ac_01 <- chart_ax_y(ac_01, cross_between = "between", num_fmt = "General")
ac_01 <- chart_ax_x(ac_01, cross_between = "midCat", num_fmt = "m/d/yy")
ac_01 <- set_theme(ac_01, mytheme)


# example ac_02 -------
ac_02 <- chart_settings(ac_01, grouping = "percentStacked")

# example ac_03 -------
ac_03 <- chart_settings(ac_01, grouping = "percentStacked", table = TRUE)
ac_03 <- chart_table(
  ac_03,
  horizontal = FALSE, vertical = FALSE,
  outline = FALSE, show_keys = TRUE)
}
\seealso{
\code{\link[=chart_settings]{chart_settings()}}, \code{\link[=chart_ax_x]{chart_ax_x()}}, \code{\link[=chart_ax_y]{chart_ax_y()}},
\code{\link[=chart_data_labels]{chart_data_labels()}}, \code{\link[=chart_theme]{chart_theme()}}, \code{\link[=chart_labels]{chart_labels()}}

Other 'Office' chart objects:
\code{\link[=ms_barchart]{ms_barchart()}},
\code{\link[=ms_boxplotchart]{ms_boxplotchart()}},
\code{\link[=ms_bubblechart]{ms_bubblechart()}},
\code{\link[=ms_chart_combine]{ms_chart_combine()}},
\code{\link[=ms_funnelchart]{ms_funnelchart()}},
\code{\link[=ms_histogramchart]{ms_histogramchart()}},
\code{\link[=ms_linechart]{ms_linechart()}},
\code{\link[=ms_paretochart]{ms_paretochart()}},
\code{\link[=ms_piechart]{ms_piechart()}},
\code{\link[=ms_radarchart]{ms_radarchart()}},
\code{\link[=ms_scatterchart]{ms_scatterchart()}},
\code{\link[=ms_stockchart]{ms_stockchart()}},
\code{\link[=ms_sunburstchart]{ms_sunburstchart()}},
\code{\link[=ms_treemapchart]{ms_treemapchart()}},
\code{\link[=ms_waterfallchart]{ms_waterfallchart()}}
}
\concept{'Office' chart objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ms_treemapchart.R
\name{ms_treemapchart}
\alias{ms_treemapchart}
\title{Treemap chart object}
\usage{
ms_treemapchart(data, path, value, labels = NULL)
}
\arguments{
\item{data}{a data.frame.}

\item{path}{character vector of column names defining the hierarchy,
from outermost (root) to innermost (leaf).}

\item{value}{column name for the numeric leaf values.}

\item{labels}{unused for now; reserved for future custom data label
columns.}
}
\value{
An \code{ms_chart} object (subclass \code{ms_treemapchart}).
}
\description{
Creation of a treemap chart object that can be inserted
in a 'Microsoft' document. Treemap charts use the chartEx pipeline
(Office 2016+); older versions of 'Microsoft Office' will display a
fallback placeholder.

Data is hierarchical: one column per level (parent to leaf, left to
right) and one numeric column for the leaf values.
}
\section{Per-leaf coloring with \code{chart_data_fill()}}{

Per-leaf coloring via \code{\link[=chart_data_fill]{chart_data_fill()}} with a named vector works
correctly on flat treemaps (\code{length(path) == 1}). With a hierarchy
(\code{length(path) >= 2}), PowerPoint silently re-maps \verb{<cx:dataPt>}
indices in a way that drops \code{idx="0"} and applies the last specified
color to remaining leaves. This is a PowerPoint rendering limitation
(also reproducible from Excel-generated chartEx files); the mschart
XML output is conformant. For hierarchical treemaps, prefer a single
fill color via \code{chart_data_fill(x, "#HEX")}.
}

\examples{
library(officer)

dat <- data.frame(
  region  = c("EU", "EU", "EU", "AM", "AM"),
  country = c("FR", "FR", "DE", "US", "US"),
  city    = c("Paris", "Lyon", "Berlin", "NYC", "LA"),
  value   = c(10, 5, 12, 20, 8),
  stringsAsFactors = FALSE
)
tm <- ms_treemapchart(
  data = dat, path = c("region", "country", "city"), value = "value"
)
tm <- chart_labels(tm, title = "Sales by region")

doc <- read_pptx()
doc <- add_slide(doc)
doc <- ph_with(doc, tm, location = ph_location_fullsize())
print(doc, target = tempfile(fileext = ".pptx"))
}
\seealso{
\code{\link[=chart_labels]{chart_labels()}}, \code{\link[=chart_theme]{chart_theme()}}

Other 'Office' chart objects:
\code{\link[=ms_areachart]{ms_areachart()}},
\code{\link[=ms_barchart]{ms_barchart()}},
\code{\link[=ms_boxplotchart]{ms_boxplotchart()}},
\code{\link[=ms_bubblechart]{ms_bubblechart()}},
\code{\link[=ms_chart_combine]{ms_chart_combine()}},
\code{\link[=ms_funnelchart]{ms_funnelchart()}},
\code{\link[=ms_histogramchart]{ms_histogramchart()}},
\code{\link[=ms_linechart]{ms_linechart()}},
\code{\link[=ms_paretochart]{ms_paretochart()}},
\code{\link[=ms_piechart]{ms_piechart()}},
\code{\link[=ms_radarchart]{ms_radarchart()}},
\code{\link[=ms_scatterchart]{ms_scatterchart()}},
\code{\link[=ms_stockchart]{ms_stockchart()}},
\code{\link[=ms_sunburstchart]{ms_sunburstchart()}},
\code{\link[=ms_waterfallchart]{ms_waterfallchart()}}
}
\concept{'Office' chart objects}
